(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('react-dom')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'react-dom'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ReactFlowEditorEngine = {}, global.React, global.ReactDOM));
}(this, (function (exports, React, ReactDOM) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
      Object.keys(e).forEach(function (k) {
        if (k !== 'default') {
          var d = Object.getOwnPropertyDescriptor(e, k);
          Object.defineProperty(n, k, d.get ? d : {
            enumerable: true,
            get: function () {
              return e[k];
            }
          });
        }
      });
    }
    n['default'] = e;
    return Object.freeze(n);
  }

  var React__namespace = /*#__PURE__*/_interopNamespace(React);
  var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
  var ReactDOM__namespace = /*#__PURE__*/_interopNamespace(ReactDOM);

  var SELECTED = 1;
  exports.GraphEdgeState = void 0;
  (function (GraphEdgeState) {
      GraphEdgeState[GraphEdgeState["default"] = 0] = "default";
      GraphEdgeState[GraphEdgeState["selected"] = SELECTED] = "selected";
      GraphEdgeState[GraphEdgeState["activated"] = 2] = "activated";
      GraphEdgeState[GraphEdgeState["connectedToSelected"] = 4] = "connectedToSelected";
      GraphEdgeState[GraphEdgeState["unconnectedToSelected"] = 8] = "unconnectedToSelected";
      GraphEdgeState[GraphEdgeState["editing"] = 16] = "editing";
  })(exports.GraphEdgeState || (exports.GraphEdgeState = {}));
  exports.GraphNodeState = void 0;
  (function (GraphNodeState) {
      GraphNodeState[GraphNodeState["default"] = 0] = "default";
      GraphNodeState[GraphNodeState["selected"] = SELECTED] = "selected";
      GraphNodeState[GraphNodeState["activated"] = 2] = "activated";
      GraphNodeState[GraphNodeState["editing"] = 4] = "editing";
      GraphNodeState[GraphNodeState["connectedToSelected"] = 8] = "connectedToSelected";
      GraphNodeState[GraphNodeState["unconnectedToSelected"] = 16] = "unconnectedToSelected";
  })(exports.GraphNodeState || (exports.GraphNodeState = {}));
  exports.GraphNodeStateConst = void 0;
  (function (GraphNodeStateConst) {
      GraphNodeStateConst["default"] = "default";
      GraphNodeStateConst["selected"] = "selected";
      GraphNodeStateConst["activated"] = "activated";
      GraphNodeStateConst["editing"] = "editing";
      GraphNodeStateConst["connectedToSelected"] = "connectedToSelected";
      GraphNodeStateConst["unconnectedToSelected"] = "unconnectedToSelected";
  })(exports.GraphNodeStateConst || (exports.GraphNodeStateConst = {}));
  exports.GraphPortState = void 0;
  (function (GraphPortState) {
      GraphPortState[GraphPortState["default"] = 0] = "default";
      GraphPortState[GraphPortState["selected"] = SELECTED] = "selected";
      GraphPortState[GraphPortState["activated"] = 2] = "activated";
      GraphPortState[GraphPortState["connecting"] = 4] = "connecting";
      GraphPortState[GraphPortState["connectingAsTarget"] = 8] = "connectingAsTarget";
  })(exports.GraphPortState || (exports.GraphPortState = {}));

  var Debug = (function () {
      function Debug() {
      }
      Debug.log = function (message) {
          if (process.env.NODE_ENV !== "production") {
              console.log(message);
          }
      };
      Debug.warn = function (message) {
          if (process.env.NODE_ENV !== "production") {
              console.warn(message);
          }
      };
      Debug.error = function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          console.error.apply(console, args);
      };
      Debug.never = function (value, message) {
          throw new Error(message !== null && message !== void 0 ? message : value + " is unexpected");
      };
      return Debug;
  }());

  var getNodeConfig = function (node, graphConfig) {
      var nodeConfig = graphConfig.getNodeConfigByName(node.shape);
      if (!nodeConfig) {
          Debug.warn("invalid shape in node " + JSON.stringify(node));
          return undefined;
      }
      return nodeConfig;
  };

  function getRectWidth(rectConfig, rect) {
      var _a;
      var minWidth = (_a = rectConfig === null || rectConfig === void 0 ? void 0 : rectConfig.getMinWidth(rect)) !== null && _a !== void 0 ? _a : 0;
      if (rect.width && rect.width >= minWidth) {
          return rect.width;
      }
      return minWidth;
  }
  function getRectHeight(rectConfig, rect) {
      var _a;
      var minHeight = (_a = rectConfig === null || rectConfig === void 0 ? void 0 : rectConfig.getMinHeight(rect)) !== null && _a !== void 0 ? _a : 0;
      if (rect.height && rect.height >= minHeight) {
          return rect.height;
      }
      return minHeight;
  }
  function getNodeSize(node, graphConfig) {
      var nodeConfig = getNodeConfig(node, graphConfig);
      var width = getRectWidth(nodeConfig, node);
      var height = getRectHeight(nodeConfig, node);
      return {
          height: height,
          width: width
      };
  }
  function getGroupRect(group, nodes, graphConfig) {
      var _a, _b, _c, _d, _e, _f, _g, _h;
      var childrenIds = new Set(group.nodeIds);
      var children = Array.from(nodes.values()).filter(function (node) { return childrenIds.has(node.id); });
      var minX = Math.min.apply(Math, children.map(function (node) { return node.x; }));
      var maxX = Math.max.apply(Math, children.map(function (node) { return node.x + getNodeSize(node, graphConfig).width; }));
      var minY = Math.min.apply(Math, children.map(function (node) { return node.y; }));
      var maxY = Math.max.apply(Math, children.map(function (node) { return node.y + getNodeSize(node, graphConfig).height; }));
      var x = minX - ((_b = (_a = group.padding) === null || _a === void 0 ? void 0 : _a.left) !== null && _b !== void 0 ? _b : 0);
      var y = minY - ((_d = (_c = group.padding) === null || _c === void 0 ? void 0 : _c.top) !== null && _d !== void 0 ? _d : 0);
      var height = maxY - y + ((_f = (_e = group.padding) === null || _e === void 0 ? void 0 : _e.bottom) !== null && _f !== void 0 ? _f : 0);
      var width = maxX - x + ((_h = (_g = group.padding) === null || _g === void 0 ? void 0 : _g.left) !== null && _h !== void 0 ? _h : 0);
      return {
          x: x,
          y: y,
          width: width,
          height: height
      };
  }

  var getPortPosition = function (node, port, nodeConfig) {
      var width = getRectWidth(nodeConfig, node);
      var height = getRectHeight(nodeConfig, node);
      var xOffset = port.position ? port.position[0] * width : width * 0.5;
      var x = node.x + xOffset;
      var yOffset = port.position ? port.position[1] * height : height;
      var y = node.y + yOffset;
      return {
          x: x,
          y: y
      };
  };
  var getPortPositionByPortId = function (node, portId, graphConfig) {
      var nodeConfig = getNodeConfig(node, graphConfig);
      if (!nodeConfig) {
          return undefined;
      }
      var ports = node.ports || [];
      var port = ports.find(function (p) { return p.id === portId; });
      if (!port) {
          Debug.warn("invalid port id " + JSON.stringify(port));
          return undefined;
      }
      return getPortPosition(node, port, nodeConfig);
  };

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      if (typeof b !== "function" && b !== null)
          throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  function __rest(s, e) {
      var t = {};
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
          t[p] = s[p];
      if (s != null && typeof Object.getOwnPropertySymbols === "function")
          for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
              if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                  t[p[i]] = s[p[i]];
          }
      return t;
  }

  function __awaiter(thisArg, _arguments, P, generator) {
      function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
      return new (P || (P = Promise))(function (resolve, reject) {
          function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
          function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
          function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
          step((generator = generator.apply(thisArg, _arguments || [])).next());
      });
  }

  function __generator(thisArg, body) {
      var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
      return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
      function verb(n) { return function (v) { return step([n, v]); }; }
      function step(op) {
          if (f) throw new TypeError("Generator is already executing.");
          while (_) try {
              if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
              if (y = 0, t) op = [op[0] & 2, t.value];
              switch (op[0]) {
                  case 0: case 1: t = op; break;
                  case 4: _.label++; return { value: op[1], done: false };
                  case 5: _.label++; y = op[1]; op = [0]; continue;
                  case 7: op = _.ops.pop(); _.trys.pop(); continue;
                  default:
                      if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                      if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                      if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                      if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                      if (t[2]) _.ops.pop();
                      _.trys.pop(); continue;
              }
              op = body.call(thisArg, _);
          } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
          if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
      }
  }

  /** @deprecated */
  function __spreadArrays() {
      for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
      for (var r = Array(s), k = 0, i = 0; i < il; i++)
          for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
              r[k] = a[j];
      return r;
  }

  function __spreadArray(to, from) {
      for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
          to[j] = from[i];
      return to;
  }

  var MouseEventButton;
  (function (MouseEventButton) {
      MouseEventButton[MouseEventButton["Primary"] = 0] = "Primary";
      MouseEventButton[MouseEventButton["Auxiliary"] = 1] = "Auxiliary";
      MouseEventButton[MouseEventButton["Secondary"] = 2] = "Secondary";
      MouseEventButton[MouseEventButton["Fourth"] = 4] = "Fourth";
      MouseEventButton[MouseEventButton["Fifth"] = 5] = "Fifth";
  })(MouseEventButton || (MouseEventButton = {}));
  var MouseEventButtons;
  (function (MouseEventButtons) {
      MouseEventButtons[MouseEventButtons["None"] = 0] = "None";
      MouseEventButtons[MouseEventButtons["Left"] = 1] = "Left";
      MouseEventButtons[MouseEventButtons["Right"] = 2] = "Right";
      MouseEventButtons[MouseEventButtons["Middle"] = 4] = "Middle";
  })(MouseEventButtons || (MouseEventButtons = {}));
  var DEFAULT_AUTO_ALIGN_THRESHOLD = 50;
  var COPIED_NODE_SPACING = 50;
  var NODE_MIN_VISIBLE_LENGTH = 5;
  var NODE_MAX_VISIBLE_LENGTH = 500;

  var identical = function (data) { return data; };

  var isMobile = function () {
      var toMatch = [
          /Android/i,
          /webOS/i,
          /iPhone/i,
          /iPad/i,
          /iPod/i,
          /BlackBerry/i,
          /Windows Phone/i
      ];
      return toMatch.some(function (toMatchItem) {
          return navigator.userAgent.match(toMatchItem);
      });
  };
  exports.BrowserType = void 0;
  (function (BrowserType) {
      BrowserType["unknown"] = "unknown";
      BrowserType["edge"] = "edge";
      BrowserType["edgeChromium"] = "edgeChromium";
      BrowserType["opera"] = "opera";
      BrowserType["chrome"] = "chrome";
      BrowserType["ie"] = "ie";
      BrowserType["firefox"] = "firefox";
      BrowserType["safari"] = "safari";
      BrowserType["electron"] = "electron";
  })(exports.BrowserType || (exports.BrowserType = {}));
  var getBrowser = function () {
      var agent = navigator.userAgent.toLowerCase();
      if (agent.indexOf("electron") > -1) {
          return exports.BrowserType.electron;
      }
      switch (true) {
          case agent.indexOf("edge") > -1:
              return exports.BrowserType.edge;
          case agent.indexOf("edg") > -1:
              return exports.BrowserType.edgeChromium;
          case agent.indexOf("opr") > -1 && !!window.opr:
              return exports.BrowserType.opera;
          case agent.indexOf("chrome") > -1 && !!window.chrome:
              return exports.BrowserType.chrome;
          case agent.indexOf("trident") > -1:
              return exports.BrowserType.ie;
          case agent.indexOf("firefox") > -1:
              return exports.BrowserType.firefox;
          case agent.indexOf("safari") > -1:
              return exports.BrowserType.safari;
          default:
              return exports.BrowserType.unknown;
      }
  };
  var isSupported = function () {
      if (isMobile()) {
          return false;
      }
      var browser = getBrowser();
      var supported = [
          exports.BrowserType.chrome,
          exports.BrowserType.edgeChromium,
          exports.BrowserType.firefox,
          exports.BrowserType.safari,
          exports.BrowserType.electron
      ];
      return supported.indexOf(browser) > -1;
  };
  var isMacOs = navigator.userAgent.includes("Macintosh");

  var metaControl = function (e) {
      return isMacOs ? e.metaKey : e.ctrlKey;
  };
  var checkIsMultiSelect = function (e) {
      return e.shiftKey || metaControl(e);
  };

  var hasState = function (mask) { return function (state) { return Boolean(pickState(mask)(state)); }; };
  var addState = function (mask) { return function (state) {
      var t = state || 0;
      if (Array.isArray(mask)) {
          return mask.reduce(function (c, m) { return c | m; }, t);
      }
      else {
          return t | mask;
      }
  }; };
  var toggleState = function (mask) { return function (state) {
      var t = state || 0;
      return t ^ mask;
  }; };
  var pickState = function (mask) { return function (state) {
      var t = state || 0;
      return t & mask;
  }; };
  var updateNodeState = function (maskObj, state) {
      var t = state || 0;
      Object.keys(maskObj).forEach(function (k) {
          if (maskObj[k]) {
              t = addState(exports.GraphNodeState[k])(t);
          }
          else if (!maskObj[k]) {
              t = removeState(exports.GraphNodeState[k])(t);
          }
      });
      return t;
  };
  var removeState = function (mask) { return function (state) {
      var t = state || 0;
      if (Array.isArray(mask)) {
          return mask.reduce(function (c, m) { return c & ~m; }, t);
      }
      else {
          return t & ~mask;
      }
  }; };
  var resetState = function (state) { return function () { return state; }; };
  var resetConnectState = function (state) { return function (prevState) {
      return (prevState & exports.GraphNodeState.activated) | state;
  }; };
  var updateState = function (updater) { return function (value) {
      var nextState = updater(value.state);
      if (nextState === value.state) {
          return value;
      }
      return __assign(__assign({}, value), { state: updater(value.state) });
  }; };
  var isNodeEditing = function (node) { return hasState(exports.GraphNodeState.editing)(node.state); };
  var isNodeActivated = function (node) { return hasState(exports.GraphNodeState.activated)(node.state); };
  var isSelected = function (node) { return hasState(SELECTED)(node.state); };
  var notSelected = function (node) { return !hasState(SELECTED)(node.state); };
  var hasSelectedPort = function (node) {
      var ports = node.ports || [];
      for (var _i = 0, ports_1 = ports; _i < ports_1.length; _i++) {
          var port = ports_1[_i];
          if (hasState(SELECTED)(port.state)) {
              return true;
          }
      }
      return false;
  };

  var transformPoint = function (x, y, transformMatrix) {
      return {
          x: transformMatrix[0] * x + transformMatrix[2] * y + transformMatrix[4],
          y: transformMatrix[1] * x + transformMatrix[3] * y + transformMatrix[5]
      };
  };
  var reverseTransformPoint = function (x, y, transformMatrix) {
      var t0 = transformMatrix[0], t1 = transformMatrix[1], t2 = transformMatrix[2], t3 = transformMatrix[3], t4 = transformMatrix[4], t5 = transformMatrix[5];
      return {
          x: ((x - t4) * t3 - (y - t5) * t2) / (t0 * t3 - t1 * t2),
          y: ((x - t4) * t1 - (y - t5) * t0) / (t1 * t2 - t0 * t3)
      };
  };
  var getPointDeltaByClientDelta = function (x, y, transformMatrix) {
      var a = transformMatrix[0], b = transformMatrix[1], c = transformMatrix[2], d = transformMatrix[3];
      var realDx = (d * x) / (a * d - b * c) + (c * y) / (b * c - a * d);
      var realDy = (b * x) / (b * c - a * d) + (a * y) / (a * d - b * c);
      return {
          x: realDx,
          y: realDy
      };
  };
  var getClientDeltaByPointDelta = function (x, y, transformMatrix) {
      if (!transformMatrix) {
          return {
              x: x,
              y: y
          };
      }
      var a = transformMatrix[0], b = transformMatrix[1], c = transformMatrix[2], d = transformMatrix[3];
      return transformPoint(x, y, [a, b, c, d, 0, 0]);
  };
  var getRealPointFromClientPoint = function (clientX, clientY, viewport) {
      var rect = viewport.rect;
      var x = clientX - rect.left;
      var y = clientY - rect.top;
      return reverseTransformPoint(x, y, viewport.transformMatrix);
  };
  var getClientPointFromRealPoint = function (realX, realY, viewport) {
      var _a = transformPoint(realX, realY, viewport.transformMatrix), x = _a.x, y = _a.y;
      var rect = viewport.rect;
      return {
          x: x + rect.left,
          y: y + rect.top
      };
  };
  var getContainerClientPoint = function (realX, realY, viewport) {
      var client = getClientPointFromRealPoint(realX, realY, viewport);
      var rect = viewport.rect;
      return {
          x: client.x - rect.left,
          y: client.y - rect.top
      };
  };

  function markEdgeDirty(edges, id) {
      edges.update(id, function (edge) { return edge.shallow(); });
  }
  var getNearestConnectablePort = function (params) {
      var node = params.parentNode, clientX = params.clientX, clientY = params.clientY, graphConfig = params.graphConfig, viewport = params.viewport;
      var minDistance = Infinity;
      var nearestPort;
      if (!node.ports) {
          return undefined;
      }
      var point = getRealPointFromClientPoint(clientX, clientY, viewport);
      node.ports.forEach(function (port) {
          if (isConnectable(graphConfig, __assign(__assign({}, params), { model: port }))) {
              var portPos = getPortPositionByPortId(node, port.id, graphConfig);
              if (!portPos) {
                  return;
              }
              var diffX = point.x - portPos.x;
              var diffY = point.y - portPos.y;
              var distance = diffX * diffX + diffY * diffY;
              if (distance < minDistance) {
                  minDistance = distance;
                  nearestPort = port;
              }
          }
      });
      return nearestPort;
  };
  var isConnectable = function (graphConfig, params) {
      var portConfig = graphConfig.getPortConfigByName(params.model.shape);
      return portConfig ? portConfig.getIsConnectable(params) : false;
  };
  function resetNodePortsState(node) {
      return node.updatePorts(updateState(resetState(exports.GraphPortState.default)));
  }
  var filterSelectedItems = function (data) {
      var nodes = new Map();
      var edges = [];
      data.nodes.forEach(function (_a) {
          var inner = _a.inner;
          if (isSelected(inner)) {
              nodes.set(inner.id, inner);
          }
      });
      data.edges.forEach(function (_a) {
          var inner = _a.inner;
          if (isSelected(inner) || (nodes.has(inner.source) && nodes.has(inner.target))) {
              edges.push(inner);
          }
      });
      return {
          nodes: Array.from(nodes.values()),
          edges: edges
      };
  };
  var getNeighborPorts = function (data, nodeId, portId) {
      var neighbors = [];
      var edgesBySource = data.getEdgesBySource(nodeId, portId);
      var edgesByTarget = data.getEdgesByTarget(nodeId, portId);
      edgesBySource === null || edgesBySource === void 0 ? void 0 : edgesBySource.forEach(function (edgeId) {
          var edge = data.edges.get(edgeId);
          if (edge) {
              neighbors.push({
                  nodeId: edge.target,
                  portId: edge.targetPortId
              });
          }
      });
      edgesByTarget === null || edgesByTarget === void 0 ? void 0 : edgesByTarget.forEach(function (edgeId) {
          var edge = data.edges.get(edgeId);
          if (edge) {
              neighbors.push({
                  nodeId: edge.source,
                  portId: edge.sourcePortId
              });
          }
      });
      return neighbors;
  };
  var unSelectAllEntity = function () {
      return function (data) {
          return data
              .mapNodes(function (n) {
              return n.update(function (curNode) {
                  var _a;
                  var nextNode = __assign(__assign({}, curNode), { ports: (_a = curNode.ports) === null || _a === void 0 ? void 0 : _a.map(updateState(resetState(exports.GraphPortState.default))) });
                  return updateState(resetState(exports.GraphNodeState.default))(nextNode);
              });
          })
              .mapEdges(function (e) { return e.update(updateState(resetState(exports.GraphEdgeState.default))); });
      };
  };
  var nodeSelection = function (e, target) {
      if (isNodeEditing(target)) {
          return identical;
      }
      var isMultiSelect = checkIsMultiSelect(e);
      if (isSelected(target) && !isMultiSelect) {
          return identical;
      }
      return function (data) {
          var predicate = isMultiSelect
              ? function (node) {
                  if (node.id !== target.id) {
                      return isSelected(node);
                  }
                  else if (e.button === MouseEventButton.Secondary) {
                      return true;
                  }
                  else {
                      return !isSelected(target);
                  }
              }
              : function (node) { return node.id === target.id; };
          return data.selectNodes(predicate, target.id);
      };
  };
  var getNodeAutomationId = function (node) {
      var _a;
      return "node-container-" + ((_a = node.name) !== null && _a !== void 0 ? _a : "unnamed") + "-" + node.id;
  };
  var getPortAutomationId = function (port, parentNode) {
      return "port-" + parentNode.name + "-" + parentNode.id + "-" + port.name + "-" + port.id;
  };
  var getNodeUid = function (graphId, node) {
      return "node:" + graphId + ":" + node.id;
  };
  var getPortUid = function (graphId, node, port) {
      return "port:" + graphId + ":" + node.id + ":" + port.id;
  };
  var getEdgeUid = function (graphId, edge) {
      return "edge:" + graphId + ":" + edge.id;
  };

  var getCurvePathD = function (x1, x2, y1, y2) {
      return "M" + x1 + "," + y1 + "C" + x1 + "," + (y1 -
          getControlPointDistance(y1, y2)) + "," + x2 + "," + (y2 +
          5 +
          getControlPointDistance(y1, y2)) + "," + x2 + "," + (y2 + 5);
  };
  var getControlPointDistance = function (y1, y2) {
      return Math.min(5 * 15, Math.max(5 * 3, Math.abs((y1 - (y2 + 5)) / 2)));
  };

  var isWithinThreshold = function (dx, dy, threshold) {
      return Math.abs(dx) < threshold && Math.abs(dy) < threshold;
  };
  var isWithinRect = function (p0, p, w, h) {
      var x0 = p0.x, y0 = p0.y;
      var x = p.x, y = p.y;
      if (x0 < x && x0 + w > x && y0 < y && y0 + h > y) {
          return true;
      }
      return false;
  };

  var checkRectIntersect = function (rectA, rectB) {
      var isALeftOfB = rectA.maxX < rectB.minX;
      var isARightOfB = rectA.minX > rectB.maxX;
      var isABelowB = rectA.minY > rectB.maxY;
      var isAAboveB = rectA.maxY < rectB.minY;
      return !(isALeftOfB || isARightOfB || isABelowB || isAAboveB);
  };
  var isPointInRect = function (rect, point) {
      var minX = rect.minX, minY = rect.minY, maxX = rect.maxX, maxY = rect.maxY;
      var x = point.x, y = point.y;
      return x > minX && x < maxX && y > minY && y < maxY;
  };
  var square = function (x) { return Math.pow(x, 2); };
  var distance = function (x1, y1, x2, y2) {
      return Math.sqrt(square(x2 - x1) + square(y2 - y1));
  };
  var getLinearFunction = function (x1, y1, x2, y2) {
      if (x1 === x2) {
          return function () { return Number.MAX_SAFE_INTEGER; };
      }
      return function (x) { return ((y2 - y1) / (x2 - x1)) * x + (y1 * x2 - y2 * x1) / (x2 - x1); };
  };
  var LinearEquation = (function () {
      function LinearEquation(x1, y1, x2, y2) {
          this.a = y1 - y2;
          this.b = x1 - x2;
          this.c = x2 * y1 - x1 * y2;
      }
      LinearEquation.prototype.getX = function (y) {
          if (this.a === 0) {
              return undefined;
          }
          return (this.b * y + this.c) / this.a;
      };
      LinearEquation.prototype.getY = function (x) {
          if (this.b === 0) {
              return undefined;
          }
          return (this.a * x - this.c) / this.b;
      };
      return LinearEquation;
  }());

  var deepClone = function (obj) {
      if (typeof obj !== "object") {
          return obj;
      }
      var newObj = obj instanceof Array ? [] : {};
      for (var key in obj) {
          if (typeof obj[key] === "object" && obj[key] !== null) {
              newObj[key] = deepClone(obj[key]);
          }
          else {
              newObj[key] = obj[key];
          }
      }
      return newObj;
  };

  var diffGraphNode = function (preData, data) {
      var isAdded = data.nodes.length > preData.nodes.length;
      var larger = isAdded ? data : preData;
      var smaller = isAdded ? preData : data;
      var nodeIdSet = new Set();
      smaller.nodes.forEach(function (n) { return nodeIdSet.add(n.id); });
      var nodes = larger.nodes.filter(function (n) { return !nodeIdSet.has(n.id); });
      return { isAdded: isAdded, nodes: nodes };
  };

  var diffGraphEdge = function (preData, data) {
      var isAdded = data.edges.length > preData.edges.length;
      var larger = isAdded ? data : preData;
      var smaller = isAdded ? preData : data;
      var edgeIdSet = new Set();
      smaller.edges.forEach(function (n) { return edgeIdSet.add(n.id); });
      var edges = larger.edges.filter(function (n) { return !edgeIdSet.has(n.id); });
      return { isAdded: isAdded, edges: edges };
  };

  var diffGraphPort = function (preData, data) {
      var prePortIdSet = new Set();
      var preNodes = new Map();
      var portIdSet = new Set();
      var nodes = new Map();
      var CONNECT = "--CONNECT--";
      preData.nodes.forEach(function (n) {
          preNodes.set(n.id, n);
          if (n.ports) {
              n.ports.forEach(function (p) { return prePortIdSet.add("" + n.id + CONNECT + p.id); });
          }
      });
      data.nodes.forEach(function (n) {
          nodes.set(n.id, n);
          if (n.ports) {
              n.ports.forEach(function (p) { return portIdSet.add("" + n.id + CONNECT + p.id); });
          }
      });
      var isAdded = portIdSet.size > prePortIdSet.size;
      var small = isAdded ? prePortIdSet : portIdSet;
      var large = isAdded ? portIdSet : prePortIdSet;
      var largeNodes = isAdded ? nodes : preNodes;
      var map = new Map();
      large.forEach(function (id) {
          var _a;
          if (!small.has(id)) {
              var _b = id.split(CONNECT), nodeId = _b[0], portId_1 = _b[1];
              var node = largeNodes.get(nodeId);
              if (node && node.ports) {
                  if (!map.get(nodeId)) {
                      map.set(nodeId, { node: node, ports: [] });
                  }
                  var port = node.ports.filter(function (p) { return p.id === portId_1; })[0];
                  (_a = map.get(nodeId)) === null || _a === void 0 ? void 0 : _a.ports.push(port);
              }
          }
      });
      var diffPorts = Array.from(map.values());
      return { isAdded: isAdded, diffPorts: diffPorts };
  };

  exports.Direction = void 0;
  (function (Direction) {
      Direction[Direction["X"] = 0] = "X";
      Direction[Direction["Y"] = 1] = "Y";
      Direction[Direction["XY"] = 2] = "XY";
  })(exports.Direction || (exports.Direction = {}));

  var isViewportComplete = function (viewport) {
      return !!viewport.rect;
  };
  var isViewportEmpty = function (viewport) {
      if (!isViewportComplete(viewport)) {
          return true;
      }
      return !viewport.rect.width || !viewport.rect.height;
  };
  var getNodeRect = function (node, graphConfig) {
      var x = node.x, y = node.y;
      var _a = getNodeSize(node, graphConfig), width = _a.width, height = _a.height;
      return {
          x: x,
          y: y,
          width: width,
          height: height
      };
  };
  var isNodeVisible = function (node, viewport, graphConfig) {
      return isRectVisible(getNodeRect(node, graphConfig), viewport);
  };
  var isRectVisible = function (shapeRect, viewport) {
      var x = shapeRect.x, y = shapeRect.y, width = shapeRect.width, height = shapeRect.height;
      return (isPointVisible({ x: x, y: y }, viewport) ||
          isPointVisible({ x: x + width, y: y }, viewport) ||
          isPointVisible({ x: x + width, y: y + height }, viewport) ||
          isPointVisible({ x: x, y: y + height }, viewport));
  };
  var isPointVisible = function (point, viewport) {
      var _a = getContainerClientPoint(point.x, point.y, viewport), x = _a.x, y = _a.y;
      var _b = viewport.rect, height = _b.height, width = _b.width;
      return x > 0 && x < width && y > 0 && y < height;
  };
  var getVisibleNodes = function (nodes, viewport, graphConfig) {
      var result = [];
      nodes.forEach(function (n) {
          if (isNodeVisible(n, viewport, graphConfig)) {
              result.push(n.inner);
          }
      });
      return result;
  };
  var getRenderedNodes = function (nodes, viewport) {
      var result = [];
      var renderedArea = getRenderedArea(viewport);
      nodes.forEach(function (n) {
          if (isNodeInRenderedArea(n, renderedArea)) {
              result.push(n.inner);
          }
      });
      return result;
  };
  var isNodeInRenderedArea = function (node, renderedArea) {
      return isPointInRect(renderedArea, node);
  };
  var getRenderedEdges = function (edges, nodes, graphConfig, viewport) {
      var result = [];
      var renderedArea = getRenderedArea(viewport);
      edges.forEach(function (e) {
          var edgeCoordinate = getEdgeSourceTargetCoordinate(e, nodes, graphConfig);
          if (edgeCoordinate.source &&
              edgeCoordinate.target &&
              isEdgeInRenderedArea(edgeCoordinate.source, edgeCoordinate.target, renderedArea)) {
              result.push(e.inner);
          }
      });
      return result;
  };
  var isEdgeInRenderedArea = function (source, target, renderedArea) {
      var isSourceVisible = isPointInRect(renderedArea, source);
      var isTargetVisible = isPointInRect(renderedArea, target);
      return isSourceVisible || isTargetVisible;
  };
  var getVisibleArea = function (viewport) {
      if (!isViewportComplete(viewport)) {
          return {
              minX: 0,
              minY: 0,
              maxX: 0,
              maxY: 0
          };
      }
      var rect = viewport.rect, transformMatrix = viewport.transformMatrix;
      var minX = 0;
      var minY = 0;
      var maxX = rect.width;
      var maxY = rect.height;
      var min = reverseTransformPoint(minX, minY, transformMatrix);
      var max = reverseTransformPoint(maxX, maxY, transformMatrix);
      return {
          minX: min.x,
          minY: min.y,
          maxX: max.x,
          maxY: max.y
      };
  };
  var getRenderedArea = function (viewport) {
      if (!isViewportComplete(viewport)) {
          return {
              minX: 0,
              minY: 0,
              maxX: 0,
              maxY: 0
          };
      }
      var rect = viewport.rect, transformMatrix = viewport.transformMatrix;
      var minX = 0;
      var minY = 0;
      var maxX = rect.width;
      var maxY = rect.height;
      var min = reverseTransformPoint(minX - rect.width, minY - rect.height, transformMatrix);
      var max = reverseTransformPoint(maxX + rect.width, maxY + rect.height, transformMatrix);
      return {
          minX: min.x,
          minY: min.y,
          maxX: max.x,
          maxY: max.y
      };
  };
  var getEdgeSourceTargetCoordinate = function (edge, nodes, graphConfig) {
      var _a, _b;
      return {
          source: (_a = nodes.get(edge.source)) === null || _a === void 0 ? void 0 : _a.getPortPosition(edge.sourcePortId, graphConfig),
          target: (_b = nodes.get(edge.target)) === null || _b === void 0 ? void 0 : _b.getPortPosition(edge.targetPortId, graphConfig)
      };
  };
  var normalizeSpacing = function (spacing) {
      if (!spacing) {
          return {
              top: 0,
              right: 0,
              bottom: 0,
              left: 0
          };
      }
      if (typeof spacing === "number") {
          return {
              top: spacing,
              right: spacing,
              bottom: spacing,
              left: spacing
          };
      }
      return __assign({ top: 0, right: 0, bottom: 0, left: 0 }, spacing);
  };
  var zoomTo = function (_a) {
      var scale = _a.scale, anchor = _a.anchor, direction = _a.direction, limitScale = _a.limitScale;
      return function (prevState) {
          var scaleX = limitScale(scale) / prevState.transformMatrix[0];
          var scaleY = limitScale(scale) / prevState.transformMatrix[3];
          var x = anchor.x, y = anchor.y;
          var dx = x * (1 - scaleX);
          var dy = y * (1 - scaleY);
          var transformMatrix;
          switch (direction) {
              case exports.Direction.X:
                  transformMatrix = [
                      scale,
                      0,
                      0,
                      prevState.transformMatrix[3],
                      prevState.transformMatrix[4] * scaleX + dx,
                      prevState.transformMatrix[5]
                  ];
                  break;
              case exports.Direction.Y:
                  transformMatrix = [
                      prevState.transformMatrix[0],
                      0,
                      0,
                      scale,
                      prevState.transformMatrix[4],
                      prevState.transformMatrix[5] * scaleY + dy
                  ];
                  break;
              case exports.Direction.XY:
              default:
                  transformMatrix = [
                      scale,
                      0,
                      0,
                      scale,
                      prevState.transformMatrix[4] * scaleX + dx,
                      prevState.transformMatrix[5] * scaleY + dy
                  ];
          }
          return __assign(__assign({}, prevState), { transformMatrix: transformMatrix });
      };
  };
  var zoom = function (_a) {
      var scale = _a.scale, anchor = _a.anchor, direction = _a.direction, limitScale = _a.limitScale;
      if (scale === 1) {
          return identical;
      }
      return function (prevState) {
          var transformMatrix;
          switch (direction) {
              case exports.Direction.X:
                  return zoomTo({
                      anchor: anchor,
                      direction: direction,
                      limitScale: limitScale,
                      scale: prevState.transformMatrix[0] * scale
                  })(prevState);
              case exports.Direction.Y:
                  return zoomTo({
                      anchor: anchor,
                      direction: direction,
                      limitScale: limitScale,
                      scale: prevState.transformMatrix[3] * scale
                  })(prevState);
              case exports.Direction.XY:
              default: {
                  var resultX = limitScale(prevState.transformMatrix[0] * scale);
                  var resultY = limitScale(prevState.transformMatrix[3] * scale);
                  var scaleX = resultX / prevState.transformMatrix[0];
                  var scaleY = resultY / prevState.transformMatrix[3];
                  var x = anchor.x, y = anchor.y;
                  var dx = x * (1 - scaleX);
                  var dy = y * (1 - scaleY);
                  transformMatrix = [
                      resultX,
                      0,
                      0,
                      resultY,
                      prevState.transformMatrix[4] * scaleX + dx,
                      prevState.transformMatrix[5] * scaleY + dy
                  ];
              }
          }
          return __assign(__assign({}, prevState), { transformMatrix: transformMatrix });
      };
  };
  var pan = function (dx, dy) {
      if (dx === 0 && dy === 0) {
          return identical;
      }
      return function (prevState) {
          return __assign(__assign({}, prevState), { transformMatrix: [
                  prevState.transformMatrix[0],
                  prevState.transformMatrix[1],
                  prevState.transformMatrix[2],
                  prevState.transformMatrix[3],
                  prevState.transformMatrix[4] + dx,
                  prevState.transformMatrix[5] + dy
              ] });
      };
  };
  var minimapPan = function (dx, dy) {
      if (dx === 0 && dy === 0) {
          return identical;
      }
      return function (prevState) {
          var _a = prevState.transformMatrix, a = _a[0], b = _a[1], c = _a[2], d = _a[3];
          return __assign(__assign({}, prevState), { transformMatrix: [
                  a,
                  b,
                  c,
                  d,
                  prevState.transformMatrix[4] + a * dx + b * dy,
                  prevState.transformMatrix[5] + c * dx + d * dy
              ] });
      };
  };
  var getContentArea$1 = function (data, graphConfig, nodeIds) {
      var minNodeWidth = Infinity;
      var minNodeHeight = Infinity;
      var minNodeX = Infinity;
      var minNodeY = Infinity;
      var maxNodeX = -Infinity;
      var maxNodeY = -Infinity;
      var forEachNode = function (fn) { return data.nodes.forEach(fn); };
      var forEachByIds = function (fn) {
          return nodeIds === null || nodeIds === void 0 ? void 0 : nodeIds.forEach(function (id) {
              var node = data.nodes.get(id);
              if (node) {
                  fn(node);
              }
          });
      };
      var forEach = nodeIds === undefined ? forEachNode : forEachByIds;
      forEach(function (node) {
          var _a = getNodeSize(node, graphConfig), nodeWidth = _a.width, nodeHeight = _a.height;
          if (node.x < minNodeX) {
              minNodeX = node.x;
          }
          if (node.y < minNodeY) {
              minNodeY = node.y;
          }
          if (node.x + nodeWidth > maxNodeX) {
              maxNodeX = node.x + nodeWidth;
          }
          if (node.y + nodeHeight > maxNodeY) {
              maxNodeY = node.y + nodeHeight;
          }
          if (nodeWidth < minNodeWidth) {
              minNodeWidth = nodeWidth;
          }
          if (nodeHeight < minNodeHeight) {
              minNodeHeight = nodeHeight;
          }
      });
      return {
          minNodeWidth: minNodeWidth,
          minNodeHeight: minNodeHeight,
          minNodeX: minNodeX,
          minNodeY: minNodeY,
          maxNodeX: maxNodeX,
          maxNodeY: maxNodeY
      };
  };
  var normalizeNodeVisibleMinMax = function (_a) {
      var nodeMinVisibleSize = _a.nodeMinVisibleSize, nodeMaxVisibleSize = _a.nodeMaxVisibleSize;
      var nodeMinVisibleWidth = nodeMinVisibleSize.width, nodeMinVisibleHeight = nodeMinVisibleSize.height;
      var nodeMaxVisibleWidth = nodeMaxVisibleSize.width, nodeMaxVisibleHeight = nodeMaxVisibleSize.height;
      if (nodeMinVisibleWidth > nodeMaxVisibleWidth) {
          var temp = nodeMinVisibleWidth;
          nodeMinVisibleWidth = nodeMaxVisibleWidth;
          nodeMaxVisibleWidth = temp;
      }
      if (nodeMinVisibleHeight > nodeMaxVisibleHeight) {
          var temp = nodeMinVisibleHeight;
          nodeMinVisibleHeight = nodeMaxVisibleHeight;
          nodeMaxVisibleHeight = temp;
      }
      return {
          nodeMinVisibleWidth: nodeMinVisibleWidth,
          nodeMinVisibleHeight: nodeMinVisibleHeight,
          nodeMaxVisibleWidth: nodeMaxVisibleWidth,
          nodeMaxVisibleHeight: nodeMaxVisibleHeight
      };
  };
  var getScaleRange = function (params, _a) {
      var minNodeWidth = _a.width, minNodeHeight = _a.height;
      var _b = normalizeNodeVisibleMinMax(params), nodeMinVisibleWidth = _b.nodeMinVisibleWidth, nodeMinVisibleHeight = _b.nodeMinVisibleHeight, nodeMaxVisibleWidth = _b.nodeMaxVisibleWidth, nodeMaxVisibleHeight = _b.nodeMaxVisibleHeight;
      var minScaleX = 0;
      var minScaleY = 0;
      var maxScaleX = Infinity;
      var maxScaleY = Infinity;
      if (minNodeWidth) {
          minScaleX = nodeMinVisibleWidth / minNodeWidth;
          maxScaleX = nodeMaxVisibleWidth / minNodeWidth;
      }
      if (minNodeHeight) {
          minScaleY = nodeMinVisibleHeight / minNodeHeight;
          maxScaleY = nodeMaxVisibleHeight / minNodeHeight;
      }
      return {
          minScaleX: minScaleX,
          minScaleY: minScaleY,
          maxScaleX: maxScaleX,
          maxScaleY: maxScaleY
      };
  };
  var getZoomFitMatrix = function (args) {
      var data = args.data, graphConfig = args.graphConfig, disablePan = args.disablePan, direction = args.direction, rect = args.rect;
      var nodes = data.nodes;
      if (nodes.size === 0) {
          return [1, 0, 0, 1, 0, 0];
      }
      var _a = getContentArea$1(data, graphConfig), minNodeWidth = _a.minNodeWidth, minNodeHeight = _a.minNodeHeight, minNodeX = _a.minNodeX, minNodeY = _a.minNodeY, maxNodeX = _a.maxNodeX, maxNodeY = _a.maxNodeY;
      var _b = getScaleRange(args, {
          width: minNodeWidth,
          height: minNodeHeight
      }), minScaleX = _b.minScaleX, minScaleY = _b.minScaleY, maxScaleX = _b.maxScaleX, maxScaleY = _b.maxScaleY;
      var spacing = normalizeSpacing(args.spacing);
      var width = rect.width, height = rect.height;
      var scaleX = width / (maxNodeX - minNodeX + spacing.left + spacing.right);
      var scaleY = height / (maxNodeY - minNodeY + spacing.top + spacing.bottom);
      var scaleCommon = direction === exports.Direction.Y
          ? Math.min(Math.max(minScaleX, minScaleY, scaleY), maxScaleX, maxScaleY)
          : Math.min(Math.max(minScaleX, minScaleY, Math.min(scaleX, scaleY)), maxScaleY, maxScaleY);
      var newScaleX = direction === exports.Direction.XY ? Math.min(Math.max(minScaleX, scaleX), maxScaleX) : scaleCommon;
      var newScaleY = direction === exports.Direction.XY ? Math.min(Math.max(minScaleY, scaleY), maxScaleY) : scaleCommon;
      if (disablePan) {
          return [newScaleX, 0, 0, newScaleY, 0, 0];
      }
      var dx = -newScaleX * (minNodeX - spacing.left);
      var dy = -newScaleY * (minNodeY - spacing.top);
      var visibleNodes = getVisibleNodes(data.nodes, {
          rect: rect,
          transformMatrix: [newScaleX, 0, 0, newScaleY, dx, dy]
      }, graphConfig);
      if (visibleNodes.length > 0) {
          return [newScaleX, 0, 0, newScaleY, dx, dy];
      }
      var focusNode = data.nodes.first();
      if (focusNode) {
          data.nodes.forEach(function (node) {
              if (focusNode.y > node.y) {
                  focusNode = node;
              }
          });
      }
      return [
          newScaleX,
          0,
          0,
          newScaleY,
          -newScaleX * (focusNode.x - spacing.left),
          -newScaleY * (focusNode.y - spacing.top)
      ];
  };
  var focusArea = function (minX, minY, maxX, maxY, viewport) {
      var width = maxX - minX;
      var height = maxY - minY;
      var scale = Math.min(viewport.rect.width / width, viewport.rect.height / height);
      var dx = -scale * (minX + width / 2) + viewport.rect.width / 2;
      var dy = -scale * (minY + height / 2) + viewport.rect.height / 2;
      return __assign(__assign({}, viewport), { transformMatrix: [scale, 0, 0, scale, dx, dy] });
  };
  var getContainer = function (svgRef) { var _a; return (_a = svgRef.current) === null || _a === void 0 ? void 0 : _a.closest(".react-dag-editor-container"); };
  function getContainerCenter(rectRef) {
      var rect = rectRef.current;
      if (!rect) {
          return undefined;
      }
      var x = rect.width / 2;
      var y = rect.height / 2;
      return {
          x: x,
          y: y
      };
  }
  function getRelativePoint(rect, e) {
      var x = e.clientX - rect.left;
      var y = e.clientY - rect.top;
      return {
          x: x,
          y: y
      };
  }
  var scrollIntoView = function (x, y, rect, alwaysScroll, position) {
      if (!rect) {
          return identical;
      }
      var width = rect.width, height = rect.height;
      var shouldScroll = x < 0 || x > width || y < 0 || y > height;
      if (!shouldScroll && !alwaysScroll) {
          return identical;
      }
      return function (prevState) {
          var dx = position ? position.x - x : width / 2 - x;
          var dy = position ? position.y - y : height / 2 - y;
          return __assign(__assign({}, prevState), { transformMatrix: [
                  prevState.transformMatrix[0],
                  prevState.transformMatrix[1],
                  prevState.transformMatrix[2],
                  prevState.transformMatrix[3],
                  prevState.transformMatrix[4] + dx,
                  prevState.transformMatrix[5] + dy
              ] });
      };
  };
  var getScaleLimit = function (data, settings) {
      var _a = getContentArea$1(data, settings.graphConfig), minNodeWidth = _a.minNodeWidth, minNodeHeight = _a.minNodeHeight;
      var _b = getScaleRange(settings, {
          width: minNodeWidth,
          height: minNodeHeight
      }), minScaleX = _b.minScaleX, minScaleY = _b.minScaleY;
      return Math.max(minScaleX, minScaleY);
  };

  var maxCost;
  var Task = (function () {
      function Task(name, cost, dependencies) {
          var _this = this;
          this.dependencies = new Set();
          this.name = name;
          this.cost = cost;
          dependencies === null || dependencies === void 0 ? void 0 : dependencies.forEach(function (t) {
              _this.dependencies.add(t);
          });
          this.earlyFinish = -1;
      }
      Task.prototype.isDependent = function (t) {
          if (this.dependencies.has(t)) {
              return true;
          }
          return Array.from(this.dependencies).some(function (dep) { return dep.isDependent(t); });
      };
      return Task;
  }());
  var criticalPath = function (tasks) {
      var completed = new Set();
      var remaining = new Set(tasks);
      var _loop_1 = function () {
          var progress = false;
          remaining.forEach(function (task) {
              if (containAll(completed, task.dependencies)) {
                  var critical_1 = 0;
                  task.dependencies.forEach(function (t) {
                      if (t.criticalCost > critical_1) {
                          critical_1 = t.criticalCost;
                      }
                  });
                  task.criticalCost = critical_1 + task.cost;
                  completed.add(task);
                  remaining.delete(task);
                  progress = true;
              }
          });
          if (!progress) {
              throw new Error("Cyclic dependency, algorithm stopped!");
          }
      };
      while (remaining.size !== 0) {
          _loop_1();
      }
      getMaxCost(tasks);
      calculateEarly(tasks);
      var ret = Array.from(completed);
      ret.sort(function (o1, o2) {
          var i = o2.criticalCost - o1.criticalCost;
          if (i !== 0) {
              return i;
          }
          if (o1.isDependent(o2)) {
              return -1;
          }
          if (o2.isDependent(o1)) {
              return 1;
          }
          return 0;
      });
      return ret[0];
  };
  var setEarly = function (task) {
      var completionTime = task.earlyFinish;
      task.dependencies.forEach(function (t) {
          if (completionTime >= t.earlyStart) {
              t.earlyStart = completionTime;
              t.earlyFinish = completionTime + t.cost;
          }
          setEarly(t);
      });
  };
  var setLatest = function (task) {
      task.latestStart = maxCost - task.criticalCost;
      task.latestFinish = task.latestStart + task.cost;
  };
  var calculateEarly = function (tasks) {
      tasks.forEach(function (t) {
          t.earlyStart = 0;
          t.earlyFinish = t.cost;
          setEarly(t);
      });
  };
  var getMaxCost = function (tasks) {
      var max = -1;
      tasks.forEach(function (t) {
          if (t.criticalCost > max) {
              max = t.criticalCost;
          }
      });
      maxCost = max;
      tasks.forEach(function (t) {
          setLatest(t);
      });
  };
  var containAll = function (AllItems, items) {
      return Array.from(items).every(function (item) { return AllItems.has(item); });
  };

  /**
   * Topological sorting function
   *
   * @param {Array} edges
   * @returns {Array}
   */

  var array = toposort;

  function toposort(nodes, edges) {
    var cursor = nodes.length
      , sorted = new Array(cursor)
      , visited = {}
      , i = cursor;

    while (i--) {
      if (!visited[i]) visit(nodes[i], i, []);
    }

    return sorted

    function visit(node, i, predecessors) {
      if(predecessors.indexOf(node) >= 0) {
        var nodeRep; 
        try {
          nodeRep = ", node was:" + JSON.stringify(node);
        } catch(e) {
          nodeRep = "";
        }
        throw new Error('Cyclic dependency' + nodeRep)
      }

      if (!~nodes.indexOf(node)) {
        throw new Error('Found unknown node. Make sure to provided all involved nodes. Unknown node: '+JSON.stringify(node))
      }

      if (visited[i]) return;
      visited[i] = true;

      // outgoing edges
      var outgoing = edges.filter(function(edge){
        return edge[0] === node
      });
      if (i = outgoing.length) {
        var preds = predecessors.concat(node);
        do {
          var child = outgoing[--i][1];
          visit(child, nodes.indexOf(child), preds);
        } while (i)
      }

      sorted[--cursor] = node;
    }
  }

  var getTopoSortingNodes = function (canvasData) {
      var edges = canvasData.edges, nodes = canvasData.nodes;
      if (!nodes.length) {
          return [];
      }
      if (!edges.length) {
          return __spreadArray([], nodes);
      }
      var edgesForTopo = edges.map(function (edge) { return [edge.source, edge.target]; });
      var nodesForTopo = nodes.map(function (node) { return node.id; });
      var orderOfNodeIds = array(nodesForTopo, edgesForTopo);
      var newNodes = [];
      orderOfNodeIds.map(function (id) {
          var node = nodes.find(function (it) { return it.id === id; });
          if (node) {
              newNodes.push(node);
          }
      });
      return newNodes;
  };

  var getCriticalPath = function (canvasData, durations, isNodesSorted) {
      var edges = canvasData.edges;
      var nodesSorted = isNodesSorted ? __spreadArray([], canvasData.nodes) : getTopoSortingNodes(canvasData);
      var allTasks = new Set();
      var curAllTasksMap = new Map();
      nodesSorted.reverse().forEach(function (node) {
          allTasks === null || allTasks === void 0 ? void 0 : allTasks.forEach(function (t) {
              if (!curAllTasksMap.has(t.name)) {
                  curAllTasksMap.set(t.name, t);
              }
          });
          var task = new Task(node.id, durations.get(node.id) || 0, getSuccessors(node.id, edges, curAllTasksMap));
          allTasks.add(task);
      });
      var path = criticalPath(allTasks);
      var result = new Map();
      markCriticalPath(path, result, true);
      return result;
  };
  var markCriticalPath = function (path, ret, isCritical, parentTask) {
      var _a;
      if (isCritical === void 0) { isCritical = true; }
      if (isCritical && (!parentTask || ((_a = ret.get(parentTask.name)) === null || _a === void 0 ? void 0 : _a.isCritical))) {
          ret.set(path.name, { task: path, isCritical: isCritical });
      }
      else if (!ret.has(path.name)) {
          ret.set(path.name, { task: path, isCritical: false });
      }
      var dependencies = path.dependencies;
      if (dependencies.size) {
          var criticalCostTask_1 = getMaxCriticalCostTask(path.dependencies);
          markCriticalPath(criticalCostTask_1, ret, true, path);
          dependencies.forEach(function (t) {
              if (criticalCostTask_1.name !== t.name) {
                  markCriticalPath(t, ret, false, path);
              }
          });
      }
  };
  var getMaxCriticalCostTask = function (tasks) {
      var max = -Infinity;
      var criticalTask = tasks.values().next().value;
      tasks.forEach(function (t) {
          if (t.criticalCost > max) {
              max = t.criticalCost;
              criticalTask = t;
          }
      });
      return criticalTask;
  };
  var getSuccessors = function (nodeId, edges, allTasksMap) {
      var successors = new Set();
      edges.forEach(function (e) {
          if (nodeId === e.source) {
              var task = allTasksMap.get(e.target);
              if (task) {
                  successors.add(task);
              }
          }
      });
      return successors;
  };

  var is$1 = Object.is;

  var shallowEqual = function (a, b) {
      if (!a) {
          return false;
      }
      if (a.length !== b.length) {
          return false;
      }
      for (var i = 0; i < a.length; i += 1) {
          if (!is$1(a[i], b[i])) {
              return false;
          }
      }
      return true;
  };
  function memoize$1(f) {
      var prev;
      var value;
      return function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          if (shallowEqual(prev, args)) {
              return value;
          }
          prev = args;
          value = f.apply(undefined, args);
          return value;
      };
  }

  var getContentArea = memoize$1(getContentArea$1);
  var getOffsetLimit = function (_a) {
      var _b, _c, _d, _e;
      var data = _a.data, graphConfig = _a.graphConfig, rect = _a.rect, transformMatrix = _a.transformMatrix, canvasBoundaryPadding = _a.canvasBoundaryPadding, groupPadding = _a.groupPadding;
      var contentArea = getContentArea(data, graphConfig);
      var nodeMin = getClientDeltaByPointDelta(contentArea.minNodeX - ((groupPadding === null || groupPadding === void 0 ? void 0 : groupPadding.left) || 0), contentArea.minNodeY - ((groupPadding === null || groupPadding === void 0 ? void 0 : groupPadding.top) || 0), transformMatrix);
      nodeMin.x -= (_b = canvasBoundaryPadding === null || canvasBoundaryPadding === void 0 ? void 0 : canvasBoundaryPadding.left) !== null && _b !== void 0 ? _b : 0;
      nodeMin.y -= (_c = canvasBoundaryPadding === null || canvasBoundaryPadding === void 0 ? void 0 : canvasBoundaryPadding.top) !== null && _c !== void 0 ? _c : 0;
      var nodeMax = getClientDeltaByPointDelta(contentArea.maxNodeX + ((groupPadding === null || groupPadding === void 0 ? void 0 : groupPadding.right) || 0), contentArea.maxNodeY + ((groupPadding === null || groupPadding === void 0 ? void 0 : groupPadding.bottom) || 0), transformMatrix);
      nodeMax.x += (_d = canvasBoundaryPadding === null || canvasBoundaryPadding === void 0 ? void 0 : canvasBoundaryPadding.right) !== null && _d !== void 0 ? _d : 0;
      nodeMax.y += (_e = canvasBoundaryPadding === null || canvasBoundaryPadding === void 0 ? void 0 : canvasBoundaryPadding.bottom) !== null && _e !== void 0 ? _e : 0;
      var minX = -nodeMin.x || 0;
      var minY = -nodeMin.y || 0;
      var maxX = rect.width - nodeMax.x || 0;
      var maxY = rect.height - nodeMax.y || 0;
      if (maxX < minX) {
          var temp = maxX;
          maxX = minX;
          minX = temp;
      }
      if (maxY < minY) {
          var temp = maxY;
          maxY = minY;
          minY = temp;
      }
      return {
          minX: minX,
          minY: minY,
          maxX: maxX,
          maxY: maxY
      };
  };

  function isEdgeEvent(event) {
      return event.type.startsWith("onEdge");
  }
  function isNodeEvent(event) {
      return event.type.startsWith("onNode");
  }
  function isPortEvent(event) {
      return event.type.startsWith("onPort");
  }
  function isCanvasEvent(event) {
      return event.type.startsWith("onCanvas");
  }

  function resolveValueOrCallback(valueOrCallback, c) {
      if (typeof valueOrCallback === "function") {
          return valueOrCallback(c);
      }
      return valueOrCallback;
  }

  function isDef(v) {
      return v !== undefined && v !== null;
  }

  var debounce = function (callback, timeout, options) {
      var _a = options || {}, instance = _a.instance, maxWait = _a.maxWait;
      var timer = 0;
      var firstInvokeTime;
      return function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          window.clearTimeout(timer);
          if (isDef(maxWait)) {
              var now = Date.now();
              if (!isDef(firstInvokeTime)) {
                  firstInvokeTime = now;
              }
              else {
                  if (now - firstInvokeTime >= maxWait) {
                      firstInvokeTime = undefined;
                      invokeCallback(args);
                      return;
                  }
              }
          }
          timer = window.setTimeout(function () {
              invokeCallback(args);
          }, timeout);
      };
      function invokeCallback(args) {
          callback.apply(instance, args);
      }
  };

  function arrayEquals(a, b) {
      if (a.length !== b.length) {
          return false;
      }
      for (var i = 0; i < a.length; i++) {
          if (!Object.is(a[i], b[i])) {
              return false;
          }
      }
      return true;
  }

  var emptyArrayInstance = [];
  function constantEmptyArray() {
      return emptyArrayInstance;
  }

  function stopPropagation(e) {
      e.stopPropagation();
  }

  function clamp(min, max, value) {
      if (min > value) {
          return min;
      }
      if (max < value) {
          return max;
      }
      return value;
  }

  var pushHistory = function (history, data, mapPresent) {
      if (mapPresent === void 0) { mapPresent = identical; }
      return {
          present: data,
          past: {
              next: history.past,
              value: mapPresent(history.present)
          },
          future: null
      };
  };
  var undo = function (history) {
      if (!history.past) {
          return history;
      }
      return {
          present: history.past.value,
          past: history.past.next,
          future: {
              next: history.future,
              value: history.present
          }
      };
  };
  var redo = function (history) {
      if (!history.future) {
          return history;
      }
      return {
          present: history.future.value,
          past: {
              next: history.past,
              value: history.present
          },
          future: history.future.next
      };
  };
  var resetUndoStack = function (data) {
      return {
          present: data,
          future: null,
          past: null
      };
  };
  var canUndo = function (history) { return history.past !== null; };
  var canRedo = function (history) { return history.future !== null; };

  var RectComponent = function (props) {
      var style = props.style, node = props.node, width = props.width, height = props.height, textY = props.textY;
      var comment = node.data && node.data.comment ? node.data.comment : "";
      var isEditing = isNodeEditing(node);
      return (React__namespace.createElement("g", { key: node.id },
          React__namespace.createElement("rect", { width: width, height: height, x: node.x, y: node.y, style: style, rx: style.borderRadius }),
          React__namespace.createElement("text", { x: node.x, y: textY, fontSize: 12 }, node.name),
          node.data && node.data.comment && !isEditing && (React__namespace.createElement("text", { x: node.x, y: textY + 20, fontSize: 12, className: "comment-" + node.id }, node.data.comment)),
          isEditing && (React__namespace.createElement("foreignObject", { x: node.x, y: textY, height: height / 2.5, width: width - 5 },
              React__namespace.createElement("input", { value: comment, placeholder: "Input your comment here" })))));
  };

  var rect = {
      getMinHeight: function () {
          return 150;
      },
      getMinWidth: function () {
          return 150;
      },
      getStyle: function (node, theme) {
          if (hasState(exports.GraphNodeState.selected | exports.GraphNodeState.activated)(node.state)) {
              return {
                  fill: theme.nodeActivateFill,
                  stroke: theme.nodeActivateStroke
              };
          }
          return {
              fill: theme.nodeFill,
              fillOpacity: 0.1,
              stroke: theme.nodeStroke,
              borderRadius: "5"
          };
      },
      render: function (args) {
          var node = args.model;
          var width = getRectWidth(rect, node);
          var height = getRectHeight(rect, node);
          var style = rect.getStyle ? rect.getStyle(node, args.theme) : {};
          var textY = node.y + height / 3;
          return React__namespace.createElement(RectComponent, { style: style, node: node, width: width, height: height, textY: textY });
      }
  };

  var line = {
      getStyle: function (edge, theme) {
          return {
              cursor: "crosshair",
              stroke: hasState(exports.GraphEdgeState.selected)(edge.state) ? theme.edgeColorSelected : theme.edgeColor,
              strokeWidth: "2"
          };
      },
      render: function (args) {
          var edge = args.model;
          var style = line.getStyle ? line.getStyle(edge, args.theme) : {};
          return (React__namespace.createElement("path", { key: edge.id, d: getCurvePathD(args.x2, args.x1, args.y2, args.y1), fill: "none", style: style, id: "edge" + edge.id }));
      }
  };

  var DefaultPort = (function () {
      function DefaultPort() {
      }
      DefaultPort.prototype.getStyle = function (port, parentNode, data, theme, connectedAsSource, connectedAsTarget) {
          var stroke = theme.portStroke;
          var fill = theme.portFill;
          if (connectedAsSource || connectedAsTarget) {
              fill = theme.connectedPortColor;
          }
          if (hasState(exports.GraphPortState.activated)(port.state)) {
              fill = theme.primaryColor;
          }
          return {
              stroke: stroke,
              fill: fill
          };
      };
      DefaultPort.prototype.getIsConnectable = function () {
          return true;
      };
      DefaultPort.prototype.render = function (args) {
          var port = args.model, data = args.data, parentNode = args.parentNode;
          var connectedAsSource = data.isPortConnectedAsSource(parentNode.id, port.id);
          var connectedAsTarget = data.isPortConnectedAsTarget(parentNode.id, port.id);
          var style = this.getStyle(port, parentNode, data, args.theme, connectedAsSource, connectedAsTarget);
          var x = args.x, y = args.y;
          var polygonPoints = x - 5 + " " + y + ", " + (x + 7) + " " + y + ", " + (x + 1) + " " + (y + 8);
          return (React__namespace.createElement(React__namespace.Fragment, null, connectedAsTarget ? (React__namespace.createElement("polygon", { points: polygonPoints, style: style })) : (React__namespace.createElement("circle", { key: args.parentNode.id + "-" + args.model.id, r: 5, cx: x, cy: y, style: style }))));
      };
      return DefaultPort;
  }());
  var defaultPort = new DefaultPort();

  // Unique ID creation requires a high quality random # generator. In the browser we therefore
  // require the crypto API and do not support built-in fallback to lower quality random number
  // generators (like Math.random()).
  // getRandomValues needs to be invoked in a context where "this" is a Crypto implementation. Also,
  // find the complete implementation of crypto (msCrypto) on IE11.
  var getRandomValues = typeof crypto != 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto) || typeof msCrypto != 'undefined' && typeof msCrypto.getRandomValues == 'function' && msCrypto.getRandomValues.bind(msCrypto);
  var rnds8 = new Uint8Array(16); // eslint-disable-line no-undef

  function rng() {
    if (!getRandomValues) {
      throw new Error('crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported');
    }

    return getRandomValues(rnds8);
  }

  /**
   * Convert array of 16 byte values to UUID string format of the form:
   * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
   */
  var byteToHex = [];

  for (var i = 0; i < 256; ++i) {
    byteToHex[i] = (i + 0x100).toString(16).substr(1);
  }

  function bytesToUuid(buf, offset) {
    var i = offset || 0;
    var bth = byteToHex; // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4

    return [bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]]].join('');
  }

  function v4(options, buf, offset) {
    var i = buf && offset || 0;

    if (typeof options == 'string') {
      buf = options === 'binary' ? new Array(16) : null;
      options = null;
    }

    options = options || {};
    var rnds = options.random || (options.rng || rng)(); // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`

    rnds[6] = rnds[6] & 0x0f | 0x40;
    rnds[8] = rnds[8] & 0x3f | 0x80; // Copy bytes to buffer, if provided

    if (buf) {
      for (var ii = 0; ii < 16; ++ii) {
        buf[i + ii] = rnds[ii];
      }
    }

    return buf || bytesToUuid(rnds);
  }

  var DefaultClipboard = (function () {
      function DefaultClipboard(storage) {
          this.storage = storage;
      }
      DefaultClipboard.prototype.write = function (data) {
          this.storage.setItem("graph-clipboard", JSON.stringify({
              nodes: data.nodes.map(function (n) { return (__assign(__assign({}, n), { data: {} })); }),
              edges: data.edges.map(function (e) { return (__assign(__assign({}, e), { data: {} })); })
          }));
      };
      DefaultClipboard.prototype.read = function () {
          var str = this.storage.getItem("graph-clipboard");
          if (!str) {
              return null;
          }
          try {
              var data = JSON.parse(str);
              var nodeIdHash_1 = new Map();
              return {
                  nodes: data.nodes.map(function (n) {
                      var newId = v4();
                      nodeIdHash_1.set(n.id, newId);
                      return __assign(__assign({}, n), { x: n.x + COPIED_NODE_SPACING, y: n.y + COPIED_NODE_SPACING, id: newId });
                  }),
                  edges: data.edges.map(function (e) { return (__assign(__assign({}, e), { id: v4(), source: nodeIdHash_1.get(e.source) || "", target: nodeIdHash_1.get(e.target) || "" })); })
              };
          }
          catch (error) {
              return null;
          }
      };
      return DefaultClipboard;
  }());

  var DefaultStorage = (function () {
      function DefaultStorage() {
          this.key = function () { return "DefaultLocalStorage"; };
          this.items = {};
      }
      Object.defineProperty(DefaultStorage.prototype, "length", {
          get: function () {
              return Object.keys(this.items || {}).length;
          },
          enumerable: false,
          configurable: true
      });
      DefaultStorage.prototype.clear = function () {
          this.items = {};
      };
      DefaultStorage.prototype.setItem = function (key, data) {
          this.items[key] = data;
      };
      DefaultStorage.prototype.getItem = function (key) {
          return this.items[key];
      };
      DefaultStorage.prototype.removeItem = function (key) {
          this.items[key] = undefined;
      };
      return DefaultStorage;
  }());

  function applyDefaultPortsPosition(list) {
      var inputPorts = list.filter(function (p) { return !p.isInputDisabled; });
      var outputPorts = list.filter(function (p) { return p.isInputDisabled; });
      var inputPortsCount = inputPorts.length;
      var outputPortsCount = outputPorts.length;
      var inputPortsInterval = 1 / (inputPortsCount + 1);
      var outputPortsInterval = 1 / (outputPortsCount + 1);
      var res = [];
      for (var i = 0; i < inputPortsCount; i += 1) {
          res.push(__assign(__assign({}, inputPorts[i]), { position: [(i + 1) * inputPortsInterval, 0] }));
      }
      for (var i = 0; i < outputPortsCount; i += 1) {
          var port = outputPorts[i];
          res.push(__assign(__assign({}, port), { position: [(i + 1) * outputPortsInterval, 1] }));
      }
      return res;
  }

  function applyHorizontalGraphPortsPosition(list) {
      var inputPorts = list.filter(function (p) { return !p.isInputDisabled; });
      var outputPorts = list.filter(function (p) { return !p.isOutputDisabled; });
      var inputPortsCount = inputPorts.length;
      var outputPortsCount = outputPorts.length;
      var inputPortsInterval = 1 / (inputPortsCount + 1);
      var outputPortsInterval = 1 / (outputPortsCount + 1);
      var res = [];
      for (var i = 0; i < inputPortsCount; i += 1) {
          res.push(__assign(__assign({}, inputPorts[i]), { position: [0, (i + 1) * inputPortsInterval] }));
      }
      for (var i = 0; i < outputPortsCount; i += 1) {
          res.push(__assign(__assign({}, outputPorts[i]), { position: [1, (i + 1) * outputPortsInterval] }));
      }
      return res;
  }

  var Renderer = function (props) { return (React__namespace.createElement("rect", { height: props.height, width: props.width, fill: props.group.fill })); };
  var defaultGroup = {
      render: Renderer
  };

  var GraphConfig = (function () {
      function GraphConfig(defaultEdgeShape, defaultNodeShape, defaultPortShape, defaultGroupShape, nodeConfigMap, edgeConfigMap, portConfigMap, groupConfigMap, clipboard) {
          this.defaultEdgeShape = defaultEdgeShape;
          this.defaultNodeShape = defaultNodeShape;
          this.defaultPortShape = defaultPortShape;
          this.defaultGroupShape = defaultGroupShape;
          this.nodeConfigMap = nodeConfigMap;
          this.edgeConfigMap = edgeConfigMap;
          this.portConfigMap = portConfigMap;
          this.groupConfigMap = groupConfigMap;
          this.clipboard = clipboard;
      }
      GraphConfig.prototype.getClipboard = function () {
          return this.clipboard;
      };
      GraphConfig.prototype.getEdgeConfigByName = function (name) {
          return this.edgeConfigMap.get(name || this.defaultEdgeShape);
      };
      GraphConfig.prototype.getGroupConfigByName = function (name) {
          return this.groupConfigMap.get(name || this.defaultGroupShape);
      };
      GraphConfig.prototype.getNodeConfigByName = function (name) {
          return this.nodeConfigMap.get(name || this.defaultNodeShape);
      };
      GraphConfig.prototype.getPortConfigByName = function (name) {
          return this.portConfigMap.get(name || this.defaultPortShape);
      };
      return GraphConfig;
  }());

  var GraphConfigBuilder = (function () {
      function GraphConfigBuilder(defaultEdgeShape, defaultNodeShape, defaultPortShape, defaultGroupShape, nodeConfigMap, edgeConfigMap, portConfigMap, groupConfigMap, clipboard) {
          this.defaultEdgeShape = defaultEdgeShape;
          this.defaultNodeShape = defaultNodeShape;
          this.defaultPortShape = defaultPortShape;
          this.defaultGroupShape = defaultGroupShape;
          this.nodeConfigMap = nodeConfigMap;
          this.edgeConfigMap = edgeConfigMap;
          this.portConfigMap = portConfigMap;
          this.groupConfigMap = groupConfigMap;
          this.clipboard = clipboard;
      }
      GraphConfigBuilder.default = function () {
          return new GraphConfigBuilder("default", "default", "default", "default", new Map().set("default", rect), new Map().set("default", line), new Map().set("default", defaultPort), new Map().set("default", defaultGroup), new DefaultClipboard(new DefaultStorage()));
      };
      GraphConfigBuilder.from = function (graphConfig) {
          return new GraphConfigBuilder(graphConfig.defaultEdgeShape, graphConfig.defaultNodeShape, graphConfig.defaultPortShape, graphConfig.defaultGroupShape, new Map(graphConfig.nodeConfigMap), new Map(graphConfig.edgeConfigMap), new Map(graphConfig.portConfigMap), new Map(graphConfig.groupConfigMap), graphConfig.clipboard);
      };
      GraphConfigBuilder.prototype.withDefaultEdgeShape = function (shape) {
          this.defaultEdgeShape = shape;
          return this;
      };
      GraphConfigBuilder.prototype.withDefaultNodeShape = function (shape) {
          this.defaultNodeShape = shape;
          return this;
      };
      GraphConfigBuilder.prototype.withDefaultPortShape = function (shape) {
          this.defaultPortShape = shape;
          return this;
      };
      GraphConfigBuilder.prototype.withDefaultGroupShape = function (shape) {
          this.defaultGroupShape = shape;
          return this;
      };
      GraphConfigBuilder.prototype.withClipboard = function (clipboard) {
          this.clipboard = clipboard;
          return this;
      };
      GraphConfigBuilder.prototype.registerNode = function (name, config) {
          this.nodeConfigMap.set(name, config);
          return this;
      };
      GraphConfigBuilder.prototype.registerEdge = function (name, config) {
          this.edgeConfigMap.set(name, config);
          return this;
      };
      GraphConfigBuilder.prototype.registerPort = function (name, config) {
          this.portConfigMap.set(name, config);
          return this;
      };
      GraphConfigBuilder.prototype.registerGroup = function (name, config) {
          this.groupConfigMap.set(name, config);
          return this;
      };
      GraphConfigBuilder.prototype.build = function () {
          return new GraphConfig(this.defaultEdgeShape, this.defaultNodeShape, this.defaultPortShape, this.defaultGroupShape, new Map(this.nodeConfigMap), new Map(this.edgeConfigMap), new Map(this.portConfigMap), new Map(this.groupConfigMap), this.clipboard);
      };
      return GraphConfigBuilder;
  }());

  var GraphConfigContext = React.createContext(GraphConfigBuilder.default().build());

  exports.MenuType = void 0;
  (function (MenuType) {
      MenuType["Node"] = "node";
      MenuType["Edge"] = "edge";
      MenuType["Port"] = "port";
      MenuType["Canvas"] = "canvas";
      MenuType["Multi"] = "multi";
  })(exports.MenuType || (exports.MenuType = {}));
  var ContextMenuConfig = (function () {
      function ContextMenuConfig() {
          this.contextMenu = {};
      }
      ContextMenuConfig.prototype.registerContextMenu = function (props) {
          this.contextMenuProps = __assign({}, props);
      };
      ContextMenuConfig.prototype.registerMenu = function (element, menuType) {
          this.contextMenu[menuType] = element;
      };
      ContextMenuConfig.prototype.getMenu = function (menuType) {
          if (this.contextMenuProps && this.contextMenu[menuType]) {
              var _a = this.contextMenuProps, className = _a.className, styles = _a.styles;
              var reactElement = React__namespace.createElement("div", {
                  className: className,
                  style: styles
              }, this.contextMenu[menuType]);
              return reactElement;
          }
          return null;
      };
      return ContextMenuConfig;
  }());
  var ContextMenuConfigContext = React__namespace.createContext(new ContextMenuConfig());

  var ThemeContext$1 = React__namespace.createContext({
      theme: {},
      setTheme: function () {
      }
  });

  var emptyDummyNodes = function () { return ({
      dx: 0,
      dy: 0,
      dWidth: 0,
      dHeight: 0,
      alignedDX: undefined,
      alignedDY: undefined,
      nodes: [],
      isVisible: false
  }); };

  var useTheme = function () {
      return React__namespace.useContext(ThemeContext$1);
  };

  var emptySelectBoxPosition = function () { return ({
      startX: 0,
      startY: 0,
      height: 0,
      width: 0
  }); };
  var SelectBox = function (props) {
      var selectBoxPosition = props.selectBoxPosition, style = props.style;
      var theme = useTheme().theme;
      var selectBoxD = "m" + selectBoxPosition.startX + " " + selectBoxPosition.startY + " v " + selectBoxPosition.height + " h " + selectBoxPosition.width + " v" + -selectBoxPosition.height + " h " + -selectBoxPosition.width;
      var pathStyle = style !== null && style !== void 0 ? style : { fill: "none", stroke: theme.defaultColor };
      return React__namespace.createElement("path", { style: pathStyle, d: selectBoxD });
  };

  exports.GraphFeatures = void 0;
  (function (GraphFeatures) {
      GraphFeatures["nodeDraggable"] = "nodeDraggable";
      GraphFeatures["nodeResizable"] = "nodeResizable";
      GraphFeatures["clickNodeToSelect"] = "clickNodeToSelect";
      GraphFeatures["panCanvas"] = "panCanvas";
      GraphFeatures["multipleSelect"] = "multipleSelect";
      GraphFeatures["lassoSelect"] = "lassoSelect";
      GraphFeatures["delete"] = "delete";
      GraphFeatures["addNewNodes"] = "addNewNodes";
      GraphFeatures["addNewEdges"] = "addNewEdges";
      GraphFeatures["addNewPorts"] = "addNewPorts";
      GraphFeatures["autoFit"] = "autoFit";
      GraphFeatures["canvasHorizontalScrollable"] = "canvasHorizontalScrollable";
      GraphFeatures["canvasVerticalScrollable"] = "canvasVerticalScrollable";
      GraphFeatures["nodeHoverView"] = "nodeHoverView";
      GraphFeatures["portHoverView"] = "portHoverView";
      GraphFeatures["addEdgesByKeyboard"] = "addEdgesByKeyboard";
      GraphFeatures["a11yFeatures"] = "a11yFeatures";
      GraphFeatures["editNode"] = "editNode";
      GraphFeatures["autoAlign"] = "autoAlign";
      GraphFeatures["undoStack"] = "undoStack";
      GraphFeatures["ctrlKeyZoom"] = "ctrlKeyZoom";
      GraphFeatures["limitBoundary"] = "limitBoundary";
      GraphFeatures["editEdge"] = "editEdge";
  })(exports.GraphFeatures || (exports.GraphFeatures = {}));
  var allFeatures = new Set([
      exports.GraphFeatures.nodeDraggable,
      exports.GraphFeatures.nodeResizable,
      exports.GraphFeatures.clickNodeToSelect,
      exports.GraphFeatures.panCanvas,
      exports.GraphFeatures.multipleSelect,
      exports.GraphFeatures.lassoSelect,
      exports.GraphFeatures.delete,
      exports.GraphFeatures.addNewNodes,
      exports.GraphFeatures.addNewEdges,
      exports.GraphFeatures.addNewPorts,
      exports.GraphFeatures.canvasHorizontalScrollable,
      exports.GraphFeatures.canvasVerticalScrollable,
      exports.GraphFeatures.nodeHoverView,
      exports.GraphFeatures.portHoverView,
      exports.GraphFeatures.addEdgesByKeyboard,
      exports.GraphFeatures.a11yFeatures,
      exports.GraphFeatures.autoFit,
      exports.GraphFeatures.editNode,
      exports.GraphFeatures.autoAlign,
      exports.GraphFeatures.undoStack,
      exports.GraphFeatures.ctrlKeyZoom,
      exports.GraphFeatures.limitBoundary,
      exports.GraphFeatures.editEdge
  ]);
  var defaultFeatures = new Set([
      exports.GraphFeatures.nodeDraggable,
      exports.GraphFeatures.nodeResizable,
      exports.GraphFeatures.clickNodeToSelect,
      exports.GraphFeatures.panCanvas,
      exports.GraphFeatures.multipleSelect,
      exports.GraphFeatures.delete,
      exports.GraphFeatures.addNewNodes,
      exports.GraphFeatures.addNewEdges,
      exports.GraphFeatures.addNewPorts,
      exports.GraphFeatures.canvasHorizontalScrollable,
      exports.GraphFeatures.canvasVerticalScrollable,
      exports.GraphFeatures.nodeHoverView,
      exports.GraphFeatures.portHoverView,
      exports.GraphFeatures.addEdgesByKeyboard,
      exports.GraphFeatures.a11yFeatures,
      exports.GraphFeatures.editNode,
      exports.GraphFeatures.autoAlign,
      exports.GraphFeatures.undoStack,
      exports.GraphFeatures.ctrlKeyZoom,
      exports.GraphFeatures.limitBoundary
  ]);
  var dataReadonlyMode = new Set([
      exports.GraphFeatures.nodeDraggable,
      exports.GraphFeatures.nodeResizable,
      exports.GraphFeatures.clickNodeToSelect,
      exports.GraphFeatures.panCanvas,
      exports.GraphFeatures.multipleSelect,
      exports.GraphFeatures.canvasHorizontalScrollable,
      exports.GraphFeatures.canvasVerticalScrollable,
      exports.GraphFeatures.nodeHoverView,
      exports.GraphFeatures.portHoverView,
      exports.GraphFeatures.a11yFeatures,
      exports.GraphFeatures.ctrlKeyZoom,
      exports.GraphFeatures.limitBoundary
  ]);
  var GanttChartFeatures = new Set([
      exports.GraphFeatures.clickNodeToSelect,
      exports.GraphFeatures.canvasHorizontalScrollable,
      exports.GraphFeatures.canvasVerticalScrollable,
      exports.GraphFeatures.nodeHoverView,
      exports.GraphFeatures.portHoverView,
      exports.GraphFeatures.a11yFeatures,
      exports.GraphFeatures.lassoSelect,
      exports.GraphFeatures.limitBoundary
  ]);
  var previewMode = new Set([exports.GraphFeatures.nodeHoverView, exports.GraphFeatures.portHoverView, exports.GraphFeatures.autoFit]);

  var is = Object.is;
  var MapIterator = (function () {
      function MapIterator(upstream, f) {
          this.upstream = upstream;
          this.f = f;
      }
      MapIterator.prototype[Symbol.iterator] = function () {
          return this;
      };
      MapIterator.prototype.next = function () {
          var next = this.upstream.next();
          if (next.done) {
              return next;
          }
          return {
              done: false,
              value: this.f(next.value)
          };
      };
      return MapIterator;
  }());

  var NodeType$1;
  (function (NodeType) {
      NodeType[NodeType["Bitmap"] = 0] = "Bitmap";
      NodeType[NodeType["Collision"] = 1] = "Collision";
  })(NodeType$1 || (NodeType$1 = {}));
  var HASH_CODE_LENGTH = 30;
  var BIT_PARTITION_SIZE = 5;
  var FULL_MASK = 0x3fffffff;
  function bitPosFrom(mask) {
      return 1 << mask;
  }
  function indexFrom(bitmap, mask, bitPos) {
      return bitmap === FULL_MASK ? mask : bitCount(bitmap & (bitPos - 1));
  }
  function maskFrom(hash, shift) {
      return (hash >>> shift) & 31;
  }
  function bitCount(x) {
      x |= 0;
      x -= (x >>> 1) & 0x55555555;
      x = (x & 0x33333333) + ((x >>> 2) & 0x33333333);
      x = (x + (x >>> 4)) & 0x0f0f0f0f;
      x += x >>> 8;
      x += x >>> 16;
      return x & 0x7f;
  }
  var BitmapIndexedNode = (function () {
      function BitmapIndexedNode(owner, dataMap, nodeMap, keys, values, children, hashes, size) {
          this.type = NodeType$1.Bitmap;
          this.owner = owner;
          this.dataMap = dataMap;
          this.nodeMap = nodeMap;
          this.keys = keys;
          this.values = values;
          this.children = children;
          this.hashes = hashes;
          this.size = size;
      }
      Object.defineProperty(BitmapIndexedNode.prototype, "valueCount", {
          get: function () {
              return this.values.length;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(BitmapIndexedNode.prototype, "nodeCount", {
          get: function () {
              return this.children.length;
          },
          enumerable: false,
          configurable: true
      });
      BitmapIndexedNode.empty = function (owner) {
          return new BitmapIndexedNode(owner, 0, 0, [], [], [], [], 0);
      };
      BitmapIndexedNode.prototype.getKey = function (index) {
          return this.keys[index];
      };
      BitmapIndexedNode.prototype.getValue = function (index) {
          return this.values[index];
      };
      BitmapIndexedNode.prototype.getHash = function (index) {
          return this.hashes[index];
      };
      BitmapIndexedNode.prototype.getNode = function (index) {
          return this.children[index];
      };
      BitmapIndexedNode.prototype.contains = function (key, keyHash, shift) {
          var mask = maskFrom(keyHash, shift);
          var bitPos = bitPosFrom(mask);
          var _a = this, dataMap = _a.dataMap, nodeMap = _a.nodeMap;
          if ((dataMap & bitPos) !== 0) {
              var index = indexFrom(dataMap, mask, bitPos);
              var key0 = this.getKey(index);
              return is(key0, key);
          }
          else if ((nodeMap & bitPos) !== 0) {
              var index = indexFrom(nodeMap, mask, bitPos);
              return this.getNode(index).contains(key, keyHash, shift + BIT_PARTITION_SIZE);
          }
          return false;
      };
      BitmapIndexedNode.prototype.get = function (key, keyHash, shift) {
          var mask = maskFrom(keyHash, shift);
          var bitPos = bitPosFrom(mask);
          var _a = this, dataMap = _a.dataMap, nodeMap = _a.nodeMap;
          if ((dataMap & bitPos) !== 0) {
              var index = indexFrom(dataMap, mask, bitPos);
              var key0 = this.getKey(index);
              return is(key0, key) ? this.getValue(index) : undefined;
          }
          else if ((nodeMap & bitPos) !== 0) {
              var index = indexFrom(nodeMap, mask, bitPos);
              return this.getNode(index).get(key, keyHash, shift + BIT_PARTITION_SIZE);
          }
          return undefined;
      };
      BitmapIndexedNode.prototype.insert = function (owner, key, value, hash, shift) {
          var mask = maskFrom(hash, shift);
          var bitPos = bitPosFrom(mask);
          var _a = this, dataMap = _a.dataMap, nodeMap = _a.nodeMap;
          if ((dataMap & bitPos) !== 0) {
              var index = indexFrom(dataMap, mask, bitPos);
              var key0 = this.getKey(index);
              var value0 = this.getValue(index);
              var key0Hash = this.getHash(index);
              if (key0Hash === hash && is(key0, key)) {
                  return is(value0, value) ? this : this.setValue(owner, value, index);
              }
              else {
                  var subNodeNew = mergeTwoKeyValPairs(owner, key0, value0, key0Hash, key, value, hash, shift + BIT_PARTITION_SIZE);
                  return this.migrateInlineToNode(owner, bitPos, subNodeNew);
              }
          }
          else if ((nodeMap & bitPos) !== 0) {
              var index = indexFrom(nodeMap, mask, bitPos);
              var subNode = this.getNode(index);
              var subNodeNew = subNode.insert(owner, key, value, hash, shift + BIT_PARTITION_SIZE);
              return this.setNode(owner, 1, subNodeNew, bitPos);
          }
          return this.insertValue(owner, bitPos, key, hash, value);
      };
      BitmapIndexedNode.prototype.update = function (owner, key, updater, hash, shift) {
          var mask = maskFrom(hash, shift);
          var bitPos = bitPosFrom(mask);
          var _a = this, dataMap = _a.dataMap, nodeMap = _a.nodeMap;
          if ((dataMap & bitPos) !== 0) {
              var index = indexFrom(dataMap, mask, bitPos);
              var key0 = this.getKey(index);
              var key0Hash = this.getHash(index);
              if (key0Hash === hash && is(key0, key)) {
                  var value0 = this.getValue(index);
                  var value = updater(value0);
                  return is(value0, value) ? this : this.setValue(owner, value, index);
              }
          }
          else if ((nodeMap & bitPos) !== 0) {
              var index = indexFrom(nodeMap, mask, bitPos);
              var subNode = this.getNode(index);
              var subNodeNew = subNode.update(owner, key, updater, hash, shift + BIT_PARTITION_SIZE);
              return subNodeNew === subNode ? this : this.setNode(owner, 0, subNodeNew, bitPos);
          }
          return this;
      };
      BitmapIndexedNode.prototype.remove = function (owner, key, keyHash, shift) {
          var mask = maskFrom(keyHash, shift);
          var bitPos = bitPosFrom(mask);
          if ((this.dataMap & bitPos) !== 0) {
              var index = indexFrom(this.dataMap, mask, bitPos);
              var key0 = this.getKey(index);
              if (is(key0, key)) {
                  return this.removeValue(owner, bitPos);
              }
              return undefined;
          }
          else if ((this.nodeMap & bitPos) !== 0) {
              var index = indexFrom(this.nodeMap, mask, bitPos);
              var subNode = this.getNode(index);
              var remove = subNode.remove(owner, key, keyHash, shift + BIT_PARTITION_SIZE);
              if (remove === undefined) {
                  return undefined;
              }
              var subNodeNew = remove[0], removedValue = remove[1];
              var subNodeNewSize = subNodeNew.size;
              if (subNodeNewSize === 1) {
                  if (this.size === subNode.size) {
                      return [
                          new BitmapIndexedNode(owner, bitPos, 0, [subNodeNew.getKey(0)], [subNodeNew.getValue(0)], [], [subNodeNew.getHash(0)], 1),
                          removedValue
                      ];
                  }
                  return [this.migrateNodeToInline(owner, bitPos, subNodeNew), removedValue];
              }
              return [this.setNode(owner, -1, subNodeNew, bitPos), removedValue];
          }
          return undefined;
      };
      BitmapIndexedNode.prototype.toOwned = function (owner) {
          return this.owner === owner
              ? this
              : new BitmapIndexedNode(owner, this.dataMap, this.nodeMap, this.keys.slice(), this.values.slice(), this.children.slice(), this.hashes.slice(), this.size);
      };
      BitmapIndexedNode.prototype.iter = function () {
          return new BitmapIndexedNodeIterator(this);
      };
      BitmapIndexedNode.prototype.map = function (owner, f) {
          var valueCount = this.valueCount;
          var keys = [];
          var values = [];
          var children = [];
          var same = true;
          for (var i = 0; i < valueCount; i += 1) {
              var key = this.getKey(i);
              var value0 = this.getValue(i);
              var value = f(value0, key);
              same = same && is(value0, value);
              keys.push(key);
              values.push(value);
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var node0 = this.getNode(i);
              var node = node0.map(owner, f);
              same = same && node === node0;
              children.push(node);
          }
          return same
              ? this
              : new BitmapIndexedNode(owner, this.dataMap, this.nodeMap, keys, values, children, this.hashes, this.size);
      };
      BitmapIndexedNode.prototype.forEach = function (f) {
          for (var i = 0; i < this.values.length; i += 1) {
              var key = this.getKey(i);
              var value = this.getValue(i);
              f(value, key);
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var child = this.getNode(i);
              child.forEach(f);
          }
      };
      BitmapIndexedNode.prototype.find = function (predicate) {
          for (var i = 0; i < this.values.length; i += 1) {
              var value = this.getValue(i);
              if (predicate(value)) {
                  return value;
              }
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var child = this.getNode(i);
              var found = child.find(predicate);
              if (found) {
                  return found;
              }
          }
          return undefined;
      };
      BitmapIndexedNode.prototype.dataIndex = function (bitPos) {
          return bitCount(this.dataMap & (bitPos - 1));
      };
      BitmapIndexedNode.prototype.nodeIndex = function (bitPos) {
          return bitCount(this.nodeMap & (bitPos - 1));
      };
      BitmapIndexedNode.prototype.setValue = function (owner, value, index) {
          var next = this.toOwned(owner);
          next.values[index] = value;
          return next;
      };
      BitmapIndexedNode.prototype.insertValue = function (owner, bitPos, key, originalHash, value) {
          var index = this.dataIndex(bitPos);
          var next = this.toOwned(owner);
          next.size += 1;
          next.dataMap |= bitPos;
          next.keys.splice(index, 0, key);
          next.values.splice(index, 0, value);
          next.hashes.splice(index, 0, originalHash);
          return next;
      };
      BitmapIndexedNode.prototype.migrateInlineToNode = function (owner, bitPos, child) {
          var indexOld = this.dataIndex(bitPos);
          var indexNew = this.nodeIndex(bitPos);
          var next = this.toOwned(owner);
          next.dataMap ^= bitPos;
          next.nodeMap |= bitPos;
          next.keys.splice(indexOld, 1);
          next.values.splice(indexOld, 1);
          next.children.splice(indexNew, 0, child);
          next.hashes.splice(indexOld, 1);
          next.size += 1;
          return next;
      };
      BitmapIndexedNode.prototype.migrateNodeToInline = function (owner, bitPos, node) {
          var indexOld = this.nodeIndex(bitPos);
          var indexNew = this.dataIndex(bitPos);
          var key = node.getKey(0);
          var value = node.getValue(0);
          var hash = node.getHash(0);
          var next = this.toOwned(owner);
          next.dataMap = next.dataMap | bitPos;
          next.nodeMap = next.nodeMap ^ bitPos;
          next.children.splice(indexOld, 1);
          next.keys.splice(indexNew, 0, key);
          next.values.splice(indexNew, 0, value);
          next.size -= 1;
          next.hashes.splice(indexNew, 0, hash);
          return next;
      };
      BitmapIndexedNode.prototype.setNode = function (owner, sizeDelta, newNode, bitPos) {
          var index = this.nodeIndex(bitPos);
          var next = this.toOwned(owner);
          next.children[index] = newNode;
          next.size = next.size + sizeDelta;
          return next;
      };
      BitmapIndexedNode.prototype.removeValue = function (owner, bitPos) {
          var index = this.dataIndex(bitPos);
          var value = this.getValue(index);
          var next = this.toOwned(owner);
          next.dataMap ^= bitPos;
          next.keys.splice(index, 1);
          next.values.splice(index, 1);
          next.hashes.splice(index, 1);
          next.size -= 1;
          return [next, value];
      };
      return BitmapIndexedNode;
  }());
  function mergeTwoKeyValPairs(owner, key0, value0, keyHash0, key1, value1, keyHash1, shift) {
      if (shift >= HASH_CODE_LENGTH) {
          return new HashCollisionNode(owner, keyHash0, [key0, key1], [value0, value1]);
      }
      else {
          var mask0 = maskFrom(keyHash0, shift);
          var mask1 = maskFrom(keyHash1, shift);
          if (mask0 !== mask1) {
              var dataMap = bitPosFrom(mask0) | bitPosFrom(mask1);
              if (mask0 < mask1) {
                  return new BitmapIndexedNode(owner, dataMap, 0, [key0, key1], [value0, value1], [], [keyHash0, keyHash1], 2);
              }
              return new BitmapIndexedNode(owner, dataMap, 0, [key1, key0], [value1, value0], [], [keyHash1, keyHash0], 2);
          }
          else {
              var nodeMap = bitPosFrom(mask0);
              var node = mergeTwoKeyValPairs(owner, key0, value0, keyHash0, key1, value1, keyHash1, shift + BIT_PARTITION_SIZE);
              return new BitmapIndexedNode(owner, 0, nodeMap, [], [], [node], [], node.size);
          }
      }
  }
  var HashCollisionNode = (function () {
      function HashCollisionNode(owner, hash, keys, values) {
          this.type = NodeType$1.Collision;
          this.owner = owner;
          this.hash = hash;
          this.keys = keys;
          this.values = values;
      }
      Object.defineProperty(HashCollisionNode.prototype, "size", {
          get: function () {
              return this.keys.length;
          },
          enumerable: false,
          configurable: true
      });
      HashCollisionNode.prototype.toOwned = function (owner) {
          return this.owner === owner
              ? this
              : new HashCollisionNode(owner, this.hash, this.keys.slice(), this.values.slice());
      };
      HashCollisionNode.prototype.contains = function (key) {
          return this.keys.includes(key);
      };
      HashCollisionNode.prototype.get = function (key) {
          var index = this.keys.findIndex(function (it) { return is(it, key); });
          return index >= 0 ? this.values[index] : undefined;
      };
      HashCollisionNode.prototype.insert = function (owner, key, value) {
          var index = this.keys.findIndex(function (it) { return is(it, key); });
          if (index >= 0) {
              var value0 = this.values[index];
              if (is(value0, value)) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return next;
          }
          else {
              var next = this.toOwned(owner);
              next.keys.push(key);
              next.values.push(value);
              return next;
          }
      };
      HashCollisionNode.prototype.update = function (owner, key, updater) {
          var index = this.keys.findIndex(function (it) { return is(it, key); });
          if (index >= 0) {
              var value0 = this.values[index];
              var value = updater(value0);
              if (is(value0, value)) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return next;
          }
          return this;
      };
      HashCollisionNode.prototype.remove = function (owner, key) {
          var index = this.keys.findIndex(function (it) { return is(it, key); });
          if (index === -1) {
              return undefined;
          }
          var value = this.getValue(index);
          return [
              new HashCollisionNode(owner, this.hash, this.keys.filter(function (_, i) { return i !== index; }), this.values.filter(function (_, i) { return i !== index; })),
              value
          ];
      };
      HashCollisionNode.prototype.getKey = function (index) {
          return this.keys[index];
      };
      HashCollisionNode.prototype.getValue = function (index) {
          return this.values[index];
      };
      HashCollisionNode.prototype.getHash = function () {
          return this.hash;
      };
      HashCollisionNode.prototype.iter = function () {
          return new HashCollisionNodeIterator(this);
      };
      HashCollisionNode.prototype.map = function (owner, f) {
          var len = this.size;
          var values = [];
          var update = false;
          for (var i = 0; i < len; i += 1) {
              var key = this.getKey(i);
              var value0 = this.getValue(i);
              var value = f(value0, key);
              values.push(value);
              update = is(value0, value);
          }
          return update
              ? new HashCollisionNode(owner, this.hash, this.keys, values)
              : this;
      };
      HashCollisionNode.prototype.forEach = function (f) {
          var len = this.size;
          for (var i = 0; i < len; i += 1) {
              var key = this.getKey(i);
              var value0 = this.getValue(i);
              f(value0, key);
          }
      };
      HashCollisionNode.prototype.find = function (predicate) {
          return this.values.find(predicate);
      };
      return HashCollisionNode;
  }());
  var BitmapIndexedNodeIterator = (function () {
      function BitmapIndexedNodeIterator(node) {
          this.index = 0;
          this.delegate = null;
          this.done = false;
          this.node = node;
          this.valueCount = node.valueCount;
          this.nodeCount = node.nodeCount;
          this.size = this.valueCount + this.nodeCount;
      }
      BitmapIndexedNodeIterator.prototype[Symbol.iterator] = function () {
          return this.clone();
      };
      BitmapIndexedNodeIterator.prototype.next = function () {
          if (this.done) {
              return {
                  done: true,
                  value: undefined
              };
          }
          if (this.index < this.valueCount) {
              var key = this.node.getKey(this.index);
              var value = this.node.getValue(this.index);
              this.index += 1;
              return {
                  done: false,
                  value: [key, value]
              };
          }
          if (this.index < this.size) {
              if (this.delegate === null) {
                  this.delegate = this.node.getNode(this.index - this.valueCount).iter();
              }
              var next = this.delegate.next();
              if (next.done) {
                  this.index += 1;
                  this.delegate = null;
                  return this.next();
              }
              return next;
          }
          this.done = true;
          return {
              done: true,
              value: undefined
          };
      };
      BitmapIndexedNodeIterator.prototype.clone = function () {
          var iter = new BitmapIndexedNodeIterator(this.node);
          iter.index = this.index;
          iter.delegate = this.delegate;
          iter.done = this.done;
          return iter;
      };
      return BitmapIndexedNodeIterator;
  }());
  var HashCollisionNodeIterator = (function () {
      function HashCollisionNodeIterator(node) {
          this.index = 0;
          this.node = node;
      }
      HashCollisionNodeIterator.prototype[Symbol.iterator] = function () {
          return this.clone();
      };
      HashCollisionNodeIterator.prototype.next = function () {
          if (this.index >= this.node.size) {
              return {
                  done: true,
                  value: undefined
              };
          }
          var key = this.node.getKey(this.index);
          var value = this.node.getValue(this.index);
          this.index += 1;
          return {
              done: false,
              value: [key, value]
          };
      };
      HashCollisionNodeIterator.prototype.clone = function () {
          var iter = new HashCollisionNodeIterator(this.node);
          iter.index = this.index;
          return iter;
      };
      return HashCollisionNodeIterator;
  }());

  function hashing(o) {
      if (o === null) {
          return 0x42108422;
      }
      switch (typeof o) {
          case "boolean":
              return o ? 0x32108421 : 0x32108420;
          case "number":
              return hashNumber(o);
          case "string":
              return hashString(o);
          case "object":
          case "function":
          case "symbol":
              throw new Error("Using object, function and symbol as hash map key is not supported");
          case "undefined":
              return 0x32108423;
          default:
              return hashString(String(o));
      }
  }
  function hashString(str) {
      var hashed = 0;
      for (var ii = 0; ii < str.length; ii++) {
          hashed = (hashed * 31 + str.charCodeAt(ii)) | 0;
      }
      return smi(hashed);
  }
  function hashNumber(n) {
      if (n !== n || n === Infinity) {
          return 0;
      }
      var hash = n | 0;
      if (hash !== n) {
          hash ^= n * 0xffffffff;
      }
      while (n > 0xffffffff) {
          n /= 0xffffffff;
          hash ^= n;
      }
      return smi(hash);
  }
  function smi(value) {
      return value & 0x3fffffff;
  }

  var Uid = (function () {
      function Uid() {
          this.id = 0;
      }
      Uid.prototype.take = function () {
          this.id += 1;
          return this.id;
      };
      Uid.prototype.peek = function () {
          return this.id + 1;
      };
      return Uid;
  }());

  var uid$2 = new Uid();
  var HashMap = (function () {
      function HashMap(root) {
          this.id = uid$2.take();
          this.root = root;
      }
      Object.defineProperty(HashMap.prototype, "size", {
          get: function () {
              return this.root.size;
          },
          enumerable: false,
          configurable: true
      });
      HashMap.empty = function () {
          return HashMapBuilder.empty().finish();
      };
      HashMap.from = function (iterable) {
          return HashMapBuilder.from(iterable).finish();
      };
      HashMap.prototype.get = function (key) {
          var h = hashing(key);
          return this.root.get(key, h, 0);
      };
      HashMap.prototype.has = function (key) {
          var h = hashing(key);
          return this.root.contains(key, h, 0);
      };
      HashMap.prototype.set = function (key, value) {
          return this.withRoot(this.root.insert(uid$2.peek(), key, value, hashing(key), 0));
      };
      HashMap.prototype.update = function (key, updater) {
          return this.withRoot(this.root.update(uid$2.peek(), key, updater, hashing(key), 0));
      };
      HashMap.prototype.delete = function (key) {
          var h = hashing(key);
          var id = uid$2.peek();
          var remove = this.root.remove(id, key, h, 0);
          return remove === undefined ? this : new HashMap(remove[0]);
      };
      HashMap.prototype.clone = function () {
          return new HashMap(this.root);
      };
      HashMap.prototype[Symbol.iterator] = function () {
          return this.entries();
      };
      HashMap.prototype.entries = function () {
          return this.root.iter();
      };
      HashMap.prototype.values = function () {
          return new MapIterator(this.entries(), function (_a) {
              var v = _a[1];
              return v;
          });
      };
      HashMap.prototype.mutate = function () {
          return new HashMapBuilder(this.root);
      };
      HashMap.prototype.map = function (f) {
          return new HashMap(this.root.map(uid$2.peek(), f));
      };
      HashMap.prototype.filter = function (predicate) {
          var map = this.mutate();
          this.forEach(function (value, key) {
              if (!predicate(value, key)) {
                  map.delete(key);
              }
          });
          return map.finish();
      };
      HashMap.prototype.forEach = function (f) {
          this.root.forEach(f);
      };
      HashMap.prototype.find = function (predicate) {
          return this.root.find(predicate);
      };
      HashMap.prototype.withRoot = function (newRoot) {
          return newRoot === this.root ? this : new HashMap(newRoot);
      };
      return HashMap;
  }());
  var HashMapBuilder = (function () {
      function HashMapBuilder(root) {
          this.id = uid$2.take();
          this.root = root;
      }
      HashMapBuilder.empty = function () {
          var id = uid$2.peek();
          var root = BitmapIndexedNode.empty(id);
          return new HashMapBuilder(root);
      };
      HashMapBuilder.from = function (iterable) {
          if (Array.isArray(iterable)) {
              return HashMapBuilder.fromArray(iterable);
          }
          var iter = iterable[Symbol.iterator]();
          var builder = HashMapBuilder.empty();
          var next = iter.next();
          while (!next.done) {
              var _a = next.value, key = _a[0], value = _a[1];
              builder.set(key, value);
              next = iter.next();
          }
          return builder;
      };
      HashMapBuilder.fromArray = function (list) {
          var builder = HashMapBuilder.empty();
          for (var i = 0; i < list.length; i += 1) {
              var _a = list[i], key = _a[0], value = _a[1];
              builder.set(key, value);
          }
          return builder;
      };
      HashMapBuilder.prototype.get = function (key) {
          var h = hashing(key);
          return this.root.get(key, h, 0);
      };
      HashMapBuilder.prototype.has = function (key) {
          var h = hashing(key);
          return this.root.contains(key, h, 0);
      };
      HashMapBuilder.prototype.set = function (key, value) {
          this.root = this.root.insert(this.id, key, value, hashing(key), 0);
          return this;
      };
      HashMapBuilder.prototype.update = function (key, updater) {
          var h = hashing(key);
          this.root = this.root.update(this.id, key, updater, h, 0);
          return this;
      };
      HashMapBuilder.prototype.delete = function (key) {
          var h = hashing(key);
          var remove = this.root.remove(this.id, key, h, 0);
          if (remove !== undefined) {
              this.root = remove[0];
          }
          return this;
      };
      HashMapBuilder.prototype.finish = function () {
          return new HashMap(this.root);
      };
      return HashMapBuilder;
  }());

  var NodeType;
  (function (NodeType) {
      NodeType[NodeType["Internal"] = 0] = "Internal";
      NodeType[NodeType["Leaf"] = 1] = "Leaf";
  })(NodeType || (NodeType = {}));
  var MAX_SIZE = 31;
  var MIN_SIZE = 15;
  var HALF_NODE_SPLIT = 7;
  function binaryFind(list, key) {
      var start = 0;
      var end = list.length;
      while (true) {
          if (start + 1 === end) {
              return list[start] >= key ? start : end;
          }
          var mid = (start + end) >>> 1;
          if (list[mid] === key) {
              return mid;
          }
          if (key < list[mid]) {
              end = mid;
          }
          else {
              start = mid;
          }
      }
  }
  var InternalNode = (function () {
      function InternalNode(owner, keys, values, children, size) {
          this.type = NodeType.Internal;
          this.owner = owner;
          this.keys = keys;
          this.values = values;
          this.children = children;
          this.size = size;
      }
      Object.defineProperty(InternalNode.prototype, "selfSize", {
          get: function () {
              return this.keys.length;
          },
          enumerable: false,
          configurable: true
      });
      InternalNode.prototype.iter = function () {
          return new BTreeIterator(this);
      };
      InternalNode.prototype.toOwned = function (owner) {
          return this.owner === owner
              ? this
              : new InternalNode(owner, this.keys.slice(), this.values.slice(), this.children.slice(), this.size);
      };
      InternalNode.prototype.getKey = function (index) {
          return this.keys[index];
      };
      InternalNode.prototype.getValue = function (index) {
          return this.values[index];
      };
      InternalNode.prototype.getChild = function (index) {
          return this.children[index];
      };
      InternalNode.prototype.get = function (key) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          if (index !== size) {
              var key0 = this.getKey(index);
              if (key0 === key) {
                  return this.getValue(index);
              }
          }
          return this.getChild(index).get(key);
      };
      InternalNode.prototype.contains = function (key) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          if (index !== size) {
              var key0 = this.getKey(index);
              if (key0 === key) {
                  return true;
              }
          }
          return this.getChild(index).contains(key);
      };
      InternalNode.prototype.insert = function (owner, key, value) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          var key0 = this.getKey(index);
          var value0 = this.getValue(index);
          if (key0 === key) {
              if (is(value0, value)) {
                  return [this];
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return [next];
          }
          else {
              var child0 = this.getChild(index);
              var insert = child0.insert(owner, key, value);
              if (insert.length === 1) {
                  var child = insert[0];
                  if (child === child0) {
                      return [this];
                  }
                  var next = this.toOwned(owner);
                  next.children[index] = child;
                  return [next];
              }
              else if (size === MAX_SIZE) {
                  return this.updateWithSplit(owner, insert[0], insert[1], insert[2], insert[3], index);
              }
              else {
                  var next = this.toOwned(owner);
                  next.keys.splice(index, 0, insert[2]);
                  next.values.splice(index, 0, insert[3]);
                  next.children.splice(index, 1, insert[0], insert[1]);
                  next.size += 1;
                  return [next];
              }
          }
      };
      InternalNode.prototype.update = function (owner, key, updater) {
          var index = binaryFind(this.keys, key);
          var key0 = this.getKey(index);
          var value0 = this.getValue(index);
          if (key0 === key) {
              var value = updater(value0);
              if (is(value0, value)) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return next;
          }
          else {
              var child0 = this.getChild(index);
              var child = child0.update(owner, key, updater);
              if (child === child0) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.children[index] = child;
              return next;
          }
      };
      InternalNode.prototype.remove = function (owner, key) {
          var index = binaryFind(this.keys, key);
          var size = this.selfSize;
          var child0 = this.getChild(index);
          var originalSize = child0.size;
          var key0 = this.getKey(index);
          if (key0 === key) {
              var _a = child0.removeMostRight(owner), removedKey = _a[0], value = _a[1], child = _a[2];
              var next = this.toOwned(owner);
              next.size -= 1;
              next.keys[index] = removedKey;
              next.values[index] = value;
              next.children[index] = child;
              return next.balanceChild(owner, child, removedKey, value, index);
          }
          else {
              var child = child0.remove(owner, key);
              if (child.size === originalSize) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.size -= 1;
              next.children[index] = child;
              if (child.selfSize >= MIN_SIZE) {
                  return next;
              }
              if (index === size) {
                  next.balanceTail(child);
                  return next;
              }
              var value0 = this.getValue(index);
              return next.balanceChild(owner, child, key0, value0, index);
          }
      };
      InternalNode.prototype.removeMostRight = function (owner) {
          var size = this.selfSize;
          var _a = this.getChild(size).removeMostRight(owner), key = _a[0], value = _a[1], child = _a[2];
          var next = this.toOwned(owner);
          next.size -= 1;
          next.children[size] = child;
          if (child.selfSize < MIN_SIZE) {
              next.balanceTail(child);
          }
          return [key, value, next];
      };
      InternalNode.prototype.map = function (owner, f) {
          var values = [];
          var children = [];
          var same = true;
          for (var i = 0; i < this.keys.length; i += 1) {
              var key = this.getKey(i);
              var value0 = this.getValue(i);
              var value = f(value0, key);
              values.push(value);
              same = same && is(value0, value);
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var child0 = this.getChild(i);
              var child = child0.map(owner, f);
              children.push(child);
              same = same && child0 === child;
          }
          return same
              ? this
              : new InternalNode(owner, this.keys, values, children, this.size);
      };
      InternalNode.prototype.forEach = function (f) {
          for (var i = 0; i < this.keys.length; i += 1) {
              var key = this.getKey(i);
              var value = this.getValue(i);
              f(value, key);
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var child = this.getChild(i);
              child.forEach(f);
          }
      };
      InternalNode.prototype.find = function (predicate) {
          for (var i = 0; i < this.keys.length; i += 1) {
              var value = this.getValue(i);
              if (predicate(value)) {
                  return value;
              }
          }
          for (var i = 0; i < this.children.length; i += 1) {
              var child = this.getChild(i);
              var found = child.find(predicate);
              if (found) {
                  return found;
              }
          }
          return undefined;
      };
      InternalNode.prototype.balanceChild = function (owner, child, key0, value0, index) {
          var _a, _b, _c, _d, _e, _f;
          if (index === 0) {
              this.balanceHead(child);
              return this;
          }
          var isChildInternal = child.type === NodeType.Internal;
          var left0 = this.getChild(index - 1);
          var right0 = this.getChild(index + 1);
          if (left0.selfSize > MIN_SIZE) {
              this.rotateRight(child, left0, index, isChildInternal);
          }
          else if (right0.selfSize > MIN_SIZE) {
              this.rotateLeft(child, right0, index, isChildInternal);
          }
          else {
              var left = left0.toOwned(owner);
              var right = right0.toOwned(owner);
              var key = child.getKey(HALF_NODE_SPLIT);
              var value = child.getValue(HALF_NODE_SPLIT);
              left.keys.push(this.getKey(index - 1));
              left.values.push(this.getValue(index - 1));
              (_a = left.keys).push.apply(_a, child.keys.slice(0, HALF_NODE_SPLIT));
              (_b = left.values).push.apply(_b, child.values.slice(0, HALF_NODE_SPLIT));
              right.keys.unshift(key0);
              right.values.unshift(value0);
              (_c = right.keys).unshift.apply(_c, child.keys.slice(HALF_NODE_SPLIT + 1, MIN_SIZE));
              (_d = right.values).unshift.apply(_d, child.values.slice(HALF_NODE_SPLIT + 1, MIN_SIZE));
              this.keys.splice(index - 1, 2, key);
              this.values.splice(index - 1, 2, value);
              this.children.splice(index - 1, 3, left, right);
              if (isChildInternal) {
                  (_e = left.children).push.apply(_e, child.children.slice(0, HALF_NODE_SPLIT + 1));
                  (_f = right.children).unshift.apply(_f, child.children.slice(HALF_NODE_SPLIT + 1, MIN_SIZE + 1));
                  left.updateSize();
                  right.updateSize();
              }
          }
          return this;
      };
      InternalNode.prototype.rotateLeft = function (child, right0, index, isChildInternal) {
          var right = right0.toOwned(this.owner);
          var newKey = right.keys.shift();
          var newValue = right.values.shift();
          var key0 = this.getKey(index);
          var value0 = this.getValue(index);
          child.keys.push(key0);
          child.values.push(value0);
          this.keys[index] = newKey;
          this.values[index] = newValue;
          this.children[index + 1] = right;
          if (isChildInternal) {
              var grandChild = right.children.shift();
              child.children.push(grandChild);
              var delta = grandChild.size + 1;
              child.size += delta;
              right.size -= delta;
          }
      };
      InternalNode.prototype.rotateRight = function (child, left0, index, isChildInternal) {
          var left = left0.toOwned(this.owner);
          var newKey = left.keys.pop();
          var newValue = left.values.pop();
          var key0 = this.getKey(index - 1);
          var value0 = this.getValue(index - 1);
          child.keys.unshift(key0);
          child.values.unshift(value0);
          this.keys[index - 1] = newKey;
          this.values[index - 1] = newValue;
          this.children[index - 1] = left;
          if (isChildInternal) {
              var grandChild = left.children.pop();
              child.children.unshift(grandChild);
              var delta = grandChild.size + 1;
              child.size += delta;
              left.size -= delta;
          }
      };
      InternalNode.prototype.balanceTail = function (child) {
          var _a, _b, _c;
          var index = this.selfSize;
          var left0 = this.getChild(index - 1);
          var isChildInternal = child.type === NodeType.Internal;
          if (left0.selfSize === MIN_SIZE) {
              child.keys.unshift(this.getKey(index - 1));
              child.values.unshift(this.getValue(index - 1));
              (_a = child.keys).unshift.apply(_a, left0.keys);
              (_b = child.values).unshift.apply(_b, left0.values);
              this.keys.splice(index - 1, 1);
              this.values.splice(index - 1, 1);
              this.children.splice(index - 1, 1);
              if (isChildInternal) {
                  (_c = child.children).unshift.apply(_c, left0.children);
                  child.size += left0.size + 1;
              }
          }
          else {
              this.rotateRight(child, left0, index, isChildInternal);
          }
      };
      InternalNode.prototype.balanceHead = function (child) {
          var _a, _b, _c;
          var right0 = this.getChild(1);
          var isChildInternal = child.type === NodeType.Internal;
          if (right0.selfSize === MIN_SIZE) {
              child.keys.push(this.getKey(0));
              child.values.push(this.getValue(0));
              (_a = child.keys).push.apply(_a, right0.keys);
              (_b = child.values).push.apply(_b, right0.values);
              this.keys.splice(0, 1);
              this.values.splice(0, 1);
              this.children.splice(1, 1);
              if (isChildInternal) {
                  (_c = child.children).push.apply(_c, right0.children);
                  child.size += right0.size + 1;
              }
          }
          else {
              this.rotateLeft(child, right0, 0, isChildInternal);
          }
      };
      InternalNode.prototype.updateWithSplit = function (owner, child1, child2, key, value, index) {
          var next1 = this.toOwned(owner);
          next1.keys.splice(index, 0, key);
          next1.values.splice(index, 0, value);
          next1.children.splice(index, 1, child1, child2);
          var next2 = new InternalNode(owner, next1.keys.splice(16, 16), next1.values.splice(16, 16), next1.children.splice(16, 17), 0);
          var nextKey = next1.keys.pop();
          var nextValue = next1.values.pop();
          next1.updateSize();
          next2.updateSize();
          return [next1, next2, nextKey, nextValue];
      };
      InternalNode.prototype.updateSize = function () {
          var sum = this.selfSize;
          var len = this.children.length;
          for (var i = 0; i < len; i += 1) {
              sum += this.children[i].size;
          }
          this.size = sum;
      };
      return InternalNode;
  }());
  var LeafNode = (function () {
      function LeafNode(owner, keys, values) {
          this.type = NodeType.Leaf;
          this.owner = owner;
          this.keys = keys;
          this.values = values;
      }
      Object.defineProperty(LeafNode.prototype, "size", {
          get: function () {
              return this.keys.length;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(LeafNode.prototype, "selfSize", {
          get: function () {
              return this.size;
          },
          enumerable: false,
          configurable: true
      });
      LeafNode.prototype.toOwned = function (owner) {
          return this.owner === owner
              ? this
              : new LeafNode(owner, this.keys.slice(), this.values.slice());
      };
      LeafNode.prototype.getKey = function (index) {
          return this.keys[index];
      };
      LeafNode.prototype.getValue = function (index) {
          return this.values[index];
      };
      LeafNode.prototype.get = function (key) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          if (index !== size) {
              var key0 = this.getKey(index);
              return key0 === key ? this.getValue(index) : undefined;
          }
          return undefined;
      };
      LeafNode.prototype.contains = function (key) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          if (index !== size) {
              var key0 = this.getKey(index);
              return key0 === key;
          }
          return false;
      };
      LeafNode.prototype.insert = function (owner, key, value) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          var key0 = index === size ? undefined : this.getKey(index);
          if (key0 === key) {
              var value0 = this.getValue(index);
              if (is(value, value0)) {
                  return [this];
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return [next];
          }
          else {
              if (size === MAX_SIZE) {
                  return this.updateWithSplit(owner, key, value, index);
              }
              var next = this.toOwned(owner);
              next.keys.splice(index, 0, key);
              next.values.splice(index, 0, value);
              return [next];
          }
      };
      LeafNode.prototype.update = function (owner, key, updater) {
          var size = this.selfSize;
          var index = binaryFind(this.keys, key);
          var key0 = index === size ? undefined : this.getKey(index);
          if (key0 === key) {
              var value0 = this.getValue(index);
              var value = updater(value0);
              if (is(value, value0)) {
                  return this;
              }
              var next = this.toOwned(owner);
              next.values[index] = value;
              return next;
          }
          return this;
      };
      LeafNode.prototype.remove = function (owner, key) {
          var index = binaryFind(this.keys, key);
          var size = this.selfSize;
          if (index === size) {
              return this;
          }
          return this.removeIndex(owner, index);
      };
      LeafNode.prototype.removeMostRight = function (owner) {
          var index = this.selfSize - 1;
          var key = this.getKey(index);
          var value = this.getValue(index);
          var next = this.removeIndex(owner, index);
          return [key, value, next];
      };
      LeafNode.prototype.map = function (owner, f) {
          var values = [];
          var same = true;
          for (var i = 0; i < this.keys.length; i += 1) {
              var key = this.getKey(i);
              var value0 = this.getValue(i);
              var value = f(value0, key);
              values.push(value);
              same = same && is(value0, value);
          }
          return same
              ? this
              : new LeafNode(owner, this.keys, values);
      };
      LeafNode.prototype.forEach = function (f) {
          for (var i = 0; i < this.keys.length; i += 1) {
              var key = this.getKey(i);
              var value = this.getValue(i);
              f(value, key);
          }
      };
      LeafNode.prototype.find = function (predicate) {
          return this.values.find(predicate);
      };
      LeafNode.prototype.updateWithSplit = function (owner, key, value, index) {
          var next1 = this.toOwned(owner);
          next1.keys.splice(index, 0, key);
          next1.values.splice(index, 0, value);
          var next2 = new LeafNode(owner, next1.keys.splice(16, 16), next1.values.splice(16, 16));
          var nextKey = next1.keys.pop();
          var nextValue = next1.values.pop();
          return [next1, next2, nextKey, nextValue];
      };
      LeafNode.prototype.removeIndex = function (owner, index) {
          var next = this.toOwned(owner);
          next.keys.splice(index, 1);
          next.values.splice(index, 1);
          return next;
      };
      return LeafNode;
  }());
  function emptyRoot(owner) {
      return new LeafNode(owner, [], []);
  }
  function rootInsert(owner, root, key, value) {
      if (root.selfSize === 0) {
          return new LeafNode(owner, [key], [value]);
      }
      var insert = root.insert(owner, key, value);
      if (insert.length === 1) {
          return insert[0];
      }
      var next1 = insert[0], next2 = insert[1], newKey = insert[2], newValue = insert[3];
      return new InternalNode(owner, [newKey], [newValue], [next1, next2], next1.size + next2.size + 1);
  }
  function rootRemove(owner, root, key) {
      var newRoot = root.remove(owner, key);
      if (newRoot.type === NodeType.Internal && newRoot.selfSize === 0) {
          return newRoot.getChild(0);
      }
      return newRoot;
  }
  var BTreeIterator = (function () {
      function BTreeIterator(node) {
          this.delegate = null;
          this.index = 0;
          this.done = false;
          this.node = node;
          this.setDelegate(this.index);
      }
      BTreeIterator.prototype[Symbol.iterator] = function () {
          return this.clone();
      };
      BTreeIterator.prototype.next = function () {
          if (this.delegate === null) {
              return this.yieldValue();
          }
          var next = this.delegate.next();
          if (!next.done) {
              return {
                  done: false,
                  value: next.value
              };
          }
          var y = this.yieldValue();
          if (this.index <= this.node.selfSize) {
              this.setDelegate(this.index);
          }
          else {
              this.done = true;
              this.delegate = null;
          }
          return y;
      };
      BTreeIterator.prototype.clone = function () {
          var iter = new BTreeIterator(this.node);
          iter.delegate = this.delegate;
          iter.index = this.index;
          iter.done = this.done;
          return iter;
      };
      BTreeIterator.prototype.setDelegate = function (index) {
          if (this.node.type !== NodeType.Internal) {
              return;
          }
          var child = this.node.getChild(index);
          this.delegate = new BTreeIterator(child);
      };
      BTreeIterator.prototype.yieldValue = function () {
          if (!this.done && this.index < this.node.selfSize) {
              var key = this.node.getKey(this.index);
              var value = this.node.getValue(this.index);
              this.index += 1;
              return {
                  done: false,
                  value: [key, value]
              };
          }
          this.done = true;
          return {
              done: true,
              value: undefined
          };
      };
      return BTreeIterator;
  }());

  var uid$1 = new Uid();
  var SortedMap = (function () {
      function SortedMap(root) {
          this.id = uid$1.take();
          this.root = root;
      }
      Object.defineProperty(SortedMap.prototype, "size", {
          get: function () {
              return this.root.size;
          },
          enumerable: false,
          configurable: true
      });
      SortedMap.empty = function () {
          return SortedMapBuilder.empty().finish();
      };
      SortedMap.from = function (iterable) {
          return SortedMapBuilder.from(iterable).finish();
      };
      SortedMap.prototype.delete = function (key) {
          return this.withRoot(rootRemove(uid$1.peek(), this.root, key));
      };
      SortedMap.prototype.get = function (key) {
          return this.root.get(key);
      };
      SortedMap.prototype.has = function (key) {
          return this.root.contains(key);
      };
      SortedMap.prototype.set = function (key, value) {
          return this.withRoot(rootInsert(uid$1.peek(), this.root, key, value));
      };
      SortedMap.prototype.update = function (key, updater) {
          return this.withRoot(this.root.update(uid$1.peek(), key, updater));
      };
      SortedMap.prototype[Symbol.iterator] = function () {
          return this.entries();
      };
      SortedMap.prototype.clone = function () {
          return new SortedMap(this.root);
      };
      SortedMap.prototype.entries = function () {
          return new BTreeIterator(this.root);
      };
      SortedMap.prototype.values = function () {
          return new MapIterator(this.entries(), function (_a) {
              var v = _a[1];
              return v;
          });
      };
      SortedMap.prototype.mutate = function () {
          return new SortedMapBuilder(this.root);
      };
      SortedMap.prototype.map = function (f) {
          var root = this.root.map(uid$1.peek(), f);
          return new SortedMap(root);
      };
      SortedMap.prototype.filter = function (predicate) {
          var map = this.mutate();
          this.forEach(function (value, key) {
              if (!predicate(value, key)) {
                  map.delete(key);
              }
          });
          return map.finish();
      };
      SortedMap.prototype.forEach = function (f) {
          this.root.forEach(f);
      };
      SortedMap.prototype.find = function (predicate) {
          return this.root.find(predicate);
      };
      SortedMap.prototype.withRoot = function (newRoot) {
          return newRoot === this.root ? this : new SortedMap(newRoot);
      };
      return SortedMap;
  }());
  var SortedMapBuilder = (function () {
      function SortedMapBuilder(root) {
          this.id = uid$1.take();
          this.root = root;
      }
      SortedMapBuilder.empty = function () {
          var id = uid$1.peek();
          var root = emptyRoot(id);
          return new SortedMapBuilder(root);
      };
      SortedMapBuilder.from = function (iterable) {
          if (Array.isArray(iterable)) {
              return SortedMapBuilder.fromArray(iterable);
          }
          var iter = iterable[Symbol.iterator]();
          var builder = SortedMapBuilder.empty();
          var next = iter.next();
          while (!next.done) {
              var _a = next.value, key = _a[0], value = _a[1];
              builder.set(key, value);
              next = iter.next();
          }
          return builder;
      };
      SortedMapBuilder.fromArray = function (list) {
          var builder = SortedMapBuilder.empty();
          for (var i = 0; i < list.length; i += 1) {
              var _a = list[i], key = _a[0], value = _a[1];
              builder.set(key, value);
          }
          return builder;
      };
      SortedMapBuilder.prototype.delete = function (key) {
          this.root = rootRemove(this.id, this.root, key);
          return this;
      };
      SortedMapBuilder.prototype.get = function (key) {
          return this.root.get(key);
      };
      SortedMapBuilder.prototype.has = function (key) {
          return this.root.contains(key);
      };
      SortedMapBuilder.prototype.set = function (key, value) {
          this.root = rootInsert(this.id, this.root, key, value);
          return this;
      };
      SortedMapBuilder.prototype.update = function (key, updater) {
          this.root = this.root.update(this.id, key, updater);
          return this;
      };
      SortedMapBuilder.prototype.finish = function () {
          return new SortedMap(this.root);
      };
      return SortedMapBuilder;
  }());

  var uid = new Uid();
  var OrderedMap = (function () {
      function OrderedMap(itemId, hashRoot, sortedRoot) {
          this.id = uid.take();
          this.itemId = itemId;
          this.hashRoot = hashRoot;
          this.sortedRoot = sortedRoot;
      }
      Object.defineProperty(OrderedMap.prototype, "size", {
          get: function () {
              return this.hashRoot.size;
          },
          enumerable: false,
          configurable: true
      });
      OrderedMap.empty = function () {
          return OrderedMapBuilder.empty().finish();
      };
      OrderedMap.from = function (iterable) {
          return OrderedMapBuilder.from(iterable).finish();
      };
      OrderedMap.prototype.delete = function (key) {
          var tempId = uid.peek();
          var h = hashing(key);
          var remove = this.hashRoot.remove(tempId, key, h, 0);
          if (remove === undefined) {
              return this;
          }
          var hashRoot = remove[0], valueId = remove[1];
          var sortedRoot = this.sortedRoot.remove(tempId, valueId);
          return new OrderedMap(this.itemId, hashRoot, sortedRoot);
      };
      OrderedMap.prototype.get = function (key) {
          var h = hashing(key);
          var id = this.hashRoot.get(key, h, 0);
          if (id === undefined) {
              return undefined;
          }
          var got = this.sortedRoot.get(id);
          return got === null || got === void 0 ? void 0 : got[1];
      };
      OrderedMap.prototype.has = function (key) {
          var h = hashing(key);
          return this.hashRoot.contains(key, h, 0);
      };
      OrderedMap.prototype.set = function (key, value) {
          var temp = uid.peek();
          var itemId = this.hashRoot.get(key, hashing(key), 0);
          var hashRoot = this.hashRoot;
          if (!itemId) {
              itemId = this.itemId + 1;
              hashRoot = this.hashRoot.insert(temp, key, itemId, hashing(key), 0);
          }
          var sortedRoot = rootInsert(temp, this.sortedRoot, itemId, [key, value]);
          return this.withRoot(this.itemId + 1, hashRoot, sortedRoot);
      };
      OrderedMap.prototype.update = function (key, updater) {
          var itemId = this.hashRoot.get(key, hashing(key), 0);
          if (!itemId) {
              return this;
          }
          var sortedRoot = this.sortedRoot.update(uid.peek(), itemId, function (prev) {
              var prevKey = prev[0], prevValue = prev[1];
              var value = updater(prevValue);
              return is(value, prevValue) ? prev : [prevKey, value];
          });
          return this.withRoot(this.itemId, this.hashRoot, sortedRoot);
      };
      OrderedMap.prototype[Symbol.iterator] = function () {
          return this.entries();
      };
      OrderedMap.prototype.clone = function () {
          return new OrderedMap(this.itemId, this.hashRoot, this.sortedRoot);
      };
      OrderedMap.prototype.entries = function () {
          return new OrderedMapIterator(new BTreeIterator(this.sortedRoot));
      };
      OrderedMap.prototype.values = function () {
          return new MapIterator(this.entries(), function (_a) {
              var v = _a[1];
              return v;
          });
      };
      OrderedMap.prototype.mutate = function () {
          return new OrderedMapBuilder(this.itemId, this.hashRoot, this.sortedRoot);
      };
      OrderedMap.prototype.map = function (f) {
          var id = uid.peek();
          var g = function (prev) {
              var key = prev[0], value0 = prev[1];
              var value = f(value0, key);
              return is(value0, value) ? prev : [key, value];
          };
          var sortedRoot = this.sortedRoot.map(id, g);
          return new OrderedMap(this.itemId, this.hashRoot, sortedRoot);
      };
      OrderedMap.prototype.forEach = function (f) {
          this.sortedRoot.forEach(function (_a) {
              var key = _a[0], value = _a[1];
              f(value, key);
          });
      };
      OrderedMap.prototype.find = function (predicate) {
          var found = this.sortedRoot.find(function (_a) {
              var value = _a[1];
              return predicate(value);
          });
          return found ? found[1] : undefined;
      };
      OrderedMap.prototype.first = function () {
          var next = this.entries().next();
          if (next.done) {
              return undefined;
          }
          return next.value[1];
      };
      OrderedMap.prototype.filter = function (predicate) {
          var map = this.mutate();
          this.forEach(function (value, key) {
              if (!predicate(value, key)) {
                  map.delete(key);
              }
          });
          return map.finish();
      };
      OrderedMap.prototype.withRoot = function (itemId, hashRoot, sortedRoot) {
          return hashRoot === this.hashRoot && sortedRoot === this.sortedRoot
              ? this
              : new OrderedMap(itemId, hashRoot, sortedRoot);
      };
      return OrderedMap;
  }());
  var OrderedMapIterator = (function () {
      function OrderedMapIterator(delegate) {
          this.delegate = delegate;
      }
      OrderedMapIterator.prototype[Symbol.iterator] = function () {
          return this.clone();
      };
      OrderedMapIterator.prototype.next = function () {
          var next = this.delegate.next();
          if (next.done) {
              return {
                  done: true,
                  value: undefined
              };
          }
          return {
              done: false,
              value: next.value[1]
          };
      };
      OrderedMapIterator.prototype.clone = function () {
          return new OrderedMapIterator(this.delegate.clone());
      };
      return OrderedMapIterator;
  }());
  var OrderedMapBuilder = (function () {
      function OrderedMapBuilder(itemId, hashRoot, sortedRoot) {
          this.id = uid.take();
          this.itemId = itemId;
          this.hashRoot = hashRoot;
          this.sortedRoot = sortedRoot;
      }
      OrderedMapBuilder.empty = function () {
          var tempId = uid.peek();
          var hashRoot = BitmapIndexedNode.empty(tempId);
          var sortedRoot = emptyRoot(tempId);
          return new OrderedMapBuilder(0, hashRoot, sortedRoot);
      };
      OrderedMapBuilder.from = function (iterable) {
          if (Array.isArray(iterable)) {
              return OrderedMapBuilder.fromArray(iterable);
          }
          var builder = OrderedMapBuilder.empty();
          var iterator = iterable[Symbol.iterator]();
          var next = iterator.next();
          while (!next.done) {
              var _a = next.value, key = _a[0], value = _a[1];
              builder.set(key, value);
              next = iterator.next();
          }
          return builder;
      };
      OrderedMapBuilder.fromArray = function (list) {
          var builder = OrderedMapBuilder.empty();
          for (var i = 0; i < list.length; i += 1) {
              var _a = list[i], key = _a[0], value = _a[1];
              builder.set(key, value);
          }
          return builder;
      };
      OrderedMapBuilder.prototype.delete = function (key) {
          var h = hashing(key);
          var remove = this.hashRoot.remove(this.id, key, h, 0);
          if (remove === undefined) {
              return this;
          }
          var id = remove[1];
          this.hashRoot = remove[0];
          this.sortedRoot = rootRemove(this.id, this.sortedRoot, id);
          return this;
      };
      OrderedMapBuilder.prototype.get = function (key) {
          var _a;
          var h = hashing(key);
          var id = this.hashRoot.get(key, h, 0);
          if (id === undefined) {
              return undefined;
          }
          return (_a = this.sortedRoot.get(id)) === null || _a === void 0 ? void 0 : _a[1];
      };
      OrderedMapBuilder.prototype.has = function (key) {
          var h = hashing(key);
          return this.hashRoot.contains(key, h, 0);
      };
      OrderedMapBuilder.prototype.set = function (key, value) {
          var itemId = this.hashRoot.get(key, hashing(key), 0);
          if (itemId === undefined) {
              itemId = this.itemId + 1;
              this.itemId += 1;
              this.hashRoot = this.hashRoot.insert(this.id, key, itemId, hashing(key), 0);
          }
          this.sortedRoot = rootInsert(this.id, this.sortedRoot, itemId, [key, value]);
          return this;
      };
      OrderedMapBuilder.prototype.update = function (key, updater) {
          var itemId = this.hashRoot.get(key, hashing(key), 0);
          if (!itemId) {
              return this;
          }
          this.sortedRoot = this.sortedRoot.update(this.id, itemId, function (prev) {
              var prevKey = prev[0], prevValue = prev[1];
              var value = updater(prevValue);
              return is(value, prevValue) ? prev : [prevKey, value];
          });
          return this;
      };
      OrderedMapBuilder.prototype.finish = function () {
          return new OrderedMap(this.itemId, this.hashRoot, this.sortedRoot);
      };
      return OrderedMapBuilder;
  }());

  function preventSpread(obj) {
      Object.defineProperty(obj, "__preventSpread", {
          enumerable: true,
          configurable: false,
          get: function () {
              if (document.currentScript) {
                  Debug.error(obj.constructor.name + " is a class, which should not be used in the spread syntax or argument of Object.assign");
              }
              return undefined;
          }
      });
  }

  var EdgeModel = (function () {
      function EdgeModel(edge) {
          this.inner = edge;
          preventSpread(this);
      }
      Object.defineProperty(EdgeModel.prototype, "id", {
          get: function () {
              return this.inner.id;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "shape", {
          get: function () {
              return this.inner.shape;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "automationId", {
          get: function () {
              return this.inner.automationId;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "source", {
          get: function () {
              return this.inner.source;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "target", {
          get: function () {
              return this.inner.target;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "sourcePortId", {
          get: function () {
              return this.inner.sourcePortId;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "targetPortId", {
          get: function () {
              return this.inner.targetPortId;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "state", {
          get: function () {
              return this.inner.state;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(EdgeModel.prototype, "data", {
          get: function () {
              return this.inner.data;
          },
          enumerable: false,
          configurable: true
      });
      EdgeModel.fromJSON = function (inner) {
          return new EdgeModel(inner);
      };
      EdgeModel.prototype.updateState = function (f) {
          return this.update(updateState(f));
      };
      EdgeModel.prototype.update = function (f) {
          var edge = f(this.inner);
          return edge === this.inner ? this : new EdgeModel(edge);
      };
      EdgeModel.prototype.shallow = function () {
          return new EdgeModel(this.inner);
      };
      EdgeModel.prototype.toJSON = function () {
          return this.inner;
      };
      return EdgeModel;
  }());

  function mapCow(list, f) {
      var values = [];
      var same = true;
      for (var i = 0; i < list.length; i += 1) {
          var value0 = list[i];
          var value = f(value0, i);
          same = same && is$1(value0, value);
          values.push(value);
      }
      return same ? list : values;
  }

  var NodeModel = (function () {
      function NodeModel(node, portPositionCache, prev, next) {
          this.inner = node;
          this.portPositionCache = portPositionCache;
          this.prev = prev;
          this.next = next;
          preventSpread(this);
      }
      Object.defineProperty(NodeModel.prototype, "id", {
          get: function () {
              return this.inner.id;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "state", {
          get: function () {
              return this.inner.state;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "shape", {
          get: function () {
              return this.inner.shape;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "ports", {
          get: function () {
              return this.inner.ports;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "ariaLabel", {
          get: function () {
              return this.inner.ariaLabel;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "name", {
          get: function () {
              return this.inner.name;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "x", {
          get: function () {
              return this.inner.x;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "y", {
          get: function () {
              return this.inner.y;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "automationId", {
          get: function () {
              return this.inner.automationId;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "isInSearchResults", {
          get: function () {
              return this.inner.isInSearchResults;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "isCurrentSearchResult", {
          get: function () {
              return this.inner.isCurrentSearchResult;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "data", {
          get: function () {
              return this.inner.data;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "height", {
          get: function () {
              return this.inner.height;
          },
          enumerable: false,
          configurable: true
      });
      Object.defineProperty(NodeModel.prototype, "width", {
          get: function () {
              return this.inner.width;
          },
          enumerable: false,
          configurable: true
      });
      NodeModel.fromJSON = function (node, prev, next) {
          return new NodeModel(node, new Map(), prev, next);
      };
      NodeModel.prototype.getPort = function (id) {
          var _a;
          return (_a = this.ports) === null || _a === void 0 ? void 0 : _a.find(function (port) { return port.id === id; });
      };
      NodeModel.prototype.link = function (_a) {
          var prev = _a.prev, next = _a.next;
          if (prev === this.prev && next === this.next) {
              return this;
          }
          return new NodeModel(this.inner, this.portPositionCache, prev !== null && prev !== void 0 ? prev : this.prev, next !== null && next !== void 0 ? next : this.next);
      };
      NodeModel.prototype.updateState = function (f) {
          return this.update(updateState(f));
      };
      NodeModel.prototype.update = function (f) {
          var node = f(this.inner);
          return node === this.inner ? this : new NodeModel(node, new Map(), this.prev, this.next);
      };
      NodeModel.prototype.updateData = function (f) {
          if (!this.data) {
              return this;
          }
          return this.update(function (inner) {
              var data = f(inner.data);
              if (data === inner.data) {
                  return inner;
              }
              return __assign(__assign({}, inner), { data: data });
          });
      };
      NodeModel.prototype.getPortPosition = function (portId, graphConfig) {
          var point = this.portPositionCache.get(portId);
          if (!point) {
              point = getPortPositionByPortId(this.inner, portId, graphConfig);
              this.portPositionCache.set(portId, point);
          }
          return point;
      };
      NodeModel.prototype.hasPort = function (id) {
          var _a;
          return Boolean((_a = this.inner.ports) === null || _a === void 0 ? void 0 : _a.find(function (port) { return port.id === id; }));
      };
      NodeModel.prototype.updatePositionAndSize = function (dummy) {
          var x = dummy.x, y = dummy.y, width = dummy.width, height = dummy.height;
          var node = __assign(__assign({}, this.inner), { x: x,
              y: y, width: width !== null && width !== void 0 ? width : this.inner.width, height: height !== null && height !== void 0 ? height : this.inner.height });
          return new NodeModel(node, new Map(), this.prev, this.next);
      };
      NodeModel.prototype.updatePorts = function (f) {
          if (!this.inner.ports) {
              return this;
          }
          var ports = mapCow(this.inner.ports, f);
          var node = this.inner.ports === ports
              ? this.inner
              : __assign(__assign({}, this.inner), { ports: ports });
          return node === this.inner ? this : new NodeModel(node, new Map(), this.prev, this.next);
      };
      NodeModel.prototype.invalidCache = function () {
          return new NodeModel(this.inner, new Map(), this.prev, this.next);
      };
      NodeModel.prototype.toJSON = function () {
          return this.inner;
      };
      return NodeModel;
  }());

  var GraphModel = (function () {
      function GraphModel(nodes, edges, groups, head, tail, edgesBySource, edgesByTarget) {
          this.nodes = nodes;
          this.edges = edges;
          this.groups = groups;
          this.head = head;
          this.tail = tail;
          this.edgesBySource = edgesBySource;
          this.edgesByTarget = edgesByTarget;
          preventSpread(this);
      }
      GraphModel.empty = function () {
          return new GraphModel(OrderedMap.empty(), HashMap.empty(), [], undefined, undefined, HashMap.empty(), HashMap.empty());
      };
      GraphModel.fromJSON = function (init) {
          var _a;
          var nodes = OrderedMap.empty().mutate();
          var edges = HashMap.empty().mutate();
          var head;
          var tail;
          if (init.nodes.length === 0) {
              head = undefined;
              tail = undefined;
          }
          else if (init.nodes.length === 1) {
              var node = init.nodes[0];
              nodes.set(node.id, NodeModel.fromJSON(node, undefined, undefined));
              head = node.id;
              tail = node.id;
          }
          else {
              var first = init.nodes[0];
              var second = init.nodes[1];
              var last = init.nodes[init.nodes.length - 1];
              head = first.id;
              tail = last.id;
              nodes.set(first.id, NodeModel.fromJSON(first, undefined, second.id));
              var prev = init.nodes[0];
              if (init.nodes.length > 2) {
                  for (var i = 1; i < init.nodes.length - 1; i += 1) {
                      var node = init.nodes[i];
                      var next = init.nodes[i + 1];
                      nodes.set(node.id, NodeModel.fromJSON(node, prev.id, next.id));
                      prev = node;
                  }
              }
              nodes.set(last.id, NodeModel.fromJSON(last, prev.id, undefined));
          }
          var edgesBySource = HashMapBuilder.empty();
          var edgesByTarget = HashMapBuilder.empty();
          for (var _i = 0, _b = init.edges; _i < _b.length; _i++) {
              var edge = _b[_i];
              edges.set(edge.id, EdgeModel.fromJSON(edge));
              setEdgeByPortMutable(edgesBySource, edge.id, edge.source, edge.sourcePortId);
              setEdgeByPortMutable(edgesByTarget, edge.id, edge.target, edge.targetPortId);
          }
          return new GraphModel(nodes.finish(), edges.finish(), (_a = init.groups) !== null && _a !== void 0 ? _a : [], head, tail, edgesBySource.finish(), edgesByTarget.finish());
      };
      GraphModel.prototype.getNavigationFirstNode = function () {
          if (this.head === undefined) {
              return undefined;
          }
          return this.nodes.get(this.head);
      };
      GraphModel.prototype.updateNode = function (id, f) {
          var _a, _b;
          var nodes = this.nodes.update(id, function (node) { return node.update(f); });
          if (nodes === this.nodes) {
              return this;
          }
          var edges = this.edges.mutate();
          (_a = this.edgesBySource.get(id)) === null || _a === void 0 ? void 0 : _a.forEach(function (edgeIds) {
              edgeIds.forEach(function (edgeId) {
                  markEdgeDirty(edges, edgeId);
              });
          });
          (_b = this.edgesByTarget.get(id)) === null || _b === void 0 ? void 0 : _b.forEach(function (edgeIds) {
              edgeIds.forEach(function (edgeId) {
                  markEdgeDirty(edges, edgeId);
              });
          });
          return new GraphModel(nodes, edges.finish(), this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.updateNodeData = function (id, f) {
          return new GraphModel(this.nodes.update(id, function (node) { return node.updateData(f); }), this.edges, this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.updatePort = function (nodeId, portId, f) {
          var nodes = this.nodes.update(nodeId, function (node) { return node.updatePorts(function (port) { return (port.id === portId ? f(port) : port); }); });
          return new GraphModel(nodes, this.edges, this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.insertNode = function (node) {
          var nodes = this.nodes.mutate().set(node.id, NodeModel.fromJSON(node, this.tail, undefined));
          if (this.tail && !this.nodes.has(node.id)) {
              nodes.update(this.tail, function (tail) {
                  return tail.link({
                      next: node.id
                  });
              });
          }
          return new GraphModel(nodes.finish(), this.edges, this.groups, this.nodes.size === 0 ? node.id : this.head, node.id, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.deleteItems = function (predicate) {
          var _a;
          var deleted = new Set();
          var nodes = this.nodes.mutate();
          var first = this.head === undefined ? undefined : this.nodes.get(this.head);
          var node = first;
          var prev;
          var edgesBySource = this.edgesBySource.mutate();
          var edgesByTarget = this.edgesByTarget.mutate();
          while (node !== undefined) {
              var next = node.next ? this.nodes.get(node.next) : undefined;
              if (!((_a = predicate.node) === null || _a === void 0 ? void 0 : _a.call(predicate, node.inner))) {
                  nodes.delete(node.id);
                  edgesBySource.delete(node.id);
                  edgesByTarget.delete(node.id);
                  deleted.add(node.id);
                  if (prev) {
                      nodes.update(prev.id, function (prevNode) { return prevNode.link({ next: node === null || node === void 0 ? void 0 : node.next }); });
                  }
                  if (next) {
                      nodes.update(next.id, function (nextNode) { return nextNode.link({ prev: prev === null || prev === void 0 ? void 0 : prev.id }); });
                  }
                  if (node === first) {
                      first = next;
                  }
              }
              else {
                  nodes.update(node.id, function (current) {
                      return current.link({ prev: prev === null || prev === void 0 ? void 0 : prev.id }).update(function (n) {
                          if (hasState(exports.GraphNodeState.editing)(n.state)) {
                              return n;
                          }
                          return __assign(__assign({}, n), { state: exports.GraphNodeState.default });
                      });
                  });
                  prev = node;
              }
              node = next;
          }
          var edges = this.edges.mutate();
          this.edges.forEach(function (edge) {
              var _a, _b;
              if (!deleted.has(edge.source) && !deleted.has(edge.target) && ((_b = (_a = predicate.edge) === null || _a === void 0 ? void 0 : _a.call(predicate, edge)) !== null && _b !== void 0 ? _b : true)) {
                  edges.update(edge.id, function (e) { return e.update(updateState(resetState(exports.GraphEdgeState.default))); });
              }
              else {
                  edges.delete(edge.id);
                  deleteEdgeByPort(edgesBySource, edge.id, edge.source, edge.sourcePortId);
                  deleteEdgeByPort(edgesByTarget, edge.id, edge.target, edge.targetPortId);
              }
          });
          return new GraphModel(nodes.finish(), edges.finish(), this.groups, first === null || first === void 0 ? void 0 : first.id, prev === null || prev === void 0 ? void 0 : prev.id, edgesBySource.finish(), edgesByTarget.finish());
      };
      GraphModel.prototype.insertEdge = function (edge) {
          if (this.isEdgeExist(edge.source, edge.sourcePortId, edge.target, edge.targetPortId) ||
              !this.nodes.has(edge.source) ||
              !this.nodes.has(edge.target)) {
              return this;
          }
          var edgesBySource = setEdgeByPort(this.edgesBySource, edge.id, edge.source, edge.sourcePortId);
          var edgesByTarget = setEdgeByPort(this.edgesByTarget, edge.id, edge.target, edge.targetPortId);
          return new GraphModel(this.nodes.update(edge.source, function (node) { return node.invalidCache(); }).update(edge.target, function (node) { return node.invalidCache(); }), this.edges.set(edge.id, EdgeModel.fromJSON(edge)).map(function (e) { return e.updateState(resetState(exports.GraphEdgeState.default)); }), this.groups, this.head, this.tail, edgesBySource, edgesByTarget);
      };
      GraphModel.prototype.updateEdge = function (id, f) {
          return new GraphModel(this.nodes, this.edges.update(id, function (e) { return e.update(f); }), this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.deleteEdge = function (id) {
          var edge = this.edges.get(id);
          if (!edge) {
              return this;
          }
          return new GraphModel(this.nodes, this.edges.delete(id), this.groups, this.head, this.tail, deleteEdgeByPort(this.edgesBySource, edge.id, edge.source, edge.sourcePortId), deleteEdgeByPort(this.edgesByTarget, edge.id, edge.target, edge.targetPortId));
      };
      GraphModel.prototype.updateNodesPositionAndSize = function (dummies) {
          var _this = this;
          var updates = new Set();
          var nodes = this.nodes.mutate();
          var edges = this.edges.mutate();
          dummies.forEach(function (dummy) {
              var _a, _b;
              updates.add(dummy.id);
              nodes.update(dummy.id, function (node) { return node.updatePositionAndSize(dummy); });
              (_a = _this.edgesBySource.get(dummy.id)) === null || _a === void 0 ? void 0 : _a.forEach(function (ids) {
                  ids.forEach(function (id) {
                      markEdgeDirty(edges, id);
                  });
              });
              (_b = _this.edgesByTarget.get(dummy.id)) === null || _b === void 0 ? void 0 : _b.forEach(function (ids) {
                  ids.forEach(function (id) {
                      markEdgeDirty(edges, id);
                  });
              });
          });
          return new GraphModel(nodes.finish(), edges.finish(), this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.mapNodes = function (f) {
          return new GraphModel(this.nodes.map(f), this.edges, this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.mapEdges = function (f) {
          return new GraphModel(this.nodes, this.edges.map(f), this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.selectNodes = function (predicate, topNode) {
          var selected = new Set();
          var nodes = this.nodes
              .map(function (node) {
              var isNodeSelected = predicate(node.inner);
              if (isNodeSelected) {
                  selected.add(node.id);
              }
              return node
                  .updatePorts(updateState(resetState(exports.GraphPortState.default)))
                  .updateState(resetConnectState(isNodeSelected ? exports.GraphNodeState.selected : exports.GraphNodeState.unconnectedToSelected));
          })
              .mutate();
          if (selected.size === 0) {
              this.nodes.forEach(function (n) { return nodes.update(n.id, function (it) { return it.updateState(resetState(exports.GraphNodeState.default)); }); });
          }
          else if (topNode) {
              var n = nodes.get(topNode);
              if (n) {
                  nodes.delete(topNode);
                  nodes.set(n.id, n);
              }
          }
          var setConnected = function (id) {
              nodes.update(id, function (node) {
                  return node.update(updateState(resetState(isSelected(node) ? exports.GraphNodeState.selected : exports.GraphNodeState.connectedToSelected)));
              });
          };
          var edges = selected.size
              ? this.edges.map(function (edge) {
                  var state = exports.GraphEdgeState.unconnectedToSelected;
                  if (selected.has(edge.source)) {
                      setConnected(edge.target);
                      state = exports.GraphEdgeState.connectedToSelected;
                  }
                  if (selected.has(edge.target)) {
                      setConnected(edge.source);
                      state = exports.GraphEdgeState.connectedToSelected;
                  }
                  return edge.update(updateState(resetState(state)));
              })
              : this.edges.map(function (edge) { return edge.updateState(resetState(exports.GraphEdgeState.default)); });
          return new GraphModel(nodes.finish(), edges, this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.getEdgesBySource = function (nodeId, portId) {
          var _a;
          return (_a = this.edgesBySource.get(nodeId)) === null || _a === void 0 ? void 0 : _a.get(portId);
      };
      GraphModel.prototype.getEdgesByTarget = function (nodeId, portId) {
          var _a;
          return (_a = this.edgesByTarget.get(nodeId)) === null || _a === void 0 ? void 0 : _a.get(portId);
      };
      GraphModel.prototype.isPortConnectedAsSource = function (nodeId, portId) {
          var _a, _b;
          return ((_b = (_a = this.getEdgesBySource(nodeId, portId)) === null || _a === void 0 ? void 0 : _a.size) !== null && _b !== void 0 ? _b : 0) > 0;
      };
      GraphModel.prototype.isPortConnectedAsTarget = function (nodeId, portId) {
          var _a, _b;
          return ((_b = (_a = this.getEdgesByTarget(nodeId, portId)) === null || _a === void 0 ? void 0 : _a.size) !== null && _b !== void 0 ? _b : 0) > 0;
      };
      GraphModel.prototype.shallow = function () {
          return new GraphModel(this.nodes, this.edges, this.groups, this.head, this.tail, this.edgesBySource, this.edgesByTarget);
      };
      GraphModel.prototype.toJSON = function () {
          var nodes = [];
          var current = this.head && this.nodes.get(this.head);
          while (current) {
              nodes.push(current.inner);
              current = current.next && this.nodes.get(current.next);
          }
          var edges = Array.from(this.edges.values()).map(function (it) { return it.inner; });
          return {
              nodes: nodes,
              edges: edges
          };
      };
      GraphModel.prototype.isEdgeExist = function (source, sourcePortId, target, targetPortId) {
          var sources = this.getEdgesBySource(source, sourcePortId);
          var targets = this.getEdgesByTarget(target, targetPortId);
          if (!sources || !targets) {
              return false;
          }
          var exist = false;
          sources.forEach(function (id) {
              if (targets.has(id)) {
                  exist = true;
              }
          });
          return exist;
      };
      return GraphModel;
  }());
  function setEdgeByPort(edgesByPort, edgeId, nodeId, portId) {
      return edgesByPort.has(nodeId)
          ? edgesByPort.update(nodeId, function (map) {
              var edges = map.get(portId);
              return new Map(map).set(portId, (edges ? new Set(edges) : new Set()).add(edgeId));
          })
          : edgesByPort.set(nodeId, new Map([[portId, new Set([edgeId])]]));
  }
  function setEdgeByPortMutable(edgesByPort, edgeId, nodeId, portId) {
      if (edgesByPort.has(nodeId)) {
          edgesByPort.update(nodeId, function (map) {
              var set = map.get(portId);
              if (!set) {
                  set = new Set();
                  map.set(portId, set);
              }
              set.add(edgeId);
              return map;
          });
      }
      else {
          edgesByPort.set(nodeId, new Map([[portId, new Set([edgeId])]]));
      }
  }
  function deleteEdgeByPort(edgesByPort, edgeId, nodeId, portId) {
      if (!edgesByPort.has(nodeId)) {
          return edgesByPort;
      }
      return edgesByPort.update(nodeId, function (edgeByPortsMap) {
          var edgeIds = edgeByPortsMap.get(portId);
          if (!edgeIds) {
              return edgeByPortsMap;
          }
          var set = new Set(edgeIds);
          set.delete(edgeId);
          return new Map(edgeByPortsMap).set(portId, set);
      });
  }

  exports.CanvasMouseMode = void 0;
  (function (CanvasMouseMode) {
      CanvasMouseMode["pan"] = "pan";
      CanvasMouseMode["select"] = "select";
  })(exports.CanvasMouseMode || (exports.CanvasMouseMode = {}));
  exports.GraphBehavior = void 0;
  (function (GraphBehavior) {
      GraphBehavior["default"] = "default";
      GraphBehavior["dragging"] = "dragging";
      GraphBehavior["panning"] = "panning";
      GraphBehavior["multiSelect"] = "multiSelect";
      GraphBehavior["connecting"] = "connecting";
      GraphBehavior["addingNode"] = "addingNode";
  })(exports.GraphBehavior || (exports.GraphBehavior = {}));

  var EMPTY_TRANSFORM_MATRIX = [1, 0, 0, 1, 0, 0];
  var EMPTY_VIEW_PORT = {
      rect: undefined,
      transformMatrix: EMPTY_TRANSFORM_MATRIX
  };
  var EMPTY_GAP = {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
  };
  var DEFAULT_NODE_MIN_VISIBLE_SIZE = {
      width: NODE_MIN_VISIBLE_LENGTH,
      height: NODE_MIN_VISIBLE_LENGTH
  };
  var DEFAULT_NODE_MAX_VISIBLE_SIZE = {
      width: NODE_MAX_VISIBLE_LENGTH,
      height: NODE_MAX_VISIBLE_LENGTH
  };
  var DEFAULT_GRAPH_SETTINGS = {
      features: defaultFeatures,
      graphConfig: GraphConfigBuilder.default().build(),
      canvasBoundaryPadding: EMPTY_GAP,
      nodeMinVisibleSize: DEFAULT_NODE_MIN_VISIBLE_SIZE,
      nodeMaxVisibleSize: DEFAULT_NODE_MAX_VISIBLE_SIZE
  };
  var EMPTY_GRAPH_STATE = {
      settings: DEFAULT_GRAPH_SETTINGS,
      behavior: exports.GraphBehavior.default,
      data: resetUndoStack(GraphModel.empty()),
      viewport: {
          transformMatrix: [1, 0, 0, 1, 0, 0],
          rect: undefined
      },
      dummyNodes: emptyDummyNodes(),
      alignmentLines: [],
      activeKeys: new Set(),
      selectBoxPosition: emptySelectBoxPosition(),
      connectState: undefined
  };
  function warnGraphStateContext() {
      Debug.warn("Missing GraphStateContext, GraphStateContext must be used as child of GraphStateStore");
  }
  var defaultGraphStateContext = {
      get state() {
          warnGraphStateContext();
          return EMPTY_GRAPH_STATE;
      },
      dispatch: function () {
          warnGraphStateContext();
      }
  };
  var EMPTY_CONNECT_STATE = {
      sourceNode: undefined,
      sourcePort: undefined,
      targetNode: undefined,
      targetPort: undefined,
      movingPoint: {
          x: 0,
          y: 0
      }
  };
  var GraphValueContext = React__namespace.createContext(new Proxy(GraphModel.empty(), {
      get: function (target, prop) {
          console.warn("Default graph data value is being used. Please check if you forget rendering Graph component");
          return target[prop];
      }
  }));
  var GraphStateContext = React__namespace.createContext(defaultGraphStateContext);
  var setData = function (state, data) { return (__assign(__assign({}, state), { data: __assign(__assign({}, state.data), { present: data }) })); };
  var updateData = function (state, f) { return (__assign(__assign({}, state), { data: __assign(__assign({}, state.data), { present: f(state.data.present) }) })); };

  var ViewportContext = React.createContext(EMPTY_VIEW_PORT);

  var EventChannel = (function () {
      function EventChannel() {
          this.listenersRef = React.createRef();
          this.externalHandlerRef = React.createRef();
          this.queue = [];
          this.working = false;
      }
      EventChannel.prototype.trigger = function (event) {
          var _this = this;
          if (this.working) {
              this.queue.push(event);
          }
          else {
              this.working = true;
              ReactDOM.unstable_batchedUpdates(function () {
                  _this.callHandlers(event);
                  for (var i = 0; i < _this.queue.length; i += 1) {
                      var e = _this.queue[i];
                      _this.callHandlers(e);
                  }
                  _this.queue = [];
              });
              this.working = false;
          }
      };
      EventChannel.prototype.batch = function (events) {
          var _a, _b;
          if (this.working) {
              (_a = this.queue).push.apply(_a, events);
          }
          else {
              var first = events[0];
              if (!first) {
                  return;
              }
              (_b = this.queue).push.apply(_b, events.slice(1));
              this.trigger(first);
          }
      };
      EventChannel.prototype.callHandlers = function (e) {
          var _a, _b, _c, _d;
          (_b = (_a = this.listenersRef).current) === null || _b === void 0 ? void 0 : _b.call(_a, e);
          if (!e.intercepted) {
              (_d = (_c = this.externalHandlerRef).current) === null || _d === void 0 ? void 0 : _d.call(_c, e);
          }
      };
      return EventChannel;
  }());

  var GraphController = (function () {
      function GraphController(state, dispatch) {
          this.pointerId = null;
          this.canvasClickOnce = false;
          this.nodeClickOnce = null;
          this.eventChannel = new EventChannel();
          this.behavior = exports.GraphBehavior.default;
          this.state = state;
          this.UNSAFE_latestState = state;
          this.dispatch = dispatch;
      }
      GraphController.prototype.setMouseClientPosition = function (pos) {
          this.mouseClientPoint = pos;
      };
      GraphController.prototype.unsetMouseClientPosition = function () {
          this.mouseClientPoint = undefined;
      };
      GraphController.prototype.getMouseClientPosition = function () {
          return this.mouseClientPoint;
      };
      GraphController.prototype.getEnabledFeatures = function () {
          return this.state.settings.features;
      };
      GraphController.prototype.getBehavior = function () {
          return this.behavior;
      };
      GraphController.prototype.setBehavior = function (value) {
          this.behavior = value;
      };
      GraphController.prototype.getData = function () {
          return this.state.data.present;
      };
      GraphController.prototype.getGlobalEventTarget = function () {
          var _a, _b;
          return (_b = (_a = this.getGlobalEventTargetImpl) === null || _a === void 0 ? void 0 : _a.call(this)) !== null && _b !== void 0 ? _b : window;
      };
      return GraphController;
  }());

  function useConst(init) {
      var ref = React.useRef();
      if (ref.current === undefined) {
          ref.current = init();
      }
      return ref.current;
  }

  var noop = function () {
  };

  var ErrorBoundary = (function (_super) {
      __extends(ErrorBoundary, _super);
      function ErrorBoundary(props) {
          var _this = _super.call(this, props) || this;
          _this.state = { hasError: false };
          return _this;
      }
      ErrorBoundary.getDerivedStateFromError = function (error) {
          return { hasError: true, error: error };
      };
      ErrorBoundary.prototype.componentDidCatch = function (error, errorInfo) {
          console.error(error);
          this.setState({
              error: error,
              errorInfo: errorInfo
          });
      };
      ErrorBoundary.prototype.render = function () {
          var _a;
          if (!this.state.hasError) {
              return this.props.children;
          }
          if (this.props.renderOnError) {
              return (_a = this.props.renderOnError(this.state.error, this.state.errorInfo, this.props.children)) !== null && _a !== void 0 ? _a : null;
          }
          var componentStackLines = this.state.errorInfo ? this.state.errorInfo.componentStack.split("\n") : [];
          return (React__namespace.createElement("div", { style: { color: "red" } },
              React__namespace.createElement("h1", null, "Something went wrong."),
              React__namespace.createElement("p", null, "Error: " + this.state.error),
              React__namespace.createElement("p", null, "ErrorInfo: " + JSON.stringify(this.state.errorInfo)),
              React__namespace.createElement("h2", null, "Component Stack"),
              componentStackLines.map(function (line, index) { return (React__namespace.createElement("p", { key: index }, line)); })));
      };
      return ErrorBoundary;
  }(React__namespace.Component));

  var EMPTY_CONNECT_CONTEXT = {
      sourceNode: undefined,
      sourcePort: undefined,
      targetNode: undefined,
      targetPort: undefined
  };
  var ConnectingStateContext = React.createContext(EMPTY_CONNECT_CONTEXT);
  ConnectingStateContext.displayName = "ConnectingStateContext";

  var ConnectingState = function (_a) {
      var children = _a.children, data = _a.data, connectState = _a.connectState;
      var sourceNode;
      var sourcePort;
      var targetNode;
      var targetPort;
      if (connectState) {
          sourceNode = data.nodes.get(connectState.sourceNode);
          sourcePort = sourceNode === null || sourceNode === void 0 ? void 0 : sourceNode.getPort(connectState.sourcePort);
          targetNode = connectState.targetNode ? data.nodes.get(connectState.targetNode) : undefined;
          targetPort = connectState.targetPort ? targetNode === null || targetNode === void 0 ? void 0 : targetNode.getPort(connectState.targetPort) : undefined;
      }
      var context = React__namespace.useMemo(function () { return ({
          sourceNode: sourceNode,
          sourcePort: sourcePort,
          targetNode: targetNode,
          targetPort: targetPort
      }); }, [sourceNode, sourcePort, targetNode, targetPort]);
      return React__namespace.createElement(ConnectingStateContext.Provider, { value: context }, children);
  };
  ConnectingState.displayName = "ConnectingState";

  var AlignmentLinesContext = React.createContext([]);

  var GraphControllerContext = React.createContext(new GraphController(EMPTY_GRAPH_STATE, noop));

  function GraphStateStore(props) {
      var graphController = props.graphController, state = props.state, dispatch = props.dispatch, children = props.children;
      var contextValue = React__namespace.useMemo(function () { return ({
          state: state,
          dispatch: dispatch
      }); }, [state, dispatch]);
      return (React__namespace.createElement(GraphConfigContext.Provider, { value: state.settings.graphConfig },
          React__namespace.createElement(GraphControllerContext.Provider, { value: graphController },
              React__namespace.createElement(ConnectingState, { data: state.data.present, connectState: state.connectState },
                  React__namespace.createElement(GraphStateContext.Provider, { value: contextValue },
                      React__namespace.createElement(ViewportContext.Provider, { value: state.viewport },
                          React__namespace.createElement(GraphValueContext.Provider, { value: state.data.present },
                              React__namespace.createElement(AlignmentLinesContext.Provider, { value: state.alignmentLines }, children))))))));
  }

  var defaultTheme = {
      controlPointColor: "#333333",
      primaryColor: "#0078D4",
      defaultColor: "#CCCCCC",
      borderColor: "#B3B0AD",
      defaultBorderColor: "#FFFFFF",
      unConnectableBgColor: "#E1DFDD",
      defaultBackgroundColor: "#FFFFFF",
      portStroke: "#ccc",
      portFill: "#fff",
      connectedPortColor: "gray",
      nodeActivateFill: "#ffffff",
      nodeActivateStroke: "#0078D4",
      nodeFill: "#ffffff",
      nodeStroke: "#cccccc",
      contextMenuBackground: "#FFFFFF",
      contextMenuBorder: "#E1DFDD",
      contextMenuHoverBackground: "rgba(0, 120, 212, 0.05)",
      fontColor: "#000000",
      canvasBackground: "#EDEDED",
      minimapBackground: "#EDEDED",
      edgeColor: "#ccc",
      edgeColorSelected: "#015cda",
      minimapShadow: "#000000",
      outlineStyle: "none",
      focusOutlineColor: "#000000",
      dummyNodeStroke: "#015cda",
      inputFocusBorderAlt: "#0078d4",
      buttonBorder: "#797775",
      scrollbarColor: "#c8c8c8"
  };
  var ThemeProvider = function (_a) {
      var theme = _a.theme, _b = _a.setTheme, setTheme = _b === void 0 ? noop : _b, children = _a.children;
      var themeContext = React__namespace.useMemo(function () { return ({
          theme: __assign(__assign({}, defaultTheme), theme),
          setTheme: setTheme
      }); }, [setTheme, theme]);
      return React__namespace.createElement(ThemeContext$1.Provider, { value: themeContext }, children);
  };

  var ReactDagEditor = function (props) {
      var _a;
      React__namespace.useEffect(function () {
          if (props.handleWarning) {
              Debug.warn = props.handleWarning;
          }
      }, []);
      var handleError = (_a = props.handleError) === null || _a === void 0 ? void 0 : _a.bind(null);
      var theme = props.theme, setTheme = props.setTheme, state = props.state, dispatch = props.dispatch, getGlobalEventTarget = props.getGlobalEventTarget;
      var graphController = useConst(function () { return new GraphController(state, dispatch); });
      graphController.UNSAFE_latestState = state;
      React__namespace.useLayoutEffect(function () {
          graphController.state = state;
          graphController.dispatch = dispatch;
          graphController.getGlobalEventTargetImpl = getGlobalEventTarget;
      }, [dispatch, getGlobalEventTarget, graphController, state]);
      React__namespace.useEffect(function () {
          return function () {
              graphController.dispatch = noop;
          };
      }, [graphController]);
      return (React__namespace.createElement(ErrorBoundary, { renderOnError: handleError },
          React__namespace.createElement(GraphStateStore, { state: state, dispatch: dispatch, graphController: graphController },
              React__namespace.createElement(ContextMenuConfigContext.Provider, { value: useConst(function () { return new ContextMenuConfig(); }) },
                  React__namespace.createElement(ThemeProvider, { theme: theme, setTheme: setTheme },
                      React__namespace.createElement("div", { style: props.style, className: props.className }, props.children))))));
  };

  var useContextMenuConfigContext = function () {
      return React__namespace.useContext(ContextMenuConfigContext);
  };

  exports.GraphNodeEvent = void 0;
  (function (GraphNodeEvent) {
      GraphNodeEvent["Click"] = "[Node]Click";
      GraphNodeEvent["DoubleClick"] = "[Node]DoubleClick";
      GraphNodeEvent["MouseDown"] = "[Node]MouseDown";
      GraphNodeEvent["MouseUp"] = "[Node]MouseUp";
      GraphNodeEvent["MouseEnter"] = "[Node]MouseEnter";
      GraphNodeEvent["MouseLeave"] = "[Node]MouseLeave";
      GraphNodeEvent["MouseOver"] = "[Node]MouseOver";
      GraphNodeEvent["MouseOut"] = "[Node]MouseOut";
      GraphNodeEvent["MouseMove"] = "[Node]MouseMove";
      GraphNodeEvent["ContextMenu"] = "[Node]ContextMenu";
      GraphNodeEvent["Drag"] = "[Node]Drag";
      GraphNodeEvent["DragStart"] = "[Node]DragStart";
      GraphNodeEvent["DragEnd"] = "[Node]DragEnd";
      GraphNodeEvent["PointerDown"] = "[Node]PointerDown";
      GraphNodeEvent["PointerEnter"] = "[Node]PointerEnter";
      GraphNodeEvent["PointerMove"] = "[Node]PointerMove";
      GraphNodeEvent["PointerLeave"] = "[Node]PointerLeave";
      GraphNodeEvent["PointerUp"] = "[Node]PointerUp";
      GraphNodeEvent["Resizing"] = "[Node]Resizing";
      GraphNodeEvent["ResizingStart"] = "[Node]ResizingStart";
      GraphNodeEvent["ResizingEnd"] = "[Node]ResizingEnd";
      GraphNodeEvent["KeyDown"] = "[Node]KeyDown";
      GraphNodeEvent["Select"] = "[Node]Select";
      GraphNodeEvent["SelectAll"] = "[Node]SelectAll";
      GraphNodeEvent["Centralize"] = "[Node]Centralize";
      GraphNodeEvent["Locate"] = "[Node]Locate";
      GraphNodeEvent["Add"] = "[Node]Add";
  })(exports.GraphNodeEvent || (exports.GraphNodeEvent = {}));
  exports.GraphEdgeEvent = void 0;
  (function (GraphEdgeEvent) {
      GraphEdgeEvent["Click"] = "[Edge]Click";
      GraphEdgeEvent["DoubleClick"] = "[Edge]DoubleClick";
      GraphEdgeEvent["MouseEnter"] = "[Edge]MouseEnter";
      GraphEdgeEvent["MouseLeave"] = "[Edge]MouseLeave";
      GraphEdgeEvent["MouseOver"] = "[Edge]MouseOver";
      GraphEdgeEvent["MouseOut"] = "[Edge]MouseOut";
      GraphEdgeEvent["MouseMove"] = "[Edge]MouseMove";
      GraphEdgeEvent["MouseDown"] = "[Edge]MouseDown";
      GraphEdgeEvent["MouseUp"] = "[Edge]MouseUp";
      GraphEdgeEvent["ContextMenu"] = "[Edge]ContextMenu";
      GraphEdgeEvent["ConnectStart"] = "[Edge]ConnectStart";
      GraphEdgeEvent["ConnectMove"] = "[Edge]ConnectMove";
      GraphEdgeEvent["ConnectEnd"] = "[Edge]ConnectEnd";
      GraphEdgeEvent["ConnectNavigate"] = "[Edge]ConnectNavigate";
      GraphEdgeEvent["Add"] = "[Edge]Add";
  })(exports.GraphEdgeEvent || (exports.GraphEdgeEvent = {}));
  exports.GraphPortEvent = void 0;
  (function (GraphPortEvent) {
      GraphPortEvent["Click"] = "[Port]Click";
      GraphPortEvent["DoubleClick"] = "[Port]DoubleClick";
      GraphPortEvent["MouseDown"] = "[Port]MouseDown";
      GraphPortEvent["PointerDown"] = "[Port]PointerDown";
      GraphPortEvent["PointerUp"] = "[Port]PointerUp";
      GraphPortEvent["PointerEnter"] = "[Port]PointerEnter";
      GraphPortEvent["PointerLeave"] = "[Port]PointerLeave";
      GraphPortEvent["MouseUp"] = "[Port]MouseUp";
      GraphPortEvent["MouseEnter"] = "[Port]MouseEnter";
      GraphPortEvent["MouseLeave"] = "[Port]MouseLeave";
      GraphPortEvent["MouseOver"] = "[Port]MouseOver";
      GraphPortEvent["MouseOut"] = "[Port]MouseOut";
      GraphPortEvent["MouseMove"] = "[Port]MouseMove";
      GraphPortEvent["ContextMenu"] = "[Port]ContextMenu";
      GraphPortEvent["KeyDown"] = "[Port]KeyDown";
      GraphPortEvent["Focus"] = "[Port]Focus";
      GraphPortEvent["Blur"] = "[Port]Blur";
  })(exports.GraphPortEvent || (exports.GraphPortEvent = {}));
  exports.GraphCanvasEvent = void 0;
  (function (GraphCanvasEvent) {
      GraphCanvasEvent["Click"] = "[Canvas]Click";
      GraphCanvasEvent["DoubleClick"] = "[Canvas]DoubleClick";
      GraphCanvasEvent["MouseDown"] = "[Canvas]MouseDown";
      GraphCanvasEvent["MouseUp"] = "[Canvas]MouseUp";
      GraphCanvasEvent["MouseEnter"] = "[Canvas]MouseEnter";
      GraphCanvasEvent["MouseLeave"] = "[Canvas]MouseLeave";
      GraphCanvasEvent["MouseOver"] = "[Canvas]MouseOver";
      GraphCanvasEvent["MouseOut"] = "[Canvas]MouseOut";
      GraphCanvasEvent["MouseMove"] = "[Canvas]MouseMove";
      GraphCanvasEvent["ContextMenu"] = "[Canvas]ContextMenu";
      GraphCanvasEvent["DragStart"] = "[Canvas]DragStart";
      GraphCanvasEvent["Drag"] = "[Canvas]Drag";
      GraphCanvasEvent["DragEnd"] = "[Canvas]DragEnd";
      GraphCanvasEvent["Pan"] = "[Canvas]Pan";
      GraphCanvasEvent["Focus"] = "[Canvas]Focus";
      GraphCanvasEvent["Blur"] = "[Canvas]Blur";
      GraphCanvasEvent["Zoom"] = "[Canvas]Zoom";
      GraphCanvasEvent["Pinch"] = "[Canvas]Pinch";
      GraphCanvasEvent["KeyDown"] = "[Canvas]KeyDown";
      GraphCanvasEvent["KeyUp"] = "[Canvas]KeyUp";
      GraphCanvasEvent["SelectStart"] = "[Canvas]SelectStart";
      GraphCanvasEvent["SelectMove"] = "[Canvas]SelectMove";
      GraphCanvasEvent["SelectEnd"] = "[Canvas]SelectEnd";
      GraphCanvasEvent["UpdateNodeSelectionBySelectBox"] = "[Canvas]UpdateNodeSelectionBySelectBox";
      GraphCanvasEvent["MouseWheelScroll"] = "[Canvas]MouseWheelScroll";
      GraphCanvasEvent["DraggingNodeFromItemPanel"] = "[Canvas]DraggingNodeFromItemPanel";
      GraphCanvasEvent["DraggingNodeFromItemPanelStart"] = "[Canvas]DraggingNodeFromItemPanelStart";
      GraphCanvasEvent["DraggingNodeFromItemPanelEnd"] = "[Canvas]DraggingNodeFromItemPanelEnd";
      GraphCanvasEvent["ViewportResize"] = "[Canvas]ViewportResize";
      GraphCanvasEvent["Navigate"] = "[Canvas]Navigate";
      GraphCanvasEvent["VirtualizationRecalculated"] = "[Canvas]VirtualizationRecalculated";
      GraphCanvasEvent["ResetSelection"] = "[Canvas]ResetSelection";
      GraphCanvasEvent["Copy"] = "[Canvas]Copy";
      GraphCanvasEvent["Paste"] = "[Canvas]Paste";
      GraphCanvasEvent["Delete"] = "[Canvas]Delete";
      GraphCanvasEvent["Undo"] = "[Canvas]Undo";
      GraphCanvasEvent["Redo"] = "[Canvas]Redo";
      GraphCanvasEvent["ScrollIntoView"] = "[Canvas]ScrollIntoView";
      GraphCanvasEvent["ResetUndoStack"] = "[Canvas]ResetUndoStack";
      GraphCanvasEvent["ResetViewport"] = "[Canvas]ResetViewport";
      GraphCanvasEvent["ZoomTo"] = "[Canvas]ZoomTo";
      GraphCanvasEvent["ZoomToFit"] = "[Canvas]ZoomToFit";
      GraphCanvasEvent["SetData"] = "[Canvas]SetData";
      GraphCanvasEvent["UpdateData"] = "[Canvas]UpdateData";
      GraphCanvasEvent["ScrollTo"] = "[Canvas]ScrollTo";
      GraphCanvasEvent["UpdateSettings"] = "[Canvas]UpdateSettings";
  })(exports.GraphCanvasEvent || (exports.GraphCanvasEvent = {}));
  exports.GraphScrollBarEvent = void 0;
  (function (GraphScrollBarEvent) {
      GraphScrollBarEvent["ScrollStart"] = "[ScrollBar]ScrollStart";
      GraphScrollBarEvent["Scroll"] = "[ScrollBar]Scroll";
      GraphScrollBarEvent["ScrollEnd"] = "[ScrollBar]ScrollEnd";
  })(exports.GraphScrollBarEvent || (exports.GraphScrollBarEvent = {}));
  exports.GraphMinimapEvent = void 0;
  (function (GraphMinimapEvent) {
      GraphMinimapEvent["PanStart"] = "[Minimap]PanStart";
      GraphMinimapEvent["Pan"] = "[Minimap]Pan";
      GraphMinimapEvent["PanEnd"] = "[Minimap]PanEnd";
      GraphMinimapEvent["Click"] = "[Minimap]Click";
  })(exports.GraphMinimapEvent || (exports.GraphMinimapEvent = {}));
  exports.GraphContextMenuEvent = void 0;
  (function (GraphContextMenuEvent) {
      GraphContextMenuEvent["Open"] = "[ContextMenu]Open";
      GraphContextMenuEvent["Close"] = "[ContextMenu]Close";
  })(exports.GraphContextMenuEvent || (exports.GraphContextMenuEvent = {}));

  function getScrollLineHeight() {
      try {
          var iframe = document.createElement("iframe");
          iframe.src = "#";
          document.body.appendChild(iframe);
          var contentWindow = iframe.contentWindow;
          if (!contentWindow) {
              throw new Error("Fail to create iframe");
          }
          var doc = contentWindow.document;
          if (!doc) {
              throw new Error("Fail to create iframe");
          }
          doc.open();
          doc.write("<!DOCTYPE html><html><head></head><body><span>a</span></body></html>");
          doc.close();
          var span = doc.body.firstElementChild;
          var height = span.offsetHeight;
          document.body.removeChild(iframe);
          return height;
      }
      catch (e) {
          Debug.error("failed to calculate scroll line height", e);
          return 16;
      }
  }
  var scrollLineHeight = getScrollLineHeight();
  var normalizeWheelDelta = typeof WheelEvent === "function"
      ? function (deltaMode, delta) {
          switch (deltaMode) {
              case WheelEvent.DOM_DELTA_PIXEL:
                  return delta;
              case WheelEvent.DOM_DELTA_LINE:
                  return delta * scrollLineHeight;
              case WheelEvent.DOM_DELTA_PAGE:
                  return delta * window.innerHeight;
              default:
                  return delta;
          }
      }
      : function (deltaMode, delta) { return delta; };

  var useWheelHandler = function (args) {
      var containerRef = args.containerRef, svgRef = args.svgRef, rectRef = args.rectRef, zoomSensitivity = args.zoomSensitivity, scrollSensitivity = args.scrollSensitivity, isHorizontalScrollDisabled = args.isHorizontalScrollDisabled, isVerticalScrollDisabled = args.isVerticalScrollDisabled, isCtrlKeyZoomEnable = args.isCtrlKeyZoomEnable, eventChannel = args.eventChannel, graphConfig = args.graphConfig, dispatch = args.dispatch;
      React.useLayoutEffect(function () {
          var svg = svgRef.current;
          var container = containerRef.current;
          if (!svg || !container) {
              return noop;
          }
          var onWheel = function (e) {
              var rect = rectRef.current;
              if (!rect) {
                  return;
              }
              if (!svg.contains(document.activeElement)) {
                  return;
              }
              e.preventDefault();
              if (e.ctrlKey && isCtrlKeyZoomEnable) {
                  var deltaY = normalizeWheelDelta(e.deltaMode, e.deltaY);
                  var scale = (deltaY > 0 ? -zoomSensitivity : zoomSensitivity) + 1;
                  eventChannel.trigger({
                      type: exports.GraphCanvasEvent.Zoom,
                      rawEvent: e,
                      scale: scale,
                      anchor: getRelativePoint(rect, e)
                  });
                  return;
              }
              var dx = isHorizontalScrollDisabled
                  ? 0
                  : -normalizeWheelDelta(e.deltaMode, e.shiftKey ? e.deltaY : e.deltaX) * scrollSensitivity;
              var dy = isVerticalScrollDisabled || e.shiftKey ? 0 : -normalizeWheelDelta(e.deltaMode, e.deltaY) * scrollSensitivity;
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.MouseWheelScroll,
                  dx: dx,
                  dy: dy,
                  rawEvent: e
              });
          };
          container.addEventListener("wheel", onWheel);
          return function () {
              container.removeEventListener("wheel", onWheel);
          };
      }, [
          svgRef,
          rectRef,
          zoomSensitivity,
          scrollSensitivity,
          dispatch,
          isHorizontalScrollDisabled,
          isVerticalScrollDisabled,
          graphConfig,
          eventChannel,
          isCtrlKeyZoomEnable
      ]);
  };

  var useClientRect = function () {
      var _a = React.useState(null), rect = _a[0], setRect = _a[1];
      var ref = React.useCallback(function (node) {
          if (node !== null) {
              setRect(node.getBoundingClientRect());
          }
      }, []);
      return [rect, ref];
  };

  function nextFrame(callback) {
      requestAnimationFrame(function () {
          requestAnimationFrame(callback);
      });
  }

  var LIMIT = 20;
  var isRectChanged = function (a, b) {
      if (!a || !b) {
          return true;
      }
      if (a === b) {
          return false;
      }
      return a.top !== b.top || a.left !== b.left || a.width !== b.width || a.height !== b.height;
  };
  var useUpdateViewportCallback = function (rectRef, svgRef, eventChannel) {
      return React.useCallback(function () {
          var _a;
          var viewportRect = (_a = svgRef.current) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
          if (isRectChanged(rectRef.current, viewportRect)) {
              rectRef.current = viewportRect;
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.ViewportResize,
                  viewportRect: viewportRect
              });
          }
      }, [eventChannel, rectRef, svgRef]);
  };
  var useContainerRect = function (svgRef, containerRef, updateViewport) {
      React.useLayoutEffect(updateViewport, [updateViewport]);
      React.useEffect(function () {
          var container = containerRef.current;
          if (!container) {
              return noop;
          }
          var onResize = debounce(function () {
              return nextFrame(function () {
                  updateViewport();
              });
          }, LIMIT);
          if (typeof ResizeObserver !== "undefined") {
              var resizeObserver_1 = new ResizeObserver(onResize);
              resizeObserver_1.observe(container);
              return function () {
                  resizeObserver_1.unobserve(container);
                  resizeObserver_1.disconnect();
              };
          }
          window.addEventListener("resize", onResize);
          return function () {
              window.removeEventListener("resize", onResize);
          };
      }, [containerRef, updateViewport]);
      React.useEffect(function () {
          var listener = debounce(function (e) {
              var svg = svgRef.current;
              if (!svg || !(e.target instanceof Element) || !e.target.contains(svg)) {
                  return;
              }
              updateViewport();
          }, LIMIT);
          var options = {
              capture: true,
              passive: true
          };
          document.body.addEventListener("scroll", listener, options);
          return function () {
              document.body.removeEventListener("scroll", listener, options);
          };
      }, [svgRef, updateViewport]);
  };

  function makeScheduledCallback(callback, schedule, cancel) {
      var scheduled = false;
      var currentArgs;
      var task;
      var scheduledCallback = function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          currentArgs = args;
          if (!scheduled) {
              scheduled = true;
              task = schedule(function () {
                  scheduled = false;
                  ReactDOM.unstable_batchedUpdates(function () {
                      callback.apply(null, currentArgs);
                  });
              });
          }
      };
      scheduledCallback.cancel = function () {
          cancel(task);
      };
      return scheduledCallback;
  }
  var animationFramed = function (callback) {
      return makeScheduledCallback(callback, requestAnimationFrame, cancelAnimationFrame);
  };
  var throttle = function (callback, limit) {
      return makeScheduledCallback(callback, function (cb) { return setTimeout(cb, limit); }, clearTimeout);
  };

  function useThrottle(callback, limit, deps) {
      return React.useMemo(function () { return throttle(callback, limit); }, deps);
  }

  var useRenderedArea = function (viewport, isVirtualizationEnabled) {
      return React.useMemo(function () {
          return isVirtualizationEnabled
              ? getRenderedArea(viewport)
              : {
                  minX: -Number.MAX_SAFE_INTEGER,
                  minY: -Number.MAX_SAFE_INTEGER,
                  maxX: Number.MAX_SAFE_INTEGER,
                  maxY: Number.MAX_SAFE_INTEGER
              };
      }, [viewport, isVirtualizationEnabled]);
  };

  var getMinimapRect = function (svgRef) {
      var _a, _b;
      return (_b = (_a = svgRef.current) === null || _a === void 0 ? void 0 : _a.closest(".minimap-container")) === null || _b === void 0 ? void 0 : _b.getBoundingClientRect();
  };
  var useMinimapRect = function (svgRef) {
      var _a = React.useState(undefined), rect = _a[0], setRect = _a[1];
      React.useLayoutEffect(function () {
          setRect(getMinimapRect(svgRef));
      }, [svgRef]);
      React.useEffect(function () {
          var onResize = throttle(function () {
              setRect(getMinimapRect(svgRef));
          }, 20);
          window.addEventListener("resize", onResize);
          return function () {
              window.removeEventListener("resize", onResize);
          };
      }, [svgRef]);
      return rect;
  };

  var DragController = (function () {
      function DragController(eventProvider, getPositionFromEvent) {
          var _this = this;
          this.onMove = noop;
          this.onEnd = noop;
          this.lastEvent = null;
          this.startX = 0;
          this.startY = 0;
          this.prevClientX = 0;
          this.prevClientY = 0;
          this.onMouseUp = function (e) {
              _this.lastEvent = e;
              _this.doOnMouseUp(e);
              _this.lastEvent = null;
          };
          this.onMouseMove = function (e) {
              _this.lastEvent = e;
              e.preventDefault();
              _this.mouseMove(e);
          };
          this.eventProvider = eventProvider;
          this.getPositionFromEvent = getPositionFromEvent;
          this.mouseMove = animationFramed(function (e) {
              _this.doOnMouseMove(e);
          });
      }
      DragController.prototype.start = function (e) {
          this.lastEvent = e;
          var _a = this.getPositionFromEvent(e), x = _a.x, y = _a.y;
          this.startX = x;
          this.startY = y;
          this.prevClientX = x;
          this.prevClientY = y;
          this.eventProvider.on("move", this.onMouseMove);
          this.eventProvider.on("end", this.onMouseUp);
      };
      DragController.prototype.stop = function () {
          this.mouseMove.cancel();
          this.eventProvider.off("move", this.onMouseMove);
          this.eventProvider.off("end", this.onMouseUp);
      };
      DragController.prototype.getDelta = function (x, y) {
          var clientDX = x - this.prevClientX;
          var clientDY = y - this.prevClientY;
          this.prevClientX = x;
          this.prevClientY = y;
          return {
              x: clientDX,
              y: clientDY
          };
      };
      DragController.prototype.getTotalDelta = function (e) {
          var x = e.clientX - this.startX;
          var y = e.clientY - this.startY;
          return {
              x: x,
              y: y
          };
      };
      DragController.prototype.doOnMouseMove = function (e) {
          var _a = this.getPositionFromEvent(e), x = _a.x, y = _a.y;
          var _b = this.getDelta(x, y), dx = _b.x, dy = _b.y;
          var _c = this.getTotalDelta(e), totalDX = _c.x, totalDY = _c.y;
          this.onMove({
              clientX: x,
              clientY: y,
              dx: dx,
              dy: dy,
              totalDX: totalDX,
              totalDY: totalDY,
              e: e
          });
      };
      DragController.prototype.doOnMouseUp = function (e) {
          e.preventDefault();
          var _a = this.getTotalDelta(e), totalDX = _a.x, totalDY = _a.y;
          this.onEnd({ totalDX: totalDX, totalDY: totalDY, e: e });
          this.stop();
      };
      return DragController;
  }());
  function defaultGetPositionFromEvent(e) {
      return {
          x: e.clientX,
          y: e.clientY
      };
  }

  var DragNodeController = (function (_super) {
      __extends(DragNodeController, _super);
      function DragNodeController(globalMoveEventProvider, getPositionFromEvent, rectRef) {
          var _this = _super.call(this, globalMoveEventProvider, getPositionFromEvent) || this;
          _this.rectRef = rectRef;
          return _this;
      }
      DragNodeController.prototype.doOnMouseMove = function (e) {
          _super.prototype.doOnMouseMove.call(this, e);
          var rect = this.rectRef.current;
          if (!rect || !this.lastEvent) {
              return;
          }
          if (e.clientX < rect.left || e.clientX > rect.right || e.clientY < rect.top || e.clientY > rect.bottom) {
              this.mouseMove(this.lastEvent);
          }
      };
      return DragNodeController;
  }(DragController));

  var TouchController = (function () {
      function TouchController(handlers) {
          var _this = this;
          this.eventHandlers = {
              onPointerDown: function (e) {
                  var args = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      args[_i - 1] = arguments[_i];
                  }
                  if (e.pointerType !== "touch") {
                      return;
                  }
                  e.preventDefault();
                  _this.pointers = new Map(_this.pointers);
                  _this.pointers.set(e.pointerId, e.nativeEvent);
                  _this.updateHandler.apply(_this, __spreadArray([e.nativeEvent], args));
              },
              onPointerMove: function (e) {
                  var args = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      args[_i - 1] = arguments[_i];
                  }
                  if (e.pointerType !== "touch") {
                      return;
                  }
                  e.preventDefault();
                  _this.pointers.set(e.pointerId, e.nativeEvent);
                  _this.onMove.apply(_this, __spreadArray([e.nativeEvent], args));
              },
              onPointerUp: function (e) {
                  var args = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      args[_i - 1] = arguments[_i];
                  }
                  if (e.pointerType !== "touch") {
                      return;
                  }
                  e.preventDefault();
                  _this.pointers = new Map(_this.pointers);
                  _this.pointers.delete(e.pointerId);
                  _this.updateHandler.apply(_this, __spreadArray([e.nativeEvent], args));
              }
          };
          this.pointers = new Map();
          this.onMove = animationFramed(function (e) {
              var _a;
              var args = [];
              for (var _i = 1; _i < arguments.length; _i++) {
                  args[_i - 1] = arguments[_i];
              }
              (_a = _this.currentHandler) === null || _a === void 0 ? void 0 : _a.onMove.apply(_a, __spreadArray([_this.pointers, e], args));
          });
          this.handlers = handlers;
      }
      TouchController.prototype.updateHandler = function (e) {
          var _a, _b;
          var args = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              args[_i - 1] = arguments[_i];
          }
          var handler = this.handlers.get(this.pointers.size);
          if (handler !== this.currentHandler) {
              (_a = this.currentHandler) === null || _a === void 0 ? void 0 : _a.onEnd.apply(_a, __spreadArray([e], args));
              this.currentHandler = handler;
              (_b = this.currentHandler) === null || _b === void 0 ? void 0 : _b.onStart.apply(_b, __spreadArray([this.pointers, e], args));
          }
      };
      return TouchController;
  }());

  var TwoFingerHandler = (function () {
      function TwoFingerHandler(rectRef, eventChannel) {
          this.prevDistance = 0;
          this.rectRef = rectRef;
          this.eventChannel = eventChannel;
      }
      TwoFingerHandler.prototype.onEnd = function () {
      };
      TwoFingerHandler.prototype.onMove = function (pointers, e) {
          var events = Array.from(pointers.values());
          var currentDistance = distance(events[0].clientX, events[0].clientY, events[1].clientX, events[1].clientY);
          var _a = this, prevEvents = _a.prevEvents, prevDistance = _a.prevDistance;
          this.prevDistance = currentDistance;
          this.prevEvents = events;
          if (!prevEvents) {
              return;
          }
          var dx1 = events[0].clientX - prevEvents[0].clientX;
          var dx2 = events[1].clientX - prevEvents[1].clientX;
          var dy1 = events[0].clientY - prevEvents[0].clientY;
          var dy2 = events[1].clientY - prevEvents[1].clientY;
          var dx = (dx1 + dx2) / 2;
          var dy = (dy1 + dy2) / 2;
          var scale = (currentDistance - prevDistance) / prevDistance + 1;
          var anchor = getContainerCenter(this.rectRef);
          if (!anchor) {
              return;
          }
          this.eventChannel.trigger({
              type: exports.GraphCanvasEvent.Pinch,
              rawEvent: e,
              dx: dx,
              dy: dy,
              scale: scale,
              anchor: anchor
          });
      };
      TwoFingerHandler.prototype.onStart = function (pointers) {
          if (pointers.size !== 2) {
              throw new Error("Unexpected touch event with " + pointers.size + " touches");
          }
          this.prevEvents = Array.from(pointers.values());
          this.prevDistance = distance(this.prevEvents[0].clientX, this.prevEvents[0].clientY, this.prevEvents[1].clientX, this.prevEvents[1].clientY);
      };
      return TwoFingerHandler;
  }());

  var useGraphTouchHandler = function (rectRef, eventChannel) {
      return React__namespace.useMemo(function () {
          var touchEvents = new TouchController(new Map().set(2, new TwoFingerHandler(rectRef, eventChannel)));
          return touchEvents.eventHandlers;
      }, [rectRef, eventChannel]);
  };

  var isSafari = getBrowser() === exports.BrowserType.safari;
  var prevScale = 0;
  function useSafariScale(_a) {
      var rectRef = _a.rectRef, svgRef = _a.svgRef, eventChannel = _a.eventChannel;
      React.useEffect(function () {
          var el = svgRef.current;
          if (!isSafari || !el || isMobile()) {
              return function () {
              };
          }
          var onGestureEvent = animationFramed(function (e) {
              var scale = e.scale;
              var delta = scale / prevScale;
              prevScale = scale;
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.Zoom,
                  rawEvent: e,
                  scale: delta,
                  anchor: getContainerCenter(rectRef)
              });
          });
          var onGestureStart = function (e) {
              e.stopPropagation();
              e.preventDefault();
              prevScale = e.scale;
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.Zoom,
                  rawEvent: e,
                  scale: e.scale,
                  anchor: getContainerCenter(rectRef)
              });
          };
          var onGestureChange = function (e) {
              e.stopPropagation();
              e.preventDefault();
              onGestureEvent(e);
          };
          var onGestureEnd = function (e) {
              e.stopPropagation();
              e.preventDefault();
              onGestureEvent(e);
          };
          el.addEventListener("gesturestart", onGestureStart);
          el.addEventListener("gesturechange", onGestureChange);
          el.addEventListener("gestureend", onGestureEnd);
          return function () {
              el.removeEventListener("gesturestart", onGestureStart);
              el.removeEventListener("gesturechange", onGestureChange);
              el.removeEventListener("gestureend", onGestureEnd);
          };
      }, []);
  }

  function useDeferredValue(value, _a) {
      var timeout = _a.timeout;
      var _b = React.useState(value), state = _b[0], setState = _b[1];
      React.useEffect(function () {
          var timer = setTimeout(function () {
              setState(value);
          }, timeout);
          return function () {
              clearTimeout(timer);
          };
      }, [value, timeout]);
      return state;
  }

  var useSelectBox = function (dispatch, selectBoxPositionState) {
      var selectBoxPosition = useDeferredValue(selectBoxPositionState, { timeout: 100 });
      React__namespace.useEffect(function () {
          dispatch({
              type: exports.GraphCanvasEvent.UpdateNodeSelectionBySelectBox
          });
      }, [selectBoxPosition]);
  };

  var useGraphState = function () {
      return React.useContext(GraphStateContext);
  };
  var useGraphData = function () {
      return React.useContext(GraphValueContext);
  };

  var EMPTY_RECT = {
      height: 0,
      width: 0,
      x: 0,
      y: 0,
      bottom: 0,
      left: 0,
      right: 0,
      top: 0,
      toJSON: function () {
          return this;
      }
  };
  var VirtualizationContext = React.createContext({
      viewport: {
          rect: EMPTY_RECT,
          transformMatrix: EMPTY_TRANSFORM_MATRIX
      },
      renderedArea: {
          minX: 0,
          minY: 0,
          maxX: 0,
          maxY: 0
      },
      visibleArea: {
          minX: 0,
          minY: 0,
          maxX: 0,
          maxY: 0
      },
      renderedNodes: new Set(),
      renderedEdges: new Set(),
      timestamp: 0
  });

  function useGraphConfig() {
      return React.useContext(GraphConfigContext);
  }
  function useGraphController() {
      return React.useContext(GraphControllerContext);
  }
  function useViewport() {
      return React.useContext(ViewportContext);
  }
  function useAlignmentLines() {
      return React.useContext(AlignmentLinesContext);
  }
  function useConnectingState() {
      return React.useContext(ConnectingStateContext);
  }
  function useVirtualization() {
      return React.useContext(VirtualizationContext);
  }

  function getDefaultExportFromCjs (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  function createCommonjsModule(fn, basedir, module) {
  	return module = {
  		path: basedir,
  		exports: {},
  		require: function (path, base) {
  			return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
  		}
  	}, fn(module, module.exports), module.exports;
  }

  function commonjsRequire () {
  	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
  }

  var eventemitter3 = createCommonjsModule(function (module) {

  var has = Object.prototype.hasOwnProperty
    , prefix = '~';

  /**
   * Constructor to create a storage for our `EE` objects.
   * An `Events` instance is a plain object whose properties are event names.
   *
   * @constructor
   * @private
   */
  function Events() {}

  //
  // We try to not inherit from `Object.prototype`. In some engines creating an
  // instance in this way is faster than calling `Object.create(null)` directly.
  // If `Object.create(null)` is not supported we prefix the event names with a
  // character to make sure that the built-in object properties are not
  // overridden or used as an attack vector.
  //
  if (Object.create) {
    Events.prototype = Object.create(null);

    //
    // This hack is needed because the `__proto__` property is still inherited in
    // some old browsers like Android 4, iPhone 5.1, Opera 11 and Safari 5.
    //
    if (!new Events().__proto__) prefix = false;
  }

  /**
   * Representation of a single event listener.
   *
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} [once=false] Specify if the listener is a one-time listener.
   * @constructor
   * @private
   */
  function EE(fn, context, once) {
    this.fn = fn;
    this.context = context;
    this.once = once || false;
  }

  /**
   * Add a listener for a given event.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} once Specify if the listener is a one-time listener.
   * @returns {EventEmitter}
   * @private
   */
  function addListener(emitter, event, fn, context, once) {
    if (typeof fn !== 'function') {
      throw new TypeError('The listener must be a function');
    }

    var listener = new EE(fn, context || emitter, once)
      , evt = prefix ? prefix + event : event;

    if (!emitter._events[evt]) emitter._events[evt] = listener, emitter._eventsCount++;
    else if (!emitter._events[evt].fn) emitter._events[evt].push(listener);
    else emitter._events[evt] = [emitter._events[evt], listener];

    return emitter;
  }

  /**
   * Clear event by name.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} evt The Event name.
   * @private
   */
  function clearEvent(emitter, evt) {
    if (--emitter._eventsCount === 0) emitter._events = new Events();
    else delete emitter._events[evt];
  }

  /**
   * Minimal `EventEmitter` interface that is molded against the Node.js
   * `EventEmitter` interface.
   *
   * @constructor
   * @public
   */
  function EventEmitter() {
    this._events = new Events();
    this._eventsCount = 0;
  }

  /**
   * Return an array listing the events for which the emitter has registered
   * listeners.
   *
   * @returns {Array}
   * @public
   */
  EventEmitter.prototype.eventNames = function eventNames() {
    var names = []
      , events
      , name;

    if (this._eventsCount === 0) return names;

    for (name in (events = this._events)) {
      if (has.call(events, name)) names.push(prefix ? name.slice(1) : name);
    }

    if (Object.getOwnPropertySymbols) {
      return names.concat(Object.getOwnPropertySymbols(events));
    }

    return names;
  };

  /**
   * Return the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Array} The registered listeners.
   * @public
   */
  EventEmitter.prototype.listeners = function listeners(event) {
    var evt = prefix ? prefix + event : event
      , handlers = this._events[evt];

    if (!handlers) return [];
    if (handlers.fn) return [handlers.fn];

    for (var i = 0, l = handlers.length, ee = new Array(l); i < l; i++) {
      ee[i] = handlers[i].fn;
    }

    return ee;
  };

  /**
   * Return the number of listeners listening to a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Number} The number of listeners.
   * @public
   */
  EventEmitter.prototype.listenerCount = function listenerCount(event) {
    var evt = prefix ? prefix + event : event
      , listeners = this._events[evt];

    if (!listeners) return 0;
    if (listeners.fn) return 1;
    return listeners.length;
  };

  /**
   * Calls each of the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Boolean} `true` if the event had listeners, else `false`.
   * @public
   */
  EventEmitter.prototype.emit = function emit(event, a1, a2, a3, a4, a5) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return false;

    var listeners = this._events[evt]
      , len = arguments.length
      , args
      , i;

    if (listeners.fn) {
      if (listeners.once) this.removeListener(event, listeners.fn, undefined, true);

      switch (len) {
        case 1: return listeners.fn.call(listeners.context), true;
        case 2: return listeners.fn.call(listeners.context, a1), true;
        case 3: return listeners.fn.call(listeners.context, a1, a2), true;
        case 4: return listeners.fn.call(listeners.context, a1, a2, a3), true;
        case 5: return listeners.fn.call(listeners.context, a1, a2, a3, a4), true;
        case 6: return listeners.fn.call(listeners.context, a1, a2, a3, a4, a5), true;
      }

      for (i = 1, args = new Array(len -1); i < len; i++) {
        args[i - 1] = arguments[i];
      }

      listeners.fn.apply(listeners.context, args);
    } else {
      var length = listeners.length
        , j;

      for (i = 0; i < length; i++) {
        if (listeners[i].once) this.removeListener(event, listeners[i].fn, undefined, true);

        switch (len) {
          case 1: listeners[i].fn.call(listeners[i].context); break;
          case 2: listeners[i].fn.call(listeners[i].context, a1); break;
          case 3: listeners[i].fn.call(listeners[i].context, a1, a2); break;
          case 4: listeners[i].fn.call(listeners[i].context, a1, a2, a3); break;
          default:
            if (!args) for (j = 1, args = new Array(len -1); j < len; j++) {
              args[j - 1] = arguments[j];
            }

            listeners[i].fn.apply(listeners[i].context, args);
        }
      }
    }

    return true;
  };

  /**
   * Add a listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.on = function on(event, fn, context) {
    return addListener(this, event, fn, context, false);
  };

  /**
   * Add a one-time listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.once = function once(event, fn, context) {
    return addListener(this, event, fn, context, true);
  };

  /**
   * Remove the listeners of a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn Only remove the listeners that match this function.
   * @param {*} context Only remove the listeners that have this context.
   * @param {Boolean} once Only remove one-time listeners.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeListener = function removeListener(event, fn, context, once) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return this;
    if (!fn) {
      clearEvent(this, evt);
      return this;
    }

    var listeners = this._events[evt];

    if (listeners.fn) {
      if (
        listeners.fn === fn &&
        (!once || listeners.once) &&
        (!context || listeners.context === context)
      ) {
        clearEvent(this, evt);
      }
    } else {
      for (var i = 0, events = [], length = listeners.length; i < length; i++) {
        if (
          listeners[i].fn !== fn ||
          (once && !listeners[i].once) ||
          (context && listeners[i].context !== context)
        ) {
          events.push(listeners[i]);
        }
      }

      //
      // Reset the array, or remove it completely if we have no more listeners.
      //
      if (events.length) this._events[evt] = events.length === 1 ? events[0] : events;
      else clearEvent(this, evt);
    }

    return this;
  };

  /**
   * Remove all listeners, or those of the specified event.
   *
   * @param {(String|Symbol)} [event] The event name.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeAllListeners = function removeAllListeners(event) {
    var evt;

    if (event) {
      evt = prefix ? prefix + event : event;
      if (this._events[evt]) clearEvent(this, evt);
    } else {
      this._events = new Events();
      this._eventsCount = 0;
    }

    return this;
  };

  //
  // Alias methods names because people roll like that.
  //
  EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  //
  // Expose the prefix.
  //
  EventEmitter.prefixed = prefix;

  //
  // Allow `EventEmitter` to be imported as module namespace.
  //
  EventEmitter.EventEmitter = EventEmitter;

  //
  // Expose the module.
  //
  {
    module.exports = EventEmitter;
  }
  });

  var PointerEventProvider = (function () {
      function PointerEventProvider(target, pointerId) {
          var _this = this;
          if (pointerId === void 0) { pointerId = null; }
          this.eventEmitter = new eventemitter3.EventEmitter();
          this.onMove = function (e) {
              if (_this.pointerId === null || _this.pointerId === e.pointerId) {
                  _this.eventEmitter.emit("move", e);
              }
          };
          this.onUp = function (e) {
              if (_this.pointerId === null || _this.pointerId === e.pointerId) {
                  _this.eventEmitter.emit("end", e);
              }
          };
          this.target = target;
          this.pointerId = pointerId;
      }
      PointerEventProvider.prototype.off = function (type, callback) {
          this.eventEmitter.off(type, callback);
          this.ensureRemoveListener(type);
          return this;
      };
      PointerEventProvider.prototype.on = function (type, callback) {
          this.ensureAddListener(type);
          this.eventEmitter.on(type, callback);
          return this;
      };
      PointerEventProvider.prototype.ensureAddListener = function (type) {
          if (!this.eventEmitter.listeners(type).length) {
              switch (type) {
                  case "move":
                      this.target.addEventListener("pointermove", this.onMove);
                      break;
                  case "end":
                      this.target.addEventListener("pointerup", this.onUp);
                      break;
              }
          }
      };
      PointerEventProvider.prototype.ensureRemoveListener = function (type) {
          if (!this.eventEmitter.listeners(type).length) {
              switch (type) {
                  case "move":
                      this.target.removeEventListener("pointermove", this.onMove);
                      break;
                  case "end":
                      this.target.removeEventListener("pointerup", this.onUp);
                      break;
              }
          }
      };
      return PointerEventProvider;
  }());

  var MouseMoveEventProvider = (function () {
      function MouseMoveEventProvider(target) {
          this.target = target;
      }
      MouseMoveEventProvider.prototype.off = function (type, callback) {
          switch (type) {
              case "move":
                  this.target.removeEventListener("mousemove", callback);
                  break;
              case "end":
                  this.target.removeEventListener("mouseup", callback);
                  break;
          }
          return this;
      };
      MouseMoveEventProvider.prototype.on = function (type, callback) {
          switch (type) {
              case "move":
                  this.target.addEventListener("mousemove", callback);
                  break;
              case "end":
                  this.target.addEventListener("mouseup", callback);
                  break;
          }
          return this;
      };
      return MouseMoveEventProvider;
  }());

  var withSimulatedClick = function (params, type) { return function (_a) {
      var _b;
      var totalDX = _a.totalDX, totalDY = _a.totalDY, rawEvent = _a.e;
      var eventChannel = params.eventChannel, dragThreshold = params.dragThreshold, containerRef = params.containerRef;
      var events = [];
      events.push({
          type: type,
          rawEvent: rawEvent
      });
      if (rawEvent.target instanceof Node &&
          ((_b = containerRef.current) === null || _b === void 0 ? void 0 : _b.contains(rawEvent.target)) &&
          isWithinThreshold(totalDX, totalDY, dragThreshold)) {
          events.push({
              type: exports.GraphCanvasEvent.Click,
              rawEvent: rawEvent
          });
      }
      eventChannel.batch(events);
  }; };
  var dragMultiSelect = function (e, params) {
      var getPositionFromEvent = params.getPositionFromEvent, graphController = params.graphController, eventChannel = params.eventChannel;
      var dragging = new DragController(new MouseMoveEventProvider(graphController.getGlobalEventTarget()), getPositionFromEvent);
      dragging.onMove = function (_a) {
          var dx = _a.dx, dy = _a.dy, rawEvent = _a.e;
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.SelectMove,
              rawEvent: rawEvent,
              dx: dx,
              dy: dy
          });
      };
      dragging.onEnd = withSimulatedClick(params, exports.GraphCanvasEvent.SelectEnd);
      eventChannel.trigger({
          type: exports.GraphCanvasEvent.SelectStart,
          rawEvent: e
      });
      dragging.start(e);
  };
  var dragPan = function (e, params) {
      var getPositionFromEvent = params.getPositionFromEvent, graphController = params.graphController, eventChannel = params.eventChannel;
      var dragging = new DragController(new MouseMoveEventProvider(graphController.getGlobalEventTarget()), getPositionFromEvent);
      dragging.onMove = function (_a) {
          var dx = _a.dx, dy = _a.dy, rawEvent = _a.e;
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.Drag,
              rawEvent: rawEvent,
              dx: dx,
              dy: dy
          });
      };
      dragging.onEnd = withSimulatedClick(params, exports.GraphCanvasEvent.DragEnd);
      dragging.start(e);
      eventChannel.trigger({
          type: exports.GraphCanvasEvent.DragStart,
          rawEvent: e
      });
  };
  var onContainerMouseDown = function (e, params) {
      var _a;
      e.preventDefault();
      e.stopPropagation();
      if (e.button !== MouseEventButton.Primary) {
          return;
      }
      var canvasMouseMode = params.canvasMouseMode, isPanDisabled = params.isPanDisabled, isMultiSelectDisabled = params.isMultiSelectDisabled, state = params.state, isLassoSelectEnable = params.isLassoSelectEnable, graphController = params.graphController;
      var isPanMode = (canvasMouseMode === exports.CanvasMouseMode.pan && !e.ctrlKey && !e.shiftKey && !e.metaKey) || ((_a = state.activeKeys) === null || _a === void 0 ? void 0 : _a.has(" "));
      if (!isPanDisabled && isPanMode) {
          dragPan(e.nativeEvent, params);
      }
      else if (!isMultiSelectDisabled || (isLassoSelectEnable && !e.ctrlKey && !e.metaKey)) {
          dragMultiSelect(e.nativeEvent, params);
      }
      else {
          graphController.canvasClickOnce = true;
      }
  };

  var onNodePointerDown = function (evt, target, params) {
      evt.preventDefault();
      var svgRef = params.svgRef, isNodesDraggable = params.isNodesDraggable, getPositionFromEvent = params.getPositionFromEvent, isClickNodeToSelectDisabled = params.isClickNodeToSelectDisabled, eventChannel = params.eventChannel, dragThreshold = params.dragThreshold, rectRef = params.rectRef, isAutoAlignEnable = params.isAutoAlignEnable, autoAlignThreshold = params.autoAlignThreshold, graphController = params.graphController;
      if (isNodesDraggable) {
          evt.stopPropagation();
      }
      var isMouseRight = evt.pointerType === "mouse" && evt.button !== MouseEventButton.Primary;
      if (isClickNodeToSelectDisabled || isMouseRight) {
          return;
      }
      if (svgRef.current) {
          svgRef.current.focus({ preventScroll: true });
      }
      var isMultiSelect = checkIsMultiSelect(evt);
      var dragging = new DragNodeController(new PointerEventProvider(graphController.getGlobalEventTarget(), evt.pointerId), getPositionFromEvent, rectRef);
      dragging.onMove = function (_a) {
          var dx = _a.dx, dy = _a.dy, totalDX = _a.totalDX, totalDY = _a.totalDY, e = _a.e;
          if (isNodesDraggable) {
              eventChannel.trigger({
                  type: exports.GraphNodeEvent.Drag,
                  node: target,
                  dx: dx,
                  dy: dy,
                  rawEvent: e,
                  isVisible: !isWithinThreshold(totalDX, totalDY, dragThreshold),
                  isAutoAlignEnable: isAutoAlignEnable,
                  autoAlignThreshold: autoAlignThreshold
              });
          }
      };
      dragging.onEnd = function (_a) {
          var totalDX = _a.totalDX, totalDY = _a.totalDY, e = _a.e;
          graphController.pointerId = null;
          var events = [];
          var isDragCanceled = isWithinThreshold(totalDX, totalDY, dragThreshold);
          events.push({
              type: exports.GraphNodeEvent.DragEnd,
              node: target,
              rawEvent: e,
              isDragCanceled: isDragCanceled
          });
          if (isDragCanceled || !isNodesDraggable) {
              graphController.nodeClickOnce = target;
          }
          eventChannel.batch(events);
      };
      graphController.pointerId = evt.pointerId;
      if (evt.target instanceof Element && evt.pointerType !== "mouse") {
          evt.target.releasePointerCapture(evt.pointerId);
      }
      eventChannel.trigger({
          type: exports.GraphNodeEvent.DragStart,
          node: target,
          rawEvent: evt,
          isMultiSelect: isMultiSelect
      });
      dragging.start(evt.nativeEvent);
  };

  var handleBehaviorChange = function (prevBehavior, event) {
      switch (event.type) {
          case exports.GraphNodeEvent.DragStart:
              return exports.GraphBehavior.dragging;
          case exports.GraphEdgeEvent.ConnectStart:
              return exports.GraphBehavior.connecting;
          case exports.GraphCanvasEvent.SelectStart:
              return exports.GraphBehavior.multiSelect;
          case exports.GraphCanvasEvent.DragStart:
              return exports.GraphBehavior.panning;
          case exports.GraphCanvasEvent.DraggingNodeFromItemPanelStart:
              return exports.GraphBehavior.addingNode;
          case exports.GraphNodeEvent.DragEnd:
          case exports.GraphEdgeEvent.ConnectEnd:
          case exports.GraphCanvasEvent.SelectEnd:
          case exports.GraphCanvasEvent.DragEnd:
          case exports.GraphCanvasEvent.DraggingNodeFromItemPanelEnd:
              return exports.GraphBehavior.default;
          default:
              return prevBehavior;
      }
  };

  var VisitPortHelper = function (props) {
      var neighborPorts = props.neighborPorts, data = props.data;
      var selectRef = React__namespace.useRef(null);
      var _a = React__namespace.useState(), selectedItem = _a[0], setSelectedItem = _a[1];
      var onContainerKeyDown = React__namespace.useCallback(function (evt) {
          if (evt.key === "Escape") {
              evt.stopPropagation();
              evt.preventDefault();
              if (selectedItem) {
                  props.onComplete(selectedItem);
              }
          }
      }, [selectedItem, props]);
      var onContainerBlur = React__namespace.useCallback(function (evt) {
      }, []);
      var onContainerChange = React__namespace.useCallback(function (evt) {
          var value = JSON.parse(evt.target.value);
          if (value.nodeId && value.portId) {
              setSelectedItem({ nodeId: value.nodeId, portId: value.portId });
          }
      }, [setSelectedItem]);
      React__namespace.useEffect(function () {
          if (selectRef.current) {
              selectRef.current.focus({ preventScroll: true });
          }
      }, []);
      return (React__namespace.createElement("select", { onKeyDown: onContainerKeyDown, onBlur: onContainerBlur, ref: selectRef, onChange: onContainerChange }, neighborPorts.map(function (s) {
          var isSelected = selectedItem &&
              selectedItem.portId === s.portId &&
              selectedItem.nodeId === s.nodeId;
          var value = JSON.stringify(s);
          var node = data.nodes.get(s.nodeId);
          if (!node) {
              return null;
          }
          var port = node.ports
              ? node.ports.filter(function (p) { return p.id === s.portId; })[0]
              : null;
          if (!port) {
              return null;
          }
          var label = (node.ariaLabel ||
              node.name ||
              node.id) + ": " + (port.ariaLabel || port.name || port.id);
          return (React__namespace.createElement("option", { key: s.nodeId + "-" + s.portId, value: value, "aria-selected": isSelected, "aria-label": label }, label));
      })));
  };

  var item = function (node, port) {
      if (node === void 0) { node = undefined; }
      if (port === void 0) { port = undefined; }
      return ({
          node: node,
          port: port
      });
  };
  var findDOMElement = function (svg, _a) {
      var _b, _c;
      var node = _a.node, port = _a.port;
      var id;
      if (node && port) {
          id = getPortUid((_b = svg.dataset.graphId) !== null && _b !== void 0 ? _b : "", node, port);
      }
      else if (node) {
          id = getNodeUid((_c = svg.dataset.graphId) !== null && _c !== void 0 ? _c : "", node);
      }
      else {
          return null;
      }
      return svg.getElementById(id);
  };
  var focusItem = function (svgRef, nextItem, evt, eventChannel) {
      if (!svgRef.current) {
          return;
      }
      var el = findDOMElement(svgRef.current, nextItem);
      if (el) {
          evt.preventDefault();
          evt.stopPropagation();
          el.focus({ preventScroll: true });
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.Navigate,
              node: nextItem.node,
              port: nextItem.port,
              rawEvent: evt
          });
      }
      else if (!nextItem.node && !nextItem.port) {
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.Navigate,
              node: nextItem.node,
              port: nextItem.port,
              rawEvent: evt
          });
      }
  };
  var getNextItem = function (data, curNode, port) {
      if (curNode.ports) {
          var portIndex = port ? curNode.ports.findIndex(function (p) { return p.id === port.id; }) : -1;
          var nextPortIndex = portIndex + 1;
          if (nextPortIndex < curNode.ports.length) {
              return item(curNode, curNode.ports[nextPortIndex]);
          }
      }
      var nextNode = curNode.next && data.nodes.get(curNode.next);
      if (nextNode) {
          return item(nextNode);
      }
      return item();
  };
  var getPrevItem = function (data, curNode, port) {
      if (port && curNode.ports) {
          var prevPortIndex = curNode.ports.findIndex(function (p) { return p.id === port.id; }) - 1;
          if (prevPortIndex >= 0) {
              return item(curNode, curNode.ports[prevPortIndex]);
          }
          return item(curNode);
      }
      var prevNode = curNode.prev && data.nodes.get(curNode.prev);
      if (prevNode) {
          return item(prevNode, prevNode.ports && prevNode.ports.length ? prevNode.ports[prevNode.ports.length - 1] : undefined);
      }
      return item();
  };
  var focusNextPort = function (ports, node, curPortId, svgRef, evt, eventChannel) {
      var curPortIndex = ports.findIndex(function (p) { return p.id === curPortId; });
      var nextPortIndex = (curPortIndex + 1) % ports.length;
      var nextPort = ports[nextPortIndex];
      if (nextPort && svgRef.current) {
          focusItem(svgRef, { node: node, port: nextPort }, evt, eventChannel);
      }
  };
  var focusPrevPort = function (ports, node, curPortId, svgRef, evt, eventChannel) {
      var curPortIndex = ports.findIndex(function (p) { return p.id === curPortId; });
      var prevPortIndex = (curPortIndex - 1 + ports.length) % ports.length;
      var prevPort = ports[prevPortIndex];
      if (prevPort && svgRef.current) {
          focusItem(svgRef, { node: node, port: prevPort }, evt, eventChannel);
      }
  };
  var getFocusNodeHandler = function (compareFn) { return function (data, curNodeId, svgRef, graphController, evt, eventChannel) {
      var sortedNodes = Array.from(data.nodes.values()).sort(compareFn);
      var curNodeIndex = sortedNodes.findIndex(function (n) { return n.id === curNodeId; });
      var nextNode = sortedNodes[(curNodeIndex + 1) % sortedNodes.length];
      if (nextNode && svgRef.current) {
          graphController.dispatch({
              type: exports.GraphNodeEvent.Select,
              nodes: [nextNode.id]
          });
          graphController.dispatch({
              type: exports.GraphNodeEvent.Centralize,
              nodes: [nextNode.id]
          });
          focusItem(svgRef, { node: nextNode, port: undefined }, evt, eventChannel);
      }
  }; };
  var focusLeftNode = getFocusNodeHandler(function (n1, n2) { return n1.x * 10 + n1.y - n2.x * 10 - n2.y; });
  var focusRightNode = getFocusNodeHandler(function (n1, n2) { return n2.x * 10 + n2.y - n1.x * 10 - n1.y; });
  var focusDownNode = getFocusNodeHandler(function (n1, n2) { return n1.x + n1.y * 10 - n2.x - n2.y * 10; });
  var focusUpNode = getFocusNodeHandler(function (n1, n2) { return n2.x + n2.y * 10 - n1.x - n1.y * 10; });
  var goToConnectedPort = function (data, node, port, svgRef, evt, eventChannel) {
      var _a;
      var neighborPorts = getNeighborPorts(data, node.id, port.id);
      if (neighborPorts.length === 1 && svgRef.current) {
          var targetNode = data.nodes.get(neighborPorts[0].nodeId);
          if (!targetNode) {
              return;
          }
          var targetPort = (_a = targetNode.ports) === null || _a === void 0 ? void 0 : _a.find(function (p) { return p.id === neighborPorts[0].portId; });
          if (!targetPort) {
              return;
          }
          focusItem(svgRef, { node: targetNode, port: targetPort }, evt, eventChannel);
      }
      else if (neighborPorts.length > 1 && svgRef.current) {
          var onComplete = function (nextPort) {
              var _a;
              ReactDOM__namespace.unmountComponentAtNode(visitPortHelperContainer_1);
              if (svgRef.current) {
                  var curEditorContainer = svgRef.current.closest(".react-dag-editor-container");
                  if (curEditorContainer) {
                      curEditorContainer.removeChild(visitPortHelperContainer_1);
                  }
              }
              var targetNode = data.nodes.get(nextPort.nodeId);
              if (!targetNode) {
                  return;
              }
              var targetPort = (_a = targetNode.ports) === null || _a === void 0 ? void 0 : _a.find(function (p) { return p.id === nextPort.portId; });
              if (!targetPort) {
                  return;
              }
              focusItem(svgRef, { node: targetNode, port: targetPort }, evt, eventChannel);
          };
          var visitPortHelperContainer_1 = document.createElement("div");
          var editorContainer = svgRef.current.closest(".react-dag-editor-container");
          if (editorContainer) {
              editorContainer.appendChild(visitPortHelperContainer_1);
          }
          visitPortHelperContainer_1.style.position = "fixed";
          visitPortHelperContainer_1.style.top = "0";
          ReactDOM__namespace.render(React__namespace.createElement(VisitPortHelper, { neighborPorts: neighborPorts, onComplete: onComplete, data: data }), visitPortHelperContainer_1);
      }
  };
  function defaultGetPortAriaLabel(data, node, port) {
      return port.ariaLabel;
  }
  function defaultGetNodeAriaLabel(node) {
      return node.ariaLabel;
  }

  var useCanvasKeyboardEventHandlers = function (args) {
      var featureControl = args.featureControl, graphConfig = args.graphConfig, setCurHoverNode = args.setCurHoverNode, setCurHoverPort = args.setCurHoverPort, eventChannel = args.eventChannel;
      var isDeleteDisabled = featureControl.isDeleteDisabled, isPasteDisabled = featureControl.isPasteDisabled, isUndoEnabled = featureControl.isUndoEnabled;
      return React__namespace.useMemo(function () {
          var keyDownHandlerMap = new Map();
          var deleteKeyDownHandler = function () { return function (evt) {
              evt.preventDefault();
              evt.stopPropagation();
              if (isDeleteDisabled) {
                  return;
              }
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.Delete
              });
              setCurHoverNode(undefined);
              setCurHoverPort(undefined);
          }; };
          keyDownHandlerMap.set("delete", deleteKeyDownHandler());
          keyDownHandlerMap.set("backspace", deleteKeyDownHandler());
          var cKeyDownHandler = function (evt) {
              if (metaControl(evt)) {
                  evt.preventDefault();
                  evt.stopPropagation();
                  eventChannel.trigger({
                      type: exports.GraphCanvasEvent.Copy
                  });
              }
          };
          keyDownHandlerMap.set("c", cKeyDownHandler);
          var vKeyDownHandler = function (evt) {
              if (metaControl(evt)) {
                  evt.preventDefault();
                  evt.stopPropagation();
                  if (isPasteDisabled) {
                      return;
                  }
                  var data = graphConfig.getClipboard().read();
                  if (data) {
                      eventChannel.trigger({
                          type: exports.GraphCanvasEvent.Paste,
                          data: data
                      });
                  }
              }
          };
          keyDownHandlerMap.set("v", vKeyDownHandler);
          var zKeyDownHandler = function (evt) {
              if (metaControl(evt)) {
                  evt.preventDefault();
                  evt.stopPropagation();
                  if (isUndoEnabled) {
                      eventChannel.trigger({
                          type: exports.GraphCanvasEvent.Undo
                      });
                  }
              }
          };
          keyDownHandlerMap.set("z", zKeyDownHandler);
          var yKeyDownHandler = function (evt) {
              if (metaControl(evt)) {
                  evt.preventDefault();
                  evt.stopPropagation();
                  if (isUndoEnabled) {
                      eventChannel.trigger({
                          type: exports.GraphCanvasEvent.Redo
                      });
                  }
              }
          };
          keyDownHandlerMap.set("y", yKeyDownHandler);
          var aKeyDownHandler = function (evt) {
              if (metaControl(evt)) {
                  evt.preventDefault();
                  evt.stopPropagation();
                  eventChannel.trigger({
                      type: exports.GraphNodeEvent.SelectAll
                  });
              }
          };
          keyDownHandlerMap.set("a", aKeyDownHandler);
          var spaceKeyDownHandler = function (evt) {
              evt.preventDefault();
              evt.stopPropagation();
          };
          var ctrlKeyDownHandler = function (evt) {
              evt.preventDefault();
              evt.stopPropagation();
          };
          var metaKeyDownHandler = function (evt) {
              evt.preventDefault();
              evt.stopPropagation();
          };
          var shiftKeyDownHandler = function (evt) {
              evt.preventDefault();
              evt.stopPropagation();
          };
          keyDownHandlerMap.set(" ", spaceKeyDownHandler);
          keyDownHandlerMap.set("control", ctrlKeyDownHandler);
          keyDownHandlerMap.set("meta", metaKeyDownHandler);
          keyDownHandlerMap.set("shift", shiftKeyDownHandler);
          return function (evt) {
              if (evt.repeat) {
                  return;
              }
              var key = evt.key.toLowerCase();
              var handler = keyDownHandlerMap.get(key);
              if (handler) {
                  handler.call(null, evt);
              }
          };
      }, [eventChannel, graphConfig, isDeleteDisabled, isPasteDisabled, isUndoEnabled, setCurHoverNode, setCurHoverPort]);
  };

  var prevMouseDownPortId;
  var prevMouseDownPortTime;
  function useEventChannel(_a) {
      var props = _a.props, dispatch = _a.dispatch, rectRef = _a.rectRef, svgRef = _a.svgRef, containerRef = _a.containerRef, featureControl = _a.featureControl, graphConfig = _a.graphConfig, setFocusedWithoutMouse = _a.setFocusedWithoutMouse, setCurHoverNode = _a.setCurHoverNode, setCurHoverPort = _a.setCurHoverPort, eventChannel = _a.eventChannel, updateViewport = _a.updateViewport, graphController = _a.graphController;
      var _b = props.dragThreshold, dragThreshold = _b === void 0 ? 10 : _b, _c = props.autoAlignThreshold, autoAlignThreshold = _c === void 0 ? DEFAULT_AUTO_ALIGN_THRESHOLD : _c, _d = props.getPositionFromEvent, getPositionFromEvent = _d === void 0 ? defaultGetPositionFromEvent : _d, canvasMouseMode = props.canvasMouseMode, edgeWillAdd = props.edgeWillAdd;
      var isNodesDraggable = featureControl.isNodesDraggable, isAutoAlignEnable = featureControl.isAutoAlignEnable, isClickNodeToSelectDisabled = featureControl.isClickNodeToSelectDisabled, isPanDisabled = featureControl.isPanDisabled, isMultiSelectDisabled = featureControl.isMultiSelectDisabled, isLassoSelectEnable = featureControl.isLassoSelectEnable, isConnectDisabled = featureControl.isConnectDisabled, isPortHoverViewEnable = featureControl.isPortHoverViewEnable, isNodeEditDisabled = featureControl.isNodeEditDisabled, isA11yEnable = featureControl.isA11yEnable;
      var animationFramedDispatch = React__namespace.useMemo(function () { return animationFramed(dispatch); }, [dispatch]);
      var keyDownHandler = useCanvasKeyboardEventHandlers({
          featureControl: featureControl,
          eventChannel: eventChannel,
          graphConfig: graphConfig,
          setCurHoverNode: setCurHoverNode,
          setCurHoverPort: setCurHoverPort
      });
      var focusFirstNode = function (e) {
          var data = graphController.getData();
          if (data.nodes.size > 0 && svgRef.current) {
              var firstNode = data.head && data.nodes.get(data.head);
              if (firstNode) {
                  focusItem(svgRef, { node: firstNode, port: undefined }, e, eventChannel);
              }
          }
      };
      var handleEdgeEvent = function (event) {
          switch (event.type) {
              case exports.GraphEdgeEvent.ConnectStart:
              case exports.GraphEdgeEvent.ConnectMove:
              case exports.GraphEdgeEvent.ConnectEnd:
              case exports.GraphEdgeEvent.ConnectNavigate:
              case exports.GraphEdgeEvent.Click:
              case exports.GraphEdgeEvent.MouseEnter:
              case exports.GraphEdgeEvent.MouseLeave:
              case exports.GraphEdgeEvent.DoubleClick:
                  dispatch(event);
                  break;
              case exports.GraphEdgeEvent.ContextMenu:
                  event.rawEvent.stopPropagation();
                  event.rawEvent.preventDefault();
                  dispatch(event);
                  break;
          }
      };
      var handleCanvasEvent = function (event) {
          var _a, _b;
          switch (event.type) {
              case exports.GraphCanvasEvent.ViewportResize:
              case exports.GraphCanvasEvent.Drag:
              case exports.GraphCanvasEvent.MouseWheelScroll:
              case exports.GraphCanvasEvent.Zoom:
              case exports.GraphCanvasEvent.Pinch:
              case exports.GraphCanvasEvent.Click:
              case exports.GraphCanvasEvent.SelectStart:
              case exports.GraphCanvasEvent.SelectMove:
              case exports.GraphCanvasEvent.SelectEnd:
              case exports.GraphCanvasEvent.ResetSelection:
              case exports.GraphCanvasEvent.Navigate:
              case exports.GraphCanvasEvent.Paste:
              case exports.GraphCanvasEvent.Undo:
              case exports.GraphCanvasEvent.Redo:
              case exports.GraphCanvasEvent.Delete:
              case exports.GraphCanvasEvent.KeyUp:
              case exports.GraphCanvasEvent.DraggingNodeFromItemPanelStart:
              case exports.GraphCanvasEvent.DraggingNodeFromItemPanel:
              case exports.GraphCanvasEvent.DraggingNodeFromItemPanelEnd:
                  dispatch(event);
                  break;
              case exports.GraphCanvasEvent.Copy:
                  {
                      var selectedData = filterSelectedItems(graphController.getData());
                      var clipboard = graphConfig.getClipboard();
                      clipboard.write(selectedData);
                  }
                  break;
              case exports.GraphCanvasEvent.KeyDown:
                  if (!event.rawEvent.repeat &&
                      event.rawEvent.target === event.rawEvent.currentTarget &&
                      !event.rawEvent.shiftKey &&
                      event.rawEvent.key === "Tab") {
                      event.rawEvent.preventDefault();
                      event.rawEvent.stopPropagation();
                      setFocusedWithoutMouse(true);
                      focusFirstNode(event.rawEvent);
                  }
                  else {
                      keyDownHandler(event.rawEvent);
                  }
                  dispatch(event);
                  break;
              case exports.GraphCanvasEvent.MouseDown:
                  {
                      graphController.nodeClickOnce = null;
                      (_a = svgRef.current) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true });
                      setFocusedWithoutMouse(false);
                      var evt = event.rawEvent;
                      updateViewport();
                      onContainerMouseDown(evt, {
                          state: graphController.state,
                          canvasMouseMode: canvasMouseMode,
                          isPanDisabled: isPanDisabled,
                          isMultiSelectDisabled: isMultiSelectDisabled,
                          isLassoSelectEnable: isLassoSelectEnable,
                          dragThreshold: dragThreshold,
                          containerRef: containerRef,
                          getPositionFromEvent: defaultGetPositionFromEvent,
                          eventChannel: eventChannel,
                          graphController: graphController
                      });
                  }
                  break;
              case exports.GraphCanvasEvent.MouseUp:
                  if (graphController.canvasClickOnce) {
                      graphController.canvasClickOnce = false;
                      var evt = event.rawEvent;
                      if (evt.target instanceof Node && ((_b = svgRef.current) === null || _b === void 0 ? void 0 : _b.contains(evt.target)) && evt.target.nodeName === "svg") {
                          eventChannel.trigger({
                              type: exports.GraphCanvasEvent.Click,
                              rawEvent: event.rawEvent
                          });
                      }
                  }
                  break;
              case exports.GraphCanvasEvent.ContextMenu:
                  event.rawEvent.preventDefault();
                  event.rawEvent.stopPropagation();
                  dispatch(event);
                  break;
              case exports.GraphCanvasEvent.MouseMove:
                  {
                      var evt = event.rawEvent;
                      graphController.setMouseClientPosition({
                          x: evt.clientX,
                          y: evt.clientY
                      });
                  }
                  break;
              case exports.GraphCanvasEvent.MouseLeave:
                  graphController.unsetMouseClientPosition();
                  graphController.canvasClickOnce = false;
                  break;
              case exports.GraphCanvasEvent.Blur:
                  setFocusedWithoutMouse(false);
                  break;
          }
      };
      var onNodePointerEnter = function (event) {
          var node = event.node;
          var isNodeHoverViewEnabled = featureControl.isNodeHoverViewEnabled;
          var behavior = graphController.getBehavior();
          switch (behavior) {
              case exports.GraphBehavior.connecting:
              case exports.GraphBehavior.default:
                  if (isNodeHoverViewEnabled) {
                      setCurHoverNode(node.id);
                      setCurHoverPort(undefined);
                  }
                  break;
          }
          dispatch(event);
      };
      var onNodePointerLeave = function (event) {
          dispatch(event);
          setCurHoverNode(undefined);
      };
      var onNodeDoubleClick = function (event) {
          if (isNodeEditDisabled) {
              return;
          }
          event.rawEvent.stopPropagation();
          dispatch(event);
      };
      var onNodeKeyDown = function (event) {
          if (!svgRef || !isA11yEnable) {
              return;
          }
          var data = graphController.getData();
          var node = event.node;
          var evt = event.rawEvent;
          switch (evt.key) {
              case "Tab":
                  {
                      evt.preventDefault();
                      evt.stopPropagation();
                      var nextItem = evt.shiftKey ? getPrevItem(data, node) : getNextItem(data, node);
                      focusItem(svgRef, nextItem, evt, eventChannel);
                  }
                  break;
              case "ArrowUp":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusUpNode(data, node.id, svgRef, graphController, evt, eventChannel);
                  break;
              case "ArrowDown":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusDownNode(data, node.id, svgRef, graphController, evt, eventChannel);
                  break;
              case "ArrowLeft":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusLeftNode(data, node.id, svgRef, graphController, evt, eventChannel);
                  break;
              case "ArrowRight":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusRightNode(data, node.id, svgRef, graphController, evt, eventChannel);
                  break;
          }
      };
      var handleNodeEvent = function (event) {
          var _a;
          switch (event.type) {
              case exports.GraphNodeEvent.ResizingStart:
              case exports.GraphNodeEvent.Resizing:
              case exports.GraphNodeEvent.ResizingEnd:
              case exports.GraphNodeEvent.DragStart:
              case exports.GraphNodeEvent.Drag:
              case exports.GraphNodeEvent.DragEnd:
              case exports.GraphNodeEvent.SelectAll:
                  dispatch(event);
                  break;
              case exports.GraphNodeEvent.PointerMove:
                  if (event.rawEvent.pointerId === graphController.pointerId) {
                      animationFramedDispatch(event);
                  }
                  break;
              case exports.GraphNodeEvent.PointerDown:
                  {
                      graphController.nodeClickOnce = null;
                      if (graphController.getBehavior() !== exports.GraphBehavior.default) {
                          return;
                      }
                      var evt = event.rawEvent;
                      updateViewport();
                      onNodePointerDown(evt, event.node, {
                          svgRef: svgRef,
                          rectRef: rectRef,
                          isNodesDraggable: isNodesDraggable,
                          isAutoAlignEnable: isAutoAlignEnable,
                          dragThreshold: dragThreshold,
                          getPositionFromEvent: getPositionFromEvent,
                          isClickNodeToSelectDisabled: isClickNodeToSelectDisabled,
                          autoAlignThreshold: autoAlignThreshold,
                          eventChannel: eventChannel,
                          graphController: graphController
                      });
                  }
                  break;
              case exports.GraphNodeEvent.PointerEnter:
                  onNodePointerEnter(event);
                  break;
              case exports.GraphNodeEvent.PointerLeave:
                  onNodePointerLeave(event);
                  break;
              case exports.GraphNodeEvent.MouseDown:
                  graphController.nodeClickOnce = null;
                  event.rawEvent.preventDefault();
                  if (isNodesDraggable) {
                      event.rawEvent.stopPropagation();
                  }
                  setFocusedWithoutMouse(false);
                  break;
              case exports.GraphNodeEvent.Click:
                  if (((_a = graphController.nodeClickOnce) === null || _a === void 0 ? void 0 : _a.id) === event.node.id) {
                      var currentTarget = event.rawEvent.currentTarget;
                      if (currentTarget instanceof SVGElement) {
                          currentTarget.focus({ preventScroll: true });
                      }
                      event.node = graphController.nodeClickOnce;
                      dispatch(event);
                      graphController.nodeClickOnce = null;
                  }
                  else {
                      event.intercepted = true;
                  }
                  break;
              case exports.GraphNodeEvent.ContextMenu:
                  event.rawEvent.preventDefault();
                  event.rawEvent.stopPropagation();
                  dispatch(event);
                  break;
              case exports.GraphNodeEvent.DoubleClick:
                  onNodeDoubleClick(event);
                  break;
              case exports.GraphNodeEvent.KeyDown:
                  onNodeKeyDown(event);
                  break;
          }
      };
      var onPortPointerDown = React__namespace.useCallback(function (event) {
          var evt = event.rawEvent;
          var node = event.node, port = event.port;
          setFocusedWithoutMouse(false);
          evt.stopPropagation();
          evt.preventDefault();
          prevMouseDownPortId = node.id + ":" + port.id;
          prevMouseDownPortTime = performance.now();
          if (isConnectDisabled || (evt.pointerType === "mouse" && evt.button !== MouseEventButton.Primary)) {
              return;
          }
          updateViewport();
          var globalEventTarget = graphController.getGlobalEventTarget();
          var dragging = new DragController(new PointerEventProvider(globalEventTarget, evt.pointerId), getPositionFromEvent);
          dragging.onMove = function (_a) {
              var clientX = _a.clientX, clientY = _a.clientY, e = _a.e;
              eventChannel.trigger({
                  type: exports.GraphEdgeEvent.ConnectMove,
                  rawEvent: e,
                  clientX: clientX,
                  clientY: clientY
              });
          };
          dragging.onEnd = function (_a) {
              var e = _a.e;
              eventChannel.trigger({
                  type: exports.GraphEdgeEvent.ConnectEnd,
                  rawEvent: e,
                  edgeWillAdd: edgeWillAdd,
                  isCancel: false
              });
              graphController.pointerId = null;
          };
          eventChannel.trigger({
              type: exports.GraphEdgeEvent.ConnectStart,
              nodeId: node.id,
              portId: port.id,
              rawEvent: evt,
              clientPoint: {
                  x: evt.clientX,
                  y: evt.clientY
              }
          });
          if (evt.target instanceof Element && evt.pointerType !== "mouse") {
              evt.target.releasePointerCapture(evt.pointerId);
          }
          graphController.pointerId = evt.pointerId;
          dragging.start(evt.nativeEvent);
      }, [
          edgeWillAdd,
          eventChannel,
          getPositionFromEvent,
          graphController,
          isConnectDisabled,
          setFocusedWithoutMouse,
          updateViewport
      ]);
      var onPortPointerUp = React__namespace.useCallback(function (event) {
          var evt = event.rawEvent;
          var node = event.node, port = event.port;
          if (prevMouseDownPortId === node.id + ":" + port.id && performance.now() - (prevMouseDownPortTime || 0) < 500) {
              prevMouseDownPortId = undefined;
              prevMouseDownPortTime = undefined;
              eventChannel.trigger({
                  type: exports.GraphPortEvent.Click,
                  node: node,
                  port: port,
                  rawEvent: evt
              });
          }
      }, [eventChannel]);
      var onPortPointerEnter = function (event) {
          var behavior = graphController.getBehavior();
          switch (behavior) {
              case exports.GraphBehavior.default:
                  setCurHoverPort([event.node.id, event.port.id]);
                  break;
          }
          if (isPortHoverViewEnable) {
              setCurHoverPort([event.node.id, event.port.id]);
          }
          if (event.rawEvent.pointerId === graphController.pointerId) {
              dispatch(event);
          }
      };
      var onPortPointerLeave = function (event) {
          setCurHoverPort(undefined);
          dispatch(event);
      };
      var onPortKeyDown = function (event) {
          var _a, _b, _c;
          if (!isA11yEnable) {
              return;
          }
          var evt = event.rawEvent;
          if (evt.altKey && (evt.nativeEvent.code === "KeyC" || evt.key === "c")) {
              evt.preventDefault();
              evt.stopPropagation();
              eventChannel.trigger({
                  type: exports.GraphEdgeEvent.ConnectStart,
                  nodeId: event.node.id,
                  portId: event.port.id,
                  rawEvent: evt
              });
              return;
          }
          var data = graphController.getData();
          var node = event.node, port = event.port;
          switch (evt.key) {
              case "Tab":
                  if (isA11yEnable && graphController.getBehavior() === exports.GraphBehavior.connecting) {
                      evt.preventDefault();
                      evt.stopPropagation();
                      eventChannel.trigger({
                          type: exports.GraphEdgeEvent.ConnectNavigate,
                          rawEvent: evt
                      });
                  }
                  else {
                      var nextItem = evt.shiftKey ? getPrevItem(data, node, port) : getNextItem(data, node, port);
                      focusItem(svgRef, nextItem, evt, eventChannel);
                  }
                  break;
              case "ArrowUp":
              case "ArrowLeft":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusPrevPort((_a = node.ports) !== null && _a !== void 0 ? _a : [], node, port.id, svgRef, evt, eventChannel);
                  break;
              case "ArrowDown":
              case "ArrowRight":
                  evt.preventDefault();
                  evt.stopPropagation();
                  focusNextPort((_b = node.ports) !== null && _b !== void 0 ? _b : [], node, port.id, svgRef, evt, eventChannel);
                  break;
              case "g":
                  evt.preventDefault();
                  evt.stopPropagation();
                  goToConnectedPort(data, node, port, svgRef, evt, eventChannel);
                  break;
              case "Escape":
                  if (graphController.getBehavior() === exports.GraphBehavior.connecting) {
                      evt.preventDefault();
                      evt.stopPropagation();
                      if (svgRef.current) {
                          (_c = findDOMElement(svgRef.current, { node: node, port: port })) === null || _c === void 0 ? void 0 : _c.blur();
                      }
                  }
                  break;
              case "Enter":
                  evt.preventDefault();
                  evt.stopPropagation();
                  eventChannel.trigger({
                      type: exports.GraphEdgeEvent.ConnectEnd,
                      rawEvent: evt.nativeEvent,
                      edgeWillAdd: edgeWillAdd,
                      isCancel: false
                  });
                  break;
          }
      };
      var handlePortEvent = function (event) {
          switch (event.type) {
              case exports.GraphPortEvent.Click:
                  dispatch(event);
                  break;
              case exports.GraphPortEvent.PointerDown:
                  onPortPointerDown(event);
                  break;
              case exports.GraphPortEvent.PointerUp:
                  onPortPointerUp(event);
                  break;
              case exports.GraphPortEvent.PointerEnter:
                  onPortPointerEnter(event);
                  break;
              case exports.GraphPortEvent.PointerLeave:
                  onPortPointerLeave(event);
                  break;
              case exports.GraphPortEvent.ContextMenu:
                  event.rawEvent.preventDefault();
                  event.rawEvent.stopPropagation();
                  dispatch(event);
                  break;
              case exports.GraphPortEvent.Focus:
                  event.rawEvent.stopPropagation();
                  dispatch(event);
                  break;
              case exports.GraphPortEvent.Blur:
                  if (graphController.getBehavior() === exports.GraphBehavior.connecting) {
                      eventChannel.trigger({
                          type: exports.GraphEdgeEvent.ConnectEnd,
                          rawEvent: event.rawEvent.nativeEvent,
                          edgeWillAdd: edgeWillAdd,
                          isCancel: true
                      });
                  }
                  break;
              case exports.GraphPortEvent.KeyDown:
                  onPortKeyDown(event);
                  break;
          }
      };
      var handleEvent = function (event) {
          var behavior = handleBehaviorChange(graphController.getBehavior(), event);
          graphController.setBehavior(behavior);
          handleEdgeEvent(event);
          handleCanvasEvent(event);
          handleNodeEvent(event);
          handlePortEvent(event);
          switch (event.type) {
              case exports.GraphMinimapEvent.Pan:
              case exports.GraphScrollBarEvent.Scroll:
              case exports.GraphContextMenuEvent.Open:
              case exports.GraphContextMenuEvent.Close:
                  dispatch(event);
                  break;
          }
      };
      React__namespace.useImperativeHandle(eventChannel.listenersRef, function () { return handleEvent; });
      React__namespace.useImperativeHandle(eventChannel.externalHandlerRef, function () { return props.onEvent; });
  }

  var useFeatureControl = function (features) {
      return React__namespace.useMemo(function () {
          var isNodesDraggable = features.has(exports.GraphFeatures.nodeDraggable);
          var isNodeResizable = features.has(exports.GraphFeatures.nodeResizable);
          var isAutoFitDisabled = !features.has(exports.GraphFeatures.autoFit);
          var isPanDisabled = !features.has(exports.GraphFeatures.panCanvas);
          var isMultiSelectDisabled = !features.has(exports.GraphFeatures.multipleSelect);
          var isLassoSelectEnable = features.has(exports.GraphFeatures.lassoSelect);
          var isNodeHoverViewEnabled = features.has(exports.GraphFeatures.nodeHoverView);
          var isClickNodeToSelectDisabled = !features.has(exports.GraphFeatures.clickNodeToSelect);
          var isConnectDisabled = !features.has(exports.GraphFeatures.addNewEdges);
          var isPortHoverViewEnable = features.has(exports.GraphFeatures.portHoverView);
          var isNodeEditDisabled = !features.has(exports.GraphFeatures.editNode);
          var isVerticalScrollDisabled = !features.has(exports.GraphFeatures.canvasVerticalScrollable);
          var isHorizontalScrollDisabled = !features.has(exports.GraphFeatures.canvasHorizontalScrollable);
          var isA11yEnable = features.has(exports.GraphFeatures.a11yFeatures);
          var isAutoAlignEnable = features.has(exports.GraphFeatures.autoAlign);
          var isCtrlKeyZoomEnable = features.has(exports.GraphFeatures.ctrlKeyZoom);
          var isLimitBoundary = features.has(exports.GraphFeatures.limitBoundary);
          var isVirtualizationEnabled = !features.has(exports.GraphFeatures.autoFit);
          var isEdgeEditable = features.has(exports.GraphFeatures.editEdge);
          var isDeleteDisabled = !features.has(exports.GraphFeatures.delete);
          var isPasteDisabled = !features.has(exports.GraphFeatures.addNewNodes) || !features.has(exports.GraphFeatures.addNewEdges);
          var isUndoEnabled = features.has(exports.GraphFeatures.undoStack);
          return {
              isNodesDraggable: isNodesDraggable,
              isNodeResizable: isNodeResizable,
              isAutoFitDisabled: isAutoFitDisabled,
              isPanDisabled: isPanDisabled,
              isMultiSelectDisabled: isMultiSelectDisabled,
              isLassoSelectEnable: isLassoSelectEnable,
              isNodeHoverViewEnabled: isNodeHoverViewEnabled,
              isClickNodeToSelectDisabled: isClickNodeToSelectDisabled,
              isConnectDisabled: isConnectDisabled,
              isPortHoverViewEnable: isPortHoverViewEnable,
              isNodeEditDisabled: isNodeEditDisabled,
              isVerticalScrollDisabled: isVerticalScrollDisabled,
              isHorizontalScrollDisabled: isHorizontalScrollDisabled,
              isA11yEnable: isA11yEnable,
              isAutoAlignEnable: isAutoAlignEnable,
              isCtrlKeyZoomEnable: isCtrlKeyZoomEnable,
              isLimitBoundary: isLimitBoundary,
              isVirtualizationEnabled: isVirtualizationEnabled,
              isEdgeEditable: isEdgeEditable,
              isDeleteDisabled: isDeleteDisabled,
              isPasteDisabled: isPasteDisabled,
              isUndoEnabled: isUndoEnabled
          };
      }, [features]);
  };

  var emptyLine = function () { return ({
      x1: 0,
      y1: 0,
      x2: 0,
      y2: 0,
      visible: false
  }); };
  var Line = function (props) {
      var _a;
      var line = props.line, style = props.style;
      var lineStyle = __assign(__assign({ strokeWidth: 1 }, style), { stroke: line.visible ? (_a = style === null || style === void 0 ? void 0 : style.stroke) !== null && _a !== void 0 ? _a : "#ea4300" : "none" });
      return (React__namespace.createElement("line", { className: "auto-align-hint", x1: line.x1, y1: line.y1, x2: line.x2, y2: line.y2, style: lineStyle }));
  };

  var AlignmentLines = React__namespace.memo(function (_a) {
      var style = _a.style;
      var alignmentLines = useAlignmentLines();
      return (React__namespace.createElement(React__namespace.Fragment, null, alignmentLines.map(function (l, index) {
          if (l.visible) {
              return React__namespace.createElement(Line, { key: index, line: l, style: style });
          }
          return null;
      })));
  });
  AlignmentLines.displayName = "AlignmentLines";

  var AnimatingNodeGroup = function (props) {
      var _a, _b;
      var dummyNodes = props.dummyNodes, graphData = props.graphData;
      var graphConfig = useGraphConfig();
      var theme = useTheme().theme;
      var dWidth = dummyNodes.dWidth, dHeight = dummyNodes.dHeight;
      var dx = (_a = dummyNodes.alignedDX) !== null && _a !== void 0 ? _a : dummyNodes.dx;
      var dy = (_b = dummyNodes.alignedDY) !== null && _b !== void 0 ? _b : dummyNodes.dy;
      return (React__namespace.createElement("g", null, dummyNodes.nodes.map(function (dummyNode) {
          var node = graphData.nodes.get(dummyNode.id);
          if (!node) {
              return null;
          }
          var x = dummyNode.x + dx;
          var y = dummyNode.y + dy;
          var width = dummyNode.width + dWidth;
          var height = dummyNode.height + dHeight;
          var nodeConfig = getNodeConfig(node, graphConfig);
          if (nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.renderDummy) {
              return nodeConfig.renderDummy(__assign(__assign({}, node.inner), { x: x,
                  y: y,
                  width: width,
                  height: height }), theme);
          }
          return (React__namespace.createElement("rect", { key: node.id, transform: "translate(" + x + "," + y + ")", height: height, width: width, stroke: theme.dummyNodeStroke, strokeDasharray: "4", fill: "none" }));
      })));
  };

  var ConnectingLine = function (props) {
      var autoAttachLine = props.autoAttachLine, connectingLine = props.connectingLine, styles = props.styles;
      var theme = useTheme().theme;
      var strokeColor = (styles === null || styles === void 0 ? void 0 : styles.stroke) || theme.primaryColor;
      var fill = (styles === null || styles === void 0 ? void 0 : styles.fill) || "none";
      var strokeDasharray = (styles === null || styles === void 0 ? void 0 : styles.strokeDasharray) || "4,4";
      var connectingLineStroke = connectingLine.visible ? strokeColor : "none";
      return (React__namespace.createElement("g", null,
          React__namespace.createElement("defs", null,
              React__namespace.createElement("marker", { id: "markerArrow", markerWidth: "10", markerHeight: "10", refX: "6", refY: "5", orient: "auto", markerUnits: "strokeWidth" },
                  React__namespace.createElement("path", { d: "M0,0 L6,5 L0,10", style: { stroke: connectingLineStroke, fill: "none" } }))),
          React__namespace.createElement("line", { x1: connectingLine.x1, y1: connectingLine.y1, x2: connectingLine.x2, y2: connectingLine.y2, style: { stroke: connectingLineStroke, fill: fill, strokeDasharray: strokeDasharray }, markerEnd: "url(#markerArrow)" }),
          React__namespace.createElement("path", { d: getCurvePathD(autoAttachLine.x2, autoAttachLine.x1, autoAttachLine.y2, autoAttachLine.y1), style: {
                  stroke: autoAttachLine.visible ? strokeColor : "none",
                  fill: "none"
              } })));
  };

  var Connecting = React__namespace.memo(function (props) {
      var styles = props.styles, graphConfig = props.graphConfig, viewport = props.viewport, movingPoint = props.movingPoint;
      var _a = useConnectingState(), sourcePort = _a.sourcePort, sourceNode = _a.sourceNode, targetPort = _a.targetPort, targetNode = _a.targetNode;
      if (!sourceNode || !sourcePort) {
          return null;
      }
      var sourcePortPosition = sourceNode.getPortPosition(sourcePort.id, graphConfig);
      var targetPortPosition;
      var isAttach = false;
      if (targetNode && targetPort) {
          isAttach = true;
          targetPortPosition = targetNode === null || targetNode === void 0 ? void 0 : targetNode.getPortPosition(targetPort.id, graphConfig);
      }
      else {
          targetPortPosition = sourcePortPosition;
      }
      if (!sourcePortPosition || !targetPortPosition) {
          return null;
      }
      var sourcePoint = transformPoint(sourcePortPosition.x, sourcePortPosition.y, viewport.transformMatrix);
      var targetPoint = transformPoint(targetPortPosition.x, targetPortPosition.y, viewport.transformMatrix);
      var connectingLine = movingPoint
          ? {
              x1: sourcePoint.x,
              y1: sourcePoint.y,
              x2: movingPoint.x,
              y2: movingPoint.y,
              visible: !isAttach
          }
          : emptyLine();
      var autoAttachLine = {
          x1: sourcePoint.x,
          y1: sourcePoint.y,
          x2: targetPoint.x,
          y2: targetPoint.y,
          visible: isAttach
      };
      return React__namespace.createElement(ConnectingLine, { connectingLine: connectingLine, autoAttachLine: autoAttachLine, styles: styles });
  });
  Connecting.displayName = "Connecting";

  var defaultStyle = {
      position: "fixed",
      userSelect: "none"
  };
  var GraphContextMenu = function (_a) {
      var _b, _c;
      var state = _a.state, onClick = _a.onClick;
      var ref = React__namespace.useRef(null);
      var _d = React__namespace.useState(__assign({}, defaultStyle)), style = _d[0], setStyle = _d[1];
      React__namespace.useLayoutEffect(function () {
          var el = ref.current;
          if (!el || !state.contextMenuPosition) {
              return;
          }
          var _a = state.contextMenuPosition, x = _a.x, y = _a.y;
          var _b = document.documentElement, clientWidth = _b.clientWidth, clientHeight = _b.clientHeight;
          var _c = el.getBoundingClientRect(), width = _c.width, height = _c.height;
          var nextStyle = __assign({}, defaultStyle);
          if (x + width >= clientWidth) {
              nextStyle.right = 0;
          }
          else {
              nextStyle.left = x;
          }
          if (y + height > clientHeight) {
              nextStyle.bottom = 0;
          }
          else {
              nextStyle.top = y;
          }
          setStyle(nextStyle);
      }, [(_b = state.contextMenuPosition) === null || _b === void 0 ? void 0 : _b.x, (_c = state.contextMenuPosition) === null || _c === void 0 ? void 0 : _c.y]);
      var contextMenuConfig = useContextMenuConfigContext();
      var _e = React__namespace.useState(React__namespace.createElement(React__namespace.Fragment, null)), content = _e[0], setContent = _e[1];
      React__namespace.useEffect(function () {
          var data = state.data.present;
          var selectedNodesCount = 0;
          var selectedPortsCount = 0;
          var selectedEdgesCount = 0;
          data.nodes.forEach(function (node) {
              var _a;
              if (isSelected(node)) {
                  selectedNodesCount += 1;
              }
              (_a = node.ports) === null || _a === void 0 ? void 0 : _a.forEach(function (port) {
                  if (isSelected(port)) {
                      selectedPortsCount += 1;
                  }
              });
          });
          data.edges.forEach(function (edge) {
              if (isSelected(edge)) {
                  selectedEdgesCount += 1;
              }
          });
          var menu;
          if (selectedPortsCount + selectedNodesCount + selectedEdgesCount > 1) {
              menu = contextMenuConfig.getMenu(exports.MenuType.Multi);
          }
          else if (selectedPortsCount + selectedNodesCount + selectedEdgesCount === 0) {
              menu = contextMenuConfig.getMenu(exports.MenuType.Canvas);
          }
          else if (selectedNodesCount === 1) {
              menu = contextMenuConfig.getMenu(exports.MenuType.Node);
          }
          else if (selectedPortsCount === 1) {
              menu = contextMenuConfig.getMenu(exports.MenuType.Port);
          }
          else {
              menu = contextMenuConfig.getMenu(exports.MenuType.Edge);
          }
          setContent(menu);
      }, [state.data.present, contextMenuConfig]);
      var handleContextMenu = React__namespace.useCallback(function (evt) {
          evt.stopPropagation();
          evt.preventDefault();
      }, []);
      return (React__namespace.createElement(React__namespace.Fragment, null, state.contextMenuPosition && (React__namespace.createElement("div", { ref: ref, onClick: onClick, onContextMenu: handleContextMenu, role: "button", style: style }, content))));
  };

  var Group = function (props) {
      var _a;
      var data = props.data, group = props.group;
      var graphConfig = useGraphConfig();
      var _b = React__namespace.useMemo(function () { return getGroupRect(group, data.nodes, graphConfig); }, [
          group,
          data.nodes,
          graphConfig
      ]), x = _b.x, y = _b.y, width = _b.width, height = _b.height;
      var groupConfig = (_a = graphConfig.getGroupConfigByName(group.shape)) !== null && _a !== void 0 ? _a : defaultGroup;
      var automationId = "group-container-" + group.id;
      return (React__namespace.createElement("g", { "data-automation-id": automationId, key: group.id, transform: "translate(" + x + ", " + y + ")" }, groupConfig.render({
          group: group,
          height: height,
          width: width
      })));
  };

  var GraphGroupsRenderer = function (props) {
      return (React__namespace.createElement("g", null, React__namespace.useMemo(function () {
          return props.groups.map(function (group) {
              return React__namespace.createElement(Group, { key: group.id, group: group, data: props.data });
          });
      }, [props.groups, props.data])));
  };

  var NodeTooltips = function (props) {
      var node = props.node, viewport = props.viewport;
      var graphConfig = useGraphConfig();
      var theme = useTheme().theme;
      if (!node) {
          return null;
      }
      if (!hasState(exports.GraphNodeState.activated)(node.state)) {
          return null;
      }
      var nodeConfig = getNodeConfig(node, graphConfig);
      if (!(nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.renderTooltips)) {
          return null;
      }
      return (React__namespace.createElement("div", { className: "node-tooltips" }, nodeConfig.renderTooltips({
          model: node,
          theme: theme,
          viewport: viewport
      })));
  };

  var PortTooltips = function (props) {
      var graphConfig = useGraphConfig();
      var theme = useTheme().theme;
      var parentNode = props.parentNode, port = props.port, viewport = props.viewport;
      var isPortTooltipsVisible = hasState(exports.GraphPortState.activated)(port.state);
      if (!isPortTooltipsVisible) {
          return null;
      }
      var portShape = port.shape ? port.shape : graphConfig.defaultPortShape;
      var portConfig = graphConfig.getPortConfigByName(portShape);
      if (!portConfig || !portConfig.renderTooltips) {
          return null;
      }
      var pos = parentNode.getPortPosition(port.id, graphConfig);
      if (!pos) {
          return null;
      }
      return (React__namespace.createElement("div", { className: "port-tooltips" },
          React__namespace.createElement(ConnectingStateContext.Consumer, null, function (_a) {
              var sourceNode = _a.sourceNode, sourcePort = _a.sourcePort;
              return portConfig.renderTooltips &&
                  portConfig.renderTooltips(__assign({ model: port, parentNode: parentNode, data: props.data, theme: theme, anotherNode: sourceNode, anotherPort: sourcePort, viewport: viewport }, pos));
          })));
  };

  function _extends$1() {
    _extends$1 = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends$1.apply(this, arguments);
  }

  function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };

    return _setPrototypeOf(o, p);
  }

  function _inheritsLoose$1(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass);
  }

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  /** @license React v16.13.1
   * react-is.production.min.js
   *
   * Copyright (c) Facebook, Inc. and its affiliates.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */
  var b="function"===typeof Symbol&&Symbol.for,c=b?Symbol.for("react.element"):60103,d=b?Symbol.for("react.portal"):60106,e=b?Symbol.for("react.fragment"):60107,f=b?Symbol.for("react.strict_mode"):60108,g=b?Symbol.for("react.profiler"):60114,h=b?Symbol.for("react.provider"):60109,k=b?Symbol.for("react.context"):60110,l=b?Symbol.for("react.async_mode"):60111,m=b?Symbol.for("react.concurrent_mode"):60111,n=b?Symbol.for("react.forward_ref"):60112,p=b?Symbol.for("react.suspense"):60113,q=b?
  Symbol.for("react.suspense_list"):60120,r=b?Symbol.for("react.memo"):60115,t=b?Symbol.for("react.lazy"):60116,v=b?Symbol.for("react.block"):60121,w=b?Symbol.for("react.fundamental"):60117,x=b?Symbol.for("react.responder"):60118,y=b?Symbol.for("react.scope"):60119;
  function z(a){if("object"===typeof a&&null!==a){var u=a.$$typeof;switch(u){case c:switch(a=a.type,a){case l:case m:case e:case g:case f:case p:return a;default:switch(a=a&&a.$$typeof,a){case k:case n:case t:case r:case h:return a;default:return u}}case d:return u}}}function A(a){return z(a)===m}var AsyncMode=l;var ConcurrentMode=m;var ContextConsumer=k;var ContextProvider=h;var Element$1=c;var ForwardRef=n;var Fragment=e;var Lazy=t;var Memo=r;var Portal=d;
  var Profiler=g;var StrictMode=f;var Suspense=p;var isAsyncMode=function(a){return A(a)||z(a)===l};var isConcurrentMode=A;var isContextConsumer=function(a){return z(a)===k};var isContextProvider=function(a){return z(a)===h};var isElement=function(a){return "object"===typeof a&&null!==a&&a.$$typeof===c};var isForwardRef=function(a){return z(a)===n};var isFragment=function(a){return z(a)===e};var isLazy=function(a){return z(a)===t};
  var isMemo=function(a){return z(a)===r};var isPortal=function(a){return z(a)===d};var isProfiler=function(a){return z(a)===g};var isStrictMode=function(a){return z(a)===f};var isSuspense=function(a){return z(a)===p};
  var isValidElementType=function(a){return "string"===typeof a||"function"===typeof a||a===e||a===m||a===g||a===f||a===p||a===q||"object"===typeof a&&null!==a&&(a.$$typeof===t||a.$$typeof===r||a.$$typeof===h||a.$$typeof===k||a.$$typeof===n||a.$$typeof===w||a.$$typeof===x||a.$$typeof===y||a.$$typeof===v)};var typeOf=z;

  var reactIs_production_min = {
  	AsyncMode: AsyncMode,
  	ConcurrentMode: ConcurrentMode,
  	ContextConsumer: ContextConsumer,
  	ContextProvider: ContextProvider,
  	Element: Element$1,
  	ForwardRef: ForwardRef,
  	Fragment: Fragment,
  	Lazy: Lazy,
  	Memo: Memo,
  	Portal: Portal,
  	Profiler: Profiler,
  	StrictMode: StrictMode,
  	Suspense: Suspense,
  	isAsyncMode: isAsyncMode,
  	isConcurrentMode: isConcurrentMode,
  	isContextConsumer: isContextConsumer,
  	isContextProvider: isContextProvider,
  	isElement: isElement,
  	isForwardRef: isForwardRef,
  	isFragment: isFragment,
  	isLazy: isLazy,
  	isMemo: isMemo,
  	isPortal: isPortal,
  	isProfiler: isProfiler,
  	isStrictMode: isStrictMode,
  	isSuspense: isSuspense,
  	isValidElementType: isValidElementType,
  	typeOf: typeOf
  };

  var reactIs_development = createCommonjsModule(function (module, exports) {



  if (process.env.NODE_ENV !== "production") {
    (function() {

  // The Symbol used to tag the ReactElement-like types. If there is no native Symbol
  // nor polyfill, then a plain number is used for performance.
  var hasSymbol = typeof Symbol === 'function' && Symbol.for;
  var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;
  var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;
  var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;
  var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;
  var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;
  var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;
  var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary
  // (unstable) APIs that have been removed. Can we remove the symbols?

  var REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;
  var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;
  var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;
  var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;
  var REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;
  var REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;
  var REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;
  var REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;
  var REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;
  var REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;
  var REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;

  function isValidElementType(type) {
    return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
    type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);
  }

  function typeOf(object) {
    if (typeof object === 'object' && object !== null) {
      var $$typeof = object.$$typeof;

      switch ($$typeof) {
        case REACT_ELEMENT_TYPE:
          var type = object.type;

          switch (type) {
            case REACT_ASYNC_MODE_TYPE:
            case REACT_CONCURRENT_MODE_TYPE:
            case REACT_FRAGMENT_TYPE:
            case REACT_PROFILER_TYPE:
            case REACT_STRICT_MODE_TYPE:
            case REACT_SUSPENSE_TYPE:
              return type;

            default:
              var $$typeofType = type && type.$$typeof;

              switch ($$typeofType) {
                case REACT_CONTEXT_TYPE:
                case REACT_FORWARD_REF_TYPE:
                case REACT_LAZY_TYPE:
                case REACT_MEMO_TYPE:
                case REACT_PROVIDER_TYPE:
                  return $$typeofType;

                default:
                  return $$typeof;
              }

          }

        case REACT_PORTAL_TYPE:
          return $$typeof;
      }
    }

    return undefined;
  } // AsyncMode is deprecated along with isAsyncMode

  var AsyncMode = REACT_ASYNC_MODE_TYPE;
  var ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;
  var ContextConsumer = REACT_CONTEXT_TYPE;
  var ContextProvider = REACT_PROVIDER_TYPE;
  var Element = REACT_ELEMENT_TYPE;
  var ForwardRef = REACT_FORWARD_REF_TYPE;
  var Fragment = REACT_FRAGMENT_TYPE;
  var Lazy = REACT_LAZY_TYPE;
  var Memo = REACT_MEMO_TYPE;
  var Portal = REACT_PORTAL_TYPE;
  var Profiler = REACT_PROFILER_TYPE;
  var StrictMode = REACT_STRICT_MODE_TYPE;
  var Suspense = REACT_SUSPENSE_TYPE;
  var hasWarnedAboutDeprecatedIsAsyncMode = false; // AsyncMode should be deprecated

  function isAsyncMode(object) {
    {
      if (!hasWarnedAboutDeprecatedIsAsyncMode) {
        hasWarnedAboutDeprecatedIsAsyncMode = true; // Using console['warn'] to evade Babel and ESLint

        console['warn']('The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');
      }
    }

    return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;
  }
  function isConcurrentMode(object) {
    return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;
  }
  function isContextConsumer(object) {
    return typeOf(object) === REACT_CONTEXT_TYPE;
  }
  function isContextProvider(object) {
    return typeOf(object) === REACT_PROVIDER_TYPE;
  }
  function isElement(object) {
    return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
  }
  function isForwardRef(object) {
    return typeOf(object) === REACT_FORWARD_REF_TYPE;
  }
  function isFragment(object) {
    return typeOf(object) === REACT_FRAGMENT_TYPE;
  }
  function isLazy(object) {
    return typeOf(object) === REACT_LAZY_TYPE;
  }
  function isMemo(object) {
    return typeOf(object) === REACT_MEMO_TYPE;
  }
  function isPortal(object) {
    return typeOf(object) === REACT_PORTAL_TYPE;
  }
  function isProfiler(object) {
    return typeOf(object) === REACT_PROFILER_TYPE;
  }
  function isStrictMode(object) {
    return typeOf(object) === REACT_STRICT_MODE_TYPE;
  }
  function isSuspense(object) {
    return typeOf(object) === REACT_SUSPENSE_TYPE;
  }

  exports.AsyncMode = AsyncMode;
  exports.ConcurrentMode = ConcurrentMode;
  exports.ContextConsumer = ContextConsumer;
  exports.ContextProvider = ContextProvider;
  exports.Element = Element;
  exports.ForwardRef = ForwardRef;
  exports.Fragment = Fragment;
  exports.Lazy = Lazy;
  exports.Memo = Memo;
  exports.Portal = Portal;
  exports.Profiler = Profiler;
  exports.StrictMode = StrictMode;
  exports.Suspense = Suspense;
  exports.isAsyncMode = isAsyncMode;
  exports.isConcurrentMode = isConcurrentMode;
  exports.isContextConsumer = isContextConsumer;
  exports.isContextProvider = isContextProvider;
  exports.isElement = isElement;
  exports.isForwardRef = isForwardRef;
  exports.isFragment = isFragment;
  exports.isLazy = isLazy;
  exports.isMemo = isMemo;
  exports.isPortal = isPortal;
  exports.isProfiler = isProfiler;
  exports.isStrictMode = isStrictMode;
  exports.isSuspense = isSuspense;
  exports.isValidElementType = isValidElementType;
  exports.typeOf = typeOf;
    })();
  }
  });

  var reactIs = createCommonjsModule(function (module) {

  if (process.env.NODE_ENV === 'production') {
    module.exports = reactIs_production_min;
  } else {
    module.exports = reactIs_development;
  }
  });

  /**
   * Copyright 2015, Yahoo! Inc.
   * Copyrights licensed under the New BSD License. See the accompanying LICENSE file for terms.
   */
  var REACT_STATICS = {
    childContextTypes: true,
    contextType: true,
    contextTypes: true,
    defaultProps: true,
    displayName: true,
    getDefaultProps: true,
    getDerivedStateFromError: true,
    getDerivedStateFromProps: true,
    mixins: true,
    propTypes: true,
    type: true
  };
  var KNOWN_STATICS = {
    name: true,
    length: true,
    prototype: true,
    caller: true,
    callee: true,
    arguments: true,
    arity: true
  };
  var FORWARD_REF_STATICS = {
    '$$typeof': true,
    render: true,
    defaultProps: true,
    displayName: true,
    propTypes: true
  };
  var MEMO_STATICS = {
    '$$typeof': true,
    compare: true,
    defaultProps: true,
    displayName: true,
    propTypes: true,
    type: true
  };
  var TYPE_STATICS = {};
  TYPE_STATICS[reactIs.ForwardRef] = FORWARD_REF_STATICS;
  TYPE_STATICS[reactIs.Memo] = MEMO_STATICS;

  function getStatics(component) {
    // React v16.11 and below
    if (reactIs.isMemo(component)) {
      return MEMO_STATICS;
    } // React v16.12 and above


    return TYPE_STATICS[component['$$typeof']] || REACT_STATICS;
  }

  var defineProperty = Object.defineProperty;
  var getOwnPropertyNames = Object.getOwnPropertyNames;
  var getOwnPropertySymbols$1 = Object.getOwnPropertySymbols;
  var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;
  var getPrototypeOf = Object.getPrototypeOf;
  var objectPrototype = Object.prototype;
  function hoistNonReactStatics(targetComponent, sourceComponent, blacklist) {
    if (typeof sourceComponent !== 'string') {
      // don't hoist over string (html) components
      if (objectPrototype) {
        var inheritedComponent = getPrototypeOf(sourceComponent);

        if (inheritedComponent && inheritedComponent !== objectPrototype) {
          hoistNonReactStatics(targetComponent, inheritedComponent, blacklist);
        }
      }

      var keys = getOwnPropertyNames(sourceComponent);

      if (getOwnPropertySymbols$1) {
        keys = keys.concat(getOwnPropertySymbols$1(sourceComponent));
      }

      var targetStatics = getStatics(targetComponent);
      var sourceStatics = getStatics(sourceComponent);

      for (var i = 0; i < keys.length; ++i) {
        var key = keys[i];

        if (!KNOWN_STATICS[key] && !(blacklist && blacklist[key]) && !(sourceStatics && sourceStatics[key]) && !(targetStatics && targetStatics[key])) {
          var descriptor = getOwnPropertyDescriptor(sourceComponent, key);

          try {
            // Avoid failures from read-only properties
            defineProperty(targetComponent, key, descriptor);
          } catch (e) {}
        }
      }
    }

    return targetComponent;
  }

  var hoistNonReactStatics_cjs = hoistNonReactStatics;

  var isProduction = process.env.NODE_ENV === 'production';
  function warning(condition, message) {
    if (!isProduction) {
      if (condition) {
        return;
      }

      var text = "Warning: " + message;

      if (typeof console !== 'undefined') {
        console.warn(text);
      }

      try {
        throw Error(text);
      } catch (x) {}
    }
  }

  /*
  object-assign
  (c) Sindre Sorhus
  @license MIT
  */
  /* eslint-disable no-unused-vars */
  var getOwnPropertySymbols = Object.getOwnPropertySymbols;
  var hasOwnProperty = Object.prototype.hasOwnProperty;
  var propIsEnumerable = Object.prototype.propertyIsEnumerable;

  function toObject(val) {
  	if (val === null || val === undefined) {
  		throw new TypeError('Object.assign cannot be called with null or undefined');
  	}

  	return Object(val);
  }

  function shouldUseNative() {
  	try {
  		if (!Object.assign) {
  			return false;
  		}

  		// Detect buggy property enumeration order in older V8 versions.

  		// https://bugs.chromium.org/p/v8/issues/detail?id=4118
  		var test1 = new String('abc');  // eslint-disable-line no-new-wrappers
  		test1[5] = 'de';
  		if (Object.getOwnPropertyNames(test1)[0] === '5') {
  			return false;
  		}

  		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
  		var test2 = {};
  		for (var i = 0; i < 10; i++) {
  			test2['_' + String.fromCharCode(i)] = i;
  		}
  		var order2 = Object.getOwnPropertyNames(test2).map(function (n) {
  			return test2[n];
  		});
  		if (order2.join('') !== '0123456789') {
  			return false;
  		}

  		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
  		var test3 = {};
  		'abcdefghijklmnopqrst'.split('').forEach(function (letter) {
  			test3[letter] = letter;
  		});
  		if (Object.keys(Object.assign({}, test3)).join('') !==
  				'abcdefghijklmnopqrst') {
  			return false;
  		}

  		return true;
  	} catch (err) {
  		// We don't expect any of the above to throw, but better to be safe.
  		return false;
  	}
  }

  var objectAssign = shouldUseNative() ? Object.assign : function (target, source) {
  	var from;
  	var to = toObject(target);
  	var symbols;

  	for (var s = 1; s < arguments.length; s++) {
  		from = Object(arguments[s]);

  		for (var key in from) {
  			if (hasOwnProperty.call(from, key)) {
  				to[key] = from[key];
  			}
  		}

  		if (getOwnPropertySymbols) {
  			symbols = getOwnPropertySymbols(from);
  			for (var i = 0; i < symbols.length; i++) {
  				if (propIsEnumerable.call(from, symbols[i])) {
  					to[symbols[i]] = from[symbols[i]];
  				}
  			}
  		}
  	}

  	return to;
  };

  /**
   * Copyright (c) 2013-present, Facebook, Inc.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */

  var ReactPropTypesSecret$1 = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';

  var ReactPropTypesSecret_1 = ReactPropTypesSecret$1;

  var printWarning$1 = function() {};

  if (process.env.NODE_ENV !== 'production') {
    var ReactPropTypesSecret = ReactPropTypesSecret_1;
    var loggedTypeFailures = {};
    var has$1 = Function.call.bind(Object.prototype.hasOwnProperty);

    printWarning$1 = function(text) {
      var message = 'Warning: ' + text;
      if (typeof console !== 'undefined') {
        console.error(message);
      }
      try {
        // --- Welcome to debugging React ---
        // This error was thrown as a convenience so that you can use this stack
        // to find the callsite that caused this warning to fire.
        throw new Error(message);
      } catch (x) {}
    };
  }

  /**
   * Assert that the values match with the type specs.
   * Error messages are memorized and will only be shown once.
   *
   * @param {object} typeSpecs Map of name to a ReactPropType
   * @param {object} values Runtime values that need to be type-checked
   * @param {string} location e.g. "prop", "context", "child context"
   * @param {string} componentName Name of the component for error messages.
   * @param {?Function} getStack Returns the component stack.
   * @private
   */
  function checkPropTypes(typeSpecs, values, location, componentName, getStack) {
    if (process.env.NODE_ENV !== 'production') {
      for (var typeSpecName in typeSpecs) {
        if (has$1(typeSpecs, typeSpecName)) {
          var error;
          // Prop type validation may throw. In case they do, we don't want to
          // fail the render phase where it didn't fail before. So we log it.
          // After these have been cleaned up, we'll let them throw.
          try {
            // This is intentionally an invariant that gets caught. It's the same
            // behavior as without this statement except with a better message.
            if (typeof typeSpecs[typeSpecName] !== 'function') {
              var err = Error(
                (componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' +
                'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.'
              );
              err.name = 'Invariant Violation';
              throw err;
            }
            error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret);
          } catch (ex) {
            error = ex;
          }
          if (error && !(error instanceof Error)) {
            printWarning$1(
              (componentName || 'React class') + ': type specification of ' +
              location + ' `' + typeSpecName + '` is invalid; the type checker ' +
              'function must return `null` or an `Error` but returned a ' + typeof error + '. ' +
              'You may have forgotten to pass an argument to the type checker ' +
              'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' +
              'shape all require an argument).'
            );
          }
          if (error instanceof Error && !(error.message in loggedTypeFailures)) {
            // Only monitor this failure once because there tends to be a lot of the
            // same error.
            loggedTypeFailures[error.message] = true;

            var stack = getStack ? getStack() : '';

            printWarning$1(
              'Failed ' + location + ' type: ' + error.message + (stack != null ? stack : '')
            );
          }
        }
      }
    }
  }

  /**
   * Resets warning cache when testing.
   *
   * @private
   */
  checkPropTypes.resetWarningCache = function() {
    if (process.env.NODE_ENV !== 'production') {
      loggedTypeFailures = {};
    }
  };

  var checkPropTypes_1 = checkPropTypes;

  var has = Function.call.bind(Object.prototype.hasOwnProperty);
  var printWarning = function() {};

  if (process.env.NODE_ENV !== 'production') {
    printWarning = function(text) {
      var message = 'Warning: ' + text;
      if (typeof console !== 'undefined') {
        console.error(message);
      }
      try {
        // --- Welcome to debugging React ---
        // This error was thrown as a convenience so that you can use this stack
        // to find the callsite that caused this warning to fire.
        throw new Error(message);
      } catch (x) {}
    };
  }

  function emptyFunctionThatReturnsNull() {
    return null;
  }

  var factoryWithTypeCheckers = function(isValidElement, throwOnDirectAccess) {
    /* global Symbol */
    var ITERATOR_SYMBOL = typeof Symbol === 'function' && Symbol.iterator;
    var FAUX_ITERATOR_SYMBOL = '@@iterator'; // Before Symbol spec.

    /**
     * Returns the iterator method function contained on the iterable object.
     *
     * Be sure to invoke the function with the iterable as context:
     *
     *     var iteratorFn = getIteratorFn(myIterable);
     *     if (iteratorFn) {
     *       var iterator = iteratorFn.call(myIterable);
     *       ...
     *     }
     *
     * @param {?object} maybeIterable
     * @return {?function}
     */
    function getIteratorFn(maybeIterable) {
      var iteratorFn = maybeIterable && (ITERATOR_SYMBOL && maybeIterable[ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL]);
      if (typeof iteratorFn === 'function') {
        return iteratorFn;
      }
    }

    /**
     * Collection of methods that allow declaration and validation of props that are
     * supplied to React components. Example usage:
     *
     *   var Props = require('ReactPropTypes');
     *   var MyArticle = React.createClass({
     *     propTypes: {
     *       // An optional string prop named "description".
     *       description: Props.string,
     *
     *       // A required enum prop named "category".
     *       category: Props.oneOf(['News','Photos']).isRequired,
     *
     *       // A prop named "dialog" that requires an instance of Dialog.
     *       dialog: Props.instanceOf(Dialog).isRequired
     *     },
     *     render: function() { ... }
     *   });
     *
     * A more formal specification of how these methods are used:
     *
     *   type := array|bool|func|object|number|string|oneOf([...])|instanceOf(...)
     *   decl := ReactPropTypes.{type}(.isRequired)?
     *
     * Each and every declaration produces a function with the same signature. This
     * allows the creation of custom validation functions. For example:
     *
     *  var MyLink = React.createClass({
     *    propTypes: {
     *      // An optional string or URI prop named "href".
     *      href: function(props, propName, componentName) {
     *        var propValue = props[propName];
     *        if (propValue != null && typeof propValue !== 'string' &&
     *            !(propValue instanceof URI)) {
     *          return new Error(
     *            'Expected a string or an URI for ' + propName + ' in ' +
     *            componentName
     *          );
     *        }
     *      }
     *    },
     *    render: function() {...}
     *  });
     *
     * @internal
     */

    var ANONYMOUS = '<<anonymous>>';

    // Important!
    // Keep this list in sync with production version in `./factoryWithThrowingShims.js`.
    var ReactPropTypes = {
      array: createPrimitiveTypeChecker('array'),
      bool: createPrimitiveTypeChecker('boolean'),
      func: createPrimitiveTypeChecker('function'),
      number: createPrimitiveTypeChecker('number'),
      object: createPrimitiveTypeChecker('object'),
      string: createPrimitiveTypeChecker('string'),
      symbol: createPrimitiveTypeChecker('symbol'),

      any: createAnyTypeChecker(),
      arrayOf: createArrayOfTypeChecker,
      element: createElementTypeChecker(),
      elementType: createElementTypeTypeChecker(),
      instanceOf: createInstanceTypeChecker,
      node: createNodeChecker(),
      objectOf: createObjectOfTypeChecker,
      oneOf: createEnumTypeChecker,
      oneOfType: createUnionTypeChecker,
      shape: createShapeTypeChecker,
      exact: createStrictShapeTypeChecker,
    };

    /**
     * inlined Object.is polyfill to avoid requiring consumers ship their own
     * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
     */
    /*eslint-disable no-self-compare*/
    function is(x, y) {
      // SameValue algorithm
      if (x === y) {
        // Steps 1-5, 7-10
        // Steps 6.b-6.e: +0 != -0
        return x !== 0 || 1 / x === 1 / y;
      } else {
        // Step 6.a: NaN == NaN
        return x !== x && y !== y;
      }
    }
    /*eslint-enable no-self-compare*/

    /**
     * We use an Error-like object for backward compatibility as people may call
     * PropTypes directly and inspect their output. However, we don't use real
     * Errors anymore. We don't inspect their stack anyway, and creating them
     * is prohibitively expensive if they are created too often, such as what
     * happens in oneOfType() for any type before the one that matched.
     */
    function PropTypeError(message) {
      this.message = message;
      this.stack = '';
    }
    // Make `instanceof Error` still work for returned errors.
    PropTypeError.prototype = Error.prototype;

    function createChainableTypeChecker(validate) {
      if (process.env.NODE_ENV !== 'production') {
        var manualPropTypeCallCache = {};
        var manualPropTypeWarningCount = 0;
      }
      function checkType(isRequired, props, propName, componentName, location, propFullName, secret) {
        componentName = componentName || ANONYMOUS;
        propFullName = propFullName || propName;

        if (secret !== ReactPropTypesSecret_1) {
          if (throwOnDirectAccess) {
            // New behavior only for users of `prop-types` package
            var err = new Error(
              'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +
              'Use `PropTypes.checkPropTypes()` to call them. ' +
              'Read more at http://fb.me/use-check-prop-types'
            );
            err.name = 'Invariant Violation';
            throw err;
          } else if (process.env.NODE_ENV !== 'production' && typeof console !== 'undefined') {
            // Old behavior for people using React.PropTypes
            var cacheKey = componentName + ':' + propName;
            if (
              !manualPropTypeCallCache[cacheKey] &&
              // Avoid spamming the console because they are often not actionable except for lib authors
              manualPropTypeWarningCount < 3
            ) {
              printWarning(
                'You are manually calling a React.PropTypes validation ' +
                'function for the `' + propFullName + '` prop on `' + componentName  + '`. This is deprecated ' +
                'and will throw in the standalone `prop-types` package. ' +
                'You may be seeing this warning due to a third-party PropTypes ' +
                'library. See https://fb.me/react-warning-dont-call-proptypes ' + 'for details.'
              );
              manualPropTypeCallCache[cacheKey] = true;
              manualPropTypeWarningCount++;
            }
          }
        }
        if (props[propName] == null) {
          if (isRequired) {
            if (props[propName] === null) {
              return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required ' + ('in `' + componentName + '`, but its value is `null`.'));
            }
            return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required in ' + ('`' + componentName + '`, but its value is `undefined`.'));
          }
          return null;
        } else {
          return validate(props, propName, componentName, location, propFullName);
        }
      }

      var chainedCheckType = checkType.bind(null, false);
      chainedCheckType.isRequired = checkType.bind(null, true);

      return chainedCheckType;
    }

    function createPrimitiveTypeChecker(expectedType) {
      function validate(props, propName, componentName, location, propFullName, secret) {
        var propValue = props[propName];
        var propType = getPropType(propValue);
        if (propType !== expectedType) {
          // `propValue` being instance of, say, date/regexp, pass the 'object'
          // check, but we can offer a more precise error message here rather than
          // 'of type `object`'.
          var preciseType = getPreciseType(propValue);

          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + preciseType + '` supplied to `' + componentName + '`, expected ') + ('`' + expectedType + '`.'));
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createAnyTypeChecker() {
      return createChainableTypeChecker(emptyFunctionThatReturnsNull);
    }

    function createArrayOfTypeChecker(typeChecker) {
      function validate(props, propName, componentName, location, propFullName) {
        if (typeof typeChecker !== 'function') {
          return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside arrayOf.');
        }
        var propValue = props[propName];
        if (!Array.isArray(propValue)) {
          var propType = getPropType(propValue);
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an array.'));
        }
        for (var i = 0; i < propValue.length; i++) {
          var error = typeChecker(propValue, i, componentName, location, propFullName + '[' + i + ']', ReactPropTypesSecret_1);
          if (error instanceof Error) {
            return error;
          }
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createElementTypeChecker() {
      function validate(props, propName, componentName, location, propFullName) {
        var propValue = props[propName];
        if (!isValidElement(propValue)) {
          var propType = getPropType(propValue);
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement.'));
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createElementTypeTypeChecker() {
      function validate(props, propName, componentName, location, propFullName) {
        var propValue = props[propName];
        if (!reactIs.isValidElementType(propValue)) {
          var propType = getPropType(propValue);
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement type.'));
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createInstanceTypeChecker(expectedClass) {
      function validate(props, propName, componentName, location, propFullName) {
        if (!(props[propName] instanceof expectedClass)) {
          var expectedClassName = expectedClass.name || ANONYMOUS;
          var actualClassName = getClassName(props[propName]);
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + actualClassName + '` supplied to `' + componentName + '`, expected ') + ('instance of `' + expectedClassName + '`.'));
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createEnumTypeChecker(expectedValues) {
      if (!Array.isArray(expectedValues)) {
        if (process.env.NODE_ENV !== 'production') {
          if (arguments.length > 1) {
            printWarning(
              'Invalid arguments supplied to oneOf, expected an array, got ' + arguments.length + ' arguments. ' +
              'A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z]).'
            );
          } else {
            printWarning('Invalid argument supplied to oneOf, expected an array.');
          }
        }
        return emptyFunctionThatReturnsNull;
      }

      function validate(props, propName, componentName, location, propFullName) {
        var propValue = props[propName];
        for (var i = 0; i < expectedValues.length; i++) {
          if (is(propValue, expectedValues[i])) {
            return null;
          }
        }

        var valuesString = JSON.stringify(expectedValues, function replacer(key, value) {
          var type = getPreciseType(value);
          if (type === 'symbol') {
            return String(value);
          }
          return value;
        });
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of value `' + String(propValue) + '` ' + ('supplied to `' + componentName + '`, expected one of ' + valuesString + '.'));
      }
      return createChainableTypeChecker(validate);
    }

    function createObjectOfTypeChecker(typeChecker) {
      function validate(props, propName, componentName, location, propFullName) {
        if (typeof typeChecker !== 'function') {
          return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside objectOf.');
        }
        var propValue = props[propName];
        var propType = getPropType(propValue);
        if (propType !== 'object') {
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an object.'));
        }
        for (var key in propValue) {
          if (has(propValue, key)) {
            var error = typeChecker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
            if (error instanceof Error) {
              return error;
            }
          }
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createUnionTypeChecker(arrayOfTypeCheckers) {
      if (!Array.isArray(arrayOfTypeCheckers)) {
        process.env.NODE_ENV !== 'production' ? printWarning('Invalid argument supplied to oneOfType, expected an instance of array.') : void 0;
        return emptyFunctionThatReturnsNull;
      }

      for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
        var checker = arrayOfTypeCheckers[i];
        if (typeof checker !== 'function') {
          printWarning(
            'Invalid argument supplied to oneOfType. Expected an array of check functions, but ' +
            'received ' + getPostfixForTypeWarning(checker) + ' at index ' + i + '.'
          );
          return emptyFunctionThatReturnsNull;
        }
      }

      function validate(props, propName, componentName, location, propFullName) {
        for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
          var checker = arrayOfTypeCheckers[i];
          if (checker(props, propName, componentName, location, propFullName, ReactPropTypesSecret_1) == null) {
            return null;
          }
        }

        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`.'));
      }
      return createChainableTypeChecker(validate);
    }

    function createNodeChecker() {
      function validate(props, propName, componentName, location, propFullName) {
        if (!isNode(props[propName])) {
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`, expected a ReactNode.'));
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createShapeTypeChecker(shapeTypes) {
      function validate(props, propName, componentName, location, propFullName) {
        var propValue = props[propName];
        var propType = getPropType(propValue);
        if (propType !== 'object') {
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
        }
        for (var key in shapeTypes) {
          var checker = shapeTypes[key];
          if (!checker) {
            continue;
          }
          var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
          if (error) {
            return error;
          }
        }
        return null;
      }
      return createChainableTypeChecker(validate);
    }

    function createStrictShapeTypeChecker(shapeTypes) {
      function validate(props, propName, componentName, location, propFullName) {
        var propValue = props[propName];
        var propType = getPropType(propValue);
        if (propType !== 'object') {
          return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
        }
        // We need to check all keys in case some are required but missing from
        // props.
        var allKeys = objectAssign({}, props[propName], shapeTypes);
        for (var key in allKeys) {
          var checker = shapeTypes[key];
          if (!checker) {
            return new PropTypeError(
              'Invalid ' + location + ' `' + propFullName + '` key `' + key + '` supplied to `' + componentName + '`.' +
              '\nBad object: ' + JSON.stringify(props[propName], null, '  ') +
              '\nValid keys: ' +  JSON.stringify(Object.keys(shapeTypes), null, '  ')
            );
          }
          var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
          if (error) {
            return error;
          }
        }
        return null;
      }

      return createChainableTypeChecker(validate);
    }

    function isNode(propValue) {
      switch (typeof propValue) {
        case 'number':
        case 'string':
        case 'undefined':
          return true;
        case 'boolean':
          return !propValue;
        case 'object':
          if (Array.isArray(propValue)) {
            return propValue.every(isNode);
          }
          if (propValue === null || isValidElement(propValue)) {
            return true;
          }

          var iteratorFn = getIteratorFn(propValue);
          if (iteratorFn) {
            var iterator = iteratorFn.call(propValue);
            var step;
            if (iteratorFn !== propValue.entries) {
              while (!(step = iterator.next()).done) {
                if (!isNode(step.value)) {
                  return false;
                }
              }
            } else {
              // Iterator will provide entry [k,v] tuples rather than values.
              while (!(step = iterator.next()).done) {
                var entry = step.value;
                if (entry) {
                  if (!isNode(entry[1])) {
                    return false;
                  }
                }
              }
            }
          } else {
            return false;
          }

          return true;
        default:
          return false;
      }
    }

    function isSymbol(propType, propValue) {
      // Native Symbol.
      if (propType === 'symbol') {
        return true;
      }

      // falsy value can't be a Symbol
      if (!propValue) {
        return false;
      }

      // 19.4.3.5 Symbol.prototype[@@toStringTag] === 'Symbol'
      if (propValue['@@toStringTag'] === 'Symbol') {
        return true;
      }

      // Fallback for non-spec compliant Symbols which are polyfilled.
      if (typeof Symbol === 'function' && propValue instanceof Symbol) {
        return true;
      }

      return false;
    }

    // Equivalent of `typeof` but with special handling for array and regexp.
    function getPropType(propValue) {
      var propType = typeof propValue;
      if (Array.isArray(propValue)) {
        return 'array';
      }
      if (propValue instanceof RegExp) {
        // Old webkits (at least until Android 4.0) return 'function' rather than
        // 'object' for typeof a RegExp. We'll normalize this here so that /bla/
        // passes PropTypes.object.
        return 'object';
      }
      if (isSymbol(propType, propValue)) {
        return 'symbol';
      }
      return propType;
    }

    // This handles more types than `getPropType`. Only used for error messages.
    // See `createPrimitiveTypeChecker`.
    function getPreciseType(propValue) {
      if (typeof propValue === 'undefined' || propValue === null) {
        return '' + propValue;
      }
      var propType = getPropType(propValue);
      if (propType === 'object') {
        if (propValue instanceof Date) {
          return 'date';
        } else if (propValue instanceof RegExp) {
          return 'regexp';
        }
      }
      return propType;
    }

    // Returns a string that is postfixed to a warning about an invalid type.
    // For example, "undefined" or "of type array"
    function getPostfixForTypeWarning(value) {
      var type = getPreciseType(value);
      switch (type) {
        case 'array':
        case 'object':
          return 'an ' + type;
        case 'boolean':
        case 'date':
        case 'regexp':
          return 'a ' + type;
        default:
          return type;
      }
    }

    // Returns class name of the object, if any.
    function getClassName(propValue) {
      if (!propValue.constructor || !propValue.constructor.name) {
        return ANONYMOUS;
      }
      return propValue.constructor.name;
    }

    ReactPropTypes.checkPropTypes = checkPropTypes_1;
    ReactPropTypes.resetWarningCache = checkPropTypes_1.resetWarningCache;
    ReactPropTypes.PropTypes = ReactPropTypes;

    return ReactPropTypes;
  };

  function emptyFunction() {}
  function emptyFunctionWithReset() {}
  emptyFunctionWithReset.resetWarningCache = emptyFunction;

  var factoryWithThrowingShims = function() {
    function shim(props, propName, componentName, location, propFullName, secret) {
      if (secret === ReactPropTypesSecret_1) {
        // It is still safe when called from React.
        return;
      }
      var err = new Error(
        'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +
        'Use PropTypes.checkPropTypes() to call them. ' +
        'Read more at http://fb.me/use-check-prop-types'
      );
      err.name = 'Invariant Violation';
      throw err;
    }  shim.isRequired = shim;
    function getShim() {
      return shim;
    }  // Important!
    // Keep this list in sync with production version in `./factoryWithTypeCheckers.js`.
    var ReactPropTypes = {
      array: shim,
      bool: shim,
      func: shim,
      number: shim,
      object: shim,
      string: shim,
      symbol: shim,

      any: shim,
      arrayOf: getShim,
      element: shim,
      elementType: shim,
      instanceOf: getShim,
      node: shim,
      objectOf: getShim,
      oneOf: getShim,
      oneOfType: getShim,
      shape: getShim,
      exact: getShim,

      checkPropTypes: emptyFunctionWithReset,
      resetWarningCache: emptyFunction
    };

    ReactPropTypes.PropTypes = ReactPropTypes;

    return ReactPropTypes;
  };

  var propTypes = createCommonjsModule(function (module) {
  /**
   * Copyright (c) 2013-present, Facebook, Inc.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */

  if (process.env.NODE_ENV !== 'production') {
    var ReactIs = reactIs;

    // By explicitly using `prop-types` you are opting into new development behavior.
    // http://fb.me/prop-types-in-prod
    var throwOnDirectAccess = true;
    module.exports = factoryWithTypeCheckers(ReactIs.isElement, throwOnDirectAccess);
  } else {
    // By explicitly using `prop-types` you are opting into new production behavior.
    // http://fb.me/prop-types-in-prod
    module.exports = factoryWithThrowingShims();
  }
  });

  var getDisplayName_1 = createCommonjsModule(function (module, exports) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = getDisplayName;
  function getDisplayName(Component) {
    return Component.displayName || Component.name || (typeof Component === 'string' && Component.length > 0 ? Component : 'Unknown');
  }
  });

  var getDisplayName$1 = /*@__PURE__*/getDefaultExportFromCjs(getDisplayName_1);

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    subClass.__proto__ = superClass;
  }

  function _assertThisInitialized$1(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return self;
  }

  function isObject$1(obj) {
    return obj !== null && typeof obj === 'object' && !Array.isArray(obj);
  }

  function createThemeProvider(context) {
    var ThemeProvider =
    /*#__PURE__*/
    function (_React$Component) {
      _inheritsLoose(ThemeProvider, _React$Component);

      function ThemeProvider() {
        var _this;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;

        _defineProperty(_assertThisInitialized$1(_assertThisInitialized$1(_this)), "cachedTheme", void 0);

        _defineProperty(_assertThisInitialized$1(_assertThisInitialized$1(_this)), "lastOuterTheme", void 0);

        _defineProperty(_assertThisInitialized$1(_assertThisInitialized$1(_this)), "lastTheme", void 0);

        _defineProperty(_assertThisInitialized$1(_assertThisInitialized$1(_this)), "renderProvider", function (outerTheme) {
          var children = _this.props.children;
          return React__default['default'].createElement(context.Provider, {
            value: _this.getTheme(outerTheme)
          }, children);
        });

        return _this;
      }

      var _proto = ThemeProvider.prototype;

      // Get the theme from the props, supporting both (outerTheme) => {} as well as object notation
      _proto.getTheme = function getTheme(outerTheme) {
        if (this.props.theme !== this.lastTheme || outerTheme !== this.lastOuterTheme || !this.cachedTheme) {
          this.lastOuterTheme = outerTheme;
          this.lastTheme = this.props.theme;

          if (typeof this.lastTheme === 'function') {
            var theme = this.props.theme;
            this.cachedTheme = theme(outerTheme);
            process.env.NODE_ENV !== "production" ? warning(isObject$1(this.cachedTheme), '[ThemeProvider] Please return an object from your theme function') : void 0;
          } else {
            var _theme = this.props.theme;
            process.env.NODE_ENV !== "production" ? warning(isObject$1(_theme), '[ThemeProvider] Please make your theme prop a plain object') : void 0;
            this.cachedTheme = outerTheme ? _extends({}, outerTheme, _theme) : _theme;
          }
        }

        return this.cachedTheme;
      };

      _proto.render = function render() {
        var children = this.props.children;

        if (!children) {
          return null;
        }

        return React__default['default'].createElement(context.Consumer, null, this.renderProvider);
      };

      return ThemeProvider;
    }(React__default['default'].Component);

    if (process.env.NODE_ENV !== 'production') {
      ThemeProvider.propTypes = {
        // eslint-disable-next-line react/require-default-props
        children: propTypes.node,
        theme: propTypes.oneOfType([propTypes.shape({}), propTypes.func]).isRequired
      };
    }

    return ThemeProvider;
  }

  function createWithTheme(context) {
    return function hoc(Component) {
      var withTheme = React__default['default'].forwardRef(function (props, ref) {
        return React__default['default'].createElement(context.Consumer, null, function (theme) {
          process.env.NODE_ENV !== "production" ? warning(isObject$1(theme), '[theming] Please use withTheme only with the ThemeProvider') : void 0;
          return React__default['default'].createElement(Component, _extends({
            theme: theme,
            ref: ref
          }, props));
        });
      });

      if (process.env.NODE_ENV !== 'production') {
        withTheme.displayName = "WithTheme(" + getDisplayName$1(Component) + ")";
      }

      hoistNonReactStatics_cjs(withTheme, Component);
      return withTheme;
    };
  }

  function createUseTheme(context) {
    var useTheme = function useTheme() {
      var theme = React__default['default'].useContext(context);
      process.env.NODE_ENV !== "production" ? warning(isObject$1(theme), '[theming] Please use useTheme only with the ThemeProvider') : void 0;
      return theme;
    };

    return useTheme;
  }

  var ThemeContext = React.createContext();

  function createTheming(context) {
    return {
      context: context,
      withTheme: createWithTheme(context),
      useTheme: createUseTheme(context),
      ThemeProvider: createThemeProvider(context)
    };
  }

  createTheming(ThemeContext);

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

  var isBrowser = (typeof window === "undefined" ? "undefined" : _typeof(window)) === "object" && (typeof document === "undefined" ? "undefined" : _typeof(document)) === 'object' && document.nodeType === 9;

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return self;
  }

  var plainObjectConstrurctor = {}.constructor;
  function cloneStyle(style) {
    if (style == null || typeof style !== 'object') return style;
    if (Array.isArray(style)) return style.map(cloneStyle);
    if (style.constructor !== plainObjectConstrurctor) return style;
    var newStyle = {};

    for (var name in style) {
      newStyle[name] = cloneStyle(style[name]);
    }

    return newStyle;
  }

  /**
   * Create a rule instance.
   */

  function createRule(name, decl, options) {
    if (name === void 0) {
      name = 'unnamed';
    }

    var jss = options.jss;
    var declCopy = cloneStyle(decl);
    var rule = jss.plugins.onCreateRule(name, declCopy, options);
    if (rule) return rule; // It is an at-rule and it has no instance.

    if (name[0] === '@') {
      process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Unknown rule " + name) : void 0;
    }

    return null;
  }

  var join = function join(value, by) {
    var result = '';

    for (var i = 0; i < value.length; i++) {
      // Remove !important from the value, it will be readded later.
      if (value[i] === '!important') break;
      if (result) result += by;
      result += value[i];
    }

    return result;
  };
  /**
   * Converts array values to string.
   *
   * `margin: [['5px', '10px']]` > `margin: 5px 10px;`
   * `border: ['1px', '2px']` > `border: 1px, 2px;`
   * `margin: [['5px', '10px'], '!important']` > `margin: 5px 10px !important;`
   * `color: ['red', !important]` > `color: red !important;`
   */


  function toCssValue(value, ignoreImportant) {
    if (ignoreImportant === void 0) {
      ignoreImportant = false;
    }

    if (!Array.isArray(value)) return value;
    var cssValue = ''; // Support space separated values via `[['5px', '10px']]`.

    if (Array.isArray(value[0])) {
      for (var i = 0; i < value.length; i++) {
        if (value[i] === '!important') break;
        if (cssValue) cssValue += ', ';
        cssValue += join(value[i], ' ');
      }
    } else cssValue = join(value, ', '); // Add !important, because it was ignored.


    if (!ignoreImportant && value[value.length - 1] === '!important') {
      cssValue += ' !important';
    }

    return cssValue;
  }

  /**
   * Indent a string.
   * http://jsperf.com/array-join-vs-for
   */
  function indentStr(str, indent) {
    var result = '';

    for (var index = 0; index < indent; index++) {
      result += '  ';
    }

    return result + str;
  }
  /**
   * Converts a Rule to CSS string.
   */


  function toCss(selector, style, options) {
    if (options === void 0) {
      options = {};
    }

    var result = '';
    if (!style) return result;
    var _options = options,
        _options$indent = _options.indent,
        indent = _options$indent === void 0 ? 0 : _options$indent;
    var fallbacks = style.fallbacks;
    if (selector) indent++; // Apply fallbacks first.

    if (fallbacks) {
      // Array syntax {fallbacks: [{prop: value}]}
      if (Array.isArray(fallbacks)) {
        for (var index = 0; index < fallbacks.length; index++) {
          var fallback = fallbacks[index];

          for (var prop in fallback) {
            var value = fallback[prop];

            if (value != null) {
              if (result) result += '\n';
              result += "" + indentStr(prop + ": " + toCssValue(value) + ";", indent);
            }
          }
        }
      } else {
        // Object syntax {fallbacks: {prop: value}}
        for (var _prop in fallbacks) {
          var _value = fallbacks[_prop];

          if (_value != null) {
            if (result) result += '\n';
            result += "" + indentStr(_prop + ": " + toCssValue(_value) + ";", indent);
          }
        }
      }
    }

    for (var _prop2 in style) {
      var _value2 = style[_prop2];

      if (_value2 != null && _prop2 !== 'fallbacks') {
        if (result) result += '\n';
        result += "" + indentStr(_prop2 + ": " + toCssValue(_value2) + ";", indent);
      }
    } // Allow empty style in this case, because properties will be added dynamically.


    if (!result && !options.allowEmpty) return result; // When rule is being stringified before selector was defined.

    if (!selector) return result;
    indent--;
    if (result) result = "\n" + result + "\n";
    return indentStr(selector + " {" + result, indent) + indentStr('}', indent);
  }

  var escapeRegex = /([[\].#*$><+~=|^:(),"'`\s])/g;
  var nativeEscape = typeof CSS !== 'undefined' && CSS.escape;
  var escape = (function (str) {
    return nativeEscape ? nativeEscape(str) : str.replace(escapeRegex, '\\$1');
  });

  var BaseStyleRule =
  /*#__PURE__*/
  function () {
    function BaseStyleRule(key, style, options) {
      this.type = 'style';
      this.key = void 0;
      this.isProcessed = false;
      this.style = void 0;
      this.renderer = void 0;
      this.renderable = void 0;
      this.options = void 0;
      var sheet = options.sheet,
          Renderer = options.Renderer;
      this.key = key;
      this.options = options;
      this.style = style;
      if (sheet) this.renderer = sheet.renderer;else if (Renderer) this.renderer = new Renderer();
    }
    /**
     * Get or set a style property.
     */


    var _proto = BaseStyleRule.prototype;

    _proto.prop = function prop(name, value, options) {
      // It's a getter.
      if (value === undefined) return this.style[name]; // Don't do anything if the value has not changed.

      var force = options ? options.force : false;
      if (!force && this.style[name] === value) return this;
      var newValue = value;

      if (!options || options.process !== false) {
        newValue = this.options.jss.plugins.onChangeValue(value, name, this);
      }

      var isEmpty = newValue == null || newValue === false;
      var isDefined = name in this.style; // Value is empty and wasn't defined before.

      if (isEmpty && !isDefined && !force) return this; // We are going to remove this value.

      var remove = isEmpty && isDefined;
      if (remove) delete this.style[name];else this.style[name] = newValue; // Renderable is defined if StyleSheet option `link` is true.

      if (this.renderable && this.renderer) {
        if (remove) this.renderer.removeProperty(this.renderable, name);else this.renderer.setProperty(this.renderable, name, newValue);
        return this;
      }

      var sheet = this.options.sheet;

      if (sheet && sheet.attached) {
        process.env.NODE_ENV !== "production" ? warning(false, '[JSS] Rule is not linked. Missing sheet option "link: true".') : void 0;
      }

      return this;
    };

    return BaseStyleRule;
  }();
  var StyleRule =
  /*#__PURE__*/
  function (_BaseStyleRule) {
    _inheritsLoose$1(StyleRule, _BaseStyleRule);

    function StyleRule(key, style, options) {
      var _this;

      _this = _BaseStyleRule.call(this, key, style, options) || this;
      _this.selectorText = void 0;
      _this.id = void 0;
      _this.renderable = void 0;
      var selector = options.selector,
          scoped = options.scoped,
          sheet = options.sheet,
          generateId = options.generateId;

      if (selector) {
        _this.selectorText = selector;
      } else if (scoped !== false) {
        _this.id = generateId(_assertThisInitialized(_assertThisInitialized(_this)), sheet);
        _this.selectorText = "." + escape(_this.id);
      }

      return _this;
    }
    /**
     * Set selector string.
     * Attention: use this with caution. Most browsers didn't implement
     * selectorText setter, so this may result in rerendering of entire Style Sheet.
     */


    var _proto2 = StyleRule.prototype;

    /**
     * Apply rule to an element inline.
     */
    _proto2.applyTo = function applyTo(renderable) {
      var renderer = this.renderer;

      if (renderer) {
        var json = this.toJSON();

        for (var prop in json) {
          renderer.setProperty(renderable, prop, json[prop]);
        }
      }

      return this;
    }
    /**
     * Returns JSON representation of the rule.
     * Fallbacks are not supported.
     * Useful for inline styles.
     */
    ;

    _proto2.toJSON = function toJSON() {
      var json = {};

      for (var prop in this.style) {
        var value = this.style[prop];
        if (typeof value !== 'object') json[prop] = value;else if (Array.isArray(value)) json[prop] = toCssValue(value);
      }

      return json;
    }
    /**
     * Generates a CSS string.
     */
    ;

    _proto2.toString = function toString(options) {
      var sheet = this.options.sheet;
      var link = sheet ? sheet.options.link : false;
      var opts = link ? _extends$1({}, options, {
        allowEmpty: true
      }) : options;
      return toCss(this.selectorText, this.style, opts);
    };

    _createClass(StyleRule, [{
      key: "selector",
      set: function set(selector) {
        if (selector === this.selectorText) return;
        this.selectorText = selector;
        var renderer = this.renderer,
            renderable = this.renderable;
        if (!renderable || !renderer) return;
        var hasChanged = renderer.setSelector(renderable, selector); // If selector setter is not implemented, rerender the rule.

        if (!hasChanged) {
          renderer.replaceRule(renderable, this);
        }
      }
      /**
       * Get selector string.
       */
      ,
      get: function get() {
        return this.selectorText;
      }
    }]);

    return StyleRule;
  }(BaseStyleRule);
  var pluginStyleRule = {
    onCreateRule: function onCreateRule(name, style, options) {
      if (name[0] === '@' || options.parent && options.parent.type === 'keyframes') {
        return null;
      }

      return new StyleRule(name, style, options);
    }
  };

  var defaultToStringOptions = {
    indent: 1,
    children: true
  };
  var atRegExp = /@([\w-]+)/;
  /**
   * Conditional rule for @media, @supports
   */

  var ConditionalRule =
  /*#__PURE__*/
  function () {
    function ConditionalRule(key, styles, options) {
      this.type = 'conditional';
      this.at = void 0;
      this.key = void 0;
      this.query = void 0;
      this.rules = void 0;
      this.options = void 0;
      this.isProcessed = false;
      this.renderable = void 0;
      this.key = key; // Key might contain a unique suffix in case the `name` passed by user was duplicate.

      this.query = options.name;
      var atMatch = key.match(atRegExp);
      this.at = atMatch ? atMatch[1] : 'unknown';
      this.options = options;
      this.rules = new RuleList(_extends$1({}, options, {
        parent: this
      }));

      for (var name in styles) {
        this.rules.add(name, styles[name]);
      }

      this.rules.process();
    }
    /**
     * Get a rule.
     */


    var _proto = ConditionalRule.prototype;

    _proto.getRule = function getRule(name) {
      return this.rules.get(name);
    }
    /**
     * Get index of a rule.
     */
    ;

    _proto.indexOf = function indexOf(rule) {
      return this.rules.indexOf(rule);
    }
    /**
     * Create and register rule, run plugins.
     */
    ;

    _proto.addRule = function addRule(name, style, options) {
      var rule = this.rules.add(name, style, options);
      if (!rule) return null;
      this.options.jss.plugins.onProcessRule(rule);
      return rule;
    }
    /**
     * Generates a CSS string.
     */
    ;

    _proto.toString = function toString(options) {
      if (options === void 0) {
        options = defaultToStringOptions;
      }

      if (options.indent == null) options.indent = defaultToStringOptions.indent;
      if (options.children == null) options.children = defaultToStringOptions.children;

      if (options.children === false) {
        return this.query + " {}";
      }

      var children = this.rules.toString(options);
      return children ? this.query + " {\n" + children + "\n}" : '';
    };

    return ConditionalRule;
  }();
  var keyRegExp = /@media|@supports\s+/;
  var pluginConditionalRule = {
    onCreateRule: function onCreateRule(key, styles, options) {
      return keyRegExp.test(key) ? new ConditionalRule(key, styles, options) : null;
    }
  };

  var defaultToStringOptions$1 = {
    indent: 1,
    children: true
  };
  var nameRegExp = /@keyframes\s+([\w-]+)/;
  /**
   * Rule for @keyframes
   */

  var KeyframesRule =
  /*#__PURE__*/
  function () {
    function KeyframesRule(key, frames, options) {
      this.type = 'keyframes';
      this.at = '@keyframes';
      this.key = void 0;
      this.name = void 0;
      this.id = void 0;
      this.rules = void 0;
      this.options = void 0;
      this.isProcessed = false;
      this.renderable = void 0;
      var nameMatch = key.match(nameRegExp);

      if (nameMatch && nameMatch[1]) {
        this.name = nameMatch[1];
      } else {
        this.name = 'noname';
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Bad keyframes name " + key) : void 0;
      }

      this.key = this.type + "-" + this.name;
      this.options = options;
      var scoped = options.scoped,
          sheet = options.sheet,
          generateId = options.generateId;
      this.id = scoped === false ? this.name : escape(generateId(this, sheet));
      this.rules = new RuleList(_extends$1({}, options, {
        parent: this
      }));

      for (var name in frames) {
        this.rules.add(name, frames[name], _extends$1({}, options, {
          parent: this
        }));
      }

      this.rules.process();
    }
    /**
     * Generates a CSS string.
     */


    var _proto = KeyframesRule.prototype;

    _proto.toString = function toString(options) {
      if (options === void 0) {
        options = defaultToStringOptions$1;
      }

      if (options.indent == null) options.indent = defaultToStringOptions$1.indent;
      if (options.children == null) options.children = defaultToStringOptions$1.children;

      if (options.children === false) {
        return this.at + " " + this.id + " {}";
      }

      var children = this.rules.toString(options);
      if (children) children = "\n" + children + "\n";
      return this.at + " " + this.id + " {" + children + "}";
    };

    return KeyframesRule;
  }();
  var keyRegExp$1 = /@keyframes\s+/;
  var refRegExp$1 = /\$([\w-]+)/g;

  var findReferencedKeyframe = function findReferencedKeyframe(val, keyframes) {
    if (typeof val === 'string') {
      return val.replace(refRegExp$1, function (match, name) {
        if (name in keyframes) {
          return keyframes[name];
        }

        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Referenced keyframes rule \"" + name + "\" is not defined.") : void 0;
        return match;
      });
    }

    return val;
  };
  /**
   * Replace the reference for a animation name.
   */


  var replaceRef = function replaceRef(style, prop, keyframes) {
    var value = style[prop];
    var refKeyframe = findReferencedKeyframe(value, keyframes);

    if (refKeyframe !== value) {
      style[prop] = refKeyframe;
    }
  };

  var plugin = {
    onCreateRule: function onCreateRule(key, frames, options) {
      return typeof key === 'string' && keyRegExp$1.test(key) ? new KeyframesRule(key, frames, options) : null;
    },
    // Animation name ref replacer.
    onProcessStyle: function onProcessStyle(style, rule, sheet) {
      if (rule.type !== 'style' || !sheet) return style;
      if ('animation-name' in style) replaceRef(style, 'animation-name', sheet.keyframes);
      if ('animation' in style) replaceRef(style, 'animation', sheet.keyframes);
      return style;
    },
    onChangeValue: function onChangeValue(val, prop, rule) {
      var sheet = rule.options.sheet;

      if (!sheet) {
        return val;
      }

      switch (prop) {
        case 'animation':
          return findReferencedKeyframe(val, sheet.keyframes);

        case 'animation-name':
          return findReferencedKeyframe(val, sheet.keyframes);

        default:
          return val;
      }
    }
  };

  var KeyframeRule =
  /*#__PURE__*/
  function (_BaseStyleRule) {
    _inheritsLoose$1(KeyframeRule, _BaseStyleRule);

    function KeyframeRule() {
      var _this;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      _this = _BaseStyleRule.call.apply(_BaseStyleRule, [this].concat(args)) || this;
      _this.renderable = void 0;
      return _this;
    }

    var _proto = KeyframeRule.prototype;

    /**
     * Generates a CSS string.
     */
    _proto.toString = function toString(options) {
      var sheet = this.options.sheet;
      var link = sheet ? sheet.options.link : false;
      var opts = link ? _extends$1({}, options, {
        allowEmpty: true
      }) : options;
      return toCss(this.key, this.style, opts);
    };

    return KeyframeRule;
  }(BaseStyleRule);
  var pluginKeyframeRule = {
    onCreateRule: function onCreateRule(key, style, options) {
      if (options.parent && options.parent.type === 'keyframes') {
        return new KeyframeRule(key, style, options);
      }

      return null;
    }
  };

  var FontFaceRule =
  /*#__PURE__*/
  function () {
    function FontFaceRule(key, style, options) {
      this.type = 'font-face';
      this.at = '@font-face';
      this.key = void 0;
      this.style = void 0;
      this.options = void 0;
      this.isProcessed = false;
      this.renderable = void 0;
      this.key = key;
      this.style = style;
      this.options = options;
    }
    /**
     * Generates a CSS string.
     */


    var _proto = FontFaceRule.prototype;

    _proto.toString = function toString(options) {
      if (Array.isArray(this.style)) {
        var str = '';

        for (var index = 0; index < this.style.length; index++) {
          str += toCss(this.at, this.style[index]);
          if (this.style[index + 1]) str += '\n';
        }

        return str;
      }

      return toCss(this.at, this.style, options);
    };

    return FontFaceRule;
  }();
  var keyRegExp$2 = /@font-face/;
  var pluginFontFaceRule = {
    onCreateRule: function onCreateRule(key, style, options) {
      return keyRegExp$2.test(key) ? new FontFaceRule(key, style, options) : null;
    }
  };

  var ViewportRule =
  /*#__PURE__*/
  function () {
    function ViewportRule(key, style, options) {
      this.type = 'viewport';
      this.at = '@viewport';
      this.key = void 0;
      this.style = void 0;
      this.options = void 0;
      this.isProcessed = false;
      this.renderable = void 0;
      this.key = key;
      this.style = style;
      this.options = options;
    }
    /**
     * Generates a CSS string.
     */


    var _proto = ViewportRule.prototype;

    _proto.toString = function toString(options) {
      return toCss(this.key, this.style, options);
    };

    return ViewportRule;
  }();
  var pluginViewportRule = {
    onCreateRule: function onCreateRule(key, style, options) {
      return key === '@viewport' || key === '@-ms-viewport' ? new ViewportRule(key, style, options) : null;
    }
  };

  var SimpleRule =
  /*#__PURE__*/
  function () {
    function SimpleRule(key, value, options) {
      this.type = 'simple';
      this.key = void 0;
      this.value = void 0;
      this.options = void 0;
      this.isProcessed = false;
      this.renderable = void 0;
      this.key = key;
      this.value = value;
      this.options = options;
    }
    /**
     * Generates a CSS string.
     */
    // eslint-disable-next-line no-unused-vars


    var _proto = SimpleRule.prototype;

    _proto.toString = function toString(options) {
      if (Array.isArray(this.value)) {
        var str = '';

        for (var index = 0; index < this.value.length; index++) {
          str += this.key + " " + this.value[index] + ";";
          if (this.value[index + 1]) str += '\n';
        }

        return str;
      }

      return this.key + " " + this.value + ";";
    };

    return SimpleRule;
  }();
  var keysMap = {
    '@charset': true,
    '@import': true,
    '@namespace': true
  };
  var pluginSimpleRule = {
    onCreateRule: function onCreateRule(key, value, options) {
      return key in keysMap ? new SimpleRule(key, value, options) : null;
    }
  };

  var plugins$1 = [pluginStyleRule, pluginConditionalRule, plugin, pluginKeyframeRule, pluginFontFaceRule, pluginViewportRule, pluginSimpleRule];

  var defaultUpdateOptions = {
    process: true
  };
  var forceUpdateOptions = {
    force: true,
    process: true
    /**
     * Contains rules objects and allows adding/removing etc.
     * Is used for e.g. by `StyleSheet` or `ConditionalRule`.
     */

  };

  var RuleList =
  /*#__PURE__*/
  function () {
    // Rules registry for access by .get() method.
    // It contains the same rule registered by name and by selector.
    // Original styles object.
    // Used to ensure correct rules order.
    function RuleList(options) {
      this.map = {};
      this.raw = {};
      this.index = [];
      this.counter = 0;
      this.options = void 0;
      this.classes = void 0;
      this.keyframes = void 0;
      this.options = options;
      this.classes = options.classes;
      this.keyframes = options.keyframes;
    }
    /**
     * Create and register rule.
     *
     * Will not render after Style Sheet was rendered the first time.
     */


    var _proto = RuleList.prototype;

    _proto.add = function add(name, decl, ruleOptions) {
      var _this$options = this.options,
          parent = _this$options.parent,
          sheet = _this$options.sheet,
          jss = _this$options.jss,
          Renderer = _this$options.Renderer,
          generateId = _this$options.generateId,
          scoped = _this$options.scoped;

      var options = _extends$1({
        classes: this.classes,
        parent: parent,
        sheet: sheet,
        jss: jss,
        Renderer: Renderer,
        generateId: generateId,
        scoped: scoped,
        name: name
      }, ruleOptions); // When user uses .createStyleSheet(), duplicate names are not possible, but
      // `sheet.addRule()` opens the door for any duplicate rule name. When this happens
      // we need to make the key unique within this RuleList instance scope.


      var key = name;

      if (name in this.raw) {
        key = name + "-d" + this.counter++;
      } // We need to save the original decl before creating the rule
      // because cache plugin needs to use it as a key to return a cached rule.


      this.raw[key] = decl;

      if (key in this.classes) {
        // E.g. rules inside of @media container
        options.selector = "." + escape(this.classes[key]);
      }

      var rule = createRule(key, decl, options);
      if (!rule) return null;
      this.register(rule);
      var index = options.index === undefined ? this.index.length : options.index;
      this.index.splice(index, 0, rule);
      return rule;
    }
    /**
     * Get a rule.
     */
    ;

    _proto.get = function get(name) {
      return this.map[name];
    }
    /**
     * Delete a rule.
     */
    ;

    _proto.remove = function remove(rule) {
      this.unregister(rule);
      delete this.raw[rule.key];
      this.index.splice(this.index.indexOf(rule), 1);
    }
    /**
     * Get index of a rule.
     */
    ;

    _proto.indexOf = function indexOf(rule) {
      return this.index.indexOf(rule);
    }
    /**
     * Run `onProcessRule()` plugins on every rule.
     */
    ;

    _proto.process = function process() {
      var plugins = this.options.jss.plugins; // We need to clone array because if we modify the index somewhere else during a loop
      // we end up with very hard-to-track-down side effects.

      this.index.slice(0).forEach(plugins.onProcessRule, plugins);
    }
    /**
     * Register a rule in `.map`, `.classes` and `.keyframes` maps.
     */
    ;

    _proto.register = function register(rule) {
      this.map[rule.key] = rule;

      if (rule instanceof StyleRule) {
        this.map[rule.selector] = rule;
        if (rule.id) this.classes[rule.key] = rule.id;
      } else if (rule instanceof KeyframesRule && this.keyframes) {
        this.keyframes[rule.name] = rule.id;
      }
    }
    /**
     * Unregister a rule.
     */
    ;

    _proto.unregister = function unregister(rule) {
      delete this.map[rule.key];

      if (rule instanceof StyleRule) {
        delete this.map[rule.selector];
        delete this.classes[rule.key];
      } else if (rule instanceof KeyframesRule) {
        delete this.keyframes[rule.name];
      }
    }
    /**
     * Update the function values with a new data.
     */
    ;

    _proto.update = function update() {
      var name;
      var data;
      var options;

      if (typeof (arguments.length <= 0 ? undefined : arguments[0]) === 'string') {
        name = arguments.length <= 0 ? undefined : arguments[0]; // $FlowFixMe

        data = arguments.length <= 1 ? undefined : arguments[1]; // $FlowFixMe

        options = arguments.length <= 2 ? undefined : arguments[2];
      } else {
        data = arguments.length <= 0 ? undefined : arguments[0]; // $FlowFixMe

        options = arguments.length <= 1 ? undefined : arguments[1];
        name = null;
      }

      if (name) {
        this.updateOne(this.map[name], data, options);
      } else {
        for (var index = 0; index < this.index.length; index++) {
          this.updateOne(this.index[index], data, options);
        }
      }
    }
    /**
     * Execute plugins, update rule props.
     */
    ;

    _proto.updateOne = function updateOne(rule, data, options) {
      if (options === void 0) {
        options = defaultUpdateOptions;
      }

      var _this$options2 = this.options,
          plugins = _this$options2.jss.plugins,
          sheet = _this$options2.sheet; // It is a rules container like for e.g. ConditionalRule.

      if (rule.rules instanceof RuleList) {
        rule.rules.update(data, options);
        return;
      }

      var styleRule = rule;
      var style = styleRule.style;
      plugins.onUpdate(data, rule, sheet, options); // We rely on a new `style` ref in case it was mutated during onUpdate hook.

      if (options.process && style && style !== styleRule.style) {
        // We need to run the plugins in case new `style` relies on syntax plugins.
        plugins.onProcessStyle(styleRule.style, styleRule, sheet); // Update and add props.

        for (var prop in styleRule.style) {
          var nextValue = styleRule.style[prop];
          var prevValue = style[prop]; // We need to use `force: true` because `rule.style` has been updated during onUpdate hook, so `rule.prop()` will not update the CSSOM rule.
          // We do this comparison to avoid unneeded `rule.prop()` calls, since we have the old `style` object here.

          if (nextValue !== prevValue) {
            styleRule.prop(prop, nextValue, forceUpdateOptions);
          }
        } // Remove props.


        for (var _prop in style) {
          var _nextValue = styleRule.style[_prop];
          var _prevValue = style[_prop]; // We need to use `force: true` because `rule.style` has been updated during onUpdate hook, so `rule.prop()` will not update the CSSOM rule.
          // We do this comparison to avoid unneeded `rule.prop()` calls, since we have the old `style` object here.

          if (_nextValue == null && _nextValue !== _prevValue) {
            styleRule.prop(_prop, null, forceUpdateOptions);
          }
        }
      }
    }
    /**
     * Convert rules to a CSS string.
     */
    ;

    _proto.toString = function toString(options) {
      var str = '';
      var sheet = this.options.sheet;
      var link = sheet ? sheet.options.link : false;

      for (var index = 0; index < this.index.length; index++) {
        var rule = this.index[index];
        var css = rule.toString(options); // No need to render an empty rule.

        if (!css && !link) continue;
        if (str) str += '\n';
        str += css;
      }

      return str;
    };

    return RuleList;
  }();

  var StyleSheet =
  /*#__PURE__*/
  function () {
    function StyleSheet(styles, options) {
      this.options = void 0;
      this.deployed = void 0;
      this.attached = void 0;
      this.rules = void 0;
      this.renderer = void 0;
      this.classes = void 0;
      this.keyframes = void 0;
      this.queue = void 0;
      this.attached = false;
      this.deployed = false;
      this.classes = {};
      this.keyframes = {};
      this.options = _extends$1({}, options, {
        sheet: this,
        parent: this,
        classes: this.classes,
        keyframes: this.keyframes
      });

      if (options.Renderer) {
        this.renderer = new options.Renderer(this);
      }

      this.rules = new RuleList(this.options);

      for (var name in styles) {
        this.rules.add(name, styles[name]);
      }

      this.rules.process();
    }
    /**
     * Attach renderable to the render tree.
     */


    var _proto = StyleSheet.prototype;

    _proto.attach = function attach() {
      if (this.attached) return this;
      if (this.renderer) this.renderer.attach();
      this.attached = true; // Order is important, because we can't use insertRule API if style element is not attached.

      if (!this.deployed) this.deploy();
      return this;
    }
    /**
     * Remove renderable from render tree.
     */
    ;

    _proto.detach = function detach() {
      if (!this.attached) return this;
      if (this.renderer) this.renderer.detach();
      this.attached = false;
      return this;
    }
    /**
     * Add a rule to the current stylesheet.
     * Will insert a rule also after the stylesheet has been rendered first time.
     */
    ;

    _proto.addRule = function addRule(name, decl, options) {
      var queue = this.queue; // Plugins can create rules.
      // In order to preserve the right order, we need to queue all `.addRule` calls,
      // which happen after the first `rules.add()` call.

      if (this.attached && !queue) this.queue = [];
      var rule = this.rules.add(name, decl, options);
      if (!rule) return null;
      this.options.jss.plugins.onProcessRule(rule);

      if (this.attached) {
        if (!this.deployed) return rule; // Don't insert rule directly if there is no stringified version yet.
        // It will be inserted all together when .attach is called.

        if (queue) queue.push(rule);else {
          this.insertRule(rule);

          if (this.queue) {
            this.queue.forEach(this.insertRule, this);
            this.queue = undefined;
          }
        }
        return rule;
      } // We can't add rules to a detached style node.
      // We will redeploy the sheet once user will attach it.


      this.deployed = false;
      return rule;
    }
    /**
     * Insert rule into the StyleSheet
     */
    ;

    _proto.insertRule = function insertRule(rule) {
      if (this.renderer) {
        this.renderer.insertRule(rule);
      }
    }
    /**
     * Create and add rules.
     * Will render also after Style Sheet was rendered the first time.
     */
    ;

    _proto.addRules = function addRules(styles, options) {
      var added = [];

      for (var name in styles) {
        var rule = this.addRule(name, styles[name], options);
        if (rule) added.push(rule);
      }

      return added;
    }
    /**
     * Get a rule by name.
     */
    ;

    _proto.getRule = function getRule(name) {
      return this.rules.get(name);
    }
    /**
     * Delete a rule by name.
     * Returns `true`: if rule has been deleted from the DOM.
     */
    ;

    _proto.deleteRule = function deleteRule(name) {
      var rule = typeof name === 'object' ? name : this.rules.get(name);
      if (!rule) return false;
      this.rules.remove(rule);

      if (this.attached && rule.renderable && this.renderer) {
        return this.renderer.deleteRule(rule.renderable);
      }

      return true;
    }
    /**
     * Get index of a rule.
     */
    ;

    _proto.indexOf = function indexOf(rule) {
      return this.rules.indexOf(rule);
    }
    /**
     * Deploy pure CSS string to a renderable.
     */
    ;

    _proto.deploy = function deploy() {
      if (this.renderer) this.renderer.deploy();
      this.deployed = true;
      return this;
    }
    /**
     * Update the function values with a new data.
     */
    ;

    _proto.update = function update() {
      var _this$rules;

      (_this$rules = this.rules).update.apply(_this$rules, arguments);

      return this;
    }
    /**
     * Updates a single rule.
     */
    ;

    _proto.updateOne = function updateOne(rule, data, options) {
      this.rules.updateOne(rule, data, options);
      return this;
    }
    /**
     * Convert rules to a CSS string.
     */
    ;

    _proto.toString = function toString(options) {
      return this.rules.toString(options);
    };

    return StyleSheet;
  }();

  var PluginsRegistry =
  /*#__PURE__*/
  function () {
    function PluginsRegistry() {
      this.plugins = {
        internal: [],
        external: []
      };
      this.registry = void 0;
    }

    var _proto = PluginsRegistry.prototype;

    /**
     * Call `onCreateRule` hooks and return an object if returned by a hook.
     */
    _proto.onCreateRule = function onCreateRule(name, decl, options) {
      for (var i = 0; i < this.registry.onCreateRule.length; i++) {
        var rule = this.registry.onCreateRule[i](name, decl, options);
        if (rule) return rule;
      }

      return null;
    }
    /**
     * Call `onProcessRule` hooks.
     */
    ;

    _proto.onProcessRule = function onProcessRule(rule) {
      if (rule.isProcessed) return;
      var sheet = rule.options.sheet;

      for (var i = 0; i < this.registry.onProcessRule.length; i++) {
        this.registry.onProcessRule[i](rule, sheet);
      }

      if (rule.style) this.onProcessStyle(rule.style, rule, sheet);
      rule.isProcessed = true;
    }
    /**
     * Call `onProcessStyle` hooks.
     */
    ;

    _proto.onProcessStyle = function onProcessStyle(style, rule, sheet) {
      for (var i = 0; i < this.registry.onProcessStyle.length; i++) {
        // $FlowFixMe
        rule.style = this.registry.onProcessStyle[i](rule.style, rule, sheet);
      }
    }
    /**
     * Call `onProcessSheet` hooks.
     */
    ;

    _proto.onProcessSheet = function onProcessSheet(sheet) {
      for (var i = 0; i < this.registry.onProcessSheet.length; i++) {
        this.registry.onProcessSheet[i](sheet);
      }
    }
    /**
     * Call `onUpdate` hooks.
     */
    ;

    _proto.onUpdate = function onUpdate(data, rule, sheet, options) {
      for (var i = 0; i < this.registry.onUpdate.length; i++) {
        this.registry.onUpdate[i](data, rule, sheet, options);
      }
    }
    /**
     * Call `onChangeValue` hooks.
     */
    ;

    _proto.onChangeValue = function onChangeValue(value, prop, rule) {
      var processedValue = value;

      for (var i = 0; i < this.registry.onChangeValue.length; i++) {
        processedValue = this.registry.onChangeValue[i](processedValue, prop, rule);
      }

      return processedValue;
    }
    /**
     * Register a plugin.
     */
    ;

    _proto.use = function use(newPlugin, options) {
      if (options === void 0) {
        options = {
          queue: 'external'
        };
      }

      var plugins = this.plugins[options.queue]; // Avoids applying same plugin twice, at least based on ref.

      if (plugins.indexOf(newPlugin) !== -1) {
        return;
      }

      plugins.push(newPlugin);
      this.registry = [].concat(this.plugins.external, this.plugins.internal).reduce(function (registry, plugin) {
        for (var name in plugin) {
          if (name in registry) {
            registry[name].push(plugin[name]);
          } else {
            process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Unknown hook \"" + name + "\".") : void 0;
          }
        }

        return registry;
      }, {
        onCreateRule: [],
        onProcessRule: [],
        onProcessStyle: [],
        onProcessSheet: [],
        onChangeValue: [],
        onUpdate: []
      });
    };

    return PluginsRegistry;
  }();

  /**
   * Sheets registry to access them all at one place.
   */
  var SheetsRegistry =
  /*#__PURE__*/
  function () {
    function SheetsRegistry() {
      this.registry = [];
    }

    var _proto = SheetsRegistry.prototype;

    /**
     * Register a Style Sheet.
     */
    _proto.add = function add(sheet) {
      var registry = this.registry;
      var index = sheet.options.index;
      if (registry.indexOf(sheet) !== -1) return;

      if (registry.length === 0 || index >= this.index) {
        registry.push(sheet);
        return;
      } // Find a position.


      for (var i = 0; i < registry.length; i++) {
        if (registry[i].options.index > index) {
          registry.splice(i, 0, sheet);
          return;
        }
      }
    }
    /**
     * Reset the registry.
     */
    ;

    _proto.reset = function reset() {
      this.registry = [];
    }
    /**
     * Remove a Style Sheet.
     */
    ;

    _proto.remove = function remove(sheet) {
      var index = this.registry.indexOf(sheet);
      this.registry.splice(index, 1);
    }
    /**
     * Convert all attached sheets to a CSS string.
     */
    ;

    _proto.toString = function toString(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
          attached = _ref.attached,
          options = _objectWithoutPropertiesLoose(_ref, ["attached"]);

      var css = '';

      for (var i = 0; i < this.registry.length; i++) {
        var sheet = this.registry[i];

        if (attached != null && sheet.attached !== attached) {
          continue;
        }

        if (css) css += '\n';
        css += sheet.toString(options);
      }

      return css;
    };

    _createClass(SheetsRegistry, [{
      key: "index",

      /**
       * Current highest index number.
       */
      get: function get() {
        return this.registry.length === 0 ? 0 : this.registry[this.registry.length - 1].options.index;
      }
    }]);

    return SheetsRegistry;
  }();

  /**
   * This is a global sheets registry. Only DomRenderer will add sheets to it.
   * On the server one should use an own SheetsRegistry instance and add the
   * sheets to it, because you need to make sure to create a new registry for
   * each request in order to not leak sheets across requests.
   */

  var sheets = new SheetsRegistry();

  /* eslint-disable */
  // https://github.com/zloirock/core-js/issues/86#issuecomment-115759028
  var globalThis = typeof window != 'undefined' && window.Math == Math ? window : typeof self != 'undefined' && self.Math == Math ? self : Function('return this')();

  var ns = '2f1acc6c3a606b082e5eef5e54414ffb';
  if (globalThis[ns] == null) globalThis[ns] = 0; // Bundle may contain multiple JSS versions at the same time. In order to identify
  // the current version with just one short number and use it for classes generation
  // we use a counter. Also it is more accurate, because user can manually reevaluate
  // the module.

  var moduleId = globalThis[ns]++;

  var maxRules = 1e10;

  /**
   * Returns a function which generates unique class names based on counters.
   * When new generator function is created, rule counter is reseted.
   * We need to reset the rule counter for SSR for each request.
   */
  var createGenerateId = function createGenerateId(options) {
    if (options === void 0) {
      options = {};
    }

    var ruleCounter = 0;
    return function (rule, sheet) {
      ruleCounter += 1;

      if (ruleCounter > maxRules) {
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] You might have a memory leak. Rule counter is at " + ruleCounter + ".") : void 0;
      }

      var jssId = '';
      var prefix = '';

      if (sheet) {
        if (sheet.options.classNamePrefix) {
          prefix = sheet.options.classNamePrefix;
        }

        if (sheet.options.jss.id != null) {
          jssId = String(sheet.options.jss.id);
        }
      }

      if (options.minify) {
        // Using "c" because a number can't be the first char in a class name.
        return "" + (prefix || 'c') + moduleId + jssId + ruleCounter;
      }

      return prefix + rule.key + "-" + moduleId + (jssId ? "-" + jssId : '') + "-" + ruleCounter;
    };
  };

  /**
   * Cache the value from the first time a function is called.
   */
  var memoize = function memoize(fn) {
    var value;
    return function () {
      if (!value) value = fn();
      return value;
    };
  };
  /**
   * Get a style property value.
   */


  function getPropertyValue(cssRule, prop) {
    try {
      // Support CSSTOM.
      if (cssRule.attributeStyleMap) {
        return cssRule.attributeStyleMap.get(prop);
      }

      return cssRule.style.getPropertyValue(prop);
    } catch (err) {
      // IE may throw if property is unknown.
      return '';
    }
  }
  /**
   * Set a style property.
   */


  function setProperty(cssRule, prop, value) {
    try {
      var cssValue = value;

      if (Array.isArray(value)) {
        cssValue = toCssValue(value, true);

        if (value[value.length - 1] === '!important') {
          cssRule.style.setProperty(prop, cssValue, 'important');
          return true;
        }
      } // Support CSSTOM.


      if (cssRule.attributeStyleMap) {
        cssRule.attributeStyleMap.set(prop, cssValue);
      } else {
        cssRule.style.setProperty(prop, cssValue);
      }
    } catch (err) {
      // IE may throw if property is unknown.
      return false;
    }

    return true;
  }
  /**
   * Remove a style property.
   */


  function removeProperty(cssRule, prop) {
    try {
      // Support CSSTOM.
      if (cssRule.attributeStyleMap) {
        cssRule.attributeStyleMap.delete(prop);
      } else {
        cssRule.style.removeProperty(prop);
      }
    } catch (err) {
      process.env.NODE_ENV !== "production" ? warning(false, "[JSS] DOMException \"" + err.message + "\" was thrown. Tried to remove property \"" + prop + "\".") : void 0;
    }
  }
  /**
   * Set the selector.
   */


  function setSelector(cssRule, selectorText) {
    cssRule.selectorText = selectorText; // Return false if setter was not successful.
    // Currently works in chrome only.

    return cssRule.selectorText === selectorText;
  }
  /**
   * Gets the `head` element upon the first call and caches it.
   * We assume it can't be null.
   */


  var getHead = memoize(function () {
    return document.querySelector('head');
  });
  /**
   * Find attached sheet with an index higher than the passed one.
   */

  function findHigherSheet(registry, options) {
    for (var i = 0; i < registry.length; i++) {
      var sheet = registry[i];

      if (sheet.attached && sheet.options.index > options.index && sheet.options.insertionPoint === options.insertionPoint) {
        return sheet;
      }
    }

    return null;
  }
  /**
   * Find attached sheet with the highest index.
   */


  function findHighestSheet(registry, options) {
    for (var i = registry.length - 1; i >= 0; i--) {
      var sheet = registry[i];

      if (sheet.attached && sheet.options.insertionPoint === options.insertionPoint) {
        return sheet;
      }
    }

    return null;
  }
  /**
   * Find a comment with "jss" inside.
   */


  function findCommentNode(text) {
    var head = getHead();

    for (var i = 0; i < head.childNodes.length; i++) {
      var node = head.childNodes[i];

      if (node.nodeType === 8 && node.nodeValue.trim() === text) {
        return node;
      }
    }

    return null;
  }

  /**
   * Find a node before which we can insert the sheet.
   */
  function findPrevNode(options) {
    var registry = sheets.registry;

    if (registry.length > 0) {
      // Try to insert before the next higher sheet.
      var sheet = findHigherSheet(registry, options);

      if (sheet && sheet.renderer) {
        return {
          parent: sheet.renderer.element.parentNode,
          node: sheet.renderer.element
        };
      } // Otherwise insert after the last attached.


      sheet = findHighestSheet(registry, options);

      if (sheet && sheet.renderer) {
        return {
          parent: sheet.renderer.element.parentNode,
          node: sheet.renderer.element.nextSibling
        };
      }
    } // Try to find a comment placeholder if registry is empty.


    var insertionPoint = options.insertionPoint;

    if (insertionPoint && typeof insertionPoint === 'string') {
      var comment = findCommentNode(insertionPoint);

      if (comment) {
        return {
          parent: comment.parentNode,
          node: comment.nextSibling
        };
      } // If user specifies an insertion point and it can't be found in the document -
      // bad specificity issues may appear.


      process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Insertion point \"" + insertionPoint + "\" not found.") : void 0;
    }

    return false;
  }
  /**
   * Insert style element into the DOM.
   */


  function insertStyle(style, options) {
    var insertionPoint = options.insertionPoint;
    var nextNode = findPrevNode(options);

    if (nextNode !== false && nextNode.parent) {
      nextNode.parent.insertBefore(style, nextNode.node);
      return;
    } // Works with iframes and any node types.


    if (insertionPoint && typeof insertionPoint.nodeType === 'number') {
      // https://stackoverflow.com/questions/41328728/force-casting-in-flow
      var insertionPointElement = insertionPoint;
      var parentNode = insertionPointElement.parentNode;
      if (parentNode) parentNode.insertBefore(style, insertionPointElement.nextSibling);else process.env.NODE_ENV !== "production" ? warning(false, '[JSS] Insertion point is not in the DOM.') : void 0;
      return;
    }

    getHead().appendChild(style);
  }
  /**
   * Read jss nonce setting from the page if the user has set it.
   */


  var getNonce = memoize(function () {
    var node = document.querySelector('meta[property="csp-nonce"]');
    return node ? node.getAttribute('content') : null;
  });

  var _insertRule = function insertRule(container, rule, index) {
    var maxIndex = container.cssRules.length; // In case previous insertion fails, passed index might be wrong

    if (index === undefined || index > maxIndex) {
      // eslint-disable-next-line no-param-reassign
      index = maxIndex;
    }

    try {
      if ('insertRule' in container) {
        var c = container;
        c.insertRule(rule, index);
      } // Keyframes rule.
      else if ('appendRule' in container) {
          var _c = container;

          _c.appendRule(rule);
        }
    } catch (err) {
      process.env.NODE_ENV !== "production" ? warning(false, "[JSS] " + err.message) : void 0;
      return false;
    }

    return container.cssRules[index];
  };

  var createStyle = function createStyle() {
    var el = document.createElement('style'); // Without it, IE will have a broken source order specificity if we
    // insert rules after we insert the style tag.
    // It seems to kick-off the source order specificity algorithm.

    el.textContent = '\n';
    return el;
  };

  var DomRenderer =
  /*#__PURE__*/
  function () {
    // HTMLStyleElement needs fixing https://github.com/facebook/flow/issues/2696
    function DomRenderer(sheet) {
      this.getPropertyValue = getPropertyValue;
      this.setProperty = setProperty;
      this.removeProperty = removeProperty;
      this.setSelector = setSelector;
      this.element = void 0;
      this.sheet = void 0;
      this.hasInsertedRules = false;
      // There is no sheet when the renderer is used from a standalone StyleRule.
      if (sheet) sheets.add(sheet);
      this.sheet = sheet;

      var _ref = this.sheet ? this.sheet.options : {},
          media = _ref.media,
          meta = _ref.meta,
          element = _ref.element;

      this.element = element || createStyle();
      this.element.setAttribute('data-jss', '');
      if (media) this.element.setAttribute('media', media);
      if (meta) this.element.setAttribute('data-meta', meta);
      var nonce = getNonce();
      if (nonce) this.element.setAttribute('nonce', nonce);
    }
    /**
     * Insert style element into render tree.
     */


    var _proto = DomRenderer.prototype;

    _proto.attach = function attach() {
      // In the case the element node is external and it is already in the DOM.
      if (this.element.parentNode || !this.sheet) return;
      insertStyle(this.element, this.sheet.options); // When rules are inserted using `insertRule` API, after `sheet.detach().attach()`
      // most browsers create a new CSSStyleSheet, except of all IEs.

      var deployed = Boolean(this.sheet && this.sheet.deployed);

      if (this.hasInsertedRules && deployed) {
        this.hasInsertedRules = false;
        this.deploy();
      }
    }
    /**
     * Remove style element from render tree.
     */
    ;

    _proto.detach = function detach() {
      var parentNode = this.element.parentNode;
      if (parentNode) parentNode.removeChild(this.element);
    }
    /**
     * Inject CSS string into element.
     */
    ;

    _proto.deploy = function deploy() {
      var sheet = this.sheet;
      if (!sheet) return;

      if (sheet.options.link) {
        this.insertRules(sheet.rules);
        return;
      }

      this.element.textContent = "\n" + sheet.toString() + "\n";
    }
    /**
     * Insert RuleList into an element.
     */
    ;

    _proto.insertRules = function insertRules(rules, nativeParent) {
      for (var i = 0; i < rules.index.length; i++) {
        this.insertRule(rules.index[i], i, nativeParent);
      }
    }
    /**
     * Insert a rule into element.
     */
    ;

    _proto.insertRule = function insertRule(rule, index, nativeParent) {
      if (nativeParent === void 0) {
        nativeParent = this.element.sheet;
      }

      if (rule.rules) {
        var parent = rule;
        var latestNativeParent = nativeParent;

        if (rule.type === 'conditional' || rule.type === 'keyframes') {
          // We need to render the container without children first.
          latestNativeParent = _insertRule(nativeParent, parent.toString({
            children: false
          }), index);

          if (latestNativeParent === false) {
            return false;
          }
        }

        this.insertRules(parent.rules, latestNativeParent);
        return latestNativeParent;
      } // IE keeps the CSSStyleSheet after style node has been reattached,
      // so we need to check if the `renderable` reference the right style sheet and not
      // rerender those rules.


      if (rule.renderable && rule.renderable.parentStyleSheet === this.element.sheet) {
        return rule.renderable;
      }

      var ruleStr = rule.toString();
      if (!ruleStr) return false;

      var nativeRule = _insertRule(nativeParent, ruleStr, index);

      if (nativeRule === false) {
        return false;
      }

      this.hasInsertedRules = true;
      rule.renderable = nativeRule;
      return nativeRule;
    }
    /**
     * Delete a rule.
     */
    ;

    _proto.deleteRule = function deleteRule(cssRule) {
      var sheet = this.element.sheet;
      var index = this.indexOf(cssRule);
      if (index === -1) return false;
      sheet.deleteRule(index);
      return true;
    }
    /**
     * Get index of a CSS Rule.
     */
    ;

    _proto.indexOf = function indexOf(cssRule) {
      var cssRules = this.element.sheet.cssRules;

      for (var index = 0; index < cssRules.length; index++) {
        if (cssRule === cssRules[index]) return index;
      }

      return -1;
    }
    /**
     * Generate a new CSS rule and replace the existing one.
     *
     * Only used for some old browsers because they can't set a selector.
     */
    ;

    _proto.replaceRule = function replaceRule(cssRule, rule) {
      var index = this.indexOf(cssRule);
      if (index === -1) return false;
      this.element.sheet.deleteRule(index);
      return this.insertRule(rule, index);
    }
    /**
     * Get all rules elements.
     */
    ;

    _proto.getRules = function getRules() {
      return this.element.sheet.cssRules;
    };

    return DomRenderer;
  }();

  var instanceCounter = 0;

  var Jss =
  /*#__PURE__*/
  function () {
    function Jss(options) {
      this.id = instanceCounter++;
      this.version = "10.2.0";
      this.plugins = new PluginsRegistry();
      this.options = {
        id: {
          minify: false
        },
        createGenerateId: createGenerateId,
        Renderer: isBrowser ? DomRenderer : null,
        plugins: []
      };
      this.generateId = createGenerateId({
        minify: false
      });

      for (var i = 0; i < plugins$1.length; i++) {
        this.plugins.use(plugins$1[i], {
          queue: 'internal'
        });
      }

      this.setup(options);
    }
    /**
     * Prepares various options, applies plugins.
     * Should not be used twice on the same instance, because there is no plugins
     * deduplication logic.
     */


    var _proto = Jss.prototype;

    _proto.setup = function setup(options) {
      if (options === void 0) {
        options = {};
      }

      if (options.createGenerateId) {
        this.options.createGenerateId = options.createGenerateId;
      }

      if (options.id) {
        this.options.id = _extends$1({}, this.options.id, options.id);
      }

      if (options.createGenerateId || options.id) {
        this.generateId = this.options.createGenerateId(this.options.id);
      }

      if (options.insertionPoint != null) this.options.insertionPoint = options.insertionPoint;

      if ('Renderer' in options) {
        this.options.Renderer = options.Renderer;
      } // eslint-disable-next-line prefer-spread


      if (options.plugins) this.use.apply(this, options.plugins);
      return this;
    }
    /**
     * Create a Style Sheet.
     */
    ;

    _proto.createStyleSheet = function createStyleSheet(styles, options) {
      if (options === void 0) {
        options = {};
      }

      var _options = options,
          index = _options.index;

      if (typeof index !== 'number') {
        index = sheets.index === 0 ? 0 : sheets.index + 1;
      }

      var sheet = new StyleSheet(styles, _extends$1({}, options, {
        jss: this,
        generateId: options.generateId || this.generateId,
        insertionPoint: this.options.insertionPoint,
        Renderer: this.options.Renderer,
        index: index
      }));
      this.plugins.onProcessSheet(sheet);
      return sheet;
    }
    /**
     * Detach the Style Sheet and remove it from the registry.
     */
    ;

    _proto.removeStyleSheet = function removeStyleSheet(sheet) {
      sheet.detach();
      sheets.remove(sheet);
      return this;
    }
    /**
     * Create a rule without a Style Sheet.
     * [Deprecated] will be removed in the next major version.
     */
    ;

    _proto.createRule = function createRule$1(name, style, options) {
      if (style === void 0) {
        style = {};
      }

      if (options === void 0) {
        options = {};
      }

      // Enable rule without name for inline styles.
      if (typeof name === 'object') {
        return this.createRule(undefined, name, style);
      }

      var ruleOptions = _extends$1({}, options, {
        name: name,
        jss: this,
        Renderer: this.options.Renderer
      });

      if (!ruleOptions.generateId) ruleOptions.generateId = this.generateId;
      if (!ruleOptions.classes) ruleOptions.classes = {};
      if (!ruleOptions.keyframes) ruleOptions.keyframes = {};

      var rule = createRule(name, style, ruleOptions);

      if (rule) this.plugins.onProcessRule(rule);
      return rule;
    }
    /**
     * Register plugin. Passed function will be invoked with a rule instance.
     */
    ;

    _proto.use = function use() {
      var _this = this;

      for (var _len = arguments.length, plugins = new Array(_len), _key = 0; _key < _len; _key++) {
        plugins[_key] = arguments[_key];
      }

      plugins.forEach(function (plugin) {
        _this.plugins.use(plugin);
      });
      return this;
    };

    return Jss;
  }();

  /**
   * Extracts a styles object with only props that contain function values.
   */
  function getDynamicStyles(styles) {
    var to = null;

    for (var key in styles) {
      var value = styles[key];
      var type = typeof value;

      if (type === 'function') {
        if (!to) to = {};
        to[key] = value;
      } else if (type === 'object' && value !== null && !Array.isArray(value)) {
        var extracted = getDynamicStyles(value);

        if (extracted) {
          if (!to) to = {};
          to[key] = extracted;
        }
      }
    }

    return to;
  }

  /**
   * SheetsManager is like a WeakMap which is designed to count StyleSheet
   * instances and attach/detach automatically.
   */
  var SheetsManager =
  /*#__PURE__*/
  function () {
    function SheetsManager() {
      this.length = 0;
      this.sheets = new WeakMap();
    }

    var _proto = SheetsManager.prototype;

    _proto.get = function get(key) {
      var entry = this.sheets.get(key);
      return entry && entry.sheet;
    };

    _proto.add = function add(key, sheet) {
      if (this.sheets.has(key)) return;
      this.length++;
      this.sheets.set(key, {
        sheet: sheet,
        refs: 0
      });
    };

    _proto.manage = function manage(key) {
      var entry = this.sheets.get(key);

      if (entry) {
        if (entry.refs === 0) {
          entry.sheet.attach();
        }

        entry.refs++;
        return entry.sheet;
      }

      warning(false, "[JSS] SheetsManager: can't find sheet to manage");
      return undefined;
    };

    _proto.unmanage = function unmanage(key) {
      var entry = this.sheets.get(key);

      if (entry) {
        if (entry.refs > 0) {
          entry.refs--;
          if (entry.refs === 0) entry.sheet.detach();
        }
      } else {
        warning(false, "SheetsManager: can't find sheet to unmanage");
      }
    };

    _createClass(SheetsManager, [{
      key: "size",
      get: function get() {
        return this.length;
      }
    }]);

    return SheetsManager;
  }();

  /**
   * A better abstraction over CSS.
   *
   * @copyright Oleg Isonen (Slobodskoi) / Isonen 2014-present
   * @website https://github.com/cssinjs/jss
   * @license MIT
   */

  /**
   * Export a constant indicating if this browser has CSSTOM support.
   * https://developers.google.com/web/updates/2018/03/cssom
   */
  var hasCSSTOMSupport = typeof CSS !== 'undefined' && CSS && 'number' in CSS;
  /**
   * Creates a new instance of Jss.
   */

  var create = function create(options) {
    return new Jss(options);
  };
  /**
   * A global Jss instance.
   */

  var index$2 = create();

  var now = Date.now();
  var fnValuesNs = "fnValues" + now;
  var fnRuleNs = "fnStyle" + ++now;
  function functionPlugin() {
    return {
      onCreateRule: function onCreateRule(name, decl, options) {
        if (typeof decl !== 'function') return null;
        var rule = createRule(name, {}, options);
        rule[fnRuleNs] = decl;
        return rule;
      },
      onProcessStyle: function onProcessStyle(style, rule) {
        // We need to extract function values from the declaration, so that we can keep core unaware of them.
        // We need to do that only once.
        // We don't need to extract functions on each style update, since this can happen only once.
        // We don't support function values inside of function rules.
        if (fnValuesNs in rule || fnRuleNs in rule) return style;
        var fnValues = {};

        for (var prop in style) {
          var value = style[prop];
          if (typeof value !== 'function') continue;
          delete style[prop];
          fnValues[prop] = value;
        } // $FlowFixMe


        rule[fnValuesNs] = fnValues;
        return style;
      },
      onUpdate: function onUpdate(data, rule, sheet, options) {
        var styleRule = rule;
        var fnRule = styleRule[fnRuleNs]; // If we have a style function, the entire rule is dynamic and style object
        // will be returned from that function.

        if (fnRule) {
          // Empty object will remove all currently defined props
          // in case function rule returns a falsy value.
          styleRule.style = fnRule(data) || {};

          if (process.env.NODE_ENV === 'development') {
            for (var prop in styleRule.style) {
              if (typeof styleRule.style[prop] === 'function') {
                process.env.NODE_ENV !== "production" ? warning(false, '[JSS] Function values inside function rules are not supported.') : void 0;
                break;
              }
            }
          }
        }

        var fnValues = styleRule[fnValuesNs]; // If we have a fn values map, it is a rule with function values.

        if (fnValues) {
          for (var _prop in fnValues) {
            styleRule.prop(_prop, fnValues[_prop](data), options);
          }
        }
      }
    };
  }

  function symbolObservablePonyfill(root) {
  	var result;
  	var Symbol = root.Symbol;

  	if (typeof Symbol === 'function') {
  		if (Symbol.observable) {
  			result = Symbol.observable;
  		} else {
  			result = Symbol('observable');
  			Symbol.observable = result;
  		}
  	} else {
  		result = '@@observable';
  	}

  	return result;
  }

  /* global window */

  var root;

  if (typeof self !== 'undefined') {
    root = self;
  } else if (typeof window !== 'undefined') {
    root = window;
  } else if (typeof global !== 'undefined') {
    root = global;
  } else if (typeof module !== 'undefined') {
    root = module;
  } else {
    root = Function('return this')();
  }

  var result = symbolObservablePonyfill(root);

  var isObservable = function isObservable(value) {
    return value && value[result] && value === value[result]();
  };

  function observablePlugin(updateOptions) {
    return {
      onCreateRule: function onCreateRule(name, decl, options) {
        if (!isObservable(decl)) return null; // Cast `decl` to `Observable`, since it passed the type guard.

        var style$ = decl;
        var rule = createRule(name, {}, options); // TODO
        // Call `stream.subscribe()` returns a subscription, which should be explicitly
        // unsubscribed from when we know this sheet is no longer needed.

        style$.subscribe(function (style) {
          for (var prop in style) {
            rule.prop(prop, style[prop], updateOptions);
          }
        });
        return rule;
      },
      onProcessRule: function onProcessRule(rule) {
        if (rule && rule.type !== 'style') return;
        var styleRule = rule;
        var style = styleRule.style;

        var _loop = function _loop(prop) {
          var value = style[prop];
          if (!isObservable(value)) return "continue";
          delete style[prop];
          value.subscribe({
            next: function next(nextValue) {
              styleRule.prop(prop, nextValue, updateOptions);
            }
          });
        };

        for (var prop in style) {
          var _ret = _loop(prop);

          if (_ret === "continue") continue;
        }
      }
    };
  }

  var semiWithNl = /;\n/;
  /**
   * Naive CSS parser.
   * - Supports only rule body (no selectors)
   * - Requires semicolon and new line after the value (except of last line)
   * - No nested rules support
   */

  var parse = (function (cssText) {
    var style = {};
    var split = cssText.split(semiWithNl);

    for (var i = 0; i < split.length; i++) {
      var decl = (split[i] || '').trim();
      if (!decl) continue;
      var colonIndex = decl.indexOf(':');

      if (colonIndex === -1) {
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Malformed CSS string \"" + decl + "\"") : void 0;
        continue;
      }

      var prop = decl.substr(0, colonIndex).trim();
      var value = decl.substr(colonIndex + 1).trim();
      style[prop] = value;
    }

    return style;
  });

  var onProcessRule = function onProcessRule(rule) {
    if (typeof rule.style === 'string') {
      // $FlowFixMe: We can safely assume that rule has the style property
      rule.style = parse(rule.style);
    }
  };

  function templatePlugin() {
    return {
      onProcessRule: onProcessRule
    };
  }

  var at = '@global';
  var atPrefix = '@global ';

  var GlobalContainerRule =
  /*#__PURE__*/
  function () {
    function GlobalContainerRule(key, styles, options) {
      this.type = 'global';
      this.at = at;
      this.rules = void 0;
      this.options = void 0;
      this.key = void 0;
      this.isProcessed = false;
      this.key = key;
      this.options = options;
      this.rules = new RuleList(_extends$1({}, options, {
        parent: this
      }));

      for (var selector in styles) {
        this.rules.add(selector, styles[selector]);
      }

      this.rules.process();
    }
    /**
     * Get a rule.
     */


    var _proto = GlobalContainerRule.prototype;

    _proto.getRule = function getRule(name) {
      return this.rules.get(name);
    }
    /**
     * Create and register rule, run plugins.
     */
    ;

    _proto.addRule = function addRule(name, style, options) {
      var rule = this.rules.add(name, style, options);
      this.options.jss.plugins.onProcessRule(rule);
      return rule;
    }
    /**
     * Get index of a rule.
     */
    ;

    _proto.indexOf = function indexOf(rule) {
      return this.rules.indexOf(rule);
    }
    /**
     * Generates a CSS string.
     */
    ;

    _proto.toString = function toString() {
      return this.rules.toString();
    };

    return GlobalContainerRule;
  }();

  var GlobalPrefixedRule =
  /*#__PURE__*/
  function () {
    function GlobalPrefixedRule(key, style, options) {
      this.type = 'global';
      this.at = at;
      this.options = void 0;
      this.rule = void 0;
      this.isProcessed = false;
      this.key = void 0;
      this.key = key;
      this.options = options;
      var selector = key.substr(atPrefix.length);
      this.rule = options.jss.createRule(selector, style, _extends$1({}, options, {
        parent: this
      }));
    }

    var _proto2 = GlobalPrefixedRule.prototype;

    _proto2.toString = function toString(options) {
      return this.rule ? this.rule.toString(options) : '';
    };

    return GlobalPrefixedRule;
  }();

  var separatorRegExp$1 = /\s*,\s*/g;

  function addScope(selector, scope) {
    var parts = selector.split(separatorRegExp$1);
    var scoped = '';

    for (var i = 0; i < parts.length; i++) {
      scoped += scope + " " + parts[i].trim();
      if (parts[i + 1]) scoped += ', ';
    }

    return scoped;
  }

  function handleNestedGlobalContainerRule(rule) {
    var options = rule.options,
        style = rule.style;
    var rules = style ? style[at] : null;
    if (!rules) return;

    for (var name in rules) {
      options.sheet.addRule(name, rules[name], _extends$1({}, options, {
        selector: addScope(name, rule.selector)
      }));
    }

    delete style[at];
  }

  function handlePrefixedGlobalRule(rule) {
    var options = rule.options,
        style = rule.style;

    for (var prop in style) {
      if (prop[0] !== '@' || prop.substr(0, at.length) !== at) continue;
      var selector = addScope(prop.substr(at.length), rule.selector);
      options.sheet.addRule(selector, style[prop], _extends$1({}, options, {
        selector: selector
      }));
      delete style[prop];
    }
  }
  /**
   * Convert nested rules to separate, remove them from original styles.
   *
   * @param {Rule} rule
   * @api public
   */


  function jssGlobal() {
    function onCreateRule(name, styles, options) {
      if (!name) return null;

      if (name === at) {
        return new GlobalContainerRule(name, styles, options);
      }

      if (name[0] === '@' && name.substr(0, atPrefix.length) === atPrefix) {
        return new GlobalPrefixedRule(name, styles, options);
      }

      var parent = options.parent;

      if (parent) {
        if (parent.type === 'global' || parent.options.parent && parent.options.parent.type === 'global') {
          options.scoped = false;
        }
      }

      if (options.scoped === false) {
        options.selector = name;
      }

      return null;
    }

    function onProcessRule(rule) {
      if (rule.type !== 'style') return;
      handleNestedGlobalContainerRule(rule);
      handlePrefixedGlobalRule(rule);
    }

    return {
      onCreateRule: onCreateRule,
      onProcessRule: onProcessRule
    };
  }

  /* eslint-disable no-use-before-define */

  var isObject = function isObject(obj) {
    return obj && typeof obj === 'object' && !Array.isArray(obj);
  };

  var valueNs = "extendCurrValue" + Date.now();

  function mergeExtend(style, rule, sheet, newStyle) {
    var extendType = typeof style.extend; // Extend using a rule name.

    if (extendType === 'string') {
      if (!sheet) return;
      var refRule = sheet.getRule(style.extend);
      if (!refRule) return;

      if (refRule === rule) {
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] A rule tries to extend itself \n" + rule.toString()) : void 0;
        return;
      }

      var parent = refRule.options.parent;

      if (parent) {
        var originalStyle = parent.rules.raw[style.extend];
        extend(originalStyle, rule, sheet, newStyle);
      }

      return;
    } // Extend using an array of objects.


    if (Array.isArray(style.extend)) {
      for (var index = 0; index < style.extend.length; index++) {
        extend(style.extend[index], rule, sheet, newStyle);
      }

      return;
    } // Extend is a style object.


    for (var prop in style.extend) {
      if (prop === 'extend') {
        extend(style.extend.extend, rule, sheet, newStyle);
        continue;
      }

      if (isObject(style.extend[prop])) {
        if (!(prop in newStyle)) newStyle[prop] = {};
        extend(style.extend[prop], rule, sheet, newStyle[prop]);
        continue;
      }

      newStyle[prop] = style.extend[prop];
    }
  }

  function mergeRest(style, rule, sheet, newStyle) {
    // Copy base style.
    for (var prop in style) {
      if (prop === 'extend') continue;

      if (isObject(newStyle[prop]) && isObject(style[prop])) {
        extend(style[prop], rule, sheet, newStyle[prop]);
        continue;
      }

      if (isObject(style[prop])) {
        newStyle[prop] = extend(style[prop], rule, sheet);
        continue;
      }

      newStyle[prop] = style[prop];
    }
  }
  /**
   * Recursively extend styles.
   */


  function extend(style, rule, sheet, newStyle) {
    if (newStyle === void 0) {
      newStyle = {};
    }

    mergeExtend(style, rule, sheet, newStyle);
    mergeRest(style, rule, sheet, newStyle);
    return newStyle;
  }
  /**
   * Handle `extend` property.
   *
   * @param {Rule} rule
   * @api public
   */


  function jssExtend() {
    function onProcessStyle(style, rule, sheet) {
      if ('extend' in style) return extend(style, rule, sheet);
      return style;
    }

    function onChangeValue(value, prop, rule) {
      if (prop !== 'extend') return value; // Value is empty, remove properties set previously.

      if (value == null || value === false) {
        // $FlowFixMe: Flow complains because there is no indexer property in StyleRule
        for (var key in rule[valueNs]) {
          rule.prop(key, null);
        } // $FlowFixMe: Flow complains because there is no indexer property in StyleRule


        rule[valueNs] = null;
        return null;
      }

      if (typeof value === 'object') {
        // $FlowFixMe: This will be an object
        for (var _key in value) {
          rule.prop(_key, value[_key]);
        } // $FlowFixMe: Flow complains because there is no indexer property in StyleRule


        rule[valueNs] = value;
      } // Make sure we don't set the value in the core.


      return null;
    }

    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  var separatorRegExp = /\s*,\s*/g;
  var parentRegExp = /&/g;
  var refRegExp = /\$([\w-]+)/g;
  /**
   * Convert nested rules to separate, remove them from original styles.
   *
   * @param {Rule} rule
   * @api public
   */

  function jssNested() {
    // Get a function to be used for $ref replacement.
    function getReplaceRef(container, sheet) {
      return function (match, key) {
        var rule = container.getRule(key) || sheet && sheet.getRule(key);

        if (rule) {
          rule = rule;
          return rule.selector;
        }

        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Could not find the referenced rule \"" + key + "\" in \"" + (container.options.meta || container.toString()) + "\".") : void 0;
        return key;
      };
    }

    function replaceParentRefs(nestedProp, parentProp) {
      var parentSelectors = parentProp.split(separatorRegExp);
      var nestedSelectors = nestedProp.split(separatorRegExp);
      var result = '';

      for (var i = 0; i < parentSelectors.length; i++) {
        var parent = parentSelectors[i];

        for (var j = 0; j < nestedSelectors.length; j++) {
          var nested = nestedSelectors[j];
          if (result) result += ', '; // Replace all & by the parent or prefix & with the parent.

          result += nested.indexOf('&') !== -1 ? nested.replace(parentRegExp, parent) : parent + " " + nested;
        }
      }

      return result;
    }

    function getOptions(rule, container, prevOptions) {
      // Options has been already created, now we only increase index.
      if (prevOptions) return _extends$1({}, prevOptions, {
        index: prevOptions.index + 1
      });
      var nestingLevel = rule.options.nestingLevel;
      nestingLevel = nestingLevel === undefined ? 1 : nestingLevel + 1;

      var options = _extends$1({}, rule.options, {
        nestingLevel: nestingLevel,
        index: container.indexOf(rule) + 1 // We don't need the parent name to be set options for chlid.

      });

      delete options.name;
      return options;
    }

    function onProcessStyle(style, rule, sheet) {
      if (rule.type !== 'style') return style;
      var styleRule = rule;
      var container = styleRule.options.parent;
      var options;
      var replaceRef;

      for (var prop in style) {
        var isNested = prop.indexOf('&') !== -1;
        var isNestedConditional = prop[0] === '@';
        if (!isNested && !isNestedConditional) continue;
        options = getOptions(styleRule, container, options);

        if (isNested) {
          var selector = replaceParentRefs(prop, styleRule.selector); // Lazily create the ref replacer function just once for
          // all nested rules within the sheet.

          if (!replaceRef) replaceRef = getReplaceRef(container, sheet); // Replace all $refs.

          selector = selector.replace(refRegExp, replaceRef);
          container.addRule(selector, style[prop], _extends$1({}, options, {
            selector: selector
          }));
        } else if (isNestedConditional) {
          // Place conditional right after the parent rule to ensure right ordering.
          container.addRule(prop, {}, options) // Flow expects more options but they aren't required
          // And flow doesn't know this will always be a StyleRule which has the addRule method
          // $FlowFixMe
          .addRule(styleRule.key, style[prop], {
            selector: styleRule.selector
          });
        }

        delete style[prop];
      }

      return style;
    }

    return {
      onProcessStyle: onProcessStyle
    };
  }

  /**
   * Set selector.
   *
   * @param {Object} original rule
   * @param {String} className class string
   * @return {Boolean} flag, indicating function was successfull or not
   */
  function registerClass(rule, className) {
    // Skip falsy values
    if (!className) return true; // Support array of class names `{composes: ['foo', 'bar']}`

    if (Array.isArray(className)) {
      for (var index = 0; index < className.length; index++) {
        var isSetted = registerClass(rule, className[index]);
        if (!isSetted) return false;
      }

      return true;
    } // Support space separated class names `{composes: 'foo bar'}`


    if (className.indexOf(' ') > -1) {
      return registerClass(rule, className.split(' '));
    }

    var _ref = rule.options,
        parent = _ref.parent; // It is a ref to a local rule.

    if (className[0] === '$') {
      var refRule = parent.getRule(className.substr(1));

      if (!refRule) {
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Referenced rule is not defined. \n" + rule.toString()) : void 0;
        return false;
      }

      if (refRule === rule) {
        process.env.NODE_ENV !== "production" ? warning(false, "[JSS] Cyclic composition detected. \n" + rule.toString()) : void 0;
        return false;
      }

      parent.classes[rule.key] += " " + parent.classes[refRule.key];
      return true;
    }

    parent.classes[rule.key] += " " + className;
    return true;
  }
  /**
   * Convert compose property to additional class, remove property from original styles.
   *
   * @param {Rule} rule
   * @api public
   */


  function jssCompose() {
    function onProcessStyle(style, rule) {
      if (!('composes' in style)) return style;
      registerClass(rule, style.composes); // Remove composes property to prevent infinite loop.

      delete style.composes;
      return style;
    }

    return {
      onProcessStyle: onProcessStyle
    };
  }

  /* eslint-disable no-var, prefer-template */
  var uppercasePattern = /[A-Z]/g;
  var msPattern = /^ms-/;
  var cache$2 = {};

  function toHyphenLower(match) {
    return '-' + match.toLowerCase()
  }

  function hyphenateStyleName(name) {
    if (cache$2.hasOwnProperty(name)) {
      return cache$2[name]
    }

    var hName = name.replace(uppercasePattern, toHyphenLower);
    return (cache$2[name] = msPattern.test(hName) ? '-' + hName : hName)
  }

  /**
   * Convert camel cased property names to dash separated.
   *
   * @param {Object} style
   * @return {Object}
   */

  function convertCase(style) {
    var converted = {};

    for (var prop in style) {
      var key = prop.indexOf('--') === 0 ? prop : hyphenateStyleName(prop);
      converted[key] = style[prop];
    }

    if (style.fallbacks) {
      if (Array.isArray(style.fallbacks)) converted.fallbacks = style.fallbacks.map(convertCase);else converted.fallbacks = convertCase(style.fallbacks);
    }

    return converted;
  }
  /**
   * Allow camel cased property names by converting them back to dasherized.
   *
   * @param {Rule} rule
   */


  function camelCase() {
    function onProcessStyle(style) {
      if (Array.isArray(style)) {
        // Handle rules like @font-face, which can have multiple styles in an array
        for (var index = 0; index < style.length; index++) {
          style[index] = convertCase(style[index]);
        }

        return style;
      }

      return convertCase(style);
    }

    function onChangeValue(value, prop, rule) {
      if (prop.indexOf('--') === 0) {
        return value;
      }

      var hyphenatedProp = hyphenateStyleName(prop); // There was no camel case in place

      if (prop === hyphenatedProp) return value;
      rule.prop(hyphenatedProp, value); // Core will ignore that property value we set the proper one above.

      return null;
    }

    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  var px = hasCSSTOMSupport && CSS ? CSS.px : 'px';
  var ms = hasCSSTOMSupport && CSS ? CSS.ms : 'ms';
  var percent = hasCSSTOMSupport && CSS ? CSS.percent : '%';
  /**
   * Generated jss-plugin-default-unit CSS property units
   *
   * @type object
   */

  var defaultUnits = {
    // Animation properties
    'animation-delay': ms,
    'animation-duration': ms,
    // Background properties
    'background-position': px,
    'background-position-x': px,
    'background-position-y': px,
    'background-size': px,
    // Border Properties
    border: px,
    'border-bottom': px,
    'border-bottom-left-radius': px,
    'border-bottom-right-radius': px,
    'border-bottom-width': px,
    'border-left': px,
    'border-left-width': px,
    'border-radius': px,
    'border-right': px,
    'border-right-width': px,
    'border-top': px,
    'border-top-left-radius': px,
    'border-top-right-radius': px,
    'border-top-width': px,
    'border-width': px,
    // Margin properties
    margin: px,
    'margin-bottom': px,
    'margin-left': px,
    'margin-right': px,
    'margin-top': px,
    // Padding properties
    padding: px,
    'padding-bottom': px,
    'padding-left': px,
    'padding-right': px,
    'padding-top': px,
    // Mask properties
    'mask-position-x': px,
    'mask-position-y': px,
    'mask-size': px,
    // Width and height properties
    height: px,
    width: px,
    'min-height': px,
    'max-height': px,
    'min-width': px,
    'max-width': px,
    // Position properties
    bottom: px,
    left: px,
    top: px,
    right: px,
    // Shadow properties
    'box-shadow': px,
    'text-shadow': px,
    // Column properties
    'column-gap': px,
    'column-rule': px,
    'column-rule-width': px,
    'column-width': px,
    // Font and text properties
    'font-size': px,
    'font-size-delta': px,
    'letter-spacing': px,
    'text-indent': px,
    'text-stroke': px,
    'text-stroke-width': px,
    'word-spacing': px,
    // Motion properties
    motion: px,
    'motion-offset': px,
    // Outline properties
    outline: px,
    'outline-offset': px,
    'outline-width': px,
    // Perspective properties
    perspective: px,
    'perspective-origin-x': percent,
    'perspective-origin-y': percent,
    // Transform properties
    'transform-origin': percent,
    'transform-origin-x': percent,
    'transform-origin-y': percent,
    'transform-origin-z': percent,
    // Transition properties
    'transition-delay': ms,
    'transition-duration': ms,
    // Alignment properties
    'vertical-align': px,
    'flex-basis': px,
    // Some random properties
    'shape-margin': px,
    size: px,
    // Grid properties
    grid: px,
    'grid-gap': px,
    'grid-row-gap': px,
    'grid-column-gap': px,
    'grid-template-rows': px,
    'grid-template-columns': px,
    'grid-auto-rows': px,
    'grid-auto-columns': px,
    // Not existing properties.
    // Used to avoid issues with jss-plugin-expand integration.
    'box-shadow-x': px,
    'box-shadow-y': px,
    'box-shadow-blur': px,
    'box-shadow-spread': px,
    'font-line-height': px,
    'text-shadow-x': px,
    'text-shadow-y': px,
    'text-shadow-blur': px
  };

  /**
   * Clones the object and adds a camel cased property version.
   */
  function addCamelCasedVersion(obj) {
    var regExp = /(-[a-z])/g;

    var replace = function replace(str) {
      return str[1].toUpperCase();
    };

    var newObj = {};

    for (var _key in obj) {
      newObj[_key] = obj[_key];
      newObj[_key.replace(regExp, replace)] = obj[_key];
    }

    return newObj;
  }

  var units = addCamelCasedVersion(defaultUnits);
  /**
   * Recursive deep style passing function
   */

  function iterate(prop, value, options) {
    if (!value) return value;

    if (Array.isArray(value)) {
      for (var i = 0; i < value.length; i++) {
        value[i] = iterate(prop, value[i], options);
      }
    } else if (typeof value === 'object') {
      if (prop === 'fallbacks') {
        for (var innerProp in value) {
          value[innerProp] = iterate(innerProp, value[innerProp], options);
        }
      } else {
        for (var _innerProp in value) {
          value[_innerProp] = iterate(prop + "-" + _innerProp, value[_innerProp], options);
        }
      }
    } else if (typeof value === 'number') {
      var unit = options[prop] || units[prop];

      if (unit) {
        return typeof unit === 'function' ? unit(value).toString() : "" + value + unit;
      }

      return value.toString();
    }

    return value;
  }
  /**
   * Add unit to numeric values.
   */


  function defaultUnit(options) {
    if (options === void 0) {
      options = {};
    }

    var camelCasedOptions = addCamelCasedVersion(options);

    function onProcessStyle(style, rule) {
      if (rule.type !== 'style') return style;

      for (var prop in style) {
        style[prop] = iterate(prop, style[prop], camelCasedOptions);
      }

      return style;
    }

    function onChangeValue(value, prop) {
      return iterate(prop, value, camelCasedOptions);
    }

    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  /**
   * A scheme for converting properties from array to regular style.
   * All properties listed below will be transformed to a string separated by space.
   */
  var propArray = {
    'background-size': true,
    'background-position': true,
    border: true,
    'border-bottom': true,
    'border-left': true,
    'border-top': true,
    'border-right': true,
    'border-radius': true,
    'border-image': true,
    'border-width': true,
    'border-style': true,
    'border-color': true,
    'box-shadow': true,
    flex: true,
    margin: true,
    padding: true,
    outline: true,
    'transform-origin': true,
    transform: true,
    transition: true
    /**
     * A scheme for converting arrays to regular styles inside of objects.
     * For e.g.: "{position: [0, 0]}" => "background-position: 0 0;".
     */

  };
  var propArrayInObj = {
    position: true,
    // background-position
    size: true // background-size

    /**
     * A scheme for parsing and building correct styles from passed objects.
     */

  };
  var propObj = {
    padding: {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    },
    margin: {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    },
    background: {
      attachment: null,
      color: null,
      image: null,
      position: null,
      repeat: null
    },
    border: {
      width: null,
      style: null,
      color: null
    },
    'border-top': {
      width: null,
      style: null,
      color: null
    },
    'border-right': {
      width: null,
      style: null,
      color: null
    },
    'border-bottom': {
      width: null,
      style: null,
      color: null
    },
    'border-left': {
      width: null,
      style: null,
      color: null
    },
    outline: {
      width: null,
      style: null,
      color: null
    },
    'list-style': {
      type: null,
      position: null,
      image: null
    },
    transition: {
      property: null,
      duration: null,
      'timing-function': null,
      timingFunction: null,
      // Needed for avoiding comilation issues with jss-plugin-camel-case
      delay: null
    },
    animation: {
      name: null,
      duration: null,
      'timing-function': null,
      timingFunction: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      delay: null,
      'iteration-count': null,
      iterationCount: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      direction: null,
      'fill-mode': null,
      fillMode: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      'play-state': null,
      playState: null // Needed to avoid compilation issues with jss-plugin-camel-case

    },
    'box-shadow': {
      x: 0,
      y: 0,
      blur: 0,
      spread: 0,
      color: null,
      inset: null
    },
    'text-shadow': {
      x: 0,
      y: 0,
      blur: null,
      color: null
    }
    /**
     * A scheme for converting non-standart properties inside object.
     * For e.g.: include 'border-radius' property inside 'border' object.
     */

  };
  var customPropObj = {
    border: {
      radius: 'border-radius',
      image: 'border-image',
      width: 'border-width',
      style: 'border-style',
      color: 'border-color'
    },
    'border-bottom': {
      width: 'border-bottom-width',
      style: 'border-bottom-style',
      color: 'border-bottom-color'
    },
    'border-top': {
      width: 'border-top-width',
      style: 'border-top-style',
      color: 'border-top-color'
    },
    'border-left': {
      width: 'border-left-width',
      style: 'border-left-style',
      color: 'border-left-color'
    },
    'border-right': {
      width: 'border-right-width',
      style: 'border-right-style',
      color: 'border-right-color'
    },
    background: {
      size: 'background-size',
      image: 'background-image'
    },
    font: {
      style: 'font-style',
      variant: 'font-variant',
      weight: 'font-weight',
      stretch: 'font-stretch',
      size: 'font-size',
      family: 'font-family',
      lineHeight: 'line-height',
      // Needed to avoid compilation issues with jss-plugin-camel-case
      'line-height': 'line-height'
    },
    flex: {
      grow: 'flex-grow',
      basis: 'flex-basis',
      direction: 'flex-direction',
      wrap: 'flex-wrap',
      flow: 'flex-flow',
      shrink: 'flex-shrink'
    },
    align: {
      self: 'align-self',
      items: 'align-items',
      content: 'align-content'
    },
    grid: {
      'template-columns': 'grid-template-columns',
      templateColumns: 'grid-template-columns',
      'template-rows': 'grid-template-rows',
      templateRows: 'grid-template-rows',
      'template-areas': 'grid-template-areas',
      templateAreas: 'grid-template-areas',
      template: 'grid-template',
      'auto-columns': 'grid-auto-columns',
      autoColumns: 'grid-auto-columns',
      'auto-rows': 'grid-auto-rows',
      autoRows: 'grid-auto-rows',
      'auto-flow': 'grid-auto-flow',
      autoFlow: 'grid-auto-flow',
      row: 'grid-row',
      column: 'grid-column',
      'row-start': 'grid-row-start',
      rowStart: 'grid-row-start',
      'row-end': 'grid-row-end',
      rowEnd: 'grid-row-end',
      'column-start': 'grid-column-start',
      columnStart: 'grid-column-start',
      'column-end': 'grid-column-end',
      columnEnd: 'grid-column-end',
      area: 'grid-area',
      gap: 'grid-gap',
      'row-gap': 'grid-row-gap',
      rowGap: 'grid-row-gap',
      'column-gap': 'grid-column-gap',
      columnGap: 'grid-column-gap'
    }
  };

  /* eslint-disable no-use-before-define */

  /**
   * Map values by given prop.
   *
   * @param {Array} array of values
   * @param {String} original property
   * @param {String} original rule
   * @return {String} mapped values
   */
  function mapValuesByProp(value, prop, rule) {
    return value.map(function (item) {
      return objectToArray(item, prop, rule, false, true);
    });
  }
  /**
   * Convert array to nested array, if needed
   */


  function processArray(value, prop, scheme, rule) {
    if (scheme[prop] == null) return value;
    if (value.length === 0) return [];
    if (Array.isArray(value[0])) return processArray(value[0], prop, scheme, rule);

    if (typeof value[0] === 'object') {
      return mapValuesByProp(value, prop, rule);
    }

    return [value];
  }
  /**
   * Convert object to array.
   */


  function objectToArray(value, prop, rule, isFallback, isInArray) {
    if (!(propObj[prop] || customPropObj[prop])) return [];
    var result = []; // Check if exists any non-standard property

    if (customPropObj[prop]) {
      // eslint-disable-next-line no-param-reassign
      value = customPropsToStyle(value, rule, customPropObj[prop], isFallback);
    } // Pass throught all standart props


    if (Object.keys(value).length) {
      for (var baseProp in propObj[prop]) {
        if (value[baseProp]) {
          if (Array.isArray(value[baseProp])) {
            result.push(propArrayInObj[baseProp] === null ? value[baseProp] : value[baseProp].join(' '));
          } else result.push(value[baseProp]);

          continue;
        } // Add default value from props config.


        if (propObj[prop][baseProp] != null) {
          result.push(propObj[prop][baseProp]);
        }
      }
    }

    if (!result.length || isInArray) return result;
    return [result];
  }
  /**
   * Convert custom properties values to styles adding them to rule directly
   */


  function customPropsToStyle(value, rule, customProps, isFallback) {
    for (var prop in customProps) {
      var propName = customProps[prop]; // If current property doesn't exist already in rule - add new one

      if (typeof value[prop] !== 'undefined' && (isFallback || !rule.prop(propName))) {
        var _styleDetector;

        var appendedValue = styleDetector((_styleDetector = {}, _styleDetector[propName] = value[prop], _styleDetector), rule)[propName]; // Add style directly in rule

        if (isFallback) rule.style.fallbacks[propName] = appendedValue;else rule.style[propName] = appendedValue;
      } // Delete converted property to avoid double converting


      delete value[prop];
    }

    return value;
  }
  /**
   * Detect if a style needs to be converted.
   */


  function styleDetector(style, rule, isFallback) {
    for (var prop in style) {
      var value = style[prop];

      if (Array.isArray(value)) {
        // Check double arrays to avoid recursion.
        if (!Array.isArray(value[0])) {
          if (prop === 'fallbacks') {
            for (var index = 0; index < style.fallbacks.length; index++) {
              style.fallbacks[index] = styleDetector(style.fallbacks[index], rule, true);
            }

            continue;
          }

          style[prop] = processArray(value, prop, propArray, rule); // Avoid creating properties with empty values

          if (!style[prop].length) delete style[prop];
        }
      } else if (typeof value === 'object') {
        if (prop === 'fallbacks') {
          style.fallbacks = styleDetector(style.fallbacks, rule, true);
          continue;
        }

        style[prop] = objectToArray(value, prop, rule, isFallback); // Avoid creating properties with empty values

        if (!style[prop].length) delete style[prop];
      } // Maybe a computed value resulting in an empty string
      else if (style[prop] === '') delete style[prop];
    }

    return style;
  }
  /**
   * Adds possibility to write expanded styles.
   */


  function jssExpand() {
    function onProcessStyle(style, rule) {
      if (!style || rule.type !== 'style') return style;

      if (Array.isArray(style)) {
        // Pass rules one by one and reformat them
        for (var index = 0; index < style.length; index++) {
          style[index] = styleDetector(style[index], rule);
        }

        return style;
      }

      return styleDetector(style, rule);
    }

    return {
      onProcessStyle: onProcessStyle
    };
  }

  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;

    for (var i = 0, arr2 = new Array(len); i < len; i++) {
      arr2[i] = arr[i];
    }

    return arr2;
  }

  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
  }

  function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
  }

  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
  }

  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
  }

  // Export javascript style and css style vendor prefixes.
  var js = '';
  var css = '';
  var vendor = '';
  var browser = '';
  var isTouch = isBrowser && 'ontouchstart' in document.documentElement; // We should not do anything if required serverside.

  if (isBrowser) {
    // Order matters. We need to check Webkit the last one because
    // other vendors use to add Webkit prefixes to some properties
    var jsCssMap = {
      Moz: '-moz-',
      ms: '-ms-',
      O: '-o-',
      Webkit: '-webkit-'
    };

    var _document$createEleme = document.createElement('p'),
        style = _document$createEleme.style;

    var testProp = 'Transform';

    for (var key in jsCssMap) {
      if (key + testProp in style) {
        js = key;
        css = jsCssMap[key];
        break;
      }
    } // Correctly detect the Edge browser.


    if (js === 'Webkit' && 'msHyphens' in style) {
      js = 'ms';
      css = jsCssMap.ms;
      browser = 'edge';
    } // Correctly detect the Safari browser.


    if (js === 'Webkit' && '-apple-trailing-word' in style) {
      vendor = 'apple';
    }
  }
  /**
   * Vendor prefix string for the current browser.
   *
   * @type {{js: String, css: String, vendor: String, browser: String}}
   * @api public
   */


  var prefix = {
    js: js,
    css: css,
    vendor: vendor,
    browser: browser,
    isTouch: isTouch
  };

  /**
   * Test if a keyframe at-rule should be prefixed or not
   *
   * @param {String} vendor prefix string for the current browser.
   * @return {String}
   * @api public
   */

  function supportedKeyframes(key) {
    // Keyframes is already prefixed. e.g. key = '@-webkit-keyframes a'
    if (key[1] === '-') return key; // No need to prefix IE/Edge. Older browsers will ignore unsupported rules.
    // https://caniuse.com/#search=keyframes

    if (prefix.js === 'ms') return key;
    return "@" + prefix.css + "keyframes" + key.substr(10);
  }

  // https://caniuse.com/#search=appearance

  var appearence = {
    noPrefill: ['appearance'],
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'appearance') return false;
      if (prefix.js === 'ms') return "-webkit-" + prop;
      return prefix.css + prop;
    }
  };

  // https://caniuse.com/#search=color-adjust

  var colorAdjust = {
    noPrefill: ['color-adjust'],
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'color-adjust') return false;
      if (prefix.js === 'Webkit') return prefix.css + "print-" + prop;
      return prop;
    }
  };

  var regExp = /[-\s]+(.)?/g;
  /**
   * Replaces the letter with the capital letter
   *
   * @param {String} match
   * @param {String} c
   * @return {String}
   * @api private
   */

  function toUpper(match, c) {
    return c ? c.toUpperCase() : '';
  }
  /**
   * Convert dash separated strings to camel-cased.
   *
   * @param {String} str
   * @return {String}
   * @api private
   */


  function camelize(str) {
    return str.replace(regExp, toUpper);
  }

  /**
   * Convert dash separated strings to pascal cased.
   *
   * @param {String} str
   * @return {String}
   * @api private
   */

  function pascalize(str) {
    return camelize("-" + str);
  }

  // but we can use a longhand property instead.
  // https://caniuse.com/#search=mask

  var mask = {
    noPrefill: ['mask'],
    supportedProperty: function supportedProperty(prop, style) {
      if (!/^mask/.test(prop)) return false;

      if (prefix.js === 'Webkit') {
        var longhand = 'mask-image';

        if (camelize(longhand) in style) {
          return prop;
        }

        if (prefix.js + pascalize(longhand) in style) {
          return prefix.css + prop;
        }
      }

      return prop;
    }
  };

  // https://caniuse.com/#search=text-orientation

  var textOrientation = {
    noPrefill: ['text-orientation'],
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'text-orientation') return false;

      if (prefix.vendor === 'apple' && !prefix.isTouch) {
        return prefix.css + prop;
      }

      return prop;
    }
  };

  // https://caniuse.com/#search=transform

  var transform = {
    noPrefill: ['transform'],
    supportedProperty: function supportedProperty(prop, style, options) {
      if (prop !== 'transform') return false;

      if (options.transform) {
        return prop;
      }

      return prefix.css + prop;
    }
  };

  // https://caniuse.com/#search=transition

  var transition = {
    noPrefill: ['transition'],
    supportedProperty: function supportedProperty(prop, style, options) {
      if (prop !== 'transition') return false;

      if (options.transition) {
        return prop;
      }

      return prefix.css + prop;
    }
  };

  // https://caniuse.com/#search=writing-mode

  var writingMode = {
    noPrefill: ['writing-mode'],
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'writing-mode') return false;

      if (prefix.js === 'Webkit' || prefix.js === 'ms' && prefix.browser !== 'edge') {
        return prefix.css + prop;
      }

      return prop;
    }
  };

  // https://caniuse.com/#search=user-select

  var userSelect = {
    noPrefill: ['user-select'],
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'user-select') return false;

      if (prefix.js === 'Moz' || prefix.js === 'ms' || prefix.vendor === 'apple') {
        return prefix.css + prop;
      }

      return prop;
    }
  };

  // https://caniuse.com/#search=multicolumn
  // https://github.com/postcss/autoprefixer/issues/491
  // https://github.com/postcss/autoprefixer/issues/177

  var breakPropsOld = {
    supportedProperty: function supportedProperty(prop, style) {
      if (!/^break-/.test(prop)) return false;

      if (prefix.js === 'Webkit') {
        var jsProp = "WebkitColumn" + pascalize(prop);
        return jsProp in style ? prefix.css + "column-" + prop : false;
      }

      if (prefix.js === 'Moz') {
        var _jsProp = "page" + pascalize(prop);

        return _jsProp in style ? "page-" + prop : false;
      }

      return false;
    }
  };

  // See https://github.com/postcss/autoprefixer/issues/324.

  var inlineLogicalOld = {
    supportedProperty: function supportedProperty(prop, style) {
      if (!/^(border|margin|padding)-inline/.test(prop)) return false;
      if (prefix.js === 'Moz') return prop;
      var newProp = prop.replace('-inline', '');
      return prefix.js + pascalize(newProp) in style ? prefix.css + newProp : false;
    }
  };

  // Camelization is required because we can't test using.
  // CSS syntax for e.g. in FF.

  var unprefixed = {
    supportedProperty: function supportedProperty(prop, style) {
      return camelize(prop) in style ? prop : false;
    }
  };

  var prefixed = {
    supportedProperty: function supportedProperty(prop, style) {
      var pascalized = pascalize(prop); // Return custom CSS variable without prefixing.

      if (prop[0] === '-') return prop; // Return already prefixed value without prefixing.

      if (prop[0] === '-' && prop[1] === '-') return prop;
      if (prefix.js + pascalized in style) return prefix.css + prop; // Try webkit fallback.

      if (prefix.js !== 'Webkit' && "Webkit" + pascalized in style) return "-webkit-" + prop;
      return false;
    }
  };

  // https://caniuse.com/#search=scroll-snap

  var scrollSnap = {
    supportedProperty: function supportedProperty(prop) {
      if (prop.substring(0, 11) !== 'scroll-snap') return false;

      if (prefix.js === 'ms') {
        return "" + prefix.css + prop;
      }

      return prop;
    }
  };

  // https://caniuse.com/#search=overscroll-behavior

  var overscrollBehavior = {
    supportedProperty: function supportedProperty(prop) {
      if (prop !== 'overscroll-behavior') return false;

      if (prefix.js === 'ms') {
        return prefix.css + "scroll-chaining";
      }

      return prop;
    }
  };

  var propMap = {
    'flex-grow': 'flex-positive',
    'flex-shrink': 'flex-negative',
    'flex-basis': 'flex-preferred-size',
    'justify-content': 'flex-pack',
    order: 'flex-order',
    'align-items': 'flex-align',
    'align-content': 'flex-line-pack' // 'align-self' is handled by 'align-self' plugin.

  }; // Support old flex spec from 2012.

  var flex2012 = {
    supportedProperty: function supportedProperty(prop, style) {
      var newProp = propMap[prop];
      if (!newProp) return false;
      return prefix.js + pascalize(newProp) in style ? prefix.css + newProp : false;
    }
  };

  var propMap$1 = {
    flex: 'box-flex',
    'flex-grow': 'box-flex',
    'flex-direction': ['box-orient', 'box-direction'],
    order: 'box-ordinal-group',
    'align-items': 'box-align',
    'flex-flow': ['box-orient', 'box-direction'],
    'justify-content': 'box-pack'
  };
  var propKeys = Object.keys(propMap$1);

  var prefixCss = function prefixCss(p) {
    return prefix.css + p;
  }; // Support old flex spec from 2009.


  var flex2009 = {
    supportedProperty: function supportedProperty(prop, style, _ref) {
      var multiple = _ref.multiple;

      if (propKeys.indexOf(prop) > -1) {
        var newProp = propMap$1[prop];

        if (!Array.isArray(newProp)) {
          return prefix.js + pascalize(newProp) in style ? prefix.css + newProp : false;
        }

        if (!multiple) return false;

        for (var i = 0; i < newProp.length; i++) {
          if (!(prefix.js + pascalize(newProp[0]) in style)) {
            return false;
          }
        }

        return newProp.map(prefixCss);
      }

      return false;
    }
  };

  // plugins = [
  //   ...plugins,
  //    breakPropsOld,
  //    inlineLogicalOld,
  //    unprefixed,
  //    prefixed,
  //    scrollSnap,
  //    flex2012,
  //    flex2009
  // ]
  // Plugins without 'noPrefill' value, going last.
  // 'flex-*' plugins should be at the bottom.
  // 'flex2009' going after 'flex2012'.
  // 'prefixed' going after 'unprefixed'

  var plugins = [appearence, colorAdjust, mask, textOrientation, transform, transition, writingMode, userSelect, breakPropsOld, inlineLogicalOld, unprefixed, prefixed, scrollSnap, overscrollBehavior, flex2012, flex2009];
  var propertyDetectors = plugins.filter(function (p) {
    return p.supportedProperty;
  }).map(function (p) {
    return p.supportedProperty;
  });
  var noPrefill = plugins.filter(function (p) {
    return p.noPrefill;
  }).reduce(function (a, p) {
    a.push.apply(a, _toConsumableArray(p.noPrefill));
    return a;
  }, []);

  var el$1;
  var cache = {};

  if (isBrowser) {
    el$1 = document.createElement('p'); // We test every property on vendor prefix requirement.
    // Once tested, result is cached. It gives us up to 70% perf boost.
    // http://jsperf.com/element-style-object-access-vs-plain-object
    //
    // Prefill cache with known css properties to reduce amount of
    // properties we need to feature test at runtime.
    // http://davidwalsh.name/vendor-prefix

    var computed = window.getComputedStyle(document.documentElement, '');

    for (var key$1 in computed) {
      // eslint-disable-next-line no-restricted-globals
      if (!isNaN(key$1)) cache[computed[key$1]] = computed[key$1];
    } // Properties that cannot be correctly detected using the
    // cache prefill method.


    noPrefill.forEach(function (x) {
      return delete cache[x];
    });
  }
  /**
   * Test if a property is supported, returns supported property with vendor
   * prefix if required. Returns `false` if not supported.
   *
   * @param {String} prop dash separated
   * @param {Object} [options]
   * @return {String|Boolean}
   * @api public
   */


  function supportedProperty(prop, options) {
    if (options === void 0) {
      options = {};
    }

    // For server-side rendering.
    if (!el$1) return prop; // Remove cache for benchmark tests or return property from the cache.

    if (process.env.NODE_ENV !== 'benchmark' && cache[prop] != null) {
      return cache[prop];
    } // Check if 'transition' or 'transform' natively supported in browser.


    if (prop === 'transition' || prop === 'transform') {
      options[prop] = prop in el$1.style;
    } // Find a plugin for current prefix property.


    for (var i = 0; i < propertyDetectors.length; i++) {
      cache[prop] = propertyDetectors[i](prop, el$1.style, options); // Break loop, if value found.

      if (cache[prop]) break;
    } // Reset styles for current property.
    // Firefox can even throw an error for invalid properties, e.g., "0".


    try {
      el$1.style[prop] = '';
    } catch (err) {
      return false;
    }

    return cache[prop];
  }

  var cache$1 = {};
  var transitionProperties = {
    transition: 1,
    'transition-property': 1,
    '-webkit-transition': 1,
    '-webkit-transition-property': 1
  };
  var transPropsRegExp = /(^\s*[\w-]+)|, (\s*[\w-]+)(?![^()]*\))/g;
  var el$1$1;
  /**
   * Returns prefixed value transition/transform if needed.
   *
   * @param {String} match
   * @param {String} p1
   * @param {String} p2
   * @return {String}
   * @api private
   */

  function prefixTransitionCallback(match, p1, p2) {
    if (p1 === 'var') return 'var';
    if (p1 === 'all') return 'all';
    if (p2 === 'all') return ', all';
    var prefixedValue = p1 ? supportedProperty(p1) : ", " + supportedProperty(p2);
    if (!prefixedValue) return p1 || p2;
    return prefixedValue;
  }

  if (isBrowser) el$1$1 = document.createElement('p');
  /**
   * Returns prefixed value if needed. Returns `false` if value is not supported.
   *
   * @param {String} property
   * @param {String} value
   * @return {String|Boolean}
   * @api public
   */

  function supportedValue(property, value) {
    // For server-side rendering.
    var prefixedValue = value;
    if (!el$1$1 || property === 'content') return value; // It is a string or a number as a string like '1'.
    // We want only prefixable values here.
    // eslint-disable-next-line no-restricted-globals

    if (typeof prefixedValue !== 'string' || !isNaN(parseInt(prefixedValue, 10))) {
      return prefixedValue;
    } // Create cache key for current value.


    var cacheKey = property + prefixedValue; // Remove cache for benchmark tests or return value from cache.

    if (process.env.NODE_ENV !== 'benchmark' && cache$1[cacheKey] != null) {
      return cache$1[cacheKey];
    } // IE can even throw an error in some cases, for e.g. style.content = 'bar'.


    try {
      // Test value as it is.
      el$1$1.style[property] = prefixedValue;
    } catch (err) {
      // Return false if value not supported.
      cache$1[cacheKey] = false;
      return false;
    } // If 'transition' or 'transition-property' property.


    if (transitionProperties[property]) {
      prefixedValue = prefixedValue.replace(transPropsRegExp, prefixTransitionCallback);
    } else if (el$1$1.style[property] === '') {
      // Value with a vendor prefix.
      prefixedValue = prefix.css + prefixedValue; // Hardcode test to convert "flex" to "-ms-flexbox" for IE10.

      if (prefixedValue === '-ms-flex') el$1$1.style[property] = '-ms-flexbox'; // Test prefixed value.

      el$1$1.style[property] = prefixedValue; // Return false if value not supported.

      if (el$1$1.style[property] === '') {
        cache$1[cacheKey] = false;
        return false;
      }
    } // Reset styles for current property.


    el$1$1.style[property] = ''; // Write current value to cache.

    cache$1[cacheKey] = prefixedValue;
    return cache$1[cacheKey];
  }

  /**
   * Add vendor prefix to a property name when needed.
   *
   * @api public
   */

  function jssVendorPrefixer() {
    function onProcessRule(rule) {
      if (rule.type === 'keyframes') {
        var atRule = rule;
        atRule.at = supportedKeyframes(atRule.at);
      }
    }

    function prefixStyle(style) {
      for (var prop in style) {
        var value = style[prop];

        if (prop === 'fallbacks' && Array.isArray(value)) {
          style[prop] = value.map(prefixStyle);
          continue;
        }

        var changeProp = false;
        var supportedProp = supportedProperty(prop);
        if (supportedProp && supportedProp !== prop) changeProp = true;
        var changeValue = false;
        var supportedValue$1 = supportedValue(supportedProp, toCssValue(value));
        if (supportedValue$1 && supportedValue$1 !== value) changeValue = true;

        if (changeProp || changeValue) {
          if (changeProp) delete style[prop];
          style[supportedProp || prop] = supportedValue$1 || value;
        }
      }

      return style;
    }

    function onProcessStyle(style, rule) {
      if (rule.type !== 'style') return style;
      return prefixStyle(style);
    }

    function onChangeValue(value, prop) {
      return supportedValue(prop, toCssValue(value)) || value;
    }

    return {
      onProcessRule: onProcessRule,
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  /**
   * Sort props by length.
   */
  function jssPropsSort() {
    var sort = function sort(prop0, prop1) {
      if (prop0.length === prop1.length) {
        return prop0 > prop1 ? 1 : -1;
      }

      return prop0.length - prop1.length;
    };

    return {
      onProcessStyle: function onProcessStyle(style, rule) {
        if (rule.type !== 'style') return style;
        var newStyle = {};
        var props = Object.keys(style).sort(sort);

        for (var i = 0; i < props.length; i++) {
          newStyle[props[i]] = style[props[i]];
        }

        return newStyle;
      }
    };
  }

  var index$1 = (function (options) {
    if (options === void 0) {
      options = {};
    }

    return {
      plugins: [functionPlugin(), observablePlugin(options.observable), templatePlugin(), jssGlobal(), jssExtend(), jssNested(), jssCompose(), camelCase(), defaultUnit(options.defaultUnit), jssExpand(), jssVendorPrefixer(), jssPropsSort()]
    };
  });

  create(index$1());

  var JssContext = React__default['default'].createContext({
    classNamePrefix: '',
    disableStylesGeneration: false
  });

  /**
   * Global index counter to preserve source order.
   * As we create the style sheet during componentWillMount lifecycle,
   * children are handled after the parents, so the order of style elements would
   * be parent->child. It is a problem though when a parent passes a className
   * which needs to override any childs styles. StyleSheet of the child has a higher
   * specificity, because of the source order.
   * So our solution is to render sheets them in the reverse order child->sheet, so
   * that parent has a higher specificity.
   *
   * We start at [Number.MIN_SAFE_INTEGER] to always insert sheets from react-jss first before any
   * sheet which might be inserted manually by the user.
   */
  var index = Number.MIN_SAFE_INTEGER || -1e9;

  var getSheetIndex = function getSheetIndex() {
    return index++;
  };

  var defaultManagers = new Map();
  var getManager = function getManager(context, managerId) {
    // If `managers` map is present in the context, we use it in order to
    // let JssProvider reset them when new response has to render server-side.
    if (context.managers) {
      if (!context.managers[managerId]) {
        context.managers[managerId] = new SheetsManager();
      }

      return context.managers[managerId];
    }

    var manager = defaultManagers.get(managerId);

    if (!manager) {
      manager = new SheetsManager();
      defaultManagers.set(managerId, manager);
    }

    return manager;
  };
  var manageSheet = function manageSheet(options) {
    var sheet = options.sheet,
        context = options.context,
        index = options.index,
        theme = options.theme;

    if (!sheet) {
      return;
    }

    var manager = getManager(context, index);
    manager.manage(theme);

    if (context.registry) {
      context.registry.add(sheet);
    }
  };
  var unmanageSheet = function unmanageSheet(options) {
    if (!options.sheet) {
      return;
    }

    var manager = getManager(options.context, options.index);
    manager.unmanage(options.theme);
  };

  var defaultJss = create(index$1());

  var sheetsMeta = new WeakMap();
  var getMeta = function getMeta(sheet) {
    return sheetsMeta.get(sheet);
  };
  var addMeta = function addMeta(sheet, meta) {
    sheetsMeta.set(sheet, meta);
  };

  var getStyles = function getStyles(options) {
    var styles = options.styles;

    if (typeof styles !== 'function') {
      return styles;
    }

    process.env.NODE_ENV !== "production" ? warning(styles.length !== 0, "[JSS] <" + (options.name || 'Hook') + " />'s styles function doesn't rely on the \"theme\" argument. We recommend declaring styles as an object instead.") : void 0;
    return styles(options.theme);
  };

  function getSheetOptions(options, link) {
    var minify;

    if (options.context.id && options.context.id.minify != null) {
      minify = options.context.id.minify;
    }

    var classNamePrefix = options.context.classNamePrefix || '';

    if (options.name && !minify) {
      classNamePrefix += options.name.replace(/\s/g, '-') + "-";
    }

    var meta = '';
    if (options.name) meta = options.name + ", ";
    meta += typeof options.styles === 'function' ? 'Themed' : 'Unthemed';
    return _extends$1({}, options.sheetOptions, {
      index: options.index,
      meta: meta,
      classNamePrefix: classNamePrefix,
      link: link,
      generateId: options.sheetOptions.generateId || options.context.generateId
    });
  }

  var createStyleSheet = function createStyleSheet(options) {
    if (options.context.disableStylesGeneration) {
      return undefined;
    }

    var manager = getManager(options.context, options.index);
    var existingSheet = manager.get(options.theme);

    if (existingSheet) {
      return existingSheet;
    }

    var jss = options.context.jss || defaultJss;
    var styles = getStyles(options);
    var dynamicStyles = getDynamicStyles(styles);
    var sheet = jss.createStyleSheet(styles, getSheetOptions(options, dynamicStyles !== null));
    addMeta(sheet, {
      dynamicStyles: dynamicStyles,
      styles: styles
    });
    manager.add(options.theme, sheet);
    return sheet;
  };
  var removeDynamicRules = function removeDynamicRules(sheet, rules) {
    // Loop over each dynamic rule and remove the dynamic rule
    // We can't just remove the whole sheet as this has all of the rules for every component instance
    for (var key in rules) {
      sheet.deleteRule(rules[key]);
    }
  };
  var updateDynamicRules = function updateDynamicRules(data, sheet, rules) {
    // Loop over each dynamic rule and update it
    // We can't just update the whole sheet as this has all of the rules for every component instance
    for (var key in rules) {
      sheet.updateOne(rules[key], data);
    }
  };
  var addDynamicRules = function addDynamicRules(sheet, data) {
    var meta = getMeta(sheet);

    if (!meta) {
      return undefined;
    }

    var rules = {}; // Loop over each dynamic rule and add it to the stylesheet

    for (var key in meta.dynamicStyles) {
      var initialRuleCount = sheet.rules.index.length;
      var originalRule = sheet.addRule(key, meta.dynamicStyles[key]); // Loop through all created rules, fixes updating dynamic rules

      for (var i = initialRuleCount; i < sheet.rules.index.length; i++) {
        var rule = sheet.rules.index[i];
        sheet.updateOne(rule, data); // If it's the original rule, we need to add it by the correct key so the hook and hoc
        // can correctly concat the dynamic class with the static one

        rules[originalRule === rule ? key : rule.key] = rule;
      }
    }

    return rules;
  };

  var getSheetClasses = function getSheetClasses(sheet, dynamicRules) {
    if (!dynamicRules) {
      return sheet.classes;
    }

    var classes = {};
    var meta = getMeta(sheet);

    if (!meta) {
      return sheet.classes;
    }

    for (var key in meta.styles) {
      classes[key] = sheet.classes[key];

      if (key in dynamicRules) {
        classes[key] += " " + sheet.classes[dynamicRules[key].key];
      }
    }

    return classes;
  };

  var useEffectOrLayoutEffect = isBrowser ? React__default['default'].useLayoutEffect : React__default['default'].useEffect;
  var noTheme$1 = {};

  var createUseStyles = function createUseStyles(styles, options) {
    if (options === void 0) {
      options = {};
    }

    var _options = options,
        _options$index = _options.index,
        index = _options$index === void 0 ? getSheetIndex() : _options$index,
        theming = _options.theming,
        name = _options.name,
        sheetOptions = _objectWithoutPropertiesLoose(_options, ["index", "theming", "name"]);

    var ThemeContext$1 = theming && theming.context || ThemeContext;
    var useTheme = typeof styles === 'function' ? // $FlowFixMe
    function () {
      return React__default['default'].useContext(ThemeContext$1) || noTheme$1;
    } : // $FlowFixMe
    function () {
      return noTheme$1;
    };
    return function useStyles(data) {
      var isFirstMount = React__default['default'].useRef(true);
      var context = React__default['default'].useContext(JssContext);
      var theme = useTheme();

      var _React$useMemo = React__default['default'].useMemo(function () {
        var newSheet = createStyleSheet({
          context: context,
          styles: styles,
          name: name,
          theme: theme,
          index: index,
          sheetOptions: sheetOptions
        });
        var newDynamicRules = newSheet ? addDynamicRules(newSheet, data) : null;

        if (newSheet) {
          manageSheet({
            index: index,
            context: context,
            sheet: newSheet,
            theme: theme
          });
        }

        return [newSheet, newDynamicRules];
      }, [context, theme]),
          sheet = _React$useMemo[0],
          dynamicRules = _React$useMemo[1];

      useEffectOrLayoutEffect(function () {
        // We only need to update the rules on a subsequent update and not in the first mount
        if (sheet && dynamicRules && !isFirstMount.current) {
          updateDynamicRules(data, sheet, dynamicRules);
        }
      }, [data]);
      useEffectOrLayoutEffect(function () {
        return (// cleanup only
          function () {
            if (sheet) {
              unmanageSheet({
                index: index,
                context: context,
                sheet: sheet,
                theme: theme
              });
            }

            if (sheet && dynamicRules) {
              removeDynamicRules(sheet, dynamicRules);
            }
          }
        );
      }, [sheet]);
      var classes = sheet && dynamicRules ? getSheetClasses(sheet, dynamicRules) : {}; // $FlowFixMe

      React__default['default'].useDebugValue(classes); // $FlowFixMe

      React__default['default'].useDebugValue(theme === noTheme$1 ? 'No theme' : theme);
      React__default['default'].useEffect(function () {
        isFirstMount.current = false;
      });
      return classes;
    };
  };

  ({
    registry: propTypes.instanceOf(SheetsRegistry),
    jss: propTypes.instanceOf(index$2.constructor),
    generateId: propTypes.func,
    classNamePrefix: propTypes.string,
    disableStylesGeneration: propTypes.bool,
    children: propTypes.node.isRequired,
    media: propTypes.string,
    id: propTypes.shape({
      minify: propTypes.bool
    })
  });

  function useRefValue(value) {
      var ref = React.useRef(value);
      React.useLayoutEffect(function () {
          ref.current = value;
      }, [value]);
      return ref;
  }

  var SCROLL_BAR_WIDTH = 10;
  var wrapperCommonStyle = {
      position: "absolute",
      cursor: "initial"
  };
  var useStyles = createUseStyles({
      verticalScrollWrapper: __assign(__assign({}, wrapperCommonStyle), { height: "100%", width: SCROLL_BAR_WIDTH, top: 0, right: 0 }),
      horizontalScrollWrapper: __assign(__assign({}, wrapperCommonStyle), { height: SCROLL_BAR_WIDTH, width: "100%", bottom: 0, left: 0 }),
      verticalScrollStyle: function (args) { return ({
          height: args.scrollbarLayout.verticalScrollHeight,
          width: "100%",
          backgroundColor: args.theme.scrollbarColor,
          position: "absolute",
          top: 0,
          right: 0,
          transform: "translateY(" + args.scrollbarLayout.verticalScrollTop + "px)"
      }); },
      horizontalScrollStyle: function (args) { return ({
          width: args.scrollbarLayout.horizontalScrollWidth - SCROLL_BAR_WIDTH,
          height: "100%",
          backgroundColor: args.theme.scrollbarColor,
          position: "absolute",
          left: 0,
          bottom: 0,
          transform: "translateX(" + args.scrollbarLayout.horizontalScrollLeft + "px)"
      }); }
  });
  var Scrollbar = function (props) {
      var _a = props.vertical, vertical = _a === void 0 ? true : _a, _b = props.horizontal, horizontal = _b === void 0 ? true : _b, offsetLimit = props.offsetLimit, eventChannel = props.eventChannel, viewport = props.viewport;
      var graphController = useGraphController();
      var theme = useTheme().theme;
      var scrollbarLayout = getScrollbarLayout(viewport, offsetLimit);
      var classes = useStyles({ scrollbarLayout: scrollbarLayout, theme: theme });
      var scrollbarLayoutRef = useRefValue(scrollbarLayout);
      function onVerticalScrollMouseDown(e) {
          e.preventDefault();
          e.stopPropagation();
          var containerHeight = viewport.rect.height;
          var dragging = new DragController(new MouseMoveEventProvider(graphController.getGlobalEventTarget()), defaultGetPositionFromEvent);
          dragging.onMove = function (_a) {
              var scrollbarDy = _a.dy, rawEvent = _a.e;
              var totalContentHeight = scrollbarLayoutRef.current.totalContentHeight;
              var dy = -(scrollbarDy * totalContentHeight) / containerHeight;
              eventChannel.trigger({
                  type: exports.GraphScrollBarEvent.Scroll,
                  rawEvent: rawEvent,
                  dx: 0,
                  dy: dy
              });
          };
          dragging.onEnd = function () {
              eventChannel.trigger({
                  type: exports.GraphScrollBarEvent.ScrollEnd
              });
          };
          dragging.start(e.nativeEvent);
          eventChannel.trigger({
              type: exports.GraphScrollBarEvent.ScrollStart
          });
      }
      function onHorizontalScrollMouseDown(e) {
          e.preventDefault();
          e.stopPropagation();
          var containerWidth = viewport.rect.width;
          var dragging = new DragController(new MouseMoveEventProvider(graphController.getGlobalEventTarget()), defaultGetPositionFromEvent);
          dragging.onMove = function (_a) {
              var scrollbarDx = _a.dx, rawEvent = _a.e;
              var totalContentWidth = scrollbarLayoutRef.current.totalContentWidth;
              var dx = -(scrollbarDx * totalContentWidth) / containerWidth;
              eventChannel.trigger({
                  type: exports.GraphScrollBarEvent.Scroll,
                  rawEvent: rawEvent,
                  dx: dx,
                  dy: 0
              });
          };
          dragging.onEnd = function () {
              eventChannel.trigger({
                  type: exports.GraphScrollBarEvent.ScrollEnd
              });
          };
          dragging.start(e.nativeEvent);
          eventChannel.trigger({
              type: exports.GraphScrollBarEvent.ScrollStart
          });
      }
      return (React__namespace.createElement(React__namespace.Fragment, null,
          vertical && (React__namespace.createElement("div", { className: classes.verticalScrollWrapper },
              React__namespace.createElement("div", { className: classes.verticalScrollStyle, onMouseDown: onVerticalScrollMouseDown, role: "button", "aria-label": "vertical scrollbar", "aria-roledescription": "vertical scrollbar", id: "canvas-vertical-scrollbar" }))),
          horizontal && (React__namespace.createElement("div", { className: classes.horizontalScrollWrapper },
              React__namespace.createElement("div", { className: classes.horizontalScrollStyle, onMouseDown: onHorizontalScrollMouseDown, role: "button", "aria-label": "horizontal scrollbar", "aria-roledescription": "horizontal scrollbar", id: "canvas-horizontal-scrollbar" })))));
  };
  function getTotalContentHeight(containerHeight, offsetLimit) {
      var minY = offsetLimit.minY, maxY = offsetLimit.maxY;
      return containerHeight + maxY - minY;
  }
  function getTotalContentWidth(containerWidth, offsetLimit) {
      var minX = offsetLimit.minX, maxX = offsetLimit.maxX;
      return containerWidth + maxX - minX;
  }
  function getScrollbarLayout(viewport, offsetLimit) {
      var rect = viewport.rect, transformMatrix = viewport.transformMatrix;
      var totalContentHeight = getTotalContentHeight(rect.height, offsetLimit);
      var totalContentWidth = getTotalContentWidth(rect.width, offsetLimit);
      return {
          totalContentHeight: totalContentHeight,
          totalContentWidth: totalContentWidth,
          verticalScrollHeight: (rect.height * rect.height) / totalContentHeight,
          horizontalScrollWidth: (rect.width * rect.width) / totalContentWidth,
          verticalScrollTop: ((offsetLimit.maxY - transformMatrix[5]) * rect.height) / totalContentHeight,
          horizontalScrollLeft: ((offsetLimit.maxX - transformMatrix[4]) * rect.width) / totalContentWidth
      };
  }

  var Transform = function (_a) {
      var matrix = _a.matrix, children = _a.children;
      var transform = React__namespace.useMemo(function () { return "matrix(" + matrix.join(" ") + ")"; }, matrix);
      return React__namespace.createElement("g", { transform: transform }, children);
  };

  function getHintPoints(source, target, _a, yOnRightAxis, xOnBottomAxis, xOnTopAxis, yOnLeftAxis) {
      var minX = _a.minX, minY = _a.minY, maxX = _a.maxX, maxY = _a.maxY;
      if (source.x === target.x) {
          return {
              x: source.x,
              y: source.y < target.y ? maxY : minY
          };
      }
      if (source.x < target.x) {
          if (source.y < target.y) {
              return yOnRightAxis <= maxY ? { x: maxX, y: yOnRightAxis } : { x: xOnBottomAxis, y: maxY };
          }
          else {
              return yOnRightAxis >= minY ? { x: maxX, y: yOnRightAxis } : { x: xOnTopAxis, y: minY };
          }
      }
      if (source.y < target.y) {
          return xOnBottomAxis > minX ? { x: xOnBottomAxis, y: maxY } : { x: minX, y: yOnLeftAxis };
      }
      return yOnLeftAxis > minY ? { x: minX, y: yOnLeftAxis } : { x: xOnTopAxis, y: minY };
  }
  var GraphEdge = React__namespace.memo(function (props) {
      var _a;
      var edge = props.edge, graphModel = props.data, eventChannel = props.eventChannel, source = props.source, target = props.target, graphId = props.graphId;
      var graphConfig = useGraphConfig();
      var virtualization = useVirtualization();
      var viewport = virtualization.viewport, renderedArea = virtualization.renderedArea, visibleArea = virtualization.visibleArea;
      var theme = useTheme().theme;
      var edgeEvent = function (type) { return function (e) {
          e.persist();
          eventChannel.trigger({
              type: type,
              edge: edge,
              rawEvent: e
          });
      }; };
      var isSourceRendered = isPointInRect(renderedArea, source);
      var isTargetRendered = isPointInRect(renderedArea, target);
      var isVisible = isSourceRendered && isTargetRendered;
      React__namespace.useLayoutEffect(function () {
          if (isVisible) {
              virtualization.renderedEdges.add(edge.id);
          }
      }, [virtualization]);
      if (!isVisible) {
          return null;
      }
      var shape = edge.shape ? edge.shape : graphConfig.defaultEdgeShape;
      var edgeConfig = graphConfig.getEdgeConfigByName(shape);
      if (!edgeConfig) {
          Debug.warn("invalid shape in edge " + JSON.stringify(edge));
          return null;
      }
      if (!edgeConfig.render) {
          Debug.warn("Missing \"render\" method in edge config " + JSON.stringify(edge));
          return null;
      }
      var isSourceVisible = isPointInRect(visibleArea, source);
      var isTargetVisible = isPointInRect(visibleArea, target);
      var edgeNode = edgeConfig.render({
          model: edge,
          data: graphModel,
          x1: source.x,
          y1: source.y,
          x2: target.x,
          y2: target.y,
          theme: theme,
          viewport: viewport
      });
      if (hasState(exports.GraphEdgeState.connectedToSelected)(edge.state) && (!isSourceVisible || !isTargetVisible)) {
          var linearFunction = getLinearFunction(source.x, source.y, target.x, target.y);
          var inverseLinearFunction = getLinearFunction(source.y, source.x, target.y, target.x);
          var hintSource = isSourceVisible ? source : target;
          var hintTarget = isSourceVisible ? target : source;
          var yOnRightAxis = linearFunction(visibleArea.maxX);
          var xOnBottomAxis = inverseLinearFunction(visibleArea.maxY);
          var xOnTopAxis = inverseLinearFunction(visibleArea.minY);
          var yOnLeftAxis = linearFunction(visibleArea.minX);
          var hintPoint = getHintPoints(hintSource, hintTarget, visibleArea, yOnRightAxis, xOnBottomAxis, xOnTopAxis, yOnLeftAxis);
          if (isSourceVisible && edgeConfig.renderWithTargetHint) {
              edgeNode = edgeConfig.renderWithTargetHint({
                  model: edge,
                  data: graphModel,
                  x1: source.x,
                  y1: source.y,
                  x2: hintPoint.x,
                  y2: hintPoint.y,
                  theme: theme,
                  viewport: viewport
              });
          }
          else if (isTargetVisible && edgeConfig.renderWithSourceHint) {
              edgeNode = edgeConfig.renderWithSourceHint({
                  model: edge,
                  data: graphModel,
                  x1: hintPoint.x,
                  y1: hintPoint.y,
                  x2: target.x,
                  y2: target.y,
                  theme: theme,
                  viewport: viewport
              });
          }
      }
      var id = getEdgeUid(graphId, edge);
      var className = "edge-container-" + edge.id;
      var automationId = (_a = edge.automationId) !== null && _a !== void 0 ? _a : className;
      return (React__namespace.createElement("g", { id: id, onClick: edgeEvent(exports.GraphEdgeEvent.Click), onDoubleClick: edgeEvent(exports.GraphEdgeEvent.DoubleClick), onMouseDown: edgeEvent(exports.GraphEdgeEvent.MouseDown), onMouseUp: edgeEvent(exports.GraphEdgeEvent.MouseUp), onMouseEnter: edgeEvent(exports.GraphEdgeEvent.MouseEnter), onMouseLeave: edgeEvent(exports.GraphEdgeEvent.MouseLeave), onContextMenu: edgeEvent(exports.GraphEdgeEvent.ContextMenu), onMouseMove: edgeEvent(exports.GraphEdgeEvent.MouseMove), onMouseOver: edgeEvent(exports.GraphEdgeEvent.MouseOver), onMouseOut: edgeEvent(exports.GraphEdgeEvent.MouseOut), onFocus: undefined, onBlur: undefined, className: className, "data-automation-id": automationId }, edgeNode));
  });

  function compareEqual(prev, props) {
      return prev.node === props.node;
  }
  var EdgeChampNodeRender = React__namespace.memo(function (props) {
      var _a, _b;
      var node = props.node, data = props.data, others = __rest(props, ["node", "data"]);
      var graphConfig = useGraphConfig();
      var values = [];
      var valueCount = node.valueCount;
      for (var i = 0; i < valueCount; i += 1) {
          var it = node.getValue(i);
          var source = (_a = data.nodes.get(it.source)) === null || _a === void 0 ? void 0 : _a.getPortPosition(it.sourcePortId, graphConfig);
          var target = (_b = data.nodes.get(it.target)) === null || _b === void 0 ? void 0 : _b.getPortPosition(it.targetPortId, graphConfig);
          if (source && target) {
              values.push(React__namespace.createElement(GraphEdge, __assign({}, others, { key: it.id, data: data, edge: it, source: source, target: target })));
          }
      }
      var children = [];
      var nodeCount = node.nodeCount;
      for (var i = 0; i < nodeCount; i += 1) {
          var it = node.getNode(i);
          if (it.type === NodeType$1.Bitmap) {
              children.push(React__namespace.createElement(EdgeChampNodeRender, __assign({ key: node.getHash(i) }, props, { node: it })));
          }
          else {
              children.push(React__namespace.createElement(EdgeHashCollisionNodeRender, __assign({ key: it.getHash() }, props, { node: it })));
          }
      }
      return (React__namespace.createElement(React__namespace.Fragment, null,
          values,
          children));
  }, compareEqual);
  EdgeChampNodeRender.displayName = "EdgeChampNodeRender";
  var EdgeHashCollisionNodeRender = React__namespace.memo(function (props) {
      var data = props.data, node = props.node, others = __rest(props, ["data", "node"]);
      var graphConfig = useGraphConfig();
      return (React__namespace.createElement(React__namespace.Fragment, null, node.values.map(function (edge) {
          var _a, _b;
          var source = (_a = data.nodes.get(edge.source)) === null || _a === void 0 ? void 0 : _a.getPortPosition(edge.sourcePortId, graphConfig);
          var target = (_b = data.nodes.get(edge.target)) === null || _b === void 0 ? void 0 : _b.getPortPosition(edge.targetPortId, graphConfig);
          if (source && target) {
              return React__namespace.createElement(GraphEdge, __assign({}, others, { key: edge.id, data: data, edge: edge, source: source, target: target }));
          }
          else {
              return null;
          }
      })));
  }, compareEqual);
  EdgeHashCollisionNodeRender.displayName = "EdgeHashCollisionNodeRender";
  var EdgeTree = function (props) {
      var tree = props.tree, others = __rest(props, ["tree"]);
      return React__namespace.createElement(EdgeChampNodeRender, __assign({}, others, { node: tree.root }));
  };

  function styleInject(css, ref) {
    if ( ref === void 0 ) ref = {};
    var insertAt = ref.insertAt;

    if (!css || typeof document === 'undefined') { return; }

    var head = document.head || document.getElementsByTagName('head')[0];
    var style = document.createElement('style');
    style.type = 'text/css';

    if (insertAt === 'top') {
      if (head.firstChild) {
        head.insertBefore(style, head.firstChild);
      } else {
        head.appendChild(style);
      }
    } else {
      head.appendChild(style);
    }

    if (style.styleSheet) {
      style.styleSheet.cssText = css;
    } else {
      style.appendChild(document.createTextNode(css));
    }
  }

  var css_248z = ".svg_fd5eb451{position:absolute;overflow:visible;top:0;left:0;width:100%;height:100%}.svg_fd5eb451:focus{outline:none}.node_3cfd6437{cursor:move}.container_1eed5be4{position:relative;width:100%;height:100%;overflow:hidden;touch-action:none}.button-a11y_327f4b20{opacity:0;width:0;height:0;overflow:hidden}.adding-node-svg_9895eecf{z-index:1000000;position:fixed;left:0;top:0;width:100%;height:100%}.module-item_48fc14a0{user-select:none;cursor:pointer}.minimap_00616a7a{height:320px;width:320px;user-select:none;touch-action:none}.minimap-svg_7c6d9f3d{position:absolute;top:0;left:0;width:100%;height:100%}";
  var classes = {"svg":"svg_fd5eb451","node":"node_3cfd6437","container":"container_1eed5be4","button-a11y":"button-a11y_327f4b20","buttonA11Y":"button-a11y_327f4b20","adding-node-svg":"adding-node-svg_9895eecf","addingNodeSvg":"adding-node-svg_9895eecf","module-item":"module-item_48fc14a0","moduleItem":"module-item_48fc14a0","minimap":"minimap_00616a7a","minimap-svg":"minimap-svg_7c6d9f3d","minimapSvg":"minimap-svg_7c6d9f3d"};
  styleInject(css_248z);

  var GraphNode = function (props) {
      var _a;
      var node = props.node, eventChannel = props.eventChannel, getNodeAriaLabel = props.getNodeAriaLabel, viewport = props.viewport, graphId = props.graphId;
      var graphConfig = useGraphConfig();
      var shape = node.shape ? node.shape : graphConfig.defaultNodeShape;
      var nodeConfig = getNodeConfig(node, graphConfig);
      var theme = useTheme().theme;
      var nodeEvent = function (type) { return function (evt) {
          evt.persist();
          var e = {
              type: type,
              node: node,
              rawEvent: evt
          };
          eventChannel.trigger(e);
      }; };
      var clickEvent = function (e) {
          e.persist();
          eventChannel.trigger({
              type: exports.GraphNodeEvent.Click,
              rawEvent: e,
              isMultiSelect: checkIsMultiSelect(e),
              node: node
          });
      };
      var id = getNodeUid(graphId, node);
      var automationId = (_a = node.automationId) !== null && _a !== void 0 ? _a : getNodeAutomationId(node);
      if (!(nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.render)) {
          Debug.warn("Missing \"render\" method in node config " + shape);
          return null;
      }
      return (React__namespace.createElement("g", { id: id, focusable: "true", tabIndex: 0, className: classes.node, onPointerDown: nodeEvent(exports.GraphNodeEvent.PointerDown), onPointerEnter: nodeEvent(exports.GraphNodeEvent.PointerEnter), onPointerMove: nodeEvent(exports.GraphNodeEvent.PointerMove), onPointerLeave: nodeEvent(exports.GraphNodeEvent.PointerLeave), onPointerUp: nodeEvent(exports.GraphNodeEvent.PointerUp), onDoubleClick: nodeEvent(exports.GraphNodeEvent.DoubleClick), onMouseDown: nodeEvent(exports.GraphNodeEvent.MouseDown), onMouseUp: nodeEvent(exports.GraphNodeEvent.MouseUp), onMouseEnter: nodeEvent(exports.GraphNodeEvent.MouseEnter), onMouseLeave: nodeEvent(exports.GraphNodeEvent.MouseLeave), onContextMenu: nodeEvent(exports.GraphNodeEvent.ContextMenu), onMouseMove: nodeEvent(exports.GraphNodeEvent.MouseMove), onMouseOver: nodeEvent(exports.GraphNodeEvent.MouseOver), onMouseOut: nodeEvent(exports.GraphNodeEvent.MouseOut), onKeyDown: nodeEvent(exports.GraphNodeEvent.KeyDown), onClick: clickEvent, "aria-label": getNodeAriaLabel(node), role: "group", "aria-roledescription": "node", "data-automation-id": automationId },
          React__namespace.createElement("g", { className: "node-box-container" }, nodeConfig.render({
              model: node,
              theme: theme,
              viewport: viewport
          }))));
  };

  var BBOX_PADDING = 15;
  var ResizePoint = function (_a) {
      var x = _a.x, y = _a.y, cursor = _a.cursor, theme = _a.theme, onMouseDown = _a.onMouseDown;
      return (React__namespace.createElement("rect", { x: x, y: y, height: 8, width: 8, stroke: theme.controlPointColor, fill: "transparent", cursor: cursor, onMouseDown: onMouseDown }));
  };
  var GraphNodeControlPoints = function (props) {
      var _a, _b;
      var node = props.node, eventChannel = props.eventChannel;
      var graphConfig = useGraphConfig();
      var nodeConfig = getNodeConfig(node, graphConfig);
      var graphController = useGraphController();
      var minWidth = (_a = nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.getMinWidth(node)) !== null && _a !== void 0 ? _a : 0;
      var minHeight = (_b = nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.getMinHeight(node)) !== null && _b !== void 0 ? _b : 0;
      var theme = useTheme().theme;
      var height = getRectHeight(nodeConfig, node);
      var width = getRectWidth(nodeConfig, node);
      var getMouseDown = function (f) { return function (evt) {
          evt.preventDefault();
          evt.stopPropagation();
          eventChannel.trigger({
              type: exports.GraphNodeEvent.ResizingStart,
              rawEvent: evt,
              node: node
          });
          var drag = new DragController(new MouseMoveEventProvider(graphController.getGlobalEventTarget()), defaultGetPositionFromEvent);
          drag.onMove = function (_a) {
              var totalDX = _a.totalDX, totalDY = _a.totalDY, rawEvent = _a.e;
              eventChannel.trigger(__assign({ type: exports.GraphNodeEvent.Resizing, rawEvent: rawEvent,
                  node: node, dx: 0, dy: 0, dWidth: 0, dHeight: 0 }, f(totalDX, totalDY)));
          };
          drag.onEnd = function (_a) {
              var rawEvent = _a.e;
              eventChannel.trigger({
                  type: exports.GraphNodeEvent.ResizingEnd,
                  rawEvent: rawEvent,
                  node: node
              });
          };
          eventChannel.trigger({
              type: exports.GraphNodeEvent.ResizingStart,
              rawEvent: evt,
              node: node
          });
          drag.start(evt.nativeEvent);
      }; };
      var nw = getMouseDown(function (dx, dy) {
          var finalDx = Math.min(dx, width - minWidth);
          var finalDy = Math.min(dy, height - minHeight);
          return {
              dx: +finalDx,
              dy: +finalDy,
              dWidth: -finalDx,
              dHeight: -finalDy
          };
      });
      var n = getMouseDown(function (dx, dy) {
          var finalDy = Math.min(dy, height - minHeight);
          return {
              dy: +finalDy,
              dHeight: -finalDy
          };
      });
      var ne = getMouseDown(function (dx, dy) {
          var finalDx = Math.max(dx, minWidth - width);
          var finalDy = Math.min(dy, height - minHeight);
          return {
              dy: +finalDy,
              dWidth: +finalDx,
              dHeight: -finalDy
          };
      });
      var e = getMouseDown(function (dx) {
          var finalDx = Math.max(dx, minWidth - width);
          return {
              dWidth: +finalDx
          };
      });
      var se = getMouseDown(function (dx, dy) {
          var finalDx = Math.max(dx, minWidth - width);
          var finalDy = Math.max(dy, minHeight - height);
          return {
              dWidth: +finalDx,
              dHeight: +finalDy
          };
      });
      var s = getMouseDown(function (dx, dy) {
          var finalDy = Math.max(dy, minHeight - height);
          return {
              dHeight: +finalDy
          };
      });
      var sw = getMouseDown(function (dx, dy) {
          var finalDx = Math.min(dx, width - minWidth);
          var finalDy = Math.max(dy, minHeight - height);
          return {
              dx: +finalDx,
              dWidth: -finalDx,
              dHeight: +finalDy
          };
      });
      var w = getMouseDown(function (dx) {
          var finalDx = Math.min(dx, width - minWidth);
          return {
              dx: finalDx,
              dWidth: -finalDx
          };
      });
      return (React__namespace.createElement(React__namespace.Fragment, null,
          React__namespace.createElement(ResizePoint, { theme: theme, cursor: "nw-resize", x: node.x - BBOX_PADDING, y: node.y - BBOX_PADDING, onMouseDown: nw }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x + width / 2, y: node.y - BBOX_PADDING, cursor: "n-resize", onMouseDown: n }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x + width + BBOX_PADDING, y: node.y - BBOX_PADDING, cursor: "ne-resize", onMouseDown: ne }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x + width + BBOX_PADDING, y: node.y + height / 2, cursor: "e-resize", onMouseDown: e }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x + width + BBOX_PADDING, y: node.y + height + BBOX_PADDING, cursor: "se-resize", onMouseDown: se }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x + width / 2, y: node.y + height + BBOX_PADDING, cursor: "s-resize", onMouseDown: s }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x - BBOX_PADDING, y: node.y + height + BBOX_PADDING, cursor: "sw-resize", onMouseDown: sw }),
          React__namespace.createElement(ResizePoint, { theme: theme, x: node.x - BBOX_PADDING, y: node.y + height / 2, cursor: "w-resize", onMouseDown: w })));
  };

  var GraphOneNodePorts = function (props) {
      var data = props.data, node = props.node, getPortAriaLabel = props.getPortAriaLabel, eventChannel = props.eventChannel, viewport = props.viewport, graphId = props.graphId;
      var graphConfig = useGraphConfig();
      var theme = useTheme().theme;
      var ports = node.ports;
      if (!ports) {
          return null;
      }
      var portEvent = function (type, port) { return function (e) {
          e.persist();
          eventChannel.trigger({
              type: type,
              node: node,
              port: port,
              rawEvent: e
          });
      }; };
      return (React__namespace.createElement("g", null, ports.map(function (p) {
          var _a;
          var portShape = p.shape ? p.shape : graphConfig.defaultPortShape;
          var portConfig = graphConfig.getPortConfigByName(portShape);
          if (!portConfig || !portConfig.render) {
              Debug.warn("invalid port config " + node.id + ":" + node.name + " - " + p.id + ":" + p.name + ", shape: " + p.shape);
              return null;
          }
          var pos = node.getPortPosition(p.id, graphConfig);
          if (!pos) {
              return null;
          }
          var portKey = getPortUid(graphId, node, p);
          var automationId = (_a = p.automationId) !== null && _a !== void 0 ? _a : getPortAutomationId(p, node);
          return (React__namespace.createElement("g", { key: portKey, id: portKey, tabIndex: 0, focusable: "true", onPointerDown: portEvent(exports.GraphPortEvent.PointerDown, p), onPointerUp: portEvent(exports.GraphPortEvent.PointerUp, p), onDoubleClick: portEvent(exports.GraphPortEvent.DoubleClick, p), onMouseDown: portEvent(exports.GraphPortEvent.MouseDown, p), onMouseUp: portEvent(exports.GraphPortEvent.MouseUp, p), onContextMenu: portEvent(exports.GraphPortEvent.ContextMenu, p), onPointerEnter: portEvent(exports.GraphPortEvent.PointerEnter, p), onPointerLeave: portEvent(exports.GraphPortEvent.PointerLeave, p), onMouseMove: portEvent(exports.GraphPortEvent.MouseMove, p), onMouseOver: portEvent(exports.GraphPortEvent.MouseOver, p), onMouseOut: portEvent(exports.GraphPortEvent.MouseOut, p), onFocus: portEvent(exports.GraphPortEvent.Focus, p), onBlur: portEvent(exports.GraphPortEvent.Blur, p), onKeyDown: portEvent(exports.GraphPortEvent.KeyDown, p), "aria-label": getPortAriaLabel(data, node, p), role: "group", "aria-roledescription": "port", "data-automation-id": automationId },
              React__namespace.createElement(ConnectingStateContext.Consumer, null, function (_a) {
                  var sourceNode = _a.sourceNode, sourcePort = _a.sourcePort;
                  return portConfig === null || portConfig === void 0 ? void 0 : portConfig.render(__assign({ model: p, data: data, parentNode: node, theme: theme, anotherNode: sourceNode, anotherPort: sourcePort, viewport: viewport }, pos));
              })));
      })));
  };

  var GraphNodeParts = function (_a) {
      var node = _a.node, isNodeResizable = _a.isNodeResizable, commonProps = __rest(_a, ["node", "isNodeResizable"]);
      var virtualization = useVirtualization();
      var renderedArea = virtualization.renderedArea, viewport = virtualization.viewport;
      var isVisible = isPointInRect(renderedArea, node);
      React__namespace.useLayoutEffect(function () {
          if (isVisible) {
              virtualization.renderedEdges.add(node.id);
          }
      }, [virtualization]);
      if (!isVisible) {
          return null;
      }
      return (React__namespace.createElement(React__namespace.Fragment, null,
          React__namespace.createElement(GraphNode, __assign({}, commonProps, { node: node, viewport: viewport })),
          React__namespace.createElement(GraphOneNodePorts, __assign({}, commonProps, { node: node, viewport: viewport })),
          isNodeResizable && isNodeEditing(node) && (React__namespace.createElement(GraphNodeControlPoints, { node: node, eventChannel: commonProps.eventChannel }))));
  };
  var GraphNodePartsMemo = React__namespace.memo(GraphNodeParts);

  var NodeTreeNode = React__namespace.memo(function (_a) {
      var node = _a.node, others = __rest(_a, ["node"]);
      var values = node.values.map(function (arg) {
          var it = arg[1];
          return React__namespace.createElement(GraphNodePartsMemo, __assign({ key: it.id, node: it }, others));
      });
      var children = node.type === NodeType.Internal
          ? node.children.map(function (child, index) {
              var key = index < node.selfSize ? node.getKey(index) : "last";
              return React__namespace.createElement(NodeTreeNode, __assign({ key: key, node: child }, others));
          })
          : undefined;
      return (React__namespace.createElement(React__namespace.Fragment, null,
          values,
          children));
  }, function (prevProps, nextProps) { return prevProps.node === nextProps.node; });
  NodeTreeNode.displayName = "NodeTreeNode";
  var NodeTree = function (_a) {
      var tree = _a.tree, others = __rest(_a, ["tree"]);
      return React__namespace.createElement(NodeTreeNode, __assign({ node: tree.sortedRoot }, others));
  };

  var VirtualizationProvider = function (_a) {
      var viewport = _a.viewport, isVirtualizationEnabled = _a.isVirtualizationEnabled, virtualizationDelay = _a.virtualizationDelay, eventChannel = _a.eventChannel, children = _a.children;
      var renderedArea = useRenderedArea(viewport, isVirtualizationEnabled);
      var visibleArea = React__namespace.useMemo(function () { return getVisibleArea(viewport); }, [viewport]);
      var contextValue = React__namespace.useMemo(function () { return ({
          viewport: viewport,
          renderedArea: renderedArea,
          visibleArea: visibleArea,
          renderedEdges: new Set(),
          renderedNodes: new Set(),
          timestamp: performance.now()
      }); }, [viewport, renderedArea, visibleArea]);
      var context = useDeferredValue(contextValue, { timeout: virtualizationDelay });
      var previousContextRef = React__namespace.useRef(context);
      React__namespace.useEffect(function () {
          var previousContext = previousContextRef.current;
          previousContextRef.current = context;
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.VirtualizationRecalculated,
              performanceStartTime: context.timestamp,
              renderedNodes: previousContext.renderedNodes,
              renderedEdges: previousContext.renderedEdges,
              previousRenderedNodes: previousContext.renderedNodes,
              previousRenderedEdges: previousContext.renderedEdges
          });
      }, [context, eventChannel]);
      return React__namespace.createElement(VirtualizationContext.Provider, { value: context }, children);
  };

  var InjectionMode = {
      /**
       * Avoids style injection, use getRules() to read the styles.
       */
      none: 0,
      /**
       * Inserts rules using the insertRule api.
       */
      insertNode: 1,
      /**
       * Appends rules using appendChild.
       */
      appendChild: 2,
  };
  var STYLESHEET_SETTING = '__stylesheet__';
  /**
   * MSIE 11 doesn't cascade styles based on DOM ordering, but rather on the order that each style node
   * is created. As such, to maintain consistent priority, IE11 should reuse a single style node.
   */
  var REUSE_STYLE_NODE = typeof navigator !== 'undefined' && /rv:11.0/.test(navigator.userAgent);
  var _global = {};
  // Grab window.
  try {
      _global = window;
  }
  catch (_a) {
      /* leave as blank object */
  }
  var _stylesheet;
  /**
   * Represents the state of styles registered in the page. Abstracts
   * the surface for adding styles to the stylesheet, exposes helpers
   * for reading the styles registered in server rendered scenarios.
   *
   * @public
   */
  var Stylesheet = /** @class */ (function () {
      function Stylesheet(config) {
          this._rules = [];
          this._preservedRules = [];
          this._rulesToInsert = [];
          this._counter = 0;
          this._keyToClassName = {};
          this._onResetCallbacks = [];
          this._classNameToArgs = {};
          this._config = __assign({ injectionMode: InjectionMode.insertNode, defaultPrefix: 'css', namespace: undefined, cspSettings: undefined }, config);
          this._keyToClassName = this._config.classNameCache || {};
      }
      /**
       * Gets the singleton instance.
       */
      Stylesheet.getInstance = function () {
          _stylesheet = _global[STYLESHEET_SETTING];
          if (!_stylesheet || (_stylesheet._lastStyleElement && _stylesheet._lastStyleElement.ownerDocument !== document)) {
              var fabricConfig = (_global === null || _global === void 0 ? void 0 : _global.FabricConfig) || {};
              _stylesheet = _global[STYLESHEET_SETTING] = new Stylesheet(fabricConfig.mergeStyles);
          }
          return _stylesheet;
      };
      /**
       * Configures the stylesheet.
       */
      Stylesheet.prototype.setConfig = function (config) {
          this._config = __assign(__assign({}, this._config), config);
      };
      /**
       * Configures a reset callback.
       *
       * @param callback - A callback which will be called when the Stylesheet is reset.
       */
      Stylesheet.prototype.onReset = function (callback) {
          this._onResetCallbacks.push(callback);
      };
      /**
       * Generates a unique classname.
       *
       * @param displayName - Optional value to use as a prefix.
       */
      Stylesheet.prototype.getClassName = function (displayName) {
          var namespace = this._config.namespace;
          var prefix = displayName || this._config.defaultPrefix;
          return "" + (namespace ? namespace + '-' : '') + prefix + "-" + this._counter++;
      };
      /**
       * Used internally to cache information about a class which was
       * registered with the stylesheet.
       */
      Stylesheet.prototype.cacheClassName = function (className, key, args, rules) {
          this._keyToClassName[key] = className;
          this._classNameToArgs[className] = {
              args: args,
              rules: rules,
          };
      };
      /**
       * Gets the appropriate classname given a key which was previously
       * registered using cacheClassName.
       */
      Stylesheet.prototype.classNameFromKey = function (key) {
          return this._keyToClassName[key];
      };
      /**
       * Gets all classnames cache with the stylesheet.
       */
      Stylesheet.prototype.getClassNameCache = function () {
          return this._keyToClassName;
      };
      /**
       * Gets the arguments associated with a given classname which was
       * previously registered using cacheClassName.
       */
      Stylesheet.prototype.argsFromClassName = function (className) {
          var entry = this._classNameToArgs[className];
          return entry && entry.args;
      };
      /**
       * Gets the arguments associated with a given classname which was
       * previously registered using cacheClassName.
       */
      Stylesheet.prototype.insertedRulesFromClassName = function (className) {
          var entry = this._classNameToArgs[className];
          return entry && entry.rules;
      };
      /**
       * Inserts a css rule into the stylesheet.
       * @param preserve - Preserves the rule beyond a reset boundary.
       */
      Stylesheet.prototype.insertRule = function (rule, preserve) {
          var injectionMode = this._config.injectionMode;
          var element = injectionMode !== InjectionMode.none ? this._getStyleElement() : undefined;
          if (preserve) {
              this._preservedRules.push(rule);
          }
          if (element) {
              switch (this._config.injectionMode) {
                  case InjectionMode.insertNode:
                      var sheet = element.sheet;
                      try {
                          sheet.insertRule(rule, sheet.cssRules.length);
                      }
                      catch (e) {
                          // The browser will throw exceptions on unsupported rules (such as a moz prefix in webkit.)
                          // We need to swallow the exceptions for this scenario, otherwise we'd need to filter
                          // which could be slower and bulkier.
                      }
                      break;
                  case InjectionMode.appendChild:
                      element.appendChild(document.createTextNode(rule));
                      break;
              }
          }
          else {
              this._rules.push(rule);
          }
          if (this._config.onInsertRule) {
              this._config.onInsertRule(rule);
          }
      };
      /**
       * Gets all rules registered with the stylesheet; only valid when
       * using InsertionMode.none.
       */
      Stylesheet.prototype.getRules = function (includePreservedRules) {
          return ((includePreservedRules ? this._preservedRules.join('') : '') + this._rules.join('') + this._rulesToInsert.join(''));
      };
      /**
       * Resets the internal state of the stylesheet. Only used in server
       * rendered scenarios where we're using InsertionMode.none.
       */
      Stylesheet.prototype.reset = function () {
          this._rules = [];
          this._rulesToInsert = [];
          this._counter = 0;
          this._classNameToArgs = {};
          this._keyToClassName = {};
          this._onResetCallbacks.forEach(function (callback) { return callback(); });
      };
      // Forces the regeneration of incoming styles without totally resetting the stylesheet.
      Stylesheet.prototype.resetKeys = function () {
          this._keyToClassName = {};
      };
      Stylesheet.prototype._getStyleElement = function () {
          var _this = this;
          if (!this._styleElement && typeof document !== 'undefined') {
              this._styleElement = this._createStyleElement();
              if (!REUSE_STYLE_NODE) {
                  // Reset the style element on the next frame.
                  window.requestAnimationFrame(function () {
                      _this._styleElement = undefined;
                  });
              }
          }
          return this._styleElement;
      };
      Stylesheet.prototype._createStyleElement = function () {
          var head = document.head;
          var styleElement = document.createElement('style');
          styleElement.setAttribute('data-merge-styles', 'true');
          var cspSettings = this._config.cspSettings;
          if (cspSettings) {
              if (cspSettings.nonce) {
                  styleElement.setAttribute('nonce', cspSettings.nonce);
              }
          }
          if (this._lastStyleElement) {
              // If the `nextElementSibling` is null, then the insertBefore will act as a regular append.
              // https://developer.mozilla.org/en-US/docs/Web/API/Node/insertBefore#Syntax
              head.insertBefore(styleElement, this._lastStyleElement.nextElementSibling);
          }
          else {
              var placeholderStyleTag = this._findPlaceholderStyleTag();
              if (placeholderStyleTag) {
                  head.insertBefore(styleElement, placeholderStyleTag.nextElementSibling);
              }
              else {
                  head.insertBefore(styleElement, head.childNodes[0]);
              }
          }
          this._lastStyleElement = styleElement;
          return styleElement;
      };
      Stylesheet.prototype._findPlaceholderStyleTag = function () {
          var head = document.head;
          if (head) {
              return head.querySelector('style[data-merge-styles]');
          }
          return null;
      };
      return Stylesheet;
  }());

  /**
   * Separates the classes and style objects. Any classes that are pre-registered
   * args are auto expanded into objects.
   */
  function extractStyleParts() {
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          args[_i] = arguments[_i];
      }
      var classes = [];
      var objects = [];
      var stylesheet = Stylesheet.getInstance();
      function _processArgs(argsList) {
          for (var _i = 0, argsList_1 = argsList; _i < argsList_1.length; _i++) {
              var arg = argsList_1[_i];
              if (arg) {
                  if (typeof arg === 'string') {
                      if (arg.indexOf(' ') >= 0) {
                          _processArgs(arg.split(' '));
                      }
                      else {
                          var translatedArgs = stylesheet.argsFromClassName(arg);
                          if (translatedArgs) {
                              _processArgs(translatedArgs);
                          }
                          else {
                              // Avoid adding the same class twice.
                              if (classes.indexOf(arg) === -1) {
                                  classes.push(arg);
                              }
                          }
                      }
                  }
                  else if (Array.isArray(arg)) {
                      _processArgs(arg);
                  }
                  else if (typeof arg === 'object') {
                      objects.push(arg);
                  }
              }
          }
      }
      _processArgs(args);
      return {
          classes: classes,
          objects: objects,
      };
  }

  /**
   * Sets the current RTL value.
   */
  /**
   * Gets the current RTL value.
   */
  function getRTL() {
      if (_rtl === undefined) {
          _rtl =
              typeof document !== 'undefined' &&
                  !!document.documentElement &&
                  document.documentElement.getAttribute('dir') === 'rtl';
      }
      return _rtl;
  }
  // This has been split into 2 lines because it was working in Fabric due to the code being transpiled to es5, so this
  // was converted to var while not working in Fluent that uses babel to transpile the code to be es6-like. Splitting the
  // logic into two lines, however, allows it to work in both scenarios.
  var _rtl;
  _rtl = getRTL();
  function getStyleOptions() {
      return {
          rtl: getRTL(),
      };
  }

  var rules = {};
  function kebabRules(rulePairs, index) {
      var rule = rulePairs[index];
      if (rule.charAt(0) !== '-') {
          rulePairs[index] = rules[rule] = rules[rule] || rule.replace(/([A-Z])/g, '-$1').toLowerCase();
      }
  }

  var _vendorSettings;
  function getVendorSettings() {
      if (!_vendorSettings) {
          var doc = typeof document !== 'undefined' ? document : undefined;
          var nav = typeof navigator !== 'undefined' ? navigator : undefined;
          var userAgent = nav ? nav.userAgent.toLowerCase() : undefined;
          if (!doc) {
              _vendorSettings = {
                  isWebkit: true,
                  isMoz: true,
                  isOpera: true,
                  isMs: true,
              };
          }
          else {
              _vendorSettings = {
                  isWebkit: !!(doc && 'WebkitAppearance' in doc.documentElement.style),
                  isMoz: !!(userAgent && userAgent.indexOf('firefox') > -1),
                  isOpera: !!(userAgent && userAgent.indexOf('opera') > -1),
                  isMs: !!(nav && (/rv:11.0/i.test(nav.userAgent) || /Edge\/\d./i.test(navigator.userAgent))),
              };
          }
      }
      return _vendorSettings;
  }

  var autoPrefixNames = {
      'user-select': 1,
  };
  function prefixRules(rulePairs, index) {
      var vendorSettings = getVendorSettings();
      var name = rulePairs[index];
      if (autoPrefixNames[name]) {
          var value = rulePairs[index + 1];
          if (autoPrefixNames[name]) {
              if (vendorSettings.isWebkit) {
                  rulePairs.push('-webkit-' + name, value);
              }
              if (vendorSettings.isMoz) {
                  rulePairs.push('-moz-' + name, value);
              }
              if (vendorSettings.isMs) {
                  rulePairs.push('-ms-' + name, value);
              }
              if (vendorSettings.isOpera) {
                  rulePairs.push('-o-' + name, value);
              }
          }
      }
  }

  var NON_PIXEL_NUMBER_PROPS = [
      'column-count',
      'font-weight',
      'flex',
      'flex-grow',
      'flex-shrink',
      'fill-opacity',
      'opacity',
      'order',
      'z-index',
      'zoom',
  ];
  function provideUnits(rulePairs, index) {
      var name = rulePairs[index];
      var value = rulePairs[index + 1];
      if (typeof value === 'number') {
          var isNonPixelProp = NON_PIXEL_NUMBER_PROPS.indexOf(name) > -1;
          var isVariableOrPrefixed = name.indexOf('--') > -1;
          var unit = isNonPixelProp || isVariableOrPrefixed ? '' : 'px';
          rulePairs[index + 1] = "" + value + unit;
      }
  }

  var _a;
  var LEFT = 'left';
  var RIGHT = 'right';
  var NO_FLIP = '@noflip';
  var NAME_REPLACEMENTS = (_a = {},
      _a[LEFT] = RIGHT,
      _a[RIGHT] = LEFT,
      _a);
  var VALUE_REPLACEMENTS = {
      'w-resize': 'e-resize',
      'sw-resize': 'se-resize',
      'nw-resize': 'ne-resize',
  };
  /**
   * RTLifies the rulePair in the array at the current index. This mutates the array for performance
   * reasons.
   */
  function rtlifyRules(options, rulePairs, index) {
      if (options.rtl) {
          var name_1 = rulePairs[index];
          if (!name_1) {
              return;
          }
          var value = rulePairs[index + 1];
          if (typeof value === 'string' && value.indexOf(NO_FLIP) >= 0) {
              rulePairs[index + 1] = value.replace(/\s*(?:\/\*\s*)?\@noflip\b(?:\s*\*\/)?\s*?/g, '');
          }
          else if (name_1.indexOf(LEFT) >= 0) {
              rulePairs[index] = name_1.replace(LEFT, RIGHT);
          }
          else if (name_1.indexOf(RIGHT) >= 0) {
              rulePairs[index] = name_1.replace(RIGHT, LEFT);
          }
          else if (String(value).indexOf(LEFT) >= 0) {
              rulePairs[index + 1] = value.replace(LEFT, RIGHT);
          }
          else if (String(value).indexOf(RIGHT) >= 0) {
              rulePairs[index + 1] = value.replace(RIGHT, LEFT);
          }
          else if (NAME_REPLACEMENTS[name_1]) {
              rulePairs[index] = NAME_REPLACEMENTS[name_1];
          }
          else if (VALUE_REPLACEMENTS[value]) {
              rulePairs[index + 1] = VALUE_REPLACEMENTS[value];
          }
          else {
              switch (name_1) {
                  case 'margin':
                  case 'padding':
                      rulePairs[index + 1] = flipQuad(value);
                      break;
                  case 'box-shadow':
                      rulePairs[index + 1] = negateNum(value, 0);
                      break;
              }
          }
      }
  }
  /**
   * Given a string value in a space delimited format (e.g. "1 2 3 4"), negates a particular value.
   */
  function negateNum(value, partIndex) {
      var parts = value.split(' ');
      var numberVal = parseInt(parts[partIndex], 10);
      parts[0] = parts[0].replace(String(numberVal), String(numberVal * -1));
      return parts.join(' ');
  }
  /**
   * Given a string quad, flips the left and right values.
   */
  function flipQuad(value) {
      if (typeof value === 'string') {
          var parts = value.split(' ');
          if (parts.length === 4) {
              return parts[0] + " " + parts[3] + " " + parts[2] + " " + parts[1];
          }
      }
      return value;
  }

  /**
   * Split a string into tokens separated by whitespace, except all text within parentheses
   * is treated as a single token (whitespace is ignored within parentheses).
   *
   * Unlike String.split(' '), multiple consecutive space characters are collapsed and
   * removed from the returned array (including leading and trailing spaces).
   *
   * For example:
   * `tokenizeWithParentheses("3px calc(var(--x) / 2) 9px    0 ")`
   *   => `["3px", "calc(var(--x) / 2)", "9px", "0"]`
   *
   * @returns The array of tokens. Returns an empty array if the string was empty or contained only whitespace.
   */
  function tokenizeWithParentheses(value) {
      var parts = [];
      var partStart = 0;
      var parens = 0;
      for (var i = 0; i < value.length; i++) {
          switch (value[i]) {
              case '(':
                  parens++;
                  break;
              case ')':
                  if (parens) {
                      parens--;
                  }
                  break;
              case '\t':
              case ' ':
                  if (!parens) {
                      // Add the new part if it's not an empty string
                      if (i > partStart) {
                          parts.push(value.substring(partStart, i));
                      }
                      partStart = i + 1;
                  }
                  break;
          }
      }
      // Add the last part
      if (partStart < value.length) {
          parts.push(value.substring(partStart));
      }
      return parts;
  }

  var DISPLAY_NAME = 'displayName';
  function getDisplayName(rules) {
      var rootStyle = rules && rules['&'];
      return rootStyle ? rootStyle.displayName : undefined;
  }
  var globalSelectorRegExp = /\:global\((.+?)\)/g;
  /**
   * Finds comma separated selectors in a :global() e.g. ":global(.class1, .class2, .class3)"
   * and wraps them each in their own global ":global(.class1), :global(.class2), :global(.class3)"
   *
   * @param selectorWithGlobals The selector to process
   * @returns The updated selector
   */
  function expandCommaSeparatedGlobals(selectorWithGlobals) {
      // We the selector does not have a :global() we can shortcut
      if (!globalSelectorRegExp.test(selectorWithGlobals)) {
          return selectorWithGlobals;
      }
      var replacementInfo = [];
      var findGlobal = /\:global\((.+?)\)/g;
      var match = null;
      // Create a result list for global selectors so we can replace them.
      while ((match = findGlobal.exec(selectorWithGlobals))) {
          // Only if the found selector is a comma separated list we'll process it.
          if (match[1].indexOf(',') > -1) {
              replacementInfo.push([
                  match.index,
                  match.index + match[0].length,
                  // Wrap each of the found selectors in :global()
                  match[1]
                      .split(',')
                      .map(function (v) { return ":global(" + v.trim() + ")"; })
                      .join(', '),
              ]);
          }
      }
      // Replace the found selectors with their wrapped variants in reverse order
      return replacementInfo
          .reverse()
          .reduce(function (selector, _a) {
          var matchIndex = _a[0], matchEndIndex = _a[1], replacement = _a[2];
          var prefix = selector.slice(0, matchIndex);
          var suffix = selector.slice(matchEndIndex);
          return prefix + replacement + suffix;
      }, selectorWithGlobals);
  }
  function expandSelector(newSelector, currentSelector) {
      if (newSelector.indexOf(':global(') >= 0) {
          return newSelector.replace(globalSelectorRegExp, '$1');
      }
      else if (newSelector.indexOf(':') === 0) {
          return currentSelector + newSelector;
      }
      else if (newSelector.indexOf('&') < 0) {
          return currentSelector + ' ' + newSelector;
      }
      return newSelector;
  }
  function extractSelector(currentSelector, rules, selector, value) {
      if (rules === void 0) { rules = { __order: [] }; }
      if (selector.indexOf('@') === 0) {
          selector = selector + '{' + currentSelector;
          extractRules([value], rules, selector);
      }
      else if (selector.indexOf(',') > -1) {
          expandCommaSeparatedGlobals(selector)
              .split(',')
              .map(function (s) { return s.trim(); })
              .forEach(function (separatedSelector) {
              return extractRules([value], rules, expandSelector(separatedSelector, currentSelector));
          });
      }
      else {
          extractRules([value], rules, expandSelector(selector, currentSelector));
      }
  }
  function extractRules(args, rules, currentSelector) {
      if (rules === void 0) { rules = { __order: [] }; }
      if (currentSelector === void 0) { currentSelector = '&'; }
      var stylesheet = Stylesheet.getInstance();
      var currentRules = rules[currentSelector];
      if (!currentRules) {
          currentRules = {};
          rules[currentSelector] = currentRules;
          rules.__order.push(currentSelector);
      }
      for (var _i = 0, args_1 = args; _i < args_1.length; _i++) {
          var arg = args_1[_i];
          // If the arg is a string, we need to look up the class map and merge.
          if (typeof arg === 'string') {
              var expandedRules = stylesheet.argsFromClassName(arg);
              if (expandedRules) {
                  extractRules(expandedRules, rules, currentSelector);
              }
              // Else if the arg is an array, we need to recurse in.
          }
          else if (Array.isArray(arg)) {
              extractRules(arg, rules, currentSelector);
          }
          else {
              for (var prop in arg) {
                  if (arg.hasOwnProperty(prop)) {
                      var propValue = arg[prop];
                      if (prop === 'selectors') {
                          // every child is a selector.
                          var selectors = arg.selectors;
                          for (var newSelector in selectors) {
                              if (selectors.hasOwnProperty(newSelector)) {
                                  extractSelector(currentSelector, rules, newSelector, selectors[newSelector]);
                              }
                          }
                      }
                      else if (typeof propValue === 'object') {
                          // prop is a selector.
                          if (propValue !== null) {
                              extractSelector(currentSelector, rules, prop, propValue);
                          }
                      }
                      else {
                          if (propValue !== undefined) {
                              // Else, add the rule to the currentSelector.
                              if (prop === 'margin' || prop === 'padding') {
                                  expandQuads(currentRules, prop, propValue);
                              }
                              else {
                                  currentRules[prop] = propValue;
                              }
                          }
                      }
                  }
              }
          }
      }
      return rules;
  }
  function expandQuads(currentRules, name, value) {
      var parts = typeof value === 'string' ? tokenizeWithParentheses(value) : [value];
      if (parts.length === 0) {
          parts.push(value);
      }
      if (parts[parts.length - 1] === '!important') {
          // Remove !important from parts, and append it to each part individually
          parts = parts.slice(0, -1).map(function (p) { return p + ' !important'; });
      }
      currentRules[name + 'Top'] = parts[0];
      currentRules[name + 'Right'] = parts[1] || parts[0];
      currentRules[name + 'Bottom'] = parts[2] || parts[0];
      currentRules[name + 'Left'] = parts[3] || parts[1] || parts[0];
  }
  function getKeyForRules(options, rules) {
      var serialized = [options.rtl ? 'rtl' : 'ltr'];
      var hasProps = false;
      for (var _i = 0, _a = rules.__order; _i < _a.length; _i++) {
          var selector = _a[_i];
          serialized.push(selector);
          var rulesForSelector = rules[selector];
          for (var propName in rulesForSelector) {
              if (rulesForSelector.hasOwnProperty(propName) && rulesForSelector[propName] !== undefined) {
                  hasProps = true;
                  serialized.push(propName, rulesForSelector[propName]);
              }
          }
      }
      return hasProps ? serialized.join('') : undefined;
  }
  function repeatString(target, count) {
      if (count <= 0) {
          return '';
      }
      if (count === 1) {
          return target;
      }
      return target + repeatString(target, count - 1);
  }
  function serializeRuleEntries(options, ruleEntries) {
      if (!ruleEntries) {
          return '';
      }
      var allEntries = [];
      for (var entry in ruleEntries) {
          if (ruleEntries.hasOwnProperty(entry) && entry !== DISPLAY_NAME && ruleEntries[entry] !== undefined) {
              allEntries.push(entry, ruleEntries[entry]);
          }
      }
      // Apply transforms.
      for (var i = 0; i < allEntries.length; i += 2) {
          kebabRules(allEntries, i);
          provideUnits(allEntries, i);
          rtlifyRules(options, allEntries, i);
          prefixRules(allEntries, i);
      }
      // Apply punctuation.
      for (var i = 1; i < allEntries.length; i += 4) {
          allEntries.splice(i, 1, ':', allEntries[i], ';');
      }
      return allEntries.join('');
  }
  function styleToRegistration(options) {
      var args = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          args[_i - 1] = arguments[_i];
      }
      var rules = extractRules(args);
      var key = getKeyForRules(options, rules);
      if (key) {
          var stylesheet = Stylesheet.getInstance();
          var registration = {
              className: stylesheet.classNameFromKey(key),
              key: key,
              args: args,
          };
          if (!registration.className) {
              registration.className = stylesheet.getClassName(getDisplayName(rules));
              var rulesToInsert = [];
              for (var _a = 0, _b = rules.__order; _a < _b.length; _a++) {
                  var selector = _b[_a];
                  rulesToInsert.push(selector, serializeRuleEntries(options, rules[selector]));
              }
              registration.rulesToInsert = rulesToInsert;
          }
          return registration;
      }
      return undefined;
  }
  /**
   * Insert style to stylesheet.
   * @param registration Style registration.
   * @param specificityMultiplier Number of times classname selector is repeated in the css rule.
   * This is to increase css specificity in case it's needed. Default to 1.
   */
  function applyRegistration(registration, specificityMultiplier) {
      if (specificityMultiplier === void 0) { specificityMultiplier = 1; }
      var stylesheet = Stylesheet.getInstance();
      var className = registration.className, key = registration.key, args = registration.args, rulesToInsert = registration.rulesToInsert;
      if (rulesToInsert) {
          // rulesToInsert is an ordered array of selector/rule pairs.
          for (var i = 0; i < rulesToInsert.length; i += 2) {
              var rules = rulesToInsert[i + 1];
              if (rules) {
                  var selector = rulesToInsert[i];
                  selector = selector.replace(/&/g, repeatString("." + registration.className, specificityMultiplier));
                  // Insert. Note if a media query, we must close the query with a final bracket.
                  var processedRule = selector + "{" + rules + "}" + (selector.indexOf('@') === 0 ? '}' : '');
                  stylesheet.insertRule(processedRule);
              }
          }
          stylesheet.cacheClassName(className, key, args, rulesToInsert);
      }
  }
  function styleToClassName(options) {
      var args = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          args[_i - 1] = arguments[_i];
      }
      var registration = styleToRegistration.apply(void 0, __spreadArrays([options], args));
      if (registration) {
          applyRegistration(registration, options.specificityMultiplier);
          return registration.className;
      }
      return '';
  }

  /**
   * Concatenation helper, which can merge class names together. Skips over falsey values.
   *
   * @public
   */
  function mergeStyles() {
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          args[_i] = arguments[_i];
      }
      return mergeCss(args, getStyleOptions());
  }
  /**
   * Concatenation helper, which can merge class names together. Skips over falsey values.
   * Accepts a set of options that will be used when calculating styles.
   *
   * @public
   */
  function mergeCss(args, options) {
      var styleArgs = args instanceof Array ? args : [args];
      var _a = extractStyleParts(styleArgs), classes = _a.classes, objects = _a.objects;
      if (objects.length) {
          classes.push(styleToClassName(options || {}, objects));
      }
      return classes.join(' ');
  }

  /**
   * Combine a set of styles together (but does not register css classes).
   * @param styleSets - One or more stylesets to be merged (each param can also be falsy).
   */
  function concatStyleSets() {
      var styleSets = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          styleSets[_i] = arguments[_i];
      }
      if (styleSets && styleSets.length === 1 && styleSets[0] && !styleSets[0].subComponentStyles) {
          return styleSets[0];
      }
      var mergedSet = {};
      // We process sub component styles in two phases. First we collect them, then we combine them into 1 style function.
      var workingSubcomponentStyles = {};
      for (var _a = 0, styleSets_1 = styleSets; _a < styleSets_1.length; _a++) {
          var currentSet = styleSets_1[_a];
          if (currentSet) {
              for (var prop in currentSet) {
                  if (currentSet.hasOwnProperty(prop)) {
                      if (prop === 'subComponentStyles' && currentSet.subComponentStyles !== undefined) {
                          // subcomponent styles - style functions or objects
                          var currentComponentStyles = currentSet.subComponentStyles;
                          for (var subCompProp in currentComponentStyles) {
                              if (currentComponentStyles.hasOwnProperty(subCompProp)) {
                                  if (workingSubcomponentStyles.hasOwnProperty(subCompProp)) {
                                      workingSubcomponentStyles[subCompProp].push(currentComponentStyles[subCompProp]);
                                  }
                                  else {
                                      workingSubcomponentStyles[subCompProp] = [currentComponentStyles[subCompProp]];
                                  }
                              }
                          }
                          continue;
                      }
                      // the as any casts below is a workaround for ts 2.8.
                      // todo: remove cast to any in ts 2.9.
                      var mergedValue = mergedSet[prop];
                      var currentValue = currentSet[prop];
                      if (mergedValue === undefined) {
                          mergedSet[prop] = currentValue;
                      }
                      else {
                          mergedSet[prop] = __spreadArrays((Array.isArray(mergedValue) ? mergedValue : [mergedValue]), (Array.isArray(currentValue) ? currentValue : [currentValue]));
                      }
                  }
              }
          }
      }
      if (Object.keys(workingSubcomponentStyles).length > 0) {
          mergedSet.subComponentStyles = {};
          var mergedSubStyles = mergedSet.subComponentStyles;
          var _loop_1 = function (subCompProp) {
              if (workingSubcomponentStyles.hasOwnProperty(subCompProp)) {
                  var workingSet_1 = workingSubcomponentStyles[subCompProp];
                  mergedSubStyles[subCompProp] = function (styleProps) {
                      return concatStyleSets.apply(void 0, workingSet_1.map(function (styleFunctionOrObject) {
                          return typeof styleFunctionOrObject === 'function' ? styleFunctionOrObject(styleProps) : styleFunctionOrObject;
                      }));
                  };
              }
          };
          // now we process the subcomponent styles if there are any
          for (var subCompProp in workingSubcomponentStyles) {
              _loop_1(subCompProp);
          }
      }
      return mergedSet;
  }

  /**
   * Takes in one or more style set objects, each consisting of a set of areas,
   * each which will produce a class name. Using this is analogous to calling
   * `mergeStyles` for each property in the object, but ensures we maintain the
   * set ordering when multiple style sets are merged.
   *
   * @param styleSets - One or more style sets to be merged.
   */
  function mergeStyleSets() {
      var styleSets = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          styleSets[_i] = arguments[_i];
      }
      return mergeCssSets(styleSets, getStyleOptions());
  }
  /**
   * Takes in one or more style set objects, each1consisting of a set of areas,
   * each which will produce a class name. Using this is analogous to calling
   * `mergeCss` for each property in the object, but ensures the
   * set ordering when multiple style sets are merged.
   *
   * @param styleSets - One or more style sets to be merged.
   * @param options - (optional) Options to use when creating rules.
   */
  function mergeCssSets(styleSets, options) {
      var classNameSet = { subComponentStyles: {} };
      var styleSet = styleSets[0];
      if (!styleSet && styleSets.length <= 1) {
          return { subComponentStyles: {} };
      }
      var concatenatedStyleSet = concatStyleSets.apply(void 0, styleSets);
      var registrations = [];
      for (var styleSetArea in concatenatedStyleSet) {
          if (concatenatedStyleSet.hasOwnProperty(styleSetArea)) {
              if (styleSetArea === 'subComponentStyles') {
                  classNameSet.subComponentStyles = concatenatedStyleSet.subComponentStyles || {};
                  continue;
              }
              var styles = concatenatedStyleSet[styleSetArea];
              var _a = extractStyleParts(styles), classes = _a.classes, objects = _a.objects;
              if (objects === null || objects === void 0 ? void 0 : objects.length) {
                  var registration = styleToRegistration(options || {}, { displayName: styleSetArea }, objects);
                  if (registration) {
                      registrations.push(registration);
                      classNameSet[styleSetArea] = classes.concat([registration.className]).join(' ');
                  }
              }
              else {
                  classNameSet[styleSetArea] = classes.join(' ');
              }
          }
      }
      for (var _i = 0, registrations_1 = registrations; _i < registrations_1.length; _i++) {
          var registration = registrations_1[_i];
          if (registration) {
              applyRegistration(registration, options === null || options === void 0 ? void 0 : options.specificityMultiplier);
          }
      }
      return classNameSet;
  }

  var getCursorStyle = function (_a) {
      var canvasMouseMode = _a.canvasMouseMode, state = _a.state, isPanDisabled = _a.isPanDisabled, isMultiSelecting = _a.isMultiSelecting;
      if (state.behavior === exports.GraphBehavior.connecting) {
          return "initial";
      }
      var isClickMultipleSelect = ["meta", "control"].some(function (key) { return state.activeKeys.has(key); });
      if (isClickMultipleSelect) {
          return "initial";
      }
      var isAreaSelection = state.activeKeys.has("shift");
      if (isAreaSelection) {
          return "crosshair";
      }
      if (canvasMouseMode !== exports.CanvasMouseMode.pan) {
          if (state.activeKeys.has(" ") && !isPanDisabled) {
              return "grab";
          }
          if (isMultiSelecting) {
              return "crosshair";
          }
          return "inherit";
      }
      return isPanDisabled ? "inherit" : "grab";
  };
  function getNodeCursor(isNodeDraggable) {
      return isNodeDraggable ? "move" : "initial";
  }
  var getGraphStyles = function (props, state, theme, isPanDisabled, isNodeDraggable, focusedWithoutMouse, isMultiSelecting) {
      var _a, _b;
      var _c, _d;
      return mergeStyleSets({
          svg: [
              "react-dag-editor-svg-container",
              classes.svg,
              (_c = props.styles) === null || _c === void 0 ? void 0 : _c.svg,
              (_a = {
                      "& *:focus": {
                          outline: theme.outlineStyle
                      }
                  },
                  _a["& ." + classes.node] = {
                      cursor: getNodeCursor(isNodeDraggable)
                  },
                  _a)
          ],
          container: [
              "react-dag-editor-container",
              classes.container,
              (_b = {
                      cursor: getCursorStyle({
                          canvasMouseMode: props.canvasMouseMode,
                          state: state,
                          isPanDisabled: isPanDisabled,
                          isMultiSelecting: isMultiSelecting
                      })
                  },
                  _b["&." + classes.container] = __assign(__assign({ background: theme.canvasBackground }, props.style), (_d = props.styles) === null || _d === void 0 ? void 0 : _d.root),
                  _b),
              focusedWithoutMouse && {
                  outline: theme.focusOutlineColor + " solid 1px"
              }
          ],
          buttonA11y: ["react-dag-editor-a11y-help-button", classes.buttonA11Y],
          node: [classes.node]
      });
  };

  function Graph(props) {
      var _a, _b, _c, _d, _e, _f, _g;
      var _h = React__namespace.useState(false), focusedWithoutMouse = _h[0], setFocusedWithoutMouse = _h[1];
      var graphController = useGraphController();
      var _j = useGraphState(), state = _j.state, dispatch = _j.dispatch;
      var data = state.data.present;
      var viewport = state.viewport;
      var eventChannel = graphController.eventChannel;
      var graphId = useConst(function () { return "graph-" + v4(); });
      var defaultSVGRef = React__namespace.useRef(null);
      var _k = props.focusCanvasAccessKey, focusCanvasAccessKey = _k === void 0 ? "f" : _k, _l = props.zoomSensitivity, zoomSensitivity = _l === void 0 ? 0.1 : _l, _m = props.scrollSensitivity, scrollSensitivity = _m === void 0 ? 0.5 : _m, _o = props.svgRef, svgRef = _o === void 0 ? defaultSVGRef : _o, _p = props.virtualizationDelay, virtualizationDelay = _p === void 0 ? 500 : _p;
      var theme = useTheme().theme;
      var graphConfig = useGraphConfig();
      var featureControl = useFeatureControl(state.settings.features);
      var _q = React__namespace.useState(), curHoverNode = _q[0], setCurHoverNode = _q[1];
      var _r = React__namespace.useState(undefined), curHoverPort = _r[0], setCurHoverPort = _r[1];
      var containerRef = React__namespace.useRef(null);
      var rectRef = React__namespace.useRef(undefined);
      var updateViewport = useUpdateViewportCallback(rectRef, svgRef, eventChannel);
      useEventChannel({
          props: props,
          dispatch: dispatch,
          rectRef: rectRef,
          svgRef: svgRef,
          setFocusedWithoutMouse: setFocusedWithoutMouse,
          containerRef: containerRef,
          featureControl: featureControl,
          graphConfig: graphConfig,
          setCurHoverNode: setCurHoverNode,
          setCurHoverPort: setCurHoverPort,
          updateViewport: updateViewport,
          eventChannel: eventChannel,
          graphController: graphController
      });
      useContainerRect(svgRef, containerRef, updateViewport);
      var isNodesDraggable = featureControl.isNodesDraggable, isNodeResizable = featureControl.isNodeResizable, isPanDisabled = featureControl.isPanDisabled, isMultiSelectDisabled = featureControl.isMultiSelectDisabled, isLassoSelectEnable = featureControl.isLassoSelectEnable, isNodeEditDisabled = featureControl.isNodeEditDisabled, isVerticalScrollDisabled = featureControl.isVerticalScrollDisabled, isHorizontalScrollDisabled = featureControl.isHorizontalScrollDisabled, isA11yEnable = featureControl.isA11yEnable, isCtrlKeyZoomEnable = featureControl.isCtrlKeyZoomEnable, isLimitBoundary = featureControl.isLimitBoundary, isVirtualizationEnabled = featureControl.isVirtualizationEnabled;
      useSelectBox(dispatch, state.selectBoxPosition);
      var canvasEventHandler = function (type) { return function (rawEvent) {
          rawEvent.persist();
          eventChannel.trigger({
              type: type,
              rawEvent: rawEvent
          });
      }; };
      var classes = getGraphStyles(props, state, theme, isPanDisabled, isNodesDraggable, focusedWithoutMouse, state.behavior === exports.GraphBehavior.multiSelect);
      useWheelHandler({
          containerRef: containerRef,
          svgRef: svgRef,
          rectRef: rectRef,
          zoomSensitivity: zoomSensitivity,
          scrollSensitivity: scrollSensitivity,
          dispatch: dispatch,
          isHorizontalScrollDisabled: isHorizontalScrollDisabled,
          isVerticalScrollDisabled: isVerticalScrollDisabled,
          isCtrlKeyZoomEnable: isCtrlKeyZoomEnable,
          eventChannel: eventChannel,
          graphConfig: graphConfig
      });
      var onContextMenuClick = React__namespace.useCallback(function (evt) {
          evt.preventDefault();
          evt.stopPropagation();
          eventChannel.trigger({
              type: exports.GraphContextMenuEvent.Close
          });
          if (svgRef.current) {
              svgRef.current.focus({ preventScroll: true });
          }
      }, [eventChannel, svgRef]);
      var onFocusButtonClick = React__namespace.useCallback(function () {
          setFocusedWithoutMouse(true);
          if (svgRef.current) {
              svgRef.current.focus({ preventScroll: true });
          }
      }, [svgRef]);
      useSafariScale({
          rectRef: rectRef,
          svgRef: svgRef,
          eventChannel: eventChannel
      });
      var accessKey = isA11yEnable ? focusCanvasAccessKey : undefined;
      var touchHandlers = useGraphTouchHandler(rectRef, eventChannel);
      if (!isSupported()) {
          var _s = props.onBrowserNotSupported, onBrowserNotSupported = _s === void 0 ? function () { return React__namespace.createElement("p", null, "Your browser is not supported"); } : _s;
          return React__namespace.createElement(React__namespace.Fragment, null, onBrowserNotSupported());
      }
      var renderPortTooltip = function () {
          if (!curHoverPort || !isViewportComplete(state.viewport)) {
              return null;
          }
          var nodeId = curHoverPort[0], portId = curHoverPort[1];
          var node = data.nodes.get(nodeId);
          if (!node) {
              return null;
          }
          var port = node.getPort(portId);
          if (!port) {
              return null;
          }
          return React__namespace.createElement(PortTooltips, { port: port, parentNode: node, data: data, viewport: state.viewport });
      };
      var renderNodeTooltip = function () {
          var _a;
          if (!curHoverNode || !isViewportComplete(state.viewport)) {
              return null;
          }
          var curHoverNodeHasContextMenu = state.contextMenuPosition && curHoverNode === ((_a = state.data.present.nodes.find(isSelected)) === null || _a === void 0 ? void 0 : _a.id);
          if (curHoverNodeHasContextMenu) {
              return null;
          }
          return React__namespace.createElement(NodeTooltips, { node: data.nodes.get(curHoverNode), viewport: state.viewport });
      };
      return (React__namespace.createElement("div", __assign({ ref: containerRef, role: "application", id: graphId, className: classes.container }, touchHandlers, { onDoubleClick: canvasEventHandler(exports.GraphCanvasEvent.DoubleClick), onMouseDown: canvasEventHandler(exports.GraphCanvasEvent.MouseDown), onMouseUp: canvasEventHandler(exports.GraphCanvasEvent.MouseUp), onContextMenu: canvasEventHandler(exports.GraphCanvasEvent.ContextMenu), onMouseMove: canvasEventHandler(exports.GraphCanvasEvent.MouseMove), onMouseOver: canvasEventHandler(exports.GraphCanvasEvent.MouseOver), onMouseOut: canvasEventHandler(exports.GraphCanvasEvent.MouseOut), onFocus: canvasEventHandler(exports.GraphCanvasEvent.Focus), onBlur: canvasEventHandler(exports.GraphCanvasEvent.Blur), onKeyDown: canvasEventHandler(exports.GraphCanvasEvent.KeyDown), onKeyUp: canvasEventHandler(exports.GraphCanvasEvent.KeyUp) }),
          React__namespace.createElement("button", { className: classes.buttonA11y, onClick: onFocusButtonClick, accessKey: accessKey, hidden: true }),
          React__namespace.createElement("svg", { tabIndex: 0, focusable: "true", preserveAspectRatio: "xMidYMid meet", ref: svgRef, className: classes.svg, "data-graph-id": graphId },
              React__namespace.createElement("title", null, props.title),
              React__namespace.createElement("desc", null, props.desc),
              React__namespace.createElement(Transform, { matrix: viewport.transformMatrix },
                  state.viewport.rect && (React__namespace.createElement(VirtualizationProvider, { viewport: state.viewport, isVirtualizationEnabled: isVirtualizationEnabled, virtualizationDelay: virtualizationDelay, eventChannel: eventChannel },
                      React__namespace.createElement(GraphGroupsRenderer, { data: data, groups: (_a = data.groups) !== null && _a !== void 0 ? _a : constantEmptyArray() }),
                      React__namespace.createElement(EdgeTree, { graphId: graphId, tree: data.edges, data: data, eventChannel: eventChannel }),
                      React__namespace.createElement(NodeTree, { graphId: graphId, isNodeResizable: isNodeResizable, tree: data.nodes, data: data, isNodeEditDisabled: isNodeEditDisabled, eventChannel: eventChannel, getNodeAriaLabel: (_b = props.getNodeAriaLabel) !== null && _b !== void 0 ? _b : defaultGetNodeAriaLabel, getPortAriaLabel: (_c = props.getPortAriaLabel) !== null && _c !== void 0 ? _c : defaultGetPortAriaLabel }))),
                  state.dummyNodes.isVisible && (React__namespace.createElement(AnimatingNodeGroup, { dummyNodes: state.dummyNodes, graphData: state.data.present })),
                  React__namespace.createElement(AlignmentLines, { style: (_d = props.styles) === null || _d === void 0 ? void 0 : _d.alignmentLine })),
              (!isMultiSelectDisabled || isLassoSelectEnable) && (React__namespace.createElement(SelectBox, { selectBoxPosition: state.selectBoxPosition, style: (_e = props.styles) === null || _e === void 0 ? void 0 : _e.selectBox })),
              state.connectState && (React__namespace.createElement(Connecting, { graphConfig: graphConfig, eventChannel: eventChannel, viewport: state.viewport, styles: (_f = props.styles) === null || _f === void 0 ? void 0 : _f.connectingLine, movingPoint: state.connectState.movingPoint }))),
          (!isVerticalScrollDisabled || !isHorizontalScrollDisabled || !isPanDisabled) &&
              isLimitBoundary &&
              isViewportComplete(state.viewport) && (React__namespace.createElement(Scrollbar, { viewport: state.viewport, offsetLimit: getOffsetLimit({
                  data: data,
                  graphConfig: graphConfig,
                  rect: state.viewport.rect,
                  transformMatrix: viewport.transformMatrix,
                  canvasBoundaryPadding: state.settings.canvasBoundaryPadding,
                  groupPadding: (_g = data.groups[0]) === null || _g === void 0 ? void 0 : _g.padding
              }), dispatch: dispatch, horizontal: !isHorizontalScrollDisabled, vertical: !isVerticalScrollDisabled, eventChannel: eventChannel })),
          React__namespace.createElement(GraphContextMenu, { state: state, onClick: onContextMenuClick, "data-automation-id": "context-menu-container" }),
          renderNodeTooltip(),
          renderPortTooltip()));
  }

  var getAutoAlignDisplacement = function (alignmentLines, nodes, graphConfig, alignDirection) {
      var min = Infinity;
      var res = 0;
      var nodeHW = getDummyDraggingNode(nodes);
      var widthOrHeight = alignDirection === "x" ? nodeHW.width || 0 : nodeHW.height || 0;
      alignmentLines.forEach(function (item) {
          var alignLine;
          if (alignDirection === "x" && item.x1 === item.x2) {
              alignLine = item.x1;
          }
          else if (alignDirection === "y" && item.y1 === item.y2) {
              alignLine = item.y1;
          }
          else {
              return;
          }
          var distance1 = nodeHW[alignDirection] - alignLine;
          var distanceMiddle = nodeHW[alignDirection] + (widthOrHeight || 0) / 2 - alignLine;
          var distance2 = nodeHW[alignDirection] + (widthOrHeight || 0) - alignLine;
          if (Math.abs(distance1) < min) {
              min = Math.abs(distance1);
              res = distance1 > 0 ? -min : min;
          }
          if (Math.abs(distanceMiddle) < min) {
              min = Math.abs(distanceMiddle);
              res = distanceMiddle > 0 ? -min : min;
          }
          if (Math.abs(distance2) < min) {
              min = Math.abs(distance2);
              res = distance2 > 0 ? -min : min;
          }
      });
      return res;
  };
  var getBoundingBoxOfNodes = function (nodes) {
      var minX = Infinity;
      var minY = Infinity;
      var maxX = -Infinity;
      var maxY = -Infinity;
      nodes.forEach(function (n) {
          var tempMinX = n.x;
          var tempMinY = n.y;
          var tempMaxX = n.x + (n.width || 0);
          var tempMaxY = n.y + (n.height || 0);
          if (tempMinX < minX) {
              minX = tempMinX;
          }
          if (tempMinY < minY) {
              minY = tempMinY;
          }
          if (tempMaxX > maxX) {
              maxX = tempMaxX;
          }
          if (tempMaxY > maxY) {
              maxY = tempMaxY;
          }
      });
      return {
          x: minX,
          y: minY,
          width: maxX - minX,
          height: maxY - minY
      };
  };
  var getDummyDraggingNode = function (draggingNodes) {
      var _a = getBoundingBoxOfNodes(draggingNodes), x = _a.x, y = _a.y, width = _a.width, height = _a.height;
      var dummyDraggingNode = {
          id: v4(),
          x: x,
          y: y,
          width: width,
          height: height
      };
      return dummyDraggingNode;
  };

  function useSvgRect(svgRef) {
      var _a = React.useState(undefined), rect = _a[0], setRect = _a[1];
      React.useLayoutEffect(function () {
          if (!svgRef.current) {
              return;
          }
          setRect(svgRef.current.getBoundingClientRect());
      }, [svgRef]);
      return rect;
  }

  var AddingNodeSvg = function (_a) {
      var model = _a.model, svgRef = _a.svgRef, nextNodeRef = _a.nextNodeRef;
      var rect = useSvgRect(svgRef);
      var graphConfig = useGraphConfig();
      var graphController = useGraphController();
      var alignmentLines = useAlignmentLines();
      var viewport = useViewport();
      var dummyNode = React__namespace.useMemo(function () {
          if (!model || !viewport.rect) {
              return null;
          }
          var transformMatrix = viewport.transformMatrix;
          var _a = viewport.rect, left = _a.left, top = _a.top;
          var diffLeft = left / transformMatrix[0];
          var diffTop = top / transformMatrix[3];
          return __assign({ id: model.id, x: model.x - diffLeft, y: model.y - diffTop }, getNodeSize(model, graphConfig));
      }, [graphConfig, viewport, model]);
      React__namespace.useLayoutEffect(function () {
          if (!dummyNode) {
              return;
          }
          graphController.eventChannel.trigger({
              type: exports.GraphCanvasEvent.DraggingNodeFromItemPanel,
              node: dummyNode
          });
      }, [dummyNode === null || dummyNode === void 0 ? void 0 : dummyNode.x, dummyNode === null || dummyNode === void 0 ? void 0 : dummyNode.y]);
      var attachedNode = React__namespace.useMemo(function () {
          if (!dummyNode || !model) {
              return null;
          }
          var dxAligned = getAutoAlignDisplacement(alignmentLines, [dummyNode], graphConfig, "x");
          var dyAligned = getAutoAlignDisplacement(alignmentLines, [dummyNode], graphConfig, "y");
          return __assign(__assign({}, model), { x: model.x + dxAligned, y: model.y + dyAligned });
      }, [alignmentLines, dummyNode, graphConfig, model]);
      React__namespace.useEffect(function () {
          nextNodeRef.current = attachedNode;
      }, [nextNodeRef, attachedNode]);
      var eventChannel = useConst(function () { return new EventChannel(); });
      var tempGraphId = useConst(v4);
      var node = React__namespace.useMemo(function () { return NodeModel.fromJSON(attachedNode !== null && attachedNode !== void 0 ? attachedNode : model, undefined, undefined); }, [
          attachedNode,
          model
      ]);
      return (React__namespace.createElement("svg", { id: tempGraphId, ref: svgRef, className: classes.addingNodeSvg, preserveAspectRatio: "xMidYMid meet" },
          React__namespace.createElement(Transform, { matrix: viewport.transformMatrix }, rect && (React__namespace.createElement(GraphNode, { graphId: tempGraphId, node: node, viewport: {
                  rect: rect,
                  transformMatrix: viewport.transformMatrix
              }, eventChannel: eventChannel, getNodeAriaLabel: defaultGetNodeAriaLabel })))));
  };

  var el = document.createElement("div");
  document.body.appendChild(el);
  function isWithInBound(rect, clientX, clientY) {
      var top = rect.top, right = rect.right, bottom = rect.bottom, left = rect.left;
      return clientX >= left && clientX <= right && clientY >= top && clientY <= bottom;
  }
  var adjustedClientPoint = function (x, y, rect, transformMatrix) {
      var adjustedX = x;
      var adjustedY = y;
      if (rect) {
          var left = rect.left, top_1 = rect.top;
          adjustedX = x - left;
          adjustedY = y - top_1;
      }
      return reverseTransformPoint(adjustedX, adjustedY, transformMatrix);
  };
  var adjustPosition = function (clientX, clientY, rect, transformMatrix, node, nodeConfig) {
      var rectWidth = getRectWidth(nodeConfig, node);
      var rectHeight = getRectHeight(nodeConfig, node);
      var realWidth = transformMatrix[0] * rectWidth + transformMatrix[2] * rectHeight;
      var realHeight = transformMatrix[1] * rectWidth + transformMatrix[3] * rectHeight;
      return adjustedClientPoint(clientX - realWidth / 2, clientY - realHeight / 2, rect, transformMatrix);
  };
  var Item = function (props) {
      var graphConfig = useGraphConfig();
      var graphController = useGraphController();
      var _a = React__namespace.useState(null), workingModel = _a[0], setWorkingModel = _a[1];
      var nextNodeRef = useRefValue(workingModel);
      var svgRef = React__namespace.useRef(null);
      var style = props.style, children = props.children, model = props.model, _b = props.nodeWillAdd, nodeWillAdd = _b === void 0 ? identical : _b, _c = props.nodeDidAdd, nodeDidAdd = _c === void 0 ? noop : _c, _d = props.dragWillStart, dragWillStart = _d === void 0 ? noop : _d;
      var onPointerDown = React__namespace.useCallback(function (evt) {
          evt.stopPropagation();
          if ((evt.pointerType === "mouse" && evt.button !== MouseEventButton.Primary) ||
              !graphController.getEnabledFeatures().has(exports.GraphFeatures.addNewNodes)) {
              return;
          }
          var shape = props.shape || model.shape;
          var nodeConfig = graphConfig.getNodeConfigByName(shape);
          var node = __assign(__assign(__assign({}, deepClone(model)), adjustPosition(evt.clientX, evt.clientY, undefined, graphController.state.viewport.transformMatrix, model, nodeConfig)), { id: model.id || v4() });
          var drag = new DragController(new PointerEventProvider(graphController.getGlobalEventTarget()), defaultGetPositionFromEvent);
          var eventChannel = graphController.eventChannel;
          eventChannel.trigger({
              type: exports.GraphCanvasEvent.DraggingNodeFromItemPanelStart,
              rawEvent: evt
          });
          drag.onMove = function (_a) {
              var e = _a.e;
              setWorkingModel(function (n) {
                  if (!n) {
                      return n;
                  }
                  return __assign(__assign({}, n), adjustPosition(e.clientX, e.clientY, undefined, graphController.state.viewport.transformMatrix, model, nodeConfig));
              });
          };
          drag.onEnd = function (_a) {
              var e = _a.e;
              var viewport = graphController.state.viewport;
              var nextNode = nextNodeRef.current;
              if (!isViewportComplete(viewport) || !nextNode || !isWithInBound(viewport.rect, e.clientX, e.clientY)) {
                  setWorkingModel(null);
                  eventChannel.trigger({
                      type: exports.GraphCanvasEvent.DraggingNodeFromItemPanelEnd,
                      node: null
                  });
                  return;
              }
              nextNodeRef.current = null;
              nextNode = __assign(__assign({}, nextNode), adjustPosition(e.clientX, e.clientY, viewport.rect, viewport.transformMatrix, model, nodeConfig));
              nextNode = nodeWillAdd(nextNode);
              eventChannel.trigger({
                  type: exports.GraphCanvasEvent.DraggingNodeFromItemPanelEnd,
                  node: nextNode
              });
              nodeDidAdd(nextNode);
              setWorkingModel(null);
          };
          dragWillStart(node);
          setWorkingModel(node);
          drag.start(evt.nativeEvent);
      }, [graphController, props.shape, model, graphConfig, dragWillStart, nextNodeRef, nodeWillAdd, nodeDidAdd]);
      var className = mergeStyles(classes.moduleItem, props.className);
      return (React__namespace.createElement(React__namespace.Fragment, null,
          React__namespace.createElement("div", { className: className, style: style, onPointerDown: onPointerDown, role: "button" }, children),
          workingModel &&
              ReactDOM__namespace.createPortal(React__namespace.createElement(AddingNodeSvg, { svgRef: svgRef, model: workingModel, nextNodeRef: nextNodeRef }), el)));
  };

  var TouchDragAdapter = (function () {
      function TouchDragAdapter() {
          this.startListeners = new Set();
          this.moveListeners = new Set();
          this.endListeners = new Set();
      }
      TouchDragAdapter.prototype.off = function (type, callback) {
          switch (type) {
              case "start":
                  this.startListeners.delete(callback);
                  break;
              case "move":
                  this.moveListeners.delete(callback);
                  break;
              case "end":
                  this.endListeners.delete(callback);
                  break;
          }
          return this;
      };
      TouchDragAdapter.prototype.on = function (type, callback) {
          switch (type) {
              case "start":
                  this.startListeners.add(callback);
                  break;
              case "move":
                  this.moveListeners.add(callback);
                  break;
              case "end":
                  this.endListeners.add(callback);
                  break;
          }
          return this;
      };
      TouchDragAdapter.prototype.onStart = function (events, e) {
          var args = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              args[_i - 2] = arguments[_i];
          }
          this.startListeners.forEach(function (cb) {
              cb.call.apply(cb, __spreadArray([undefined, e], args));
          });
      };
      TouchDragAdapter.prototype.onMove = function (events, e) {
          var args = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              args[_i - 2] = arguments[_i];
          }
          this.moveListeners.forEach(function (cb) {
              cb.call.apply(cb, __spreadArray([undefined, e], args));
          });
      };
      TouchDragAdapter.prototype.onEnd = function (e) {
          var args = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              args[_i - 1] = arguments[_i];
          }
          this.endListeners.forEach(function (cb) {
              cb.call.apply(cb, __spreadArray([undefined, e], args));
          });
      };
      return TouchDragAdapter;
  }());

  var StaticNode = function (props) {
      var node = props.node;
      var graphConfig = useGraphConfig();
      var theme = useTheme().theme;
      var nodeConfig = getNodeConfig(node, graphConfig);
      if (nodeConfig === null || nodeConfig === void 0 ? void 0 : nodeConfig.renderStatic) {
          return React__namespace.createElement("g", null, nodeConfig.renderStatic({ model: node, theme: theme }));
      }
      var rectHeight = getRectHeight(nodeConfig, node);
      var rectWidth = getRectWidth(nodeConfig, node);
      return (React__namespace.createElement("rect", { transform: "translate(" + node.x + ", " + node.y + ")", height: rectHeight, width: rectWidth, fill: theme.dummyNodeStroke }));
  };
  var StaticNodeWithMemo = React__namespace.memo(StaticNode, function (prevProps, nextProps) {
      var prevNode = prevProps.node;
      var nextNode = nextProps.node;
      return (prevNode.x === nextNode.x &&
          prevNode.y === nextNode.y &&
          prevNode.height === nextNode.height &&
          prevNode.width === nextNode.width &&
          prevNode.isInSearchResults === nextNode.isInSearchResults &&
          prevNode.isCurrentSearchResult === nextNode.isCurrentSearchResult);
  });

  var ReadonlyNodeTreeNode = React__namespace.memo(function (_a) {
      var node = _a.node;
      var values = node.values.map(function (it) { return (React__namespace.createElement(StaticNodeWithMemo, { key: it[1].id, node: it[1] })); });
      var children = node.type === NodeType.Internal
          ? node.children.map(function (child, index) {
              var key = index < node.selfSize ? node.getKey(index) : "last";
              return React__namespace.createElement(ReadonlyNodeTreeNode, { key: key, node: child });
          })
          : undefined;
      return (React__namespace.createElement(React__namespace.Fragment, null,
          values,
          children));
  });
  ReadonlyNodeTreeNode.displayName = "ReadonlyNodeTreeNode";
  var ReadonlyNodeTree = function (_a) {
      var tree = _a.tree;
      return React__namespace.createElement(ReadonlyNodeTreeNode, { node: tree.sortedRoot });
  };

  var StaticGraph = function (props) {
      var nodes = props.data.nodes;
      var style = __assign({ position: "absolute", top: 0, left: 0, height: "100%", width: "100%" }, props.style);
      return (React__namespace.createElement("svg", { style: style },
          React__namespace.createElement(Transform, { matrix: props.transformMatrix },
              React__namespace.createElement(ReadonlyNodeTree, { tree: nodes }))));
  };

  var MiniMapShadow = function (_a) {
      var rect = _a.containerRect, viewport = _a.viewport, _b = _a.shadowPadding, shadowPadding = _b === void 0 ? 0 : _b, onClick = _a.onClick;
      var theme = useTheme().theme;
      if (!rect) {
          return null;
      }
      var shadowStyle = {
          fill: theme.minimapShadow,
          opacity: 0.1
      };
      var lineStyle = {
          stroke: theme.inputFocusBorderAlt,
          strokeWidth: 1
      };
      var viewportStartX = Math.max(viewport.startX, shadowPadding);
      var viewportStartY = Math.max(viewport.startY, shadowPadding);
      var viewportEndX = Math.min(viewport.endX, rect.width - shadowPadding);
      var viewportEndY = Math.min(viewport.endY, rect.height - shadowPadding);
      var sideShadowHeight = viewportEndY - viewportStartY;
      var rightShadowWidth = rect.width - viewportEndX;
      var bottomShadowHeight = rect.height - viewportEndY;
      return (React__namespace.createElement(React__namespace.Fragment, null,
          viewportStartY > 0 && (React__namespace.createElement("g", null,
              React__namespace.createElement("rect", { onClick: onClick, onMouseDown: stopPropagation, onPointerDown: stopPropagation, x: 0, y: 0, height: viewportStartY, width: rect.width, style: shadowStyle }),
              React__namespace.createElement("line", { x1: viewportStartX, y1: viewportStartY, x2: viewportEndX, y2: viewportStartY, style: lineStyle }))),
          viewportStartX && sideShadowHeight > 0 && (React__namespace.createElement("g", null,
              React__namespace.createElement("rect", { onClick: onClick, onMouseDown: stopPropagation, onPointerDown: stopPropagation, x: 0, y: viewportStartY, height: sideShadowHeight, width: viewportStartX, style: shadowStyle }),
              React__namespace.createElement("line", { x1: viewportStartX, y1: viewportStartY, x2: viewportStartX, y2: viewportEndY, style: lineStyle }))),
          rightShadowWidth && sideShadowHeight > 0 && (React__namespace.createElement("g", null,
              React__namespace.createElement("rect", { onClick: onClick, onMouseDown: stopPropagation, onPointerDown: stopPropagation, x: viewportEndX, y: viewportStartY, height: sideShadowHeight, width: rightShadowWidth, style: shadowStyle }),
              React__namespace.createElement("line", { x1: viewportEndX, y1: viewportStartY, x2: viewportEndX, y2: viewportEndY, style: lineStyle }))),
          bottomShadowHeight > 0 && (React__namespace.createElement("g", null,
              React__namespace.createElement("rect", { onClick: onClick, onMouseDown: stopPropagation, onPointerDown: stopPropagation, x: 0, y: viewportEndY, height: bottomShadowHeight, width: rect.width, style: shadowStyle }),
              React__namespace.createElement("line", { x1: viewportStartX, y1: viewportEndY, x2: viewportEndX, y2: viewportEndY, style: lineStyle })))));
  };

  var Minimap = function (props) {
      var _a = props.shadowPadding, shadowPadding = _a === void 0 ? 0 : _a, _b = props.maxNodesCountAllowed, maxNodesCountAllowed = _b === void 0 ? 150 : _b, _c = props.onRenderUnavailable, onRenderUnavailable = _c === void 0 ? function () { return null; } : _c, _d = props.renderArrow, renderArrow = _d === void 0 ? function (arrowDeg) { return undefined; } : _d;
      var graphViewport = useViewport();
      var graphController = useGraphController();
      var theme = useTheme().theme;
      var data = useGraphData();
      var minimapContainerStyle = __assign({ background: theme.minimapBackground }, props.style);
      var svgRef = React__namespace.useRef(null);
      var graphConfig = useGraphConfig();
      var rect = useMinimapRect(svgRef);
      var rectRef = useRefValue(rect);
      var minimapTransformMatrix = React__namespace.useMemo(function () {
          if (!rect) {
              return EMPTY_TRANSFORM_MATRIX;
          }
          return getZoomFitMatrix({
              data: data,
              rect: rect,
              graphConfig: graphConfig,
              nodeMaxVisibleSize: {
                  width: 0,
                  height: 0
              },
              nodeMinVisibleSize: {
                  width: Infinity,
                  height: Infinity
              }
          });
      }, [rect, data.nodes]);
      var minimapTransformMatrixRef = useRefValue(minimapTransformMatrix);
      var viewport = React__namespace.useMemo(function () {
          if (!rect || !isViewportComplete(graphViewport)) {
              return {
                  startX: 0,
                  startY: 0,
                  endX: 0,
                  endY: 0
              };
          }
          var boundaryPoints = getVisibleArea(graphViewport);
          var _a = transformPoint(boundaryPoints.minX, boundaryPoints.minY, minimapTransformMatrix), startX = _a.x, startY = _a.y;
          var _b = transformPoint(boundaryPoints.maxX, boundaryPoints.maxY, minimapTransformMatrix), endX = _b.x, endY = _b.y;
          return {
              startX: startX,
              startY: startY,
              endX: endX,
              endY: endY
          };
      }, __spreadArray([rect, minimapTransformMatrix, graphViewport.rect], graphViewport.transformMatrix));
      var onClick = React__namespace.useCallback(function (evt) {
          evt.stopPropagation();
          if (!rect) {
              return;
          }
          var viewportWidth = viewport.endX - viewport.startX;
          var viewportHeight = viewport.endY - viewport.startY;
          var point = reverseTransformPoint(clamp(shadowPadding + viewportWidth / 2, rect.width - shadowPadding - viewportWidth / 2, evt.clientX - rect.left), clamp(shadowPadding + viewportHeight / 2, rect.height - shadowPadding - viewportHeight / 2, evt.clientY - rect.top), minimapTransformMatrix);
          graphController.eventChannel.batch([
              {
                  type: exports.GraphCanvasEvent.ScrollIntoView,
                  x: point.x,
                  y: point.y
              },
              {
                  type: exports.GraphMinimapEvent.Click,
                  rawEvent: evt
              }
          ]);
      }, [rect]);
      var onStartDrag = React__namespace.useCallback(function (evt, eventProvider) {
          if (!rectRef.current) {
              return;
          }
          var _a = rectRef.current, left = _a.left, top = _a.top, right = _a.right, bottom = _a.bottom;
          var validMouseRect = {
              startX: left + shadowPadding,
              startY: top + shadowPadding,
              endX: right - shadowPadding,
              endY: bottom - shadowPadding
          };
          graphController.eventChannel.trigger({
              type: exports.GraphMinimapEvent.PanStart,
              rawEvent: evt
          });
          var drag = new DragController(eventProvider, function (e) {
              var x = clamp(validMouseRect.startX, validMouseRect.endX, e.clientX);
              var y = clamp(validMouseRect.startY, validMouseRect.endY, e.clientY);
              return {
                  x: x,
                  y: y
              };
          });
          drag.onMove = function (_a) {
              var dx = _a.dx, dy = _a.dy, e = _a.e;
              var _b = getPointDeltaByClientDelta(-dx, -dy, minimapTransformMatrixRef.current), x = _b.x, y = _b.y;
              graphController.eventChannel.trigger({
                  type: exports.GraphMinimapEvent.Pan,
                  dx: x,
                  dy: y,
                  rawEvent: e
              });
          };
          drag.start(evt);
          drag.onEnd = function () {
              graphController.eventChannel.trigger({
                  type: exports.GraphMinimapEvent.PanEnd,
                  rawEvent: evt
              });
          };
      }, [shadowPadding]);
      var arrowParams = React__namespace.useMemo(function () {
          if (!rect) {
              return {
                  showArrow: false,
                  arrowDeg: 0
              };
          }
          var showArrow = viewport.startX > rect.width - shadowPadding ||
              viewport.startY > rect.height - shadowPadding ||
              viewport.endX < shadowPadding ||
              viewport.endY < shadowPadding;
          var x0 = rect.width / 2;
          var y0 = rect.height / 2;
          var x1 = (viewport.startX + viewport.endX) / 2;
          var y1 = (viewport.startY + viewport.endY) / 2;
          var arrowDeg = (Math.atan2(y1 - y0, x1 - x0) * 180) / Math.PI;
          if (arrowDeg < 0) {
              arrowDeg = arrowDeg + 360;
          }
          return {
              showArrow: showArrow,
              arrowDeg: arrowDeg
          };
      }, [rect, viewport, shadowPadding]);
      var onMouseDown = React__namespace.useCallback(function (evt) {
          onStartDrag(evt.nativeEvent, new MouseMoveEventProvider(graphController.getGlobalEventTarget()));
      }, [onStartDrag]);
      var touchController = React__namespace.useMemo(function () {
          var handlers = new Map();
          var touchDragAdapter = new TouchDragAdapter();
          touchDragAdapter.on("start", function (e) {
              onStartDrag(e, touchDragAdapter);
          });
          handlers.set(1, touchDragAdapter);
          return new TouchController(handlers);
      }, [onStartDrag, rect]);
      var staticGraphEl = React__namespace.useMemo(function () { return React__namespace.createElement(StaticGraph, { data: data, transformMatrix: minimapTransformMatrix }); }, __spreadArray([data.nodes], minimapTransformMatrix));
      if (data.nodes.size > maxNodesCountAllowed) {
          return (React__namespace.createElement("div", { className: "minimap-container", style: minimapContainerStyle }, onRenderUnavailable()));
      }
      return (React__namespace.createElement("div", { className: "minimap-container " + classes.minimap, style: minimapContainerStyle },
          staticGraphEl,
          React__namespace.createElement("svg", __assign({ className: classes.minimapSvg }, touchController.eventHandlers, { onMouseDown: onMouseDown, ref: svgRef, "data-automation-id": "minimap-id" }),
              React__namespace.createElement(MiniMapShadow, { containerRect: rect, viewport: viewport, shadowPadding: shadowPadding, onClick: onClick })),
          arrowParams.showArrow && renderArrow(arrowParams.arrowDeg)));
  };

  var ContextMenu = function (props) {
      var theme = useTheme().theme;
      var classes = mergeStyleSets({
          contextMenu: {
              fontSize: 14,
              background: theme.contextMenuBackground,
              color: theme.fontColor,
              border: "1px solid " + theme.contextMenuBorder,
              width: 193,
              lineHeight: "20px",
              cursor: "default",
              "> *": {
                  padding: 8,
                  ":hover": {
                      background: theme.contextMenuHoverBackground
                  }
              }
          }
      });
      var propsMerged = __assign(__assign({}, props), { className: classes.contextMenu + " " + props.className });
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerContextMenu(propsMerged);
      return React__namespace.createElement("div", null, props.children);
  };

  var NodeMenu = function (props) {
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerMenu(props.children, exports.MenuType.Node);
      return null;
  };

  var EdgeMenu = function (props) {
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerMenu(props.children, exports.MenuType.Edge);
      return null;
  };

  var PortMenu = function (props) {
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerMenu(props.children, exports.MenuType.Port);
      return null;
  };

  var CanvasMenu = function (props) {
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerMenu(props.children, exports.MenuType.Canvas);
      return null;
  };

  var MultiMenu = function (props) {
      var contextMenuConfig = useContextMenuConfigContext();
      contextMenuConfig.registerMenu(props.children, exports.MenuType.Multi);
      return null;
  };

  var TestAPI = (function () {
      function TestAPI(adapter) {
          this.adapter = adapter;
      }
      TestAPI.prototype.getNodesCount = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.selectAll("g[data-automation-id*='node-container-']")];
                      case 1: return [2, (_a.sent()).length];
                  }
              });
          });
      };
      TestAPI.prototype.getEdgesCount = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.selectAll("g[data-automation-id*='edge-container-']")];
                      case 1: return [2, (_a.sent()).length];
                  }
              });
          });
      };
      TestAPI.prototype.addNodeFromItemPanel = function (options) {
          return __awaiter(this, void 0, void 0, function () {
              var addColumnsModuleInPanelElPos, canvasBBOx;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.getCentralPosBySelector(options.itemSelector)];
                      case 1:
                          addColumnsModuleInPanelElPos = _a.sent();
                          return [4, this.adapter.mouseMove(addColumnsModuleInPanelElPos.x, addColumnsModuleInPanelElPos.y)];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.mouseDown()];
                      case 3:
                          _a.sent();
                          return [4, this.getCanvasBoundingBox()];
                      case 4:
                          canvasBBOx = _a.sent();
                          return [4, this.adapter.mouseMove(canvasBBOx.x + options.x, canvasBBOx.y + options.y)];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.mouseUp()];
                      case 6:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.connectTwoNodes = function (_a) {
          var sourceNodeName = _a.sourceNodeName, sourcePortName = _a.sourcePortName, targetNodeName = _a.targetNodeName, targetPortName = _a.targetPortName;
          return __awaiter(this, void 0, void 0, function () {
              var sourcePos, targetPos;
              return __generator(this, function (_b) {
                  switch (_b.label) {
                      case 0: return [4, this.getCentralPosBySelector(this.getPortCssSelector(sourceNodeName, sourcePortName))];
                      case 1:
                          sourcePos = _b.sent();
                          return [4, this.adapter.mouseMove(sourcePos.x, sourcePos.y)];
                      case 2:
                          _b.sent();
                          return [4, this.adapter.mouseDown()];
                      case 3:
                          _b.sent();
                          return [4, this.getCentralPosBySelector(this.getPortCssSelector(targetNodeName, targetPortName))];
                      case 4:
                          targetPos = _b.sent();
                          return [4, this.adapter.mouseMove(targetPos.x, targetPos.y)];
                      case 5:
                          _b.sent();
                          return [4, this.adapter.mouseUp()];
                      case 6:
                          _b.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.draggingNode = function (nodeName, toPosition) {
          return __awaiter(this, void 0, void 0, function () {
              var nodeCenterPos, canvasBBOx;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.getCentralPosBySelector(this.getNodeCssSelector(nodeName))];
                      case 1:
                          nodeCenterPos = _a.sent();
                          return [4, this.adapter.mouseMove(nodeCenterPos.x, nodeCenterPos.y)];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.mouseDown()];
                      case 3:
                          _a.sent();
                          return [4, this.getCanvasBoundingBox()];
                      case 4:
                          canvasBBOx = _a.sent();
                          return [4, this.adapter.mouseMove(toPosition.x + canvasBBOx.x, toPosition.y + canvasBBOx.y)];
                      case 5:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.rightClickOnNode = function (nodeName) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.rightClick(this.getNodeCssSelector(nodeName))];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.clickOnNode = function (nodeName) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.click(this.getNodeCssSelector(nodeName))];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.hoverOnNode = function (nodeName) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.hover(this.getNodeCssSelector(nodeName))];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.hoverOnPort = function (nodeName, portName) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.hover(this.getPortCssSelector(nodeName, portName))];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.hoverOnEdge = function (edgeId) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.hover(this.getEdgeCssSelector(edgeId))];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.getAutoAlignHintLineCount = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.selectAll("line.auto-align-hint")];
                      case 1: return [2, (_a.sent()).length];
                  }
              });
          });
      };
      TestAPI.prototype.clickOnNodesWithCtrlKey = function (nodeNames) {
          return __awaiter(this, void 0, void 0, function () {
              var _i, nodeNames_1, name_1;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.keyDown("Control")];
                      case 1:
                          _a.sent();
                          _i = 0, nodeNames_1 = nodeNames;
                          _a.label = 2;
                      case 2:
                          if (!(_i < nodeNames_1.length)) return [3, 5];
                          name_1 = nodeNames_1[_i];
                          return [4, this.clickOnNode(name_1)];
                      case 3:
                          _a.sent();
                          _a.label = 4;
                      case 4:
                          _i++;
                          return [3, 2];
                      case 5: return [4, this.adapter.keyUp("Control")];
                      case 6:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.rightClickOnCanvas = function (x, y) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.rightClick("svg.react-dag-editor-svg-container", {
                              position: { x: x, y: y }
                          })];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.selectArea = function (start, end) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.mouseMove(start.x, start.y)];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.mouseDown()];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.mouseMove(end.x, end.y)];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.mouseUp()];
                      case 4:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.connectWithKeyboard = function (isCancelConnect) {
          if (isCancelConnect === void 0) { isCancelConnect = false; }
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyDown("Alt")];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.keyPress("c")];
                      case 6:
                          _a.sent();
                          return [4, this.adapter.keyUp("Alt")];
                      case 7:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 8:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 9:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 10:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 11:
                          _a.sent();
                          if (!isCancelConnect) return [3, 13];
                          return [4, this.adapter.keyPress("Escape")];
                      case 12:
                          _a.sent();
                          return [3, 15];
                      case 13: return [4, this.adapter.keyPress("Enter")];
                      case 14:
                          _a.sent();
                          _a.label = 15;
                      case 15: return [2];
                  }
              });
          });
      };
      TestAPI.prototype.navigateAroundNodesWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowRight")];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowLeft")];
                      case 6:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.navigateBackWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyDown("Shift")];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 6:
                          _a.sent();
                          return [4, this.adapter.keyUp("Shift")];
                      case 7:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.navigateAroundPortsWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 6:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.goToConnectedPortWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 5:
                          _a.sent();
                          return [4, this.adapter.keyDown("Alt")];
                      case 6:
                          _a.sent();
                          return [4, this.adapter.keyPress("g")];
                      case 7:
                          _a.sent();
                          return [4, this.adapter.keyUp("Alt")];
                      case 8:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.deleteNodeWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.focusOnCanvasWithKeyboard()];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyPress("Tab")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("ArrowDown")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyPress("Delete")];
                      case 4:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.panCanvas = function (from, to) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.mouseMove(from.x, from.y)];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.mouseDown()];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.mouseMove(to.x, to.y)];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.mouseUp()];
                      case 4:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.getElementBoundingBox = function (cssSelector) {
          return __awaiter(this, void 0, void 0, function () {
              var el, _a, _b, x, _c, y, _d, height, _e, width;
              return __generator(this, function (_f) {
                  switch (_f.label) {
                      case 0: return [4, this.adapter.waitForSelector(cssSelector)];
                      case 1:
                          el = _f.sent();
                          return [4, el.boundingBox()];
                      case 2:
                          _a = (_f.sent()) || {}, _b = _a.x, x = _b === void 0 ? 0 : _b, _c = _a.y, y = _c === void 0 ? 0 : _c, _d = _a.height, height = _d === void 0 ? 0 : _d, _e = _a.width, width = _e === void 0 ? 0 : _e;
                          return [2, {
                                  x: x,
                                  y: y,
                                  width: width,
                                  height: height
                              }];
                  }
              });
          });
      };
      TestAPI.prototype.getCanvasBoundingBox = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getElementBoundingBox("svg.react-dag-editor-svg-container")];
              });
          });
      };
      TestAPI.prototype.getMinimapBoundingBox = function () {
          return __awaiter(this, void 0, void 0, function () {
              var _a, x, y, height, width;
              return __generator(this, function (_b) {
                  switch (_b.label) {
                      case 0: return [4, this.getElementBoundingBox("svg[data-automation-id*='minimap-id']")];
                      case 1:
                          _a = _b.sent(), x = _a.x, y = _a.y, height = _a.height, width = _a.width;
                          return [2, {
                                  x: x,
                                  y: y,
                                  width: width,
                                  height: height
                              }];
                  }
              });
          });
      };
      TestAPI.prototype.dragOnItem = function (start, end) {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.selectArea(start, end)];
                      case 1:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      TestAPI.prototype.getNodeCentralPosition = function (nodeName) {
          return __awaiter(this, void 0, void 0, function () {
              var pos;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.getCentralPosBySelector(this.getNodeCssSelector(nodeName))];
                      case 1:
                          pos = _a.sent();
                          return [2, pos];
                  }
              });
          });
      };
      TestAPI.prototype.getNodes = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getDomElements("g[data-automation-id*='node-container-']")];
              });
          });
      };
      TestAPI.prototype.getEdges = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getDomElements("g[data-automation-id*='edge-container-']")];
              });
          });
      };
      TestAPI.prototype.getPorts = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getDomElements("g[aria-roledescription='port']")];
              });
          });
      };
      TestAPI.prototype.getNodeTooltips = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getDomElements(".node-tooltips")];
              });
          });
      };
      TestAPI.prototype.getPortTooltips = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  return [2, this.getDomElements(".port-tooltips")];
              });
          });
      };
      TestAPI.prototype.composeGraph = function () {
          return __awaiter(this, void 0, void 0, function () {
              var nodes, edges, ports, nodeTooltips, portTooltips;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.getNodes()];
                      case 1:
                          nodes = _a.sent();
                          return [4, this.getEdges()];
                      case 2:
                          edges = _a.sent();
                          return [4, this.getPorts()];
                      case 3:
                          ports = _a.sent();
                          return [4, this.getNodeTooltips()];
                      case 4:
                          nodeTooltips = _a.sent();
                          return [4, this.getPortTooltips()];
                      case 5:
                          portTooltips = _a.sent();
                          return [2, {
                                  nodes: nodes,
                                  edges: edges,
                                  ports: ports,
                                  nodeTooltips: nodeTooltips,
                                  portTooltips: portTooltips
                              }];
                  }
              });
          });
      };
      TestAPI.prototype.getCentralPosBySelector = function (cssSelector) {
          return __awaiter(this, void 0, void 0, function () {
              var _a;
              return __generator(this, function (_b) {
                  switch (_b.label) {
                      case 0:
                          _a = this.getCentralPosByBBox;
                          return [4, this.getElementBoundingBox(cssSelector)];
                      case 1: return [2, _a.apply(this, [_b.sent()])];
                  }
              });
          });
      };
      TestAPI.prototype.getDomElements = function (selector) {
          var _a;
          return __awaiter(this, void 0, void 0, function () {
              var _b, _c;
              var _this = this;
              return __generator(this, function (_d) {
                  switch (_d.label) {
                      case 0:
                          _c = (_b = Promise).all;
                          return [4, this.adapter.selectAll(selector)];
                      case 1: return [2, _c.apply(_b, [(_a = (_d.sent())) === null || _a === void 0 ? void 0 : _a.map(function (handle) { return __awaiter(_this, void 0, void 0, function () {
                                  var bbox, className, innerHTML, textContent, ariaLabel, automationId;
                                  return __generator(this, function (_a) {
                                      switch (_a.label) {
                                          case 0: return [4, handle.boundingBox()];
                                          case 1:
                                              bbox = _a.sent();
                                              return [4, handle.getAttribute("class")];
                                          case 2:
                                              className = _a.sent();
                                              return [4, handle.innerHTML()];
                                          case 3:
                                              innerHTML = _a.sent();
                                              return [4, handle.textContent()];
                                          case 4:
                                              textContent = _a.sent();
                                              return [4, handle.getAttribute("aria-label")];
                                          case 5:
                                              ariaLabel = _a.sent();
                                              return [4, handle.getAttribute("data-automation-id")];
                                          case 6:
                                              automationId = _a.sent();
                                              return [2, {
                                                      bbox: bbox,
                                                      textContent: textContent,
                                                      className: className,
                                                      innerHTML: innerHTML,
                                                      ariaLabel: ariaLabel,
                                                      automationId: automationId
                                                  }];
                                      }
                                  });
                              }); })])];
                  }
              });
          });
      };
      TestAPI.prototype.getCentralPosByBBox = function (bbox) {
          if (!bbox) {
              return { x: 0, y: 0 };
          }
          return {
              x: bbox.width / 2 + bbox.x,
              y: bbox.height / 2 + bbox.y
          };
      };
      TestAPI.prototype.getPortCssSelector = function (nodeName, portName) {
          return "g[data-automation-id*='" + nodeName + "'][data-automation-id*='" + portName + "'][data-automation-id*='port-']";
      };
      TestAPI.prototype.getNodeCssSelector = function (nodeName) {
          return "g[data-automation-id*='node-container-'][data-automation-id*='" + nodeName + "']";
      };
      TestAPI.prototype.getEdgeCssSelector = function (edgeId) {
          return "g[data-automation-id*='edge-container-" + edgeId + "']";
      };
      TestAPI.prototype.focusOnCanvasWithKeyboard = function () {
          return __awaiter(this, void 0, void 0, function () {
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0: return [4, this.adapter.keyDown("Alt")];
                      case 1:
                          _a.sent();
                          return [4, this.adapter.keyDown("Shift")];
                      case 2:
                          _a.sent();
                          return [4, this.adapter.keyPress("f")];
                      case 3:
                          _a.sent();
                          return [4, this.adapter.keyUp("Alt")];
                      case 4:
                          _a.sent();
                          return [4, this.adapter.keyUp("Shift")];
                      case 5:
                          _a.sent();
                          return [2];
                  }
              });
          });
      };
      return TestAPI;
  }());

  function composeReducers(reducers) {
      return function (next) { return reducers.reduceRight(function (prev, current) { return current(prev); }, next); };
  }

  exports.CanvasMenu = CanvasMenu;
  exports.ContextMenu = ContextMenu;
  exports.ContextMenuConfig = ContextMenuConfig;
  exports.ContextMenuConfigContext = ContextMenuConfigContext;
  exports.DEFAULT_GRAPH_SETTINGS = DEFAULT_GRAPH_SETTINGS;
  exports.DEFAULT_NODE_MAX_VISIBLE_SIZE = DEFAULT_NODE_MAX_VISIBLE_SIZE;
  exports.DEFAULT_NODE_MIN_VISIBLE_SIZE = DEFAULT_NODE_MIN_VISIBLE_SIZE;
  exports.Debug = Debug;
  exports.DefaultClipboard = DefaultClipboard;
  exports.DefaultStorage = DefaultStorage;
  exports.EMPTY_CONNECT_STATE = EMPTY_CONNECT_STATE;
  exports.EMPTY_GAP = EMPTY_GAP;
  exports.EMPTY_GRAPH_STATE = EMPTY_GRAPH_STATE;
  exports.EMPTY_TRANSFORM_MATRIX = EMPTY_TRANSFORM_MATRIX;
  exports.EMPTY_VIEW_PORT = EMPTY_VIEW_PORT;
  exports.EdgeMenu = EdgeMenu;
  exports.EdgeModel = EdgeModel;
  exports.GanttChartFeatures = GanttChartFeatures;
  exports.Graph = Graph;
  exports.GraphConfig = GraphConfig;
  exports.GraphConfigBuilder = GraphConfigBuilder;
  exports.GraphConfigContext = GraphConfigContext;
  exports.GraphModel = GraphModel;
  exports.GraphStateContext = GraphStateContext;
  exports.GraphValueContext = GraphValueContext;
  exports.Item = Item;
  exports.LinearEquation = LinearEquation;
  exports.Minimap = Minimap;
  exports.MultiMenu = MultiMenu;
  exports.NodeMenu = NodeMenu;
  exports.NodeModel = NodeModel;
  exports.PortMenu = PortMenu;
  exports.ReactDagEditor = ReactDagEditor;
  exports.SELECTED = SELECTED;
  exports.StaticGraph = StaticGraph;
  exports.TestAPI = TestAPI;
  exports.ThemeContext = ThemeContext$1;
  exports.ViewportContext = ViewportContext;
  exports.addState = addState;
  exports.allFeatures = allFeatures;
  exports.applyDefaultPortsPosition = applyDefaultPortsPosition;
  exports.applyHorizontalGraphPortsPosition = applyHorizontalGraphPortsPosition;
  exports.arrayEquals = arrayEquals;
  exports.canRedo = canRedo;
  exports.canUndo = canUndo;
  exports.checkRectIntersect = checkRectIntersect;
  exports.clamp = clamp;
  exports.composeReducers = composeReducers;
  exports.constantEmptyArray = constantEmptyArray;
  exports.dataReadonlyMode = dataReadonlyMode;
  exports.debounce = debounce;
  exports.deepClone = deepClone;
  exports.defaultFeatures = defaultFeatures;
  exports.defaultGetPositionFromEvent = defaultGetPositionFromEvent;
  exports.defaultGraphStateContext = defaultGraphStateContext;
  exports.defaultPort = defaultPort;
  exports.diffGraphEdge = diffGraphEdge;
  exports.diffGraphNode = diffGraphNode;
  exports.diffGraphPort = diffGraphPort;
  exports.distance = distance;
  exports.filterSelectedItems = filterSelectedItems;
  exports.focusArea = focusArea;
  exports.getBrowser = getBrowser;
  exports.getClientDeltaByPointDelta = getClientDeltaByPointDelta;
  exports.getClientPointFromRealPoint = getClientPointFromRealPoint;
  exports.getContainer = getContainer;
  exports.getContainerCenter = getContainerCenter;
  exports.getContainerClientPoint = getContainerClientPoint;
  exports.getContentArea = getContentArea$1;
  exports.getCriticalPath = getCriticalPath;
  exports.getCurvePathD = getCurvePathD;
  exports.getEdgeSourceTargetCoordinate = getEdgeSourceTargetCoordinate;
  exports.getEdgeUid = getEdgeUid;
  exports.getGroupRect = getGroupRect;
  exports.getLinearFunction = getLinearFunction;
  exports.getMinimapRect = getMinimapRect;
  exports.getNearestConnectablePort = getNearestConnectablePort;
  exports.getNeighborPorts = getNeighborPorts;
  exports.getNodeAutomationId = getNodeAutomationId;
  exports.getNodeConfig = getNodeConfig;
  exports.getNodeRect = getNodeRect;
  exports.getNodeSize = getNodeSize;
  exports.getNodeUid = getNodeUid;
  exports.getOffsetLimit = getOffsetLimit;
  exports.getPointDeltaByClientDelta = getPointDeltaByClientDelta;
  exports.getPortAutomationId = getPortAutomationId;
  exports.getPortPosition = getPortPosition;
  exports.getPortPositionByPortId = getPortPositionByPortId;
  exports.getPortUid = getPortUid;
  exports.getRealPointFromClientPoint = getRealPointFromClientPoint;
  exports.getRectHeight = getRectHeight;
  exports.getRectWidth = getRectWidth;
  exports.getRelativePoint = getRelativePoint;
  exports.getRenderedArea = getRenderedArea;
  exports.getRenderedEdges = getRenderedEdges;
  exports.getRenderedNodes = getRenderedNodes;
  exports.getScaleLimit = getScaleLimit;
  exports.getScaleRange = getScaleRange;
  exports.getTopoSortingNodes = getTopoSortingNodes;
  exports.getVisibleArea = getVisibleArea;
  exports.getVisibleNodes = getVisibleNodes;
  exports.getZoomFitMatrix = getZoomFitMatrix;
  exports.hasSelectedPort = hasSelectedPort;
  exports.hasState = hasState;
  exports.isCanvasEvent = isCanvasEvent;
  exports.isConnectable = isConnectable;
  exports.isEdgeEvent = isEdgeEvent;
  exports.isMacOs = isMacOs;
  exports.isMobile = isMobile;
  exports.isNodeActivated = isNodeActivated;
  exports.isNodeEditing = isNodeEditing;
  exports.isNodeEvent = isNodeEvent;
  exports.isNodeVisible = isNodeVisible;
  exports.isPointInRect = isPointInRect;
  exports.isPointVisible = isPointVisible;
  exports.isPortEvent = isPortEvent;
  exports.isRectVisible = isRectVisible;
  exports.isSelected = isSelected;
  exports.isSupported = isSupported;
  exports.isViewportComplete = isViewportComplete;
  exports.isViewportEmpty = isViewportEmpty;
  exports.isWithinRect = isWithinRect;
  exports.isWithinThreshold = isWithinThreshold;
  exports.line = line;
  exports.markCriticalPath = markCriticalPath;
  exports.markEdgeDirty = markEdgeDirty;
  exports.minimapPan = minimapPan;
  exports.nodeSelection = nodeSelection;
  exports.notSelected = notSelected;
  exports.pan = pan;
  exports.pickState = pickState;
  exports.previewMode = previewMode;
  exports.pushHistory = pushHistory;
  exports.rect = rect;
  exports.redo = redo;
  exports.removeState = removeState;
  exports.resetConnectState = resetConnectState;
  exports.resetNodePortsState = resetNodePortsState;
  exports.resetState = resetState;
  exports.resetUndoStack = resetUndoStack;
  exports.resolveValueOrCallback = resolveValueOrCallback;
  exports.reverseTransformPoint = reverseTransformPoint;
  exports.scrollIntoView = scrollIntoView;
  exports.setData = setData;
  exports.stopPropagation = stopPropagation;
  exports.toggleState = toggleState;
  exports.transformPoint = transformPoint;
  exports.unSelectAllEntity = unSelectAllEntity;
  exports.undo = undo;
  exports.updateData = updateData;
  exports.updateNodeState = updateNodeState;
  exports.updateState = updateState;
  exports.useClientRect = useClientRect;
  exports.useContainerRect = useContainerRect;
  exports.useContextMenuConfigContext = useContextMenuConfigContext;
  exports.useDeferredValue = useDeferredValue;
  exports.useGraphData = useGraphData;
  exports.useGraphState = useGraphState;
  exports.useGraphTouchHandler = useGraphTouchHandler;
  exports.useMinimapRect = useMinimapRect;
  exports.useRenderedArea = useRenderedArea;
  exports.useSafariScale = useSafariScale;
  exports.useSelectBox = useSelectBox;
  exports.useTheme = useTheme;
  exports.useThrottle = useThrottle;
  exports.useUpdateViewportCallback = useUpdateViewportCallback;
  exports.useWheelHandler = useWheelHandler;
  exports.zoom = zoom;
  exports.zoomTo = zoomTo;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.js.map
